/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Initialisation d'une bote de dialogue
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_
# define _LIBRAIRIE_X_ /* Pour avoir accs aux fonctions internes */
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "bib_interface.h"

/* -------- Initialiser une bote de dialogue ---------------------\
 |    Renvoie 0 si tout s'est bien pass, != 0 en cas de problme  |
 |                                                                 |
 |   La partie spcifique dy systme utilis est faite en fin, dans|
 | la fonction ARB_InitSpecifique()                                |
 |                                                                 |
 | arbre    : le numro de l'arbre que l'on initialise             |
 | URL_aide : la page HTML  ouvrir pour avoir de l'aide           |
 | gestion  : la fonction  appeler en cas de clic sur un objet    |
 | clavier  : la fonction  appeler pour grer le clavier          |
 \---------------------------------------------------------------- */

int ARB_Initialiser(unsigned int arbre, const char *URL_aide, int (*gestion)(unsigned int, int , int), int (*clavier)(unsigned int, int, int *))
{
  OBJECT *adresse;
  unsigned int objet;
  int retour;

  if ( arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numro d'arbre trop grand [%u / %u max]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, BI_nbr_arbres - 1);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( NULL == arbres[ arbre ].adresse ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune table d'objets dans l'arbre [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -2;
  }
  if ( VRAI == arbres[ arbre ].drapeaux.init ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : arbre dj initialis (!) [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
#endif
    return 0; /* Arbre dj initialis, inutile de le refaire ! */
  }

  if ( 0 == arbres[ arbre ].nbr_objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : ATTENTION, nombre d'objets dans l'arbre inconnu [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    arbres[ arbre ].nbr_objets = ARB_CompterObjets( arbre );
#ifdef _DEVELOPPE_
  } else { /* Contrle de cohrence interne ! */
    unsigned int nbr_objets;

    nbr_objets = ARB_CompterObjets( arbre );
    if ( nbr_objets != arbres[ arbre ].nbr_objets ) {
      (void) fprintf(stderr, "%s,%s l.%u : L'arbre %u contient %u objets, mais dit en contenir %u\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre, nbr_objets, arbres[ arbre ].nbr_objets);
      FIN_ANTICIPEE(FIN_ERREUR_GRAVE);
    }
#endif
  }

  adresse = arbres[ arbre ].adresse;
  /* Objets spciaux : mis  ROOT = 0 dans la rservation de l'arbre [calloc] : on n'y touche pas, s'ils ont t changs... */

  /* Prparation des coordonnes & infos complmentaires */
  for ( objet = ROOT; objet < arbres[ arbre ].nbr_objets ; objet++ ) { /* On parcourt tous les objets */
    /* Recherche des boutons de sortie (par heuristiques, pour Annuler...) */
    if ( adresse[ objet ].ob_flags & EXIT ) {
      const char *chaine;

      chaine = lit_chaine( arbre, objet, 0, NULL, TXT_REMPLACE );  /* Juste pour une comparaison : pas besoin d'une copie ! */
      if ( chaine != NULL ) {
	if ( ( ROOT == arbres[ arbre ].objet_annuler ) && ( strcmp(chaine, "Annuler") == 0 ) ) {
	  arbres[ arbre ].objet_annuler = objet;
	}
      }
    }

    /* Traitement des objets suivant leur type */
    switch ( adresse[ objet ].ob_type )
    {
    case G_FTEXT: case G_FBOXTEXT:
      if ( adresse[ objet ].ob_spec.tedinfo != NULL ) {
	char *texte;

	texte = adresse[ objet ].ob_spec.tedinfo->te_ptext.editable; /* si ditable, doit forcment tre crivable
									=> cre avec malloc() */
	if ( texte != NULL ) {
	  size_t lg_texte;

	  /* Arbre interne : texte en static a priori => on en fait un strdup */
	  if ( VRAI == arbres[ arbre ].drapeaux.interne ) {
	    texte = strdup( texte );
	    if ( NULL == texte ) {
	      (void) fprintf(stderr, "%s,%sl.%u : Impossible de rserver la copie du texte !\n",
			     __FILE__, __FUNCTION__, __LINE__);
	      return -43;
	    }
	    adresse[ objet ].ob_spec.tedinfo->te_ptext.editable = texte;
	  }

	  if ( ( *texte == '@' ) || ( *texte == '_') ) { /* Chane initialise, aucun texte */
	    unsigned int i;

	    for (i = 0 ; i < adresse[ objet ].ob_spec.tedinfo->te_txtlen ; i++)
	      texte[ i ] = 0;
	  }

	  /* On remplace les '_' finaux par des 0 */
	  lg_texte = strlen( texte );
	  while ( ( lg_texte > 0 ) && ( texte[ lg_texte - 1 ] == '_' ) ) {
	    texte[ lg_texte - 1 ] = 0;
	    if ( lg_texte > 1 )
	      lg_texte--;
	  }
	} else {
	  if ( FAUX == arbres[ arbre ].drapeaux.interne ) {
	    (void) fprintf(stderr, "%s,%sl.%u : Arbre externe (RSC) sans texte dans la TEDINFO : absurde !\n",
			   __FILE__, __FUNCTION__, __LINE__);
	    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	    return -48;
	  }

	  texte = (char *) calloc( adresse[ objet ].ob_spec.tedinfo->te_txtlen, sizeof( char ) );
	  if ( NULL == texte ) {
	    (void) fprintf(stderr, "%s,%sl.%u : Impossible de rserver le texte  diter\n",
			   __FILE__, __FUNCTION__, __LINE__);
	    return +4;
	  }
	  adresse[ objet ].ob_spec.tedinfo->te_ptext.editable = texte;
	}
      }
      break;

#ifdef _DEVELOPPE_
    case G_TXT_INFINI:
    case G_NOMBRE:
      break;
#endif

    case G_BUTTON:
      if ( adresse[ objet ].ob_state & SHADOWED ) {
	/* Pas de changement au vol, car difficile de distinguer facilement les menus avec des textes fixes et ceux avec des textes rservs
	   ==> seuls les menus avec textes fixes ou assimilables sont retenus ! */
	AVERTISSEMENT("L'objet %u, arbre %u, pourrait tre un sous-menu", objet, arbre );
/* 	OBJ_ChangerType( arbre, objet, G_SSMENU ); */
      }
      break;

#ifdef _DEVELOPPE_
    case G_STRING:
    case G_TITLE:
    case G_BOXTEXT:
    case G_TEXT:
    case G_ICON:
    case G_IBOX:
    case G_BOXCHAR:
    case G_BOX:
    case G_SEPARATEUR:
    case G_USERDEF:
    case G_SSMENU:
    case G_STYLELIGNE:
    case G_STYLEMARQUE:
      break;

    default:
      (void) fprintf(stderr, "%s, %s l.%u : Objet %u de type inconnu [%d]\n",
		     __FILE__, __FUNCTION__, __LINE__, objet, adresse[ objet ].ob_type);
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
#endif
    }

    /* Conversion caractres -> cran */
    RSC_CarVEcran( arbre, objet );
  }

  /* Partie spcifique du systme d'exploitation */
  retour = ARB_InitSpecifique( arbre );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation spcifique [code %d ; arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, arbre);
    return -10; /* Arbre dj initialis, inutile de le refaire ! */
  }

  /* On recherche le premier et le dernier objet ditable.
     Aprs le traitement des objets, car changements de types possibles & des textes peuvent tre crs au vol !*/
  (void) ARB_TrouverEditables( arbre, FAUX );

  /* Aide associe  la bote de dialogue */
  arbres[ arbre ].URL_aide = URL_aide;
  if ( VRAI == arbres[ arbre ].drapeaux.avec_aide ) {
    /* On rserve pour les textes d'aide */
    arbres[ arbre ].txt_aides = (const char **) calloc( arbres[ arbre ].nbr_objets, sizeof(char *));
    if ( arbres[ arbre ].txt_aides == NULL ) {
      (void) fprintf(stderr, "%s,%s l.%u : mmoire insuffisante pour allouer les textes d'aide\n --> ces textes ne seront pas affichs\n",
		     __FILE__, __FUNCTION__, __LINE__);
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "Arbre : %d - %u objets\n", arbre, arbres[ arbre ].nbr_objets);
#endif
      arbres[ arbre ].drapeaux.avec_aide = FAUX;
    }
  }

  /* Fonctions associes  la bote de dialogue */
  arbres[ arbre ].fonction.arbre = gestion;
  arbres[ arbre ].clavier = clavier;

  /* Tout s'est bien pass */
  arbres[ arbre ].drapeaux.init = VRAI;
  return 0;
}

