/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des arbres / botes de dialogue
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_
# define _LIBRAIRIE_X_ /* Pour avoir accs aux fonctions internes */
#elif defined(_WINDOWS_)
# define _LIBRAIRIE_W_
#endif

#include "bib_interface.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static OBJECT *ARB_ControleObjet(unsigned int num_arbre, unsigned int objet);
#ifdef _DEVELOPPE_
static void ARB_AfficherFils     (unsigned int num_arbre, unsigned int objet, unsigned int delta);
static void ARB_AfficherObjet    (unsigned int num_arbre, unsigned int num_objet, unsigned int delta);
#endif

/* ~-~-~-~-~-~-~-~- Gestion de la table des arbres ~-~-~-~-~-~-~-~- */

/* -------- Prparer la table des arbres -------- */

Arbre *BI_ARB_CreerArbres( unsigned int nombre_arbres )
{
  Arbre *arbres;
  unsigned int i;

  if ( 0 == nombre_arbres ) {
    return NULL;
  }

  /* Rservation de la mmoire */
  arbres = calloc( nombre_arbres, sizeof( Arbre ) );
  if ( NULL == arbres ) {
    (void) fprintf(stderr, "BI_ARB_CreerArbres : mmoire insuffisante pour crer les arbres...\n");
    return NULL;
  }

  /* Initialisation */
  /* calloc met tout  0 => les pointeurs sont  mettre  NULL */
  for ( i = 0 ; i < nombre_arbres ; i++ ) {
    arbres[ i ].adresse = NULL;
    arbres[ i ].fonction.arbre = ( int (*)(unsigned int, int, int) ) NULL;
    arbres[ i ].clavier = ( int (*)(unsigned int, int, int *) ) NULL;
    arbres[ i ].complement = NULL;
    arbres[ i ].URL_aide = NULL;
    arbres[ i ].txt_aides = NULL;
#ifdef _LIBRAIRIE_X_
    arbres[ i ].objets = NULL;
#endif
  }

  /* On a fini, tout s'est bien pass */
  return arbres;
}

/* -------- Compter le nombre d'objets dans un arbre -------- */

unsigned int ARB_CompterObjets(unsigned int arbre)
{
  OBJECT *adresse;
  unsigned int dernier_objet;

  /* Contrles prliminaires */
  if ( arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numro d'arbre trop grand [%u / %u max]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, BI_nbr_arbres - 1);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return 0;
  }
  if ( NULL == arbres[ arbre ].adresse ) {
    (void) fprintf(stderr, "%s,%s l.%u : arbre sans objets [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return 0;
  }

  adresse = arbres[ arbre ].adresse;

  /* On cherche le dernier objet dans l'arbre */
  dernier_objet = ROOT;
#ifdef _DEVELOPPE_
  (void) printf("%s, arbre %u : Objets de fin = ROOT", __FUNCTION__, arbre);
#endif
  /* 1. On va le plus loin possible en sautant les objets fils */
  while ( OB_TAIL( adresse, dernier_objet ) != -1 ) {
    dernier_objet = (unsigned int) OB_TAIL(adresse, dernier_objet);
#ifdef _DEVELOPPE_
    printf(", %u", dernier_objet );
    if ( ROOT == dernier_objet ) {
      (void) fprintf(stderr, "%s,%s l.%u : ABSURDE, ob_tail = ROOT [arbre %u, objet %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, arbre, dernier_objet);
      printf("\n");
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }
#endif
  }
#ifdef _DEVELOPPE_
  printf("\n");
#endif

  /* 2. Y a-t-il encore des objets aprs (menu, par exemple) ? Si oui, on avance un par un... */  
  while ( ( OB_FLAGS( adresse, dernier_objet ) & LASTOB ) == FAUX ) {
    dernier_objet++; /* On passe donc au suivant */
  }
  /* dern_objet contient ainsi le numro du dernier objet de l'arbre */

  /* On en dduit le nombre d'objets */
  return dernier_objet + 1; /* Les index commencent  zro = ROOT */
}

/* ~-~-~-~-~-~-~-~- Construction d'arbres  partir d'un tableau cod en dur dans le programme ~-~-~-~-~-~-~-~- */

int BI_CreerArbre_Inclus(unsigned int num_arbre, unsigned int nbr_objets, OBJECT *adresse)
{
  int retour;
  unsigned int i;

  /* Contrle des paramtres passs */
  if ( NULL == adresse ) {
    (void) fprintf(stderr, "%s,%s l.%u : Tableau des objets NULL !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }
  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numro d'arbre trop grand [%u / %u max] !\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, BI_nbr_arbres - 1);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* On cre un vrai arbre, comme les autres.
     Plus sr, car on peut alors modifier sans risque les contenus... Et utiliser le mme arbre plusieurs fois ! */
  retour = BI_CreerArbre( num_arbre, nbr_objets, adresse[ ROOT ].ob_width, adresse[ ROOT ].ob_height );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer l'arbre %u [code %d] !\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, retour);
    return -3;
  }

  /* On initialise les objets */
  for ( i = 0 ; i < nbr_objets ; i++ ) {
    arbres[ num_arbre ].adresse[ i ] = adresse[ i ];

    switch ( adresse[ i ].ob_type )
    {
    case G_STRING:
    case G_BUTTON:
    case G_TITLE:
      /* ATTENTION, criture en dur -> IMPOSSIBLE  MODIFIER APRS */
      break;

    case G_TEXT: case G_BOXTEXT:
    case G_FTEXT: case G_FBOXTEXT:
      /* On cre la TEDINFO */
      retour = OBJ_PreparerComplement( num_arbre, i );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s, %s l. %u: ERREUR en voulant prparer la TEDINFO de l'objet %u {%u sur %u} [code %d] !\n",
		       __FILE__, __FUNCTION__, __LINE__, i, i + 1, nbr_objets, retour);
	return -3;
      }

      /* On la prpare  partir de celle indique en mmoire */
      if ( adresse[ i ].ob_spec.tedinfo != NULL ) {
	const char *chaine;

	/* Tous les paramtres sont identiques */
	*(arbres[ num_arbre ].adresse[ i ].ob_spec.tedinfo) = *(adresse[ i ].ob_spec.tedinfo);

	/* MAIS le texte modifi est forcment une chane alloue avec malloc() [sinon, seg. fault] */
	chaine = adresse[ i ].ob_spec.tedinfo->te_ptext.constant;
	if ( chaine != NULL ) {
	  arbres[ num_arbre ].adresse[ i ].ob_spec.tedinfo->te_ptext.editable = strdup( chaine );
	}

	/* Le masque et la validation n'ont aucune raison d'tre changs => un texte constant convient */
      }
      break;

    case G_USERDEF:
      retour = OBJ_PreparerComplement( num_arbre, i );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s, %s l. %u: ERREUR en voulant prparer la USERBLK de l'objet %u {%u sur %u} [code %d] !\n",
		       __FILE__, __FUNCTION__, __LINE__, i, i + 1, nbr_objets, retour);
	return -4;
      }
      break;

    case G_BOXCHAR:
    case G_BOX:
    case G_IBOX:
      arbres[ num_arbre ].adresse[ i ].ob_spec.gbox = adresse[ i ].ob_spec.gbox;

    case G_ICON:
      OPTION_A_FAIRE("Objet G_ICON");
      break;

    case G_CICON:
      OPTION_A_FAIRE("Objet G_CICON");
      break;

    case G_TXT_INFINI:
      retour = OBJ_PreparerComplement( num_arbre, i );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s, %s l. %u: ERREUR en voulant prparer le complment de l'objet %u {%u sur %u} [code %d] !\n",
		       __FILE__, __FUNCTION__, __LINE__, i, i + 1, nbr_objets, retour);
	return -4;
      }
      arbres[ num_arbre ].adresse[ i ].ob_spec.txt_infini->max_visibles = arbres[ num_arbre ].adresse[ i ].ob_width;
      break;

    default: /* On se contente de crer le complment */
      retour = OBJ_PreparerComplement( num_arbre, i );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s, %s l. %u: ERREUR en voulant prparer le complment de l'objet %u {%u sur %u} [code %d] !\n",
		       __FILE__, __FUNCTION__, __LINE__, i, i + 1, nbr_objets, retour);
	return -4;
      }
    }
  }

  arbres[ num_arbre ].drapeaux.interne = VRAI; /* Alors, certaines chanes sont statiques -> non librables a priori */
  /* Arbre cr sans encombre */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Construction d'arbres pice par pice ~-~-~-~-~-~-~-~- */

/* -------- Construction de l'arbre -------- */

int BI_CreerArbre(unsigned int num_arbre, unsigned int nbr_objets, unsigned int largeur, unsigned int hauteur)
{
  OBJECT *objets;
  unsigned int i;

  ENTREE_FONCTION( __FUNCTION__, "num_arbre=%u, nbr_objets=%u, largeur=%u, hauteur=%u", num_arbre, nbr_objets, largeur, hauteur );

  /* Contrles prliminaires */
  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s, %s l.%u : numro d'arbre trop grand [%u / %u max]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, BI_nbr_arbres - 1 );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    SORTIE_FONCTION( __FUNCTION__, "%d [numro d'arbre absurde]", -1 );
    return -1;
  }

  if ( VRAI == arbres[num_arbre].drapeaux.ouvert ) {
    (void) fprintf(stderr, "%s, %s l.%u : appel rcursif impossible (arbre %u dj ouvert)\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    SORTIE_FONCTION( __FUNCTION__, "%d [arbre dj ouvert]", -2 );
    return -2;
  }

  if ( NULL != arbres[num_arbre].adresse ) {
    (void) fprintf(stderr, "%s, %s l.%u : arbre %u dj existant !\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    SORTIE_FONCTION( __FUNCTION__, "%d [arbre dj existant]", -3 );
    return -3;
  }
#ifdef _LIBRAIRIE_X_
  if ( NULL != arbres[ num_arbre ].objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : Table des objets / fentre X dj prsente !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    SORTIE_FONCTION( __FUNCTION__, "%d [fentres X dj existantes]", -25 );
    return -25;
  }
#endif

  /* On rserve l'arbre */
  objets = (OBJECT *) calloc( nbr_objets, sizeof(OBJECT) );
  if ( NULL == objets ) {
    (void) fprintf(stderr, "%s, %s l.%u : impossible de gnrer les objets !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    SORTIE_FONCTION( __FUNCTION__, "%d [Mmoire insuffisante]", -4 );
    return -4;
  }

  /* La racine */
  objets[0].ob_next  = -1;
  objets[0].ob_head  = 1;
  objets[0].ob_tail  = nbr_objets - 1;
  objets[0].ob_type  = G_BOX;
  objets[0].ob_spec.gbox.couleur_fond = GRIS;
  objets[0].ob_spec.gbox.motif = 0;
  objets[0].ob_spec.gbox.opaque = VRAI;
  objets[0].ob_spec.gbox.couleur_texte = NOIR;
  objets[0].ob_spec.gbox.couleur_cadre = GRIS;
  objets[0].ob_spec.gbox.cadre = 0;
  objets[0].ob_flags = 0;
  objets[0].ob_state = 0;
  objets[0].ob_width = largeur;
  objets[0].ob_height = hauteur;

  /* Prparation de l'arbre pour l'cran */
  arbres[num_arbre].adresse = objets;
  arbres[num_arbre].nbr_objets = nbr_objets;

  /* Les objets n'ont pas d'enfant et se suivent simplement */
  for ( i = 1 ; i < nbr_objets - 1 ; i++ ) {
    objets[ i ].ob_next = i + 1;
    objets[ i ].ob_head = -1;
    objets[ i ].ob_tail = -1;

    objets[ i ].ob_spec.texte = NULL; /* Pour tre sr de ne pas avoir de mauvaise surprise ! */
  }

  /* Le dernier objet... */
  objets[nbr_objets - 1].ob_head   = -1;
  objets[nbr_objets - 1].ob_tail   = -1;
  objets[nbr_objets - 1].ob_next   = ROOT;
  objets[nbr_objets - 1].ob_flags |= LASTOB;

  /* Tout s'est bien pass */
  SORTIE_FONCTION( __FUNCTION__, "%d [Arbre cr]", 0 );
  return 0;
}

/* -------- Contrles avant construction -------- */

static OBJECT *ARB_ControleObjet(unsigned int num_arbre, unsigned int objet)
{
  OBJECT *objets;

  if ( num_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s, %s l.%u : numro d'arbre incorrect [%u]!\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre );
    return NULL;
  }
  objets = arbres[ num_arbre ].adresse;
  if (NULL == objets) {
    (void) fprintf(stderr, "%s, %s l.%u : table des objets absente [arbre %u]!\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre );
    return NULL;
  }

  if ( objet >= arbres[ num_arbre ].nbr_objets ) {
    (void) fprintf(stderr, "%s, %s l.%u : numro d'objet incorrect [%u / arbre %u : %u]!\n",
		   __FILE__, __FUNCTION__, __LINE__, objet, num_arbre, arbres[ num_arbre ].nbr_objets);
    return NULL;
  }

  return objets;
}

/* -------- Crer un objet "simple" -------- */

int ARB_CreerObjet(unsigned int num_arbre, unsigned int objet, int type, int x, int y, unsigned int larg, unsigned int haut)
{
  OBJECT *objets;
  int retour;

  objets = ARB_ControleObjet(num_arbre, objet);
  if (NULL == objets) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans ARB_ControleObjet()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* Arborescence : faite  la cration de l'arbre, ou ailleurs... */

  /* Type & flags spciaux */
  objets[ objet ].ob_type = type;
  if ( (G_IBOX == type) || (G_BUTTON == type) )
    objets[ objet ].ob_flags |= SELECTABLE;
  else if ( ( G_FTEXT == type ) || ( G_TEXT == type ) || ( G_TXT_INFINI == type ) || ( G_NOMBRE == type ) )
    objets[ objet ].ob_flags |= EDITABLE;

  /* Prparations complmentaires en fonction du type */
  retour = OBJ_PreparerComplement( num_arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer les complments [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }
 
  /* Position, dimensions */
#ifdef _DEVELOPPE_
  if ( ( x >= (signed) objets[ ROOT ].ob_width ) || ( y >= (signed) objets[ ROOT ].ob_height ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Position absurde [objet = %u type=%d X=%d Y=%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, objet, type, x, y);
    return -50;
  }
#endif
  objets[ objet ].ob_x = x;
  objets[ objet ].ob_y = y;
  objets[ objet ].ob_width  = larg;
  objets[ objet ].ob_height = haut;

  /* Inits qui dpendent de la taille */
  if ( G_NOMBRE == type ) {
    objets[ objet ].ob_spec.nombre->texte.max_visibles = larg;
  } else if ( G_TXT_INFINI == type ) {
    objets[ objet ].ob_spec.txt_infini->max_visibles = larg;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer une suite de chanes l'une au-dessous de l'autre --------
   Utilisation : texte simple si dans ROOT
                 textes d'un dfileur si pas dans ROOT
 */

int ARB_CreerNTextes(unsigned int num_arbre, unsigned int objet_1, unsigned int nombre, int pere, int x, int y, unsigned int larg)
{
  OBJECT *objets;
  unsigned int i;

  objets = ARB_ControleObjet(num_arbre, objet_1 + nombre - 1);
  if (NULL == objets) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans ARB_ControleObjet()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On cre les objets :  la suite, sans fils */
  for (i = 0 ; i < nombre ; i++) {
    objets[ objet_1 + i ].ob_type = G_STRING;

    objets[ objet_1 + i ].ob_spec.texte = NULL; /* Pas encore de texte ! */

    objets[ objet_1 + i ].ob_x = x;
    objets[ objet_1 + i ].ob_y = y + i;
    objets[ objet_1 + i ].ob_width = (int) larg;
    objets[ objet_1 + i ].ob_height = 1;
  }

  if ( pere != ROOT ) { /* Type dfileur... */
    unsigned int dernier_fils;

    dernier_fils = objet_1 + nombre - 1;

    /* Dans le mme niveau, on doit sauter tous ces fils */
    if ( dernier_fils == arbres[ num_arbre ].nbr_objets - 1 ) {
      objets[ pere ].ob_next = ROOT;
    } else {
      objets[ pere ].ob_next = dernier_fils + 1;
    }

    /* Le pre pointe sur eux */
    objets[ pere ].ob_head = objet_1;
    objets[ pere ].ob_tail = dernier_fils;

    /* Tous les textes sont slectionnables */
    for ( i = 0 ; i < nombre ; i++ )
      objets[ objet_1 + i ].ob_flags |= SELECTABLE;

    /* Le dernier pointe sur le pre comme "next" */
    objets[ dernier_fils ].ob_next = pere;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer une chane de caractres -------- */

int ARB_CreerTexte(unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg, const char *texte)
{
  int reponse;

  reponse = ARB_CreerObjet( num_arbre, objet, G_STRING, x, y, larg, 1 );
  if ( 0 != reponse ) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans ARB_CreerObjet() [%d / %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, objet );
    return reponse;
  }

  if ( texte != NULL ) {
    arbres[ num_arbre ].adresse[ objet ].ob_spec.texte = texte;
    if ( strlen( texte ) > larg ) {
      (void) fprintf(stderr, "%s, %s l.%u : objet trop petit pour le texte [largeur : %u - texte : %lu / %s]\n",
		     __FILE__, __FUNCTION__, __LINE__, larg, (unsigned long) strlen( texte ), texte );
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer un bouton isol -------- */

int ARB_CreerBouton(unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg, const char *texte)
{
  int reponse;

  reponse = ARB_CreerObjet(num_arbre, objet, G_BUTTON, x, y, larg, 1);
  if (0 != reponse) {
    (void) fprintf(stderr, "%s,%s l.%u :  erreur dans ARB_CreerObjet() [%d / %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, num_arbre, objet);
    return reponse;
  }

  if (texte != NULL) {
    arbres[ num_arbre ].adresse[ objet ].ob_spec.texte = texte;
    if ( strlen(texte) > larg ) {
      (void) fprintf(stderr, "%s,%s l.%u : objet trop petit pour le texte [largeur : %u - texte : %lu / %s]\n",
		     __FILE__, __FUNCTION__, __LINE__, larg, (unsigned long) strlen(texte), texte);
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer une case  cocher et le texte  ct -------- */

int ARB_CreerCaseACocher(unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg_texte, const char *texte)
{
  int reponse;

  /* On cre la case  cocher */
  reponse = ARB_CreerObjet( num_arbre, objet, G_IBOX, x, y, 2, 1 );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / \"%s\" [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, texte, reponse);
    return -1;
  }

  /* On cre le texte,  ct [si ncessaire] */
  if ( texte != NULL ) {
    reponse = ARB_CreerTexte( num_arbre, objet + 1, x + 3, y, larg_texte, texte );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le texte / \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, texte, reponse);
      return -8;
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer un objet ditable, avec texte avant -------- */

int ARB_CreerEditable(unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg_texte, unsigned int larg_editable,
		      unsigned int proprietes, const char *texte)
{
  int reponse;

  /* Si demand, on cre la case  cocher */
  if ( proprietes & ARB_OPTIONS_CASES ) {
    reponse = ARB_CreerObjet( num_arbre, objet, G_IBOX, x, y, 2, 1 );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, texte, reponse);
      return -1;
    }

    /* Elle est coche par dfaut : on utilise l'option */
    arbres[ num_arbre ].adresse[ objet ].ob_state |= CHECKED;

    objet++;
    x += 3;
  }

  /* On cre le texte, s'il y en a un */
  if ( texte != NULL ) {
    reponse = ARB_CreerTexte( num_arbre, objet, x, y, larg_texte, texte );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le texte / \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, texte, reponse);
      return -8;
    }
    /* L'ditable est donc l'objet suivant... */
    objet++;

    /* Et on corrige sa position */
    x += larg_texte + 1;
  }

  /* On cre l'objet editable */
  if ( proprietes & ARB_OPTIONS_EDIT_INFINI ) {
    Texte_Infini *txt_infini;

    if ( proprietes & ARB_OPTIONS_EDIT_NOMBRE ) {
      reponse = ARB_CreerObjet( num_arbre, objet, G_NOMBRE, x, y, larg_editable, 1 );
    } else {
      reponse = ARB_CreerObjet( num_arbre, objet, G_TXT_INFINI, x, y, larg_editable, 1 );
    }
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le champ ditable / \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, texte, reponse);
      return -1;
    }

    txt_infini = arbres[ num_arbre ].adresse[ objet ].ob_spec.txt_infini;
    txt_infini->max_visibles = larg_editable;
  } else {
    TEDINFO *tedinfo;

    reponse = ARB_CreerObjet( num_arbre, objet, G_FTEXT, x, y, larg_editable, 1 );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, texte, reponse);
      return -1;
    }

    tedinfo = arbres[ num_arbre ].adresse[ objet ].ob_spec.tedinfo;
    if ( proprietes & ARB_OPTIONS_EDIT_NOMBRE ) {
      if ( proprietes & ARB_OPTIONS_EDIT_ENTIER ) {
	OPTION_A_FAIRE( "ditable de type entier, fini" );
      } else {
	tedinfo->te_ptext.editable = strdup( "01201234" );
	tedinfo->te_ptmplt = strdup( "___,_____" );
	tedinfo->te_pvalid = strdup( "X9999999" );
	tedinfo->te_txtlen = 9;  /* ATTENTION, on compte aussi le 0 final... */
	tedinfo->te_tmplen = 10; /* ATTENTION, on compte aussi le 0 final... */
      }
    } else {
      OPTION_A_FAIRE( "ditable de type texte, fini" );
    }
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Crer un sparateur -------- */

int ARB_CreerSeparateur(unsigned int arbre, unsigned int objet, int x, int y, unsigned int taille, int sens, const char *texte)
{
  int reponse;

  if ( ASC_VERTICAL == sens ) {
    reponse = ARB_CreerObjet( arbre, objet, G_USERDEF, x, y, 1, taille );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer l'objet [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, reponse);
      return -1;
    }
    arbres[ arbre ].adresse[ objet ].ob_spec.util->ub_code = USR_separateur;
  } else {
    reponse = ARB_CreerObjet( arbre, objet, G_SEPARATEUR, x, y, taille, 1 );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer l'objet [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, reponse);
      return -1;
    }
    arbres[ arbre ].adresse[ objet ].ob_spec.texte = texte;
  }

  /* Cration russie */
  return 0;
}

/* -------- Crer un bouton pour sous-menu -------- */

int ARB_CreerSousMenu(unsigned int numero_arbre, unsigned int numero_objet, int x, int y, unsigned int largeur, unsigned int largeur_avant,
		      const char *texte_avant, const char *titre, unsigned long nombre_options, const char *options[])
{
  int reponse;

  /* On cre, si ncessaire, le texte avant */
  if ( texte_avant != NULL ) {
    reponse = ARB_CreerTexte( numero_arbre, numero_objet, x, y, largeur_avant, texte_avant );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Problme en crant le texte prcdent [code %d ; arbre %u, objet %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, reponse, numero_arbre, numero_objet );
      return -1;
    }

    /* Le menu proprement dit sera donc l'objet suivant */
    numero_objet++;
    largeur_avant++; /* Et on prvoit une case de libre entre les deux ! */
  }

  /* On cre le sous-menu proprement dit */
  reponse = ARB_CreerObjet( numero_arbre, numero_objet, G_SSMENU, x + largeur_avant, y, largeur, 1 );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme en crant le bouton d'appel du menu [code %d ; arbre %u, objet %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, reponse, numero_arbre, numero_objet );
    return -10;
  }

  /* et on l'initialise */
  arbres[ numero_arbre ].adresse[ numero_objet ].ob_flags |= SHADOWED;
  if ( titre != NULL ) {
    arbres[ numero_arbre ].adresse[ numero_objet ].ob_spec.menu->titre = titre;
  }
  if ( options != NULL ) {
    arbres[ numero_arbre ].adresse[ numero_objet ].ob_spec.menu->options = options;
    arbres[ numero_arbre ].adresse[ numero_objet ].ob_spec.menu->nombre_options = nombre_options;
  }

  /* Termin sans souci */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Groupes d'objets frquents ~-~-~-~-~-~-~-~-~ */

/* -------- Cration d'un dfileur, slecteur d'objets -------- */

int ARB_CreerDefileur(unsigned int arbre, unsigned int objet_fond, int x, int y, unsigned int largeur, unsigned int nombre_textes,
		      boolean avec_ascenseur, unsigned int objet_1, unsigned int sens_ascenseur)
{
  OBJECT *objets;
  int retour;

  /* On cre les textes. Si cela fonctionne, le fond existe aussi ! */
  retour = ARB_CreerNTextes( arbre, objet_fond + 1, nombre_textes, objet_fond, 0, 0, largeur );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur dans la cration des textes [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -1;
  }

  /* On termine la cration du fond */
  objets = arbres[ arbre ].adresse;
  objets[ objet_fond ].ob_type = G_BOX;
  objets[ objet_fond ].ob_x = x;
  objets[ objet_fond ].ob_y = y;
  objets[ objet_fond ].ob_width = largeur;
  objets[ objet_fond ].ob_height = nombre_textes;

  if ( VRAI == avec_ascenseur ) {
    /* On doit crer un ascenseur :  droite si vertical, en-dessous si horizontal */
    if ( ASC_VERTICAL == sens_ascenseur ) {
      retour = ARB_CreerAscenseur( arbre, objet_1, x + largeur, y, ASC_VERTICAL, nombre_textes );
    } else {
      retour = ARB_CreerAscenseur( arbre, objet_1, x, y + nombre_textes, ASC_HORIZONTAL, largeur );
    }
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Erreur en crant l'ascenseur [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return -10;
    }
  }

  /* Termin sans souci */
  return 0;
}

/* -------- Crer une srie d'options, agences verticalement -------- */

int ARB_CreerOptions_V(unsigned int num_arbre, unsigned int objet_pere, unsigned int premiere_option, unsigned int nbr_options,
		       unsigned int proprietes, int x, int y, unsigned int delta_y, unsigned larg_txt,
		       const char *titre, const char *textes[])
{
  int reponse;
  unsigned int dernier_fils;
  OBJECT *objets;

  /* Par dfaut, premire option = premier fils = objet suivant */
  if ( ROOT == premiere_option ) {
    premiere_option = objet_pere + 1;
  }
  if ( 0 == nbr_options ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune option  crer !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +5; /* Erreur, mais mineure */
  }

  if ( proprietes & ARB_OPTIONS_CASES ) { /* On cre des cases  cocher */
    dernier_fils = premiere_option + 2 * nbr_options - 1;
  } else { /* On cre des boutons */
    dernier_fils = premiere_option + nbr_options - 1;
  }

  /* Contrle de validit */
  objets = ARB_ControleObjet( num_arbre, dernier_fils );
  if (NULL == objets) {
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans ARB_ControleObjet()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On cre le fond, si non-ROOT et si demand */
  if ( ( objet_pere != ROOT ) && ( proprietes & ARB_OPTIONS_CREER_PERE ) ) {
    unsigned int larg_pere;

    if ( proprietes & ARB_OPTIONS_CASES ) { /* On cre des cases  cocher */
      larg_pere = larg_txt + 5; /* texte, 2 = case, 1 avant, 1 aprs, 1 entre */
    } else { /* On cre des boutons */
      larg_pere = larg_txt + 2; /* texte, 1 avant, 1 aprs */
    }

    reponse = ARB_CreerObjet( num_arbre, objet_pere, G_BOX, x, y, larg_pere, 2 + nbr_options + ( nbr_options - 1 ) * delta_y );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le fond de la liste d'options [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, reponse);
      return -2;
    }

    /* Le pre a les options (cases & textes) comme enfants
       On suppose une numrotation "logique" : tous les fils  la suite du pre */
    objets[ objet_pere ].ob_head = premiere_option;
    objets[ objet_pere ].ob_tail = dernier_fils;
    if ( FAUX == ( objets[ dernier_fils ].ob_flags & LASTOB ) ) { /* On n'est pas en fin d'arbre */
      objets[ objet_pere ].ob_next = dernier_fils + 1;
    }

    /* Le dernier enfant : pointe sur le pre ! */
    objets[ dernier_fils ].ob_next = objet_pere;
  }

  if ( proprietes & ARB_OPTIONS_CASES ) { /* On cre des cases  cocher */
    unsigned int i;

    for ( i = 0 ; i < nbr_options ; i++ ) {
      /* La case  cocher */
      reponse = ARB_CreerCaseACocher( num_arbre, premiere_option + ( 2 * i ), 1, 1 + i * (1 + delta_y), larg_txt, textes[ i ] );
      if ( reponse != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / option %u \"%s\" [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, i, textes[ i ], reponse);
	return -8;
      }

      /* Radio-boutons */
      if ( proprietes & ARB_OPTIONS_BASCULE ) {
	objets[ premiere_option + ( 2 * i ) ].ob_flags |= RBUTTON;
      }
    }

    if ( proprietes & ARB_OPTIONS_BASCULE ) {
      objets[ premiere_option ].ob_state |= CHECKED;
    }
  } else { /* On cre des boutons */
    unsigned int i;

    for ( i = 0 ; i < nbr_options ; i++ ) {
      reponse = ARB_CreerBouton( num_arbre, premiere_option + i, 1, 1 + i * (1 + delta_y), larg_txt, textes[ i ] );
      if ( reponse != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / option %u \"%s\" [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, i, textes[ i ], reponse);
	return -8;
      }

      /* Radio-boutons */
      if ( proprietes & ARB_OPTIONS_BASCULE ) {
	objets[ premiere_option + i ].ob_flags |= RBUTTON;
      }
    }

    if ( proprietes & ARB_OPTIONS_BASCULE ) {
      objets[ premiere_option ].ob_state |= SELECTED;
    }
  }

#ifdef _DEVELOPPE_
/*   ARB_AfficherStructure( num_arbre ); */
#endif

  /* Le titre, s'il y en a un : pas un fils pour pouvoir faire un dcalage ensuite... */
  if ( titre != NULL ) {
    reponse = ARB_CreerTexte( num_arbre, dernier_fils + 1, x + 1, y - 1, strlen( titre ), titre );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le titre \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, titre, reponse);
      return -8;
    }
  }

  /* Cration russie */
  return 0;
}

int ARB_CreerNEditables(unsigned int num_arbre, unsigned int objet_pere, unsigned int premier_editable, unsigned int nbr_editables,
			 unsigned int proprietes, int x, int y, unsigned int delta_y, unsigned larg_texte, unsigned int larg_editable,
			 const char *titre, const char *textes[])
{
  int reponse;
  unsigned int i, di = 0;
  OBJECT *objets;

  /* Par dfaut, premire option = premier fils = objet suivant */
  if ( ROOT == premier_editable ) {
    premier_editable = objet_pere + 1;
  }
  if ( 0 == nbr_editables ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun ditable  crer !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +5; /* Erreur, mais mineure */
  }

  /* Contrle de validit */
  objets = ARB_ControleObjet( num_arbre, premier_editable + nbr_editables );
  if (NULL == objets) { 
    (void) fprintf(stderr, "%s, %s l.%u : erreur dans ARB_ControleObjet()\n",
		    __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  /* On cre le fond, si non-ROOT et si demand */
  if ( ( objet_pere != ROOT ) && ( proprietes & ARB_OPTIONS_CREER_PERE ) ) {
    unsigned int larg_pere, haut_pere;
    unsigned int dernier_fils;

    larg_pere = larg_texte + larg_editable + 3; /* texte, ditable, 1 avant, 1 aprs, 1 entre */
    if ( proprietes & ARB_OPTIONS_CASES ) { /* On cre des cases  cocher */
      larg_pere += 3; /* 2 = case, 1 entre */
    }

    haut_pere = 2 + nbr_editables;
    if ( nbr_editables >= 2 ) {
      haut_pere += ( nbr_editables - 1 ) * delta_y;
    }

    reponse = ARB_CreerObjet( num_arbre, objet_pere, G_BOX, x, y, larg_pere, haut_pere );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le fond de la liste d'options [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, reponse);
      return -2;
    }

    if ( proprietes & ARB_OPTIONS_CASES ) { /* On cre des cases  cocher */
      dernier_fils = premier_editable + 3 * nbr_editables - 1;
    } else { /* On cre des boutons */
      dernier_fils = premier_editable + 2 * nbr_editables - 1;
    }

    /* Le pre a les options (cases & textes) comme enfants
       On suppose une numrotation "logique" : tous les fils  la suite du pre */
    objets[ objet_pere ].ob_head = premier_editable;
    objets[ objet_pere ].ob_tail = dernier_fils;
    if ( FAUX == ( objets[ dernier_fils ].ob_flags & LASTOB ) ) { /* On n'est pas en fin d'arbre */
      objets[ objet_pere ].ob_next = dernier_fils + 1;
    }

    /* Le dernier enfant : pointe sur le pre ! */
    objets[ dernier_fils ].ob_next = objet_pere;

    /* Le titre, s'il y en a un : pas un fils pour pouvoir faire un dcalage ensuite... */
    if ( titre != NULL ) {
      reponse = ARB_CreerTexte( num_arbre, dernier_fils + 1, x + 1, y - 1, strlen( titre ), titre );
      if ( reponse != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le titre \"%s\" [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, titre, reponse);
	return -8;
      }
    }

    /* Les coordonnes passent en relatives au pre */
    x = 1; y = 1;
  }

  if ( proprietes & ARB_OPTIONS_CASES ) {
    di = 3;
  } else {
    di = 2;
  }
  for ( i = 0 ; i < nbr_editables ; i++ ) {
    reponse = ARB_CreerEditable( num_arbre, premier_editable + di * i, x, y + i * (1 + delta_y), larg_texte, larg_editable, proprietes, textes[ i ] );
    if ( reponse != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la case  cocher / option %u \"%s\" [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, i, textes[ i ], reponse);
      return -8;
    }
  }

  /* Cration russie */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Libration d'un arbre ~-~-~-~-~-~-~-~- */

int BI_LibererArbre(unsigned int numero_arbre)
{
  unsigned int i;
  OBJECT *objets;

  if ( numero_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s, %s l.%u : numro d'arbre incorrect [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre );
    return -1;
  }

  /* S'il a t ouvert, il reste une fentre associe -> on la fait disparatre aussi */
  FEN_Detruire( ARB_QuelleFenetre( arbres + numero_arbre, VRAI ) );

  /* Plus besoin de la correspondance objet/fentre */
  free( arbres[ numero_arbre ].objets );
  arbres[ numero_arbre ].objets = NULL; /* Sinon on le relibre quand on quitte le programme... */

  /* Libration des informations associes  un objet */
  objets = arbres[ numero_arbre ].adresse;
  for ( i = 0 ; i < arbres[ numero_arbre ].nbr_objets ; i++ ) {
    /* Autres librations possibles */
    switch ( objets[ i ].ob_type )
    {
    case G_STRING:
    case G_BUTTON:
    case G_TITLE:
      if ( FAUX == arbres[ numero_arbre ].drapeaux.interne ) {
	/* Arbres internes : les chaines peuvent tre en statique !
	   Mieux vaut ne pas librer [de toute faon, sera fait en quittant] que librer une chane statique... */
#ifdef _DEVELOPPE_
/*  	printf( "%s, libr : %s [arbre %u, objet %u]\n", __FUNCTION__, objets[ i ].ob_spec.txt_malloc, numero_arbre, i ); */
#endif
	free( objets[ i ].ob_spec.txt_malloc );
	objets[ i ].ob_spec.txt_malloc = NULL;
#ifdef _DEVELOPPE_
      } else if ( NULL != objets[ i ].ob_spec.texte ) {
/* 	printf( "%s, non-libr : %s\n", __FUNCTION__, objets[ i ].ob_spec.texte ); */
#endif
      }
      break;

    case G_SEPARATEUR:
      /* Objet cr automatiquement -> toujours du texte obtenu avec strdup()  librer */
      free( objets[ i ].ob_spec.txt_malloc );
      objets[ i ].ob_spec.txt_malloc = NULL;
    }

    /* Libration des complments ventuels
       APRS, car met ob_spec  0 -> on ne pourrait plus librer les textes... */
    OBJ_LibererComplement( numero_arbre, i );
  }

  /* Plus besoin de la table des objets */
  free( arbres[ numero_arbre ].adresse ); arbres[ numero_arbre ].adresse = NULL;

  /* L'arbre est forcment ferm, sans avoir t ni prpar, ni initialis */
  arbres[ numero_arbre ].drapeaux.ouvert = FAUX;
  arbres[ numero_arbre ].drapeaux.init = FAUX;
  arbres[ numero_arbre ].drapeaux.prepare = FAUX;

  /* Il n'a forcment plus d'objet ditable */
  arbres[ numero_arbre ].premier_edit = ROOT;
  arbres[ numero_arbre ].edit_courant = ROOT;
  arbres[ numero_arbre ].dernier_edit = ROOT;
  arbres[ numero_arbre ].pos_curseur = 0;

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Gestion gnrique de l'arbre ~-~-~-~-~-~-~-~-~ */

/* -------- Grer un clic sur un objet -------- */

int ARB_GererClic(Arbre *arbre, unsigned int numero_objet, int bouton, int x_clic, int y_clic)
{
  OBJECT *objets;
  int retour = BI_EVENEMENT_NON_TRAITE;
  unsigned int numero_arbre;

  if ( ROOT == numero_objet ) {
    /* Clic sur la racine : rien  faire ! */
    return BI_EVENEMENT_TRAITE;
  }

  objets = arbre->adresse;
  if ( NULL == objets ) {
    (void) fprintf(stderr, "%s,%s l.%u : Arbre sans table d'objets !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return BI_EVENEMENT_ERREUR;
  }

  if ( OB_STATE( objets, numero_objet ) & DISABLED ) {
    /* Clic sur un objet dsactiv : ignor */
    return BI_EVENEMENT_TRAITE;
  }

  if ( OB_FLAGS( objets, numero_objet ) & EDITABLE ) {
    /* Clic sur un objet ditable : y placer le curseur */
    (void) BI_ChangerEditable( arbre, numero_objet );
    OPTION_A_FAIRE("Placer le curseur sous la souris !");
    return BI_EVENEMENT_TRAITE;
  }

  if ( arbre->fonction.arbre != ( int (*)(unsigned int, int, int) ) NULL ) {
    /* S'il y a une fonction associe, elle est prioritaire. */
    BI_bouton = bouton;
    retour = (arbre->fonction.arbre)( numero_objet, x_clic, y_clic );
    if ( retour != BI_EVENEMENT_NON_TRAITE ) { /* On a trait... */
      return retour;
    }
  }

  /* -------- Fonctionnement par dfaut -------- */

  /* On rcupre le numro de l'arbre */
  numero_arbre = arbre - arbres;

  /* 1. Objets de type "interne" avec traitement spcial */
#ifdef G_SSMENU	/* Il existe des objets appelant un sous-menu */
  if ( G_SSMENU == OB_TYPE( objets, numero_objet ) ) {
    int reponse;
    struct infos_ssmenu *menu;

    menu = objets[ numero_objet ].ob_spec.menu;
    if ( NULL == menu ) {
      (void) fprintf(stderr, "%s,%s l.%u : Pas d'infos de menu associes au sous-menu ! [arbre %u, objet %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, numero_arbre, numero_objet);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return BI_EVENEMENT_ERREUR;
    }

    /* On enfonce le bouton */
    OBJ_choisit( numero_arbre, numero_objet, FORCE_OUI );

    /* Sous-menu interne : on l'appelle ! */
    if ( bouton < 0 ) {
      /* On simule un clic, mais l'arbre est ferm ==> ouvrir  la position de la souris ! */
      reponse = SS_menu_texte( menu->options, menu->nombre_options, menu->titre, -1, 0 );
    } else {
      reponse = SS_menu_texte( menu->options, menu->nombre_options, menu->titre, numero_arbre, numero_objet );
    }
    if ( reponse >= 0 ) {
      if ( ( void (*)(unsigned int, unsigned int, unsigned long) ) NULL != menu->gestion ) {
	/* On appelle la routine de gestion indique
	   [avant de stocker la nouvelle option, pour avoir accs  l'ancienne si ncessaire !] */
	(menu->gestion)( numero_arbre, numero_objet, reponse );
      }

      /* On a bien choisi quelque chose */
      menu->option_choisie = reponse;

      /* Le raffichage se fera en dselectionnant l'objet, en-dessous
	 ==> affichera l'option correcte ou le texte ncessaire, en fonction !*/
    }

    /* On relche le bouton */
    OBJ_choisit( numero_arbre, numero_objet, FORCE_NON );

    /* Trait... */
    retour = BI_EVENEMENT_TRAITE;
  }
#endif

  /* a. Si l'objet est slectionnable, on le change d'tat [Si pas dj trait auparavant !] */
  if ( ( OB_FLAGS( objets, numero_objet ) & SELECTABLE ) && ( BI_EVENEMENT_NON_TRAITE == retour ) ) {
    OBJ_choisit( numero_arbre, numero_objet, ECHANGE);

    retour = BI_EVENEMENT_TRAITE;
  }

  /* b. Si l'objet est une case  cocher, on la change d'tat [Si pas dj trait auparavant !] */
  if ( ( G_IBOX == OB_TYPE( objets, numero_objet ) ) && ( BI_EVENEMENT_NON_TRAITE == retour ) ) {
    AVERTISSEMENT( "Case  coher non en slectable [arbre %u, objet %u]", numero_arbre, numero_objet );
    OBJ_choisit( numero_arbre, numero_objet, ECHANGE);

    retour = BI_EVENEMENT_TRAITE;
  }

  /* c. Si l'objet est indiqu comme tel, on ferme l'arbre */
  if ( ( OB_FLAGS( objets, numero_objet ) & EXIT ) ||
       ( OB_FLAGS( objets, numero_objet ) & TOUCHEXIT ) ) {
    /* Clic sur un objet de fermeture d'arbre */
    return BI_EVENEMENT_FERMER_ARBRE;
  }

  /* d. On renvoie ce que l'on a pu faire... */
  return retour;
}

/* -------- Simuler un clic sur un objet [pour raccourcis clavier, par exemple !] -------- */

int ARB_SimulerClic(unsigned int numero_arbre, unsigned int numero_objet, int bouton)
{
  int retour;

  /* On vrifie que l'objet et l'arbre existent */
  retour = OBJ_ControlerValeurs( numero_arbre, numero_objet );
  if ( retour != 0) {
    (void) fprintf(stderr, "%s,%s l.%u : Paramtres absurdes, arbre=%u, objet=%u\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, numero_objet);
    return BI_EVENEMENT_ERREUR;
  }

  /* Maintenant que l'on sait qu'ils existent, on peut grer le clic normalement */
  retour = ARB_GererClic( arbres + numero_arbre, numero_objet, bouton, 0, 0 );
  if ( BI_EVENEMENT_ERREUR == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant cliquer [arbre=%u, objet=%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, numero_objet);
  }

  return retour;
}

/* ~-~-~-~-~-~-~-~-~ Informations associes  l'arbre ~-~-~-~-~-~-~-~-~ */

/* -------- Associer une information --------
     Renvoie 0 si l'info a pu tre associe

   arbre : l'arbre. Il ne doit avoir aucune information dj associe [sauf si complement == NULL]
   complement : les informations  associer  l'arbre [NULL : on enlve celles qui le seraient dj]
 */

int ARB_MettreComplement(unsigned int numero_arbre, void *complement)
{
  /* Contrle prliminaire */
  if ( numero_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numro d'arbre trop grand [%u / %u max]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, BI_nbr_arbres - 1);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return -1;
  }

  /* Si NULL pour complement, on enlve l'information ventuellement dj stocke */
  if ( NULL == complement ) {
    arbres[ numero_arbre ].complement = NULL;
    return 0;
  }

  /* On vrifie que l'on ne perd pas d'ancienne information */
  if ( arbres[ numero_arbre ].complement != NULL ) {
    if ( complement == arbres[ numero_arbre ].complement ) {
      /* On veut juste remettre le mme : ne pose aucun problme... */
      return 0;
    }
    (void) fprintf(stderr, "%s,%s l.%u : Il y a dj des infos, diffrentes, dans l'arbre %u [anciennes : %p, nouvelles : %p]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, arbres[ numero_arbre ].complement, complement);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  /* On peut donc associer les nouvelles */
  arbres[ numero_arbre ].complement = complement;

  /* Et tout s'est bien pass */
  return 0;
}

/* -------- Lie l'information stocke dans l'arbre */

void *ARB_LireComplement(unsigned int numero_arbre)
{
  void *complement;

  /* Contrle prliminaire */
  if ( numero_arbre >= BI_nbr_arbres ) {
    (void) fprintf(stderr, "%s,%s l.%u : numro d'arbre trop grand [%u / %u max]\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_arbre, BI_nbr_arbres - 1);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* On rcupre le complment associ directement  l'arbre */
  complement = arbres[ numero_arbre ].complement;

  /* S'il n'y en a pas : celui de la fentre associe */
  if ( NULL == complement ) {
    Fenetre *fenetre;

    AVERTISSEMENT( "Arbre %u : pas de complment, on utilise celui de la fentre", numero_arbre );

    fenetre = ARB_QuelleFenetre( arbres + numero_arbre, VRAI );
    if ( NULL == fenetre ) {
      AVERTISSEMENT( "Demande de complment pour un arbre sans fentre [%u]", numero_arbre );
      return complement;
    }

    /* On prend par dfaut le 2e complment [pour ne pas tomber sur le graphe, dans LASE !] */
    complement = FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
    if ( NULL == complement ) {
      /* Mais s'il est absent, on utilise le premier */
      complement = FEN_LireComplement( fenetre, FEN_COMPLEMENT_1 );
    }
  }

  /* On renvoie l'information stocke */
  return complement;
}

#ifdef _DEVELOPPE_
/* -------- Afficher la structure de l'arbre -------- */

void ARB_AfficherStructure(unsigned int num_arbre)
{
  int d_fils, o_suivant;

  printf("Arbre %u : %u objets\n", num_arbre, arbres[ num_arbre ].nbr_objets);
  if ( NULL == arbres[ num_arbre ].adresse ) {
    printf("----- Objets pas encore dfinis -----\n");
    return;
  }

  /* On affiche la racine & ses enfants */
  ARB_AfficherFils( num_arbre, ROOT, 0 );

  /* Dernier enfant */
  d_fils = arbres[ num_arbre ].adresse[ ROOT ].ob_tail;

  /* Y-a-t'il des objets hors-racine (menu droulant) ? */
  while ( FAUX == ( arbres[ num_arbre ].adresse[ d_fils ].ob_flags & LASTOB ) ) {
    o_suivant = d_fils + 1;

    ARB_AfficherFils( num_arbre, o_suivant, 0 );

    d_fils = arbres[ num_arbre ].adresse[ o_suivant ].ob_tail;
  }
}

static void ARB_AfficherFils(unsigned int num_arbre, unsigned int objet, unsigned int delta)
{
  int p_fils, d_fils, o_suivant;

  ARB_AfficherObjet( num_arbre, objet, delta );

  p_fils = arbres[ num_arbre ].adresse[ objet ].ob_head;
  d_fils = arbres[ num_arbre ].adresse[ objet ].ob_tail;
  if ( -1 == p_fils ) { /* Pas de fils */
    return;
  }
  if ( -1 == d_fils ) {
    (void) printf("PREMIER FILS, PAS DE DERNIER FILS\n");
  }

  o_suivant = p_fils;
  while ( o_suivant != (signed) objet ) {
    ARB_AfficherFils( num_arbre, o_suivant, delta + 1);

    o_suivant = arbres[ num_arbre ].adresse[ o_suivant ].ob_next;
  }
}

static void ARB_AfficherObjet(unsigned int num_arbre, unsigned int num_objet, unsigned int delta)
{
  unsigned int i;
  OBJECT *objet;

  objet = arbres[ num_arbre ].adresse + num_objet;

  for ( i = 0 ; i < delta ; i++ )
    printf(" ");

  (void) printf("Objet : %d, type %d [X=%d, Y=%d, L=%u, H=%u], 1er fils = %d, dernier fils = %d, suivant = %d",
		num_objet, objet->ob_type, objet->ob_x, objet->ob_y, objet->ob_width, objet->ob_height,
		objet->ob_head, objet->ob_tail, objet->ob_next );
  switch ( objet->ob_type )
  {
  case G_USERDEF:
    printf(" fonction = %p", (void *) objet->ob_spec.util->ub_code );
    break;

  case G_STRING:
  case G_BUTTON:
    printf(" texte = %s", objet->ob_spec.texte );
    break;
  }

  printf("\n");
}
#endif /* _DEVELOPPE_ */
