/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2003, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion des arbres / botes de dialogue
   Dclarations (indpendantes de LASE)
*/

#ifndef BI_ARBRES_COMMUN_H_
# define BI_ARBRES_COMMUN_H_ /* Protection / inclusion multiple */

/* ~~~~~~~~ Arbres d'arbres : onglets ~~ */

typedef struct bi_onglet {
  const char *texte;		/* Le texte affich dans l'onglet */
  unsigned int etat;		/* Masque de bits --- comme ob_state() */
  unsigned int numero_arbre;	/* Le numro de l'arbre associ  cet onglet, affich dans la partie changeante */
  int (*fermeture)(void);	/* La fonction  appeler quand on quitte l'onglet [enregistrer les valeurs de l'arbre] */
} BI_Onglet;

int BI_Onglets_Creation    (Fenetre *fenetre, unsigned int nombre_onglets, BI_Onglet *onglets);
int BI_Onglets_ChangerActif(Fenetre *fenetre, unsigned int nouvel_actif);
int BI_QuelOngletEstActif  (Fenetre *fenetre);
int BI_MemoriserOngletActif(Fenetre *fenetre, unsigned int nouvel_actif);

/* ~~~~~~~~ Cration des arbres ~~~~~~~~ */

Arbre *BI_ARB_CreerArbres( unsigned int nombre_arbres );

int BI_CreerArbre_Inclus(unsigned int num_arbre, unsigned int nbr_objets, OBJECT *adresse);
int BI_CreerArbre       (unsigned int num_arbre, unsigned int nbr_objets, unsigned int largeur, unsigned int hauteur);
int BI_LibererArbre     (unsigned int num_arbre);

unsigned int ARB_CompterObjets(unsigned int arbre);

int ARB_PositionObjet(unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg, unsigned int haut);

# ifdef _DEVELOPPE_
void ARB_AfficherStructure( unsigned int num_arbre );
# endif

/* ------ Cration d'objets dans ces arbres ------ */

int ARB_CreerBouton     (unsigned int arbre, unsigned int objet, int x, int y, unsigned int larg, const char *texte);
int ARB_CreerCaseACocher(unsigned int arbre, unsigned int objet, int x, int y, unsigned int larg_texte, const char *texte);
int ARB_CreerEditable   (unsigned int arbre, unsigned int objet, int x, int y, unsigned int larg_texte, unsigned int larg_editable,
			 unsigned int proprietes, const char *texte);
int ARB_CreerObjet      (unsigned int num_arbre, unsigned int numero, int type, int x, int y, unsigned int larg, unsigned int haut);
int ARB_CreerNEditables (unsigned int arbre, unsigned int objet_pere, unsigned int premier_editable, unsigned int nbr_editables,
			 unsigned int proprietes, int x, int y, unsigned int delta_y, unsigned larg_texte, unsigned int larg_editable,
			 const char *titre, const char *textes[]);
int ARB_CreerNTextes    (unsigned int num_arbre, unsigned int objet_1, unsigned int nombre, int pere,
			 int x, int y, unsigned int larg);
int ARB_CreerSeparateur (unsigned int arbre, unsigned int objet, int x, int y, unsigned int taille, int sens, const char *texte);
int ARB_CreerSousMenu   (unsigned int arbre, unsigned int objet, int x, int y, unsigned int largeur, unsigned int largeur_avant,
			 const char *texte_avant, const char *titre, unsigned long nombre_options, const char *options[]);
int ARB_CreerTexte      (unsigned int num_arbre, unsigned int objet, int x, int y, unsigned int larg, const char *texte);

#define ARB_OPTIONS_BASCULE      1 /* Bit mis : boutons radio */
#define ARB_OPTIONS_CREER_PERE   2 /* Bit mis : on cre l'objet pre */
#define ARB_OPTIONS_CASES        4 /* Bit mis : on cre des cases  cocher */
#define ARB_OPTIONS_EDIT_NOMBRE  8 /* Bit mis : l'ditable est pour un nombre */
#define ARB_OPTIONS_EDIT_ENTIER 16 /* Bit mis : l'ditable est pour un nombre entier */
#define ARB_OPTIONS_EDIT_INFINI 32 /* Bit mis : l'ditable est un texte infini */

#define ARB_ASCENSEUR_DROITE  0	/* Position de l'ascenseur par rapport  un dfileur... */
#define ARB_ASCENSEUR_GAUCHE  1
#define ARB_ASCENSEUR_HAUT    2
#define ARB_ASCENSEUR_BAS     3

int ARB_CreerAscenseur(unsigned int arbre, unsigned int objet_1, int x, int y, int sens, unsigned int taille);
int ARB_CreerDefileur (unsigned int arbre, unsigned int objet_fond, int x, int y, unsigned int largeur, unsigned int nombre_textes,
		       boolean avec_ascenseur, unsigned int objet_1, unsigned int sens_ascenseur);
int ARB_CreerOptions_V(unsigned int arbre, unsigned int objet_pere, unsigned int premiere_option, unsigned int nbr_options,
		       unsigned int proprietes, int x, int y, unsigned int delta_y, unsigned larg_txt,
		       const char *titre, const char *textes[]);

/* -------- Enjolivation -------- */

int ARB_EnjoliverAscenseur(unsigned int arbre, unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres,
			   int type, int delta);
int ARB_EnjoliverDefileur (unsigned int arbre, unsigned int fond_textes, unsigned int texte_1, unsigned int texte_N,
			   unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres, int position);

/* -------- Analyse et initialisation -------- */

int ARB_Initialiser   (unsigned int arbre, const char *URL_aide, int (*gestion)(unsigned int, int , int), int (*clavier)(unsigned int, int, int *));
int ARB_InitSpecifique(unsigned int arbre);

int ARB_TrouverEditables(unsigned int arbre, boolean courant);

unsigned int ARB_EditablePrecedent(unsigned int arbre, unsigned int actuel, boolean boucle);
unsigned int ARB_EditableSuivant  (unsigned int arbre, unsigned int actuel, boolean boucle);

void *ARB_LireComplement  (unsigned int arbre);
int   ARB_MettreComplement(unsigned int arbre, void *complement);

/* ~~~~~~~~ Appel & gestion globale de ces arbres ~~~~~~~~~~ */

boolean ARB_EstOuvert(unsigned int numero_arbre);

int ARB_AppelerAide(const char *URL_aide);
int ARB_GererClic  (Arbre *arbre, unsigned int numero_objet, int bouton, int x_clic, int y_clic);
int ARB_SimulerClic(unsigned int numero_arbre, unsigned int numero_objet, int bouton);

/* -------- Arbre en fentre -------- */

Fenetre *ARB_Appeler(unsigned int arbre, int attributs, const char *titre, void (*preparation)(void) );

Fenetre *ARB_QuelleFenetre(Arbre *arbre, boolean fermee);

/* -------- Arbre bloquant -------- */

int ARB_Message (unsigned int numero_arbre, unsigned int temps, Fenetre *fenetre_mere);
int ARB_Bloquant(unsigned int numero_arbre, const char *titre, Fenetre *fenetre_mere, unsigned int numero_arbre_pere);

/* ~~~~~~~~ Gestion de sous-ensembles de ces arbres ~~~~~~~~ */

/* -------- Ascenseurs -------- */

unsigned long ARB_ascenseur(unsigned int arbre, unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres, int sens,
			    unsigned long nbr_objets, unsigned int nbr_visibles, unsigned long idx_premier,
			    unsigned int objet, int x, int y, boolean mode);

int gere_ascenseur(unsigned int arbre, unsigned int avant, unsigned int fond, unsigned int bouton, unsigned int apres,
		   int mode,
		   int min, int max, int pas, int valeur, unsigned int obj_clic, int x, int y);

/* -------- Dfileurs (de texte simple) -------- */

unsigned long ARB_ChoisirDebutDefileur(unsigned long nombre_noms, unsigned long nombre_objets, unsigned long numero_objet_choisi, unsigned long premier_propose);

int ARB_PreparerDefileur_T(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long texte_choisi,
			   unsigned long premier_texte_affiche);
int ARB_GererDefileur_T   (unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_textes, const char **textes, boolean *choisi, unsigned long *texte_choisi,
			   unsigned long *premier_texte_affiche, unsigned int objet, int y_souris);

/* -------- Dfileurs (de texte "indirect") -------- */

int ARB_PreparerDefileur_O(unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_objets, boolean *choisi, unsigned long objet_choisi, unsigned long premier_objet_affiche,
			   const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero_objet) );
int ARB_GererDefileur_O   (unsigned int arbre, unsigned int obj_premier_texte, unsigned int obj_dernier_texte, unsigned int lg_max_texte,
			   unsigned int obj_avant, unsigned int obj_fond_asc, unsigned int obj_bouton, unsigned int obj_apres,
			   unsigned long nombre_objets, boolean *choisi, unsigned long *objet_choisi, unsigned long *premier_objet_affiche,
			   const char *(*objet_v_texte)(unsigned int arbre, unsigned long numero_objet), unsigned int objet, int y_souris);

#endif
