/* Bibliothque GEM / X_windows tendue
   (C) Emmanuel CURIS, mai 2006

   Dessin de textes dans la fentre utilisateur
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "txt_utils.h"

#include "bib_interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static unsigned int X_TRC_HauteurLigne(const char *texte, XFontStruct *fonte);
static unsigned int X_TRC_LargeurLigne(const char *texte, XFontStruct *fonte);

static void X_TRC_LigneTexte (Drawable destination, GC contexte, int x, int y, unsigned int mode_H, const char *ligne);
static void X_TRC_TexteDroit (Drawable destination, GC contexte, int x, int y, unsigned int mode_H, unsigned int mode_V,
			      char **lignes, unsigned long nombre_lignes);
static void X_TRC_TexteTourne(Drawable destination, GC contexte, int x, int y, unsigned int mode_H, unsigned int mode_V, unsigned int angle,
			      char **lignes, unsigned long nombre_lignes);

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Affichage d'un texte quelconque ~-~-~-~-~-~-~-~-~ */

/* -------- Remarque : la version avec zone limitative est dans X_dessin.c pour accder  la mise en place du clip -------- */

void X_TRC_Texte(struct fenetre *fenetre, int x, int y, unsigned int mode_H, unsigned int mode_V, unsigned int angle,
		 const char *texte)
{
  Drawable fenetre_X;

  if ( NULL == texte ) return;	/* Aucun texte ! */

  /* Fond blanc pour tre sr de voir quelque chose */
  XSetBackground( BI_Display, BI_GC_programme, BI_Couleurs[ BLANC ].pixel );

  /* On dtermine le Drawable final */
  if ( NULL == fenetre ) {
    fenetre_X = BI_Fen_redessin;
  } else {
    fenetre_X = fenetre->handle;

    x += fenetre->w_x; y += fenetre->w_y;
  }

  /* Dcoupage du texte en lignes */
  if ( NULL == strchr( texte, '\n' ) ) {
    if ( 0 == angle ) {
      /* Trac sans rotation : direct */
      X_TRC_TexteDroit( fenetre_X, BI_GC_programme, x, y, mode_H, mode_V, (char **) &texte, 1 );
    } else {
      X_TRC_TexteTourne( fenetre_X, BI_GC_programme, x, y, mode_H, mode_V, angle, (char **) &texte, 1 );
    }
  } else {
    char **lignes = NULL;
    unsigned long nombre_lignes = 0;

    /* Dcoupage proprement dit */
    lignes = TXT_DecomposerChaine( texte, '\n', &nombre_lignes );
    if ( NULL == lignes ) {
      (void) fprintf( stderr, "%s,%s l.%u : Impossible de dcomposer la chane en lignes distinctes [%s]\n",
		      __FILE__, __FUNCTION__, __LINE__, texte );
      return;
    }

    /* Trac du texte dcoup */
    if ( 0 == angle ) {
      /* Trac sans rotation : direct */
      X_TRC_TexteDroit( fenetre_X, BI_GC_programme, x, y, mode_H, mode_V, lignes, nombre_lignes );
    } else {
      X_TRC_TexteTourne( fenetre_X, BI_GC_programme, x, y, mode_H, mode_V, angle, lignes, nombre_lignes );
    }

    /* Plus besoin du texte dcoup */
    free( lignes[ 0 ] ); lignes[ 0 ] = NULL;
    free( lignes ); lignes = NULL;
  }  
}

/* -------- Calculer la place occupe  l'cran, en hauteur, par une chane -------- */

static unsigned int X_TRC_HauteurLigne(const char *texte, XFontStruct *fonte)
{
  unsigned int haut_texte;

  if ( NULL == texte ) return 0;
  if ( NULL == fonte ) fonte = BI_Fonte; /* On utilise la fonte par dfaut ! */

  /* Bote englobante de la fonte */
  haut_texte = fonte->max_bounds.ascent + fonte->max_bounds.descent;

  return haut_texte;
}

/* -------- Calculer la place occupe  l'cran, en largeur, par une chane -------- */

static unsigned int X_TRC_LargeurLigne(const char *texte, XFontStruct *fonte)
{
  unsigned long lg_texte;
  unsigned int largeur_ecran;

  if ( NULL == texte ) return 0;
  if ( NULL == fonte ) fonte = BI_Fonte; /* On utilise la fonte par dfaut ! */

  /* Dimensions du texte */
  lg_texte   = strlen( texte );
  largeur_ecran = XTextWidth( fonte, texte, lg_texte );

  /* Termin */
  return largeur_ecran;
}

static void X_TRC_LigneTexte(Drawable destination, GC contexte, int x, int y, unsigned int mode_H, const char *ligne)
{
  int x_texte, y_texte;
  unsigned int largeur_texte;

  /* En fonction du mode horizontal, on trouve le X */
  if ( BI_FonteUTF8 != NULL ) {
    XRectangle cadre_texte;

#ifdef X_HAVE_UTF8_STRING
    largeur_texte = Xutf8TextExtents( BI_FonteUTF8, (char *) ligne, strlen( ligne ),
				      NULL, &cadre_texte );
#else
    largeur_texte = XmbTextExtents( BI_FonteUTF8, (char *) ligne, strlen( ligne ),
				    NULL, &cadre_texte );    
#endif
  } else {
    largeur_texte = X_TRC_LargeurLigne( ligne, NULL );
  }

  /* Dtermination du coin HG du rectangle */
  switch ( mode_H )
  {
  case BI_TEXTE_FER_A_DROITE:
    /* Le X indiqu est celui du bord droit du texte */
    x_texte = x - largeur_texte;
    break;

  case BI_TEXTE_CENTRER_H:
    /* Le X indiqu est celui du centre du texte */
    x_texte = x - largeur_texte / 2;
    break;

  case BI_TEXTE_FER_A_GAUCHE:
    /* Le X indiqu est celui du bord gauche du texte (bord "naturel") */
    x_texte = x;
    break;

  default:
    AVERTISSEMENT( "Alignement horizontal inconnu [%d]", mode_H );
    x_texte = x;		      /* On fait comme si on avait ALI_LEFT */
  }

  /* Correction du Y : */
  /* Le Y indiqu est celui du coin HG du texte ; XDrawImageString attend comme Y celui de la ligne de base ! */
  y_texte = y + BI_Fonte->max_bounds.ascent;

  /* Trac du rsultat */
  if ( strchr( ligne, '\\' ) != NULL ) {
    OPTION_A_FAIRE( "Tracer un texte avec caractres spciaux" );
  } else {
    if ( BI_FonteUTF8 != NULL ) {
#ifdef X_HAVE_UTF8_STRING
      Xutf8DrawImageString( BI_Display, destination, BI_FonteUTF8, contexte,
			    x_texte, y_texte, ligne, strlen( ligne ) );
#else
      XmbDrawImageString( BI_Display, destination, BI_FonteUTF8, contexte,
			  x_texte, y_texte, ligne, strlen( ligne ) );
#endif
    } else {
      XDrawImageString( BI_Display, destination, contexte, x_texte, y_texte, ligne, strlen( ligne ) );
    }
  }
}

static void X_TRC_TexteDroit(Drawable destination, GC contexte, int x, int y, unsigned int mode_H, unsigned int mode_V,
			     char **lignes, unsigned long nombre_lignes)
{
  int y_HG; /* Coin HG du rectangle englobant */
  int y_texte;
  unsigned int hauteur_texte;
  unsigned int i;

  /* Hauteur du texte */
  hauteur_texte = 0;
  for ( i = 0 ; i < nombre_lignes ; i++ ) {
    hauteur_texte += X_TRC_HauteurLigne( lignes[ i ], NULL );
  }

  /* Dtermination du Y de dpart des tracs */
  switch ( mode_V )
  {
  case BI_TEXTE_FER_EN_HAUT:
    /* Le Y indiqu est celui du bord haut du texte (bord "naturel") */
    y_HG = y;
    break;

  case BI_TEXTE_CENTRER_V:
    /* Le Y indiqu est celui du centre du bloc de texte */
    y_HG = y - hauteur_texte / 2;
    break;

  case BI_TEXTE_LIGNE_BASE:
    /* Le Y indiqu est celui de la ligne de base du texte */
    y_HG  = y - hauteur_texte;
    y_HG += BI_Fonte->max_bounds.descent;
    break;

  case BI_TEXTE_FER_EN_BAS:
    /* Le Y indiqu est celui du bord bas du texte */
    y_HG  = y - hauteur_texte;
    break;

  default:
    AVERTISSEMENT( "Alignement vertical inconnu [%d]", mode_V );
    y_HG = y;		       /* On fait comme si on avait ALI_TOP */
  }

  /* On affiche chacune des lignes */
  for ( i = 0, y_texte = 0 ; i < nombre_lignes ; i++ ) {
    X_TRC_LigneTexte( destination, contexte, x, y_HG + y_texte, mode_H, lignes[ i ] );
    y_texte += X_TRC_HauteurLigne( lignes[ i ], NULL );
  }
}

/* -------- Trac d'un texte tourn -------- */

static void X_TRC_TexteTourne(Drawable destination, GC contexte, int x, int y, unsigned int mode_H, unsigned int mode_V, unsigned int angle,
			      char **lignes, unsigned long nombre_lignes)
{
  int x_texte, y_texte;
  unsigned int largeur_texte, largeur_ligne;
  unsigned int hauteur_texte;
  unsigned long i;

  GC gc_pixmap;
  Pixmap pixmap_texte, px_texte_tourne;
  XImage *img_texte;
  XGCValues parametres_GC;

  /* Calcul de la largeur ncessaire */
  largeur_texte = 0;
  for ( i = 0 ; i < nombre_lignes ; i++ ) {
    largeur_ligne = X_TRC_LargeurLigne( lignes[ i ], NULL );
    if ( largeur_ligne > largeur_texte ) largeur_texte = largeur_ligne;
  }

  /* Calcul de la hauteur ncessaire */
  hauteur_texte = 0;
  for ( i = 0 ; i < nombre_lignes ; i++ ) {
    hauteur_texte += X_TRC_HauteurLigne( lignes[ i ], NULL );
  }

  if ( ( 0 == largeur_texte ) || ( 0 == hauteur_texte ) ) return; /* Texte invisible de toute faon ! */

  /* 1. Crer un pixmap qui contient le texte non-tourn ; 1 plan... */
  pixmap_texte = XCreatePixmap( BI_Display, destination, largeur_texte, hauteur_texte, 1 );

  /* 2. Crer un GC associ pour pouvoir l'utiliser */
  parametres_GC.function   = GXcopy;
  parametres_GC.foreground = 1;
  parametres_GC.background = 0;
  gc_pixmap = XCreateGC( BI_Display, pixmap_texte, GCFunction | GCForeground | GCBackground, &parametres_GC );
  XSetFont( BI_Display, gc_pixmap, BI_Fonte->fid );

  /* 3. On dtermine les coordonnes dans ce pixmap en fonction du mode */
  /* 3-a : coordonne X */
  switch ( mode_H )
  {
  case BI_TEXTE_FER_A_GAUCHE:
    x_texte = 0;
    break;

  case BI_TEXTE_CENTRER_H:
    x_texte = largeur_texte / 2;
    break;

  case BI_TEXTE_FER_A_DROITE:
    x_texte = largeur_texte;
    break;

  default:
    AVERTISSEMENT( "Mode d'alignement en X inconnu [%d]", mode_H );
    x_texte = 0;
  }

  /* 3-b : coordonne Y - inutile, trac en ALI_TOP et ce sera fait lors de l'affichage final */

  /* 4. On trace le texte, en noir, dans ce pixmap */
  X_TRC_TexteDroit( pixmap_texte, gc_pixmap, x_texte, 0, mode_H, BI_TEXTE_FER_EN_HAUT, lignes, nombre_lignes );
  
  /* 5. On en fait une image */
  img_texte = XGetImage( BI_Display, pixmap_texte, 0, 0, largeur_texte, hauteur_texte, AllPlanes, XYPixmap );

  /* Le pixmap du texte original est donc inutile ! */
  XFreePixmap( BI_Display, pixmap_texte ); pixmap_texte = None;

  /* 6. Il faut faire la rotation */
  if ( 90 == angle ) {
    unsigned int i_x, i_y;

    /* O se trouve le coin HG du rectangle du texte dans la fentre ? */
    if ( BI_TEXTE_CENTRER_H == mode_H ) /* On centre sur x */
      y_texte = y - largeur_texte / 2;
    else if ( BI_TEXTE_FER_A_DROITE == mode_H ) /* On justifie  droite = fin du texte en x */
      y_texte = y - largeur_texte;
    else
      y_texte = y;

    if ( BI_TEXTE_FER_EN_HAUT == mode_V )
      x_texte = x - BI_Fonte->max_bounds.ascent;
    else if ( BI_TEXTE_CENTRER_V == mode_V )
      x_texte = x - (BI_Fonte->max_bounds.ascent - BI_Fonte->max_bounds.descent) / 2;
    else if ( BI_TEXTE_FER_EN_BAS == mode_V )
      x_texte = x - BI_Fonte->max_bounds.descent;
    else
      x_texte = x; /* Pour voir */

    /* Le pixmap qui va contenir le texte tourn */
    px_texte_tourne = XCreatePixmap( BI_Display, destination, hauteur_texte, largeur_texte, 1 );

/*     OPTION_A_FAIRE( "Vrifier s'il n'existe pas qq chose de + efficace pour la rotation" ); */

     for ( i_x = 0 ; i_x < largeur_texte ; i_x++ ) {
       for ( i_y = 0 ; i_y < hauteur_texte ; i_y++ ) {
 	XPutImage( BI_Display, px_texte_tourne, gc_pixmap, img_texte,
 		   i_x, i_y, i_y, largeur_texte - i_x - 1, 1, 1 );
       }
     } 

     /* Affichage sur la fentre */
     XCopyPlane( BI_Display, px_texte_tourne, destination, contexte, 0, 0, hauteur_texte, largeur_texte,
		 x_texte, y_texte, 1 );

    /* Le pixmap du texte tourn n'est plus utile */
    XFreePixmap( BI_Display, px_texte_tourne );
  } else {
    OPTION_A_FAIRE( "Rotation pour un angle autre que 0 ou 90 degrs" );
#ifdef _DEVELOPPE_
    fprintf( stderr, "\t angle demand = %u degrs\n", angle );
#endif
  }

  /* Plus besoin des pixmaps ni des images */
  XDestroyImage( img_texte );
  XFreeGC( BI_Display, gc_pixmap );
}
