/* Bibliothque GEM / X_windows tendue
   (C) 1998-2003 Emmanuel CURIS

   Gestion de l'tat ds objets (changement en particulier)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "Bib_Commun/bib_interface.h"

/* -------- Fonctions locales -------- */

/* -------- Ralisation pratique ----- */

/* -------- Objet actif (non disabled) ou inactif (disabled) ------------------------\
 |    Renvoie l'tat de l'objet aprs l'opration (VRAI : activ, FAUX : dsactiv)  |
 |                                                                                   |
 | arbre  : numro de l'arbre contenant l'objet  modifier                           |
 | objet  : numro de l'objet  modifier                                             |
 | mettre : opration  faire. FORCE_NON = on dsactive, FORCE_OUI = on active,      |
 |                   ECHANGE = on bascule de l'un  l'autre, QUESTION = on lit juste |
 \---------------------------------------------------------------------------------- */

int OBJ_active(unsigned int arbre, unsigned int objet, int mettre)
{
  int retour;
  OBJECT *adresse;

  retour = OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : valeurs incorrectes [arbre = %u, objet = %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, objet);
    return -1;
  }
  if ( ROOT == objet ) {
    (void) fprintf(stderr, "OBJ_active : impossible de dsactiver la racine [arbre = %u]\n", arbre);
    return -1;
  }

  /* Adresse de l'arbre, pour travail plus rapide [Non-NULL, sinon dtect dans OBJ_ControlerValeurs()] */
  adresse = arbres[ arbre ].adresse;

  /* On ne veut que demander l'tat actuel */
  if ( QUESTION == mettre ) {
    if ( OB_STATE( adresse, objet ) & DISABLED )
      return FAUX; /* On est inactif */
    else
      return VRAI;
  }

  /* On veut juste changer : inactif -> actif et actif -> inactif */
  if ( ECHANGE == mettre ) {
    if ( OB_STATE(adresse, objet) & DISABLED ) /* Objet inactif */
      mettre = FORCE_OUI;
    else
      mettre = FORCE_NON;
  }

  /* On change l'tat */
  if (mettre == FORCE_OUI) {      /* On rend actif   */
    OB_STATE(adresse, objet) &= ~DISABLED;
    retour = VRAI; /* actif */
  } else if (mettre == FORCE_NON) { /* On rend inactif */
    OB_STATE(adresse, objet) |=  DISABLED;
    retour = FAUX; /* inactif */
  }

  /* Objet ditable : adapter si ncessaire */
  if ( OB_FLAGS( adresse, objet ) & EDITABLE ) {
    ARB_TrouverEditables( arbre, VRAI ); /* Remettre  jour la liste des ditables, si possible sans changer l'ditable courant */
  }

  if ( arbres[ arbre ].drapeaux.ouvert == VRAI ) /* L'arbre est ouvert */
    OBJ_dessine( arbres[ arbre ].objets[ objet ], arbres[ arbre ].adresse + objet );
  else if ( (BI_Menu != NULL) && (adresse == BI_Menu->adresse) ) { /* Si, par hasard, c'est un menu */
    if ( OB_TYPE(adresse, objet) == G_TITLE ) { /* Titre de menu */
      OBJ_dessine( BI_Menu->titres[ objet - BI_Menu->prem_titre ], arbres[ arbre ].adresse + objet);
    }
  }

  /* On a fini... */
  return retour;  
}

/* -------- Objet choisi (selected ou checked) ou non choisi ------------------------\
 |    Renvoie l'tat de l'objet aprs l'opration (VRAI : choisi, FAUX : non choisi) |
 | Remarque : demander l'change d'tat d'un bouton radio revient  le slectionner. |
 |                                                                                   |
 | arbre  : numro de l'arbre contenant l'objet  modifier                           |
 | objet  : numro de l'objet  modifier                                             |
 | mettre : opration  faire. FORCE_NON = on dsactive, FORCE_OUI = on active,      |
 |                   ECHANGE = on bascule de l'un  l'autre, QUESTION = on lit juste |
 \---------------------------------------------------------------------------------- */

int OBJ_choisit(unsigned int arbre, unsigned int objet, int mettre)
{
  int retour, type;
  OBJECT *adresse;

  retour= OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur dans OBJ_ControlerValeur()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  if ( objet == ROOT ) { /* Racine : non slectionnable  */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : L'objet racine n'est pas modifiable [arbre %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre);
#endif
    return FAUX;
  }

  adresse = arbres[ arbre ].adresse;
  type = OB_TYPE( adresse, objet );

  /* Cases  cocher : choisir = les cocher */
  if ( G_IBOX == type ) {
    if ( QUESTION == mettre ) {
      if ( OB_STATE( adresse, objet ) & CHECKED ) {
	return VRAI;
      } else {
	return FAUX;
      }
    }

    /* On choisit le sens de slection */
    if ( ECHANGE == mettre ) {
      if ( OB_FLAGS(adresse, objet) & RBUTTON ) {
	mettre = FORCE_OUI;
      } else if ( OB_STATE(adresse, objet) & CHECKED ) {
	mettre = FORCE_NON;
      } else
	mettre = FORCE_OUI;
    }

    /* On ralise la slection proprement dite */
    if ( FORCE_OUI == mettre ) {
      if ( OB_FLAGS(adresse, objet ) & RBUTTON ) {
	int pere, i;

	/* On trouve le pre */
	pere = OBJ_TrouverPere( arbre, objet );
	if ( pere < 0 ) {
	  (void) fprintf(stderr, "OBJ_choisit : impossible de trouver le pre [arbre %u, objet %u]\n", arbre, objet);
	  return -2;
	}

	/* Et on dcoche tous les objets radio correspondant */
	i = OB_HEAD( adresse, pere );
	while ( i != pere ) {
	  if ( OB_FLAGS(adresse, i ) & RBUTTON ) {
	    OBJ_coche( arbre, i, i == (int) objet );
	  }

	  /* Fils suivant */
	  i = OB_NEXT(adresse, i);
	}

	return VRAI;
      }

      return OBJ_coche( arbre, objet, FORCE_OUI );
    } else {
#ifdef _DEVELOPPE_
      if ( OB_FLAGS(adresse, objet ) & RBUTTON ) {
	(void) fprintf(stderr, "OBJ_choisit : attention, dslection d'un bouton radio, normalement inutile.\n");
      }
#endif
      return OBJ_coche( arbre, objet, FORCE_NON );
    }
  }

  /* Except les cases  cocher, tout le reste se comporte normalement... */
  if ( QUESTION == mettre ) {
    if ( OB_STATE( adresse, objet ) & SELECTED ) {
      return VRAI;
    } else {
      return FAUX;
    }
  }

  /* Seuls les objets marqus comme tels peuvent tre choisis */
  if ( ( OB_FLAGS(adresse, objet) & SELECTABLE ) == FAUX ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Appel pour un objet non-slectionnable [arbre %u, objet %u]\n",
		   __FILE__, __FUNCTION__, __LINE__, arbre, objet);
#endif
    return FAUX;
  }

  /* On choisit le sens de slection */
  if ( ECHANGE == mettre ) {
    if ( OB_FLAGS(adresse, objet) & RBUTTON ) {
      mettre = FORCE_OUI;
    } else if ( OB_STATE(adresse, objet) & SELECTED ) {
      mettre = FORCE_NON;
    } else
      mettre = FORCE_OUI;
  }

  if ( FORCE_OUI == mettre ) {
    if ( OB_FLAGS(adresse, objet ) & RBUTTON ) {
      int pere, i;

      /* On trouve le pre */
      pere = OBJ_TrouverPere( arbre, objet );
      if ( pere < 0 ) {
	(void) fprintf(stderr, "OBJ_choisit : impossible de trouver le pre [arbre %u, objet %u]\n", arbre, objet);
	return -2;
      }

      /* Et on dcoche tous les objets radio correspondant */
      i = OB_HEAD( adresse, pere );
      while ( i != pere ) {
	if ( OB_FLAGS(adresse, i ) & RBUTTON ) {
	  if ( i != (int) objet )
	    OB_STATE(adresse, i) &= ~SELECTED;
	  else
	    OB_STATE(adresse, i) |= SELECTED;

	  if ( VRAI == arbres[ arbre ].drapeaux.ouvert )
	    BI_DessinerObjet( arbre, i );
	}

	/* Fils suivant */
	i = OB_NEXT(adresse, i);
      }

      return VRAI;
    }

    /* Bouton isol */
    OB_STATE(adresse, objet) |= SELECTED;
    if ( VRAI == arbres[ arbre ].drapeaux.ouvert )
      BI_DessinerObjet( arbre, objet );

    return VRAI;
  } else {
    if ( OB_FLAGS(adresse, objet ) & RBUTTON ) {
      AVERTISSEMENT( "Dslection d'un bouton radio [arbre %u, objet %u], normalement inutile", arbre, objet );
    }
    OB_STATE(adresse, objet) &= ~SELECTED;
    if ( VRAI == arbres[ arbre ].drapeaux.ouvert )
      BI_DessinerObjet( arbre, objet );

    return FAUX;
  }

  AVERTISSEMENT( "On ne devrait __jamais__ tre ici [arbre %u, objet %u]", arbre, objet);
  return -10;
}

/* -------------------- Objet coch ou non-coch ---------------------------------------------\
 |    Renvoie l'tat de l'objet aprs l'opration (VRAI : coch, FAUX : non-coch)            |
 |                                                                                            |
 | arbre  : numro de l'arbre contenant l'objet  modifier                                    |
 | objet  : numro de l'objet  modifier                                                      |
 | mettre : opration  faire. FORCE_NON = on coche, FORCE_OUI = on enlve la coche,          |
 |                            ECHANGE = on bascule de l'un  l'autre, QUESTION = on lit juste |
 \------------------------------------------------------------------------------------------- */

int OBJ_coche(unsigned int arbre, unsigned int objet, int mettre)
{
  int retour;
  OBJECT *adresse;

  /* Vrifications pralables */
  retour= OBJ_ControlerValeurs( arbre, objet );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "OBJ_choisit : erreur dans OBJ_ControlerValeur()\n");
    return -1;
  }
  if ( objet == ROOT ) { /* Racine : non slectionnable  */
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "OBJ_choisit : l'objet racine n'est pas modifiable [arbre %u]\n", arbre);
#endif
    return FAUX;
  }

  adresse = arbres[ arbre ].adresse;
  /* Ici, le rsultat est indpendant du type d'objet... */
  if ( QUESTION == mettre ) {
    if ( OB_STATE(adresse, objet) & CHECKED ) /* Si QUESTION, on doit tester... */
      return VRAI;
    else
      return FAUX;
  }

  /* On choisit le sens de coche */
  if (mettre == ECHANGE) {
    if ( OB_STATE(adresse, objet) & CHECKED ) /* L'objet est coch */
      mettre = FORCE_NON; /* On enlve donc la coche */
    else
      mettre = FORCE_OUI; /* On coche */
  }

  /* On coche ou dcoche */
  if (mettre == FORCE_NON) { /* On force  tre non-coch */
    OB_STATE(adresse, objet) &= ~CHECKED; /* On enlve la coche */
    if ( arbres[ arbre ].drapeaux.ouvert == VRAI ) /* Arbre effectivement ouvert */
      BI_DessinerObjet( arbre, objet );

    return FAUX;
  } else if (mettre == FORCE_OUI) { /* On force  tre coch */
    OB_STATE(adresse, objet) |= CHECKED ; /* On met la coche */
    if ( arbres[ arbre ].drapeaux.ouvert == VRAI ) /* Arbre effectivement ouvert */
      BI_DessinerObjet( arbre, objet );

    return VRAI;
  }

#ifdef _DEVELOPPE_
  (void) fprintf(stderr, "OBJ_coche : on ne devrait __jamais__ tre ici\n");
#endif
  return -10;    
}
