/**** Bibliothèque mixte GEM / X_windows
  (C) 1998-2003 Emmanuel CURIS

  Gestion des fenêtres - compatible gem_fen.c [référence]
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bib_interface.h"
#ifdef AVEC_XPM_
# include <X11/xpm.h>
#endif

#ifndef M_PI
# define M_PI 3.141592654
#endif

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~~~~~~~~ Fonctions à usage interne ~~~~~~~~~~~~~~~~~~~~~~~ */

static Fenetre *FEN_TrouverFenetre(Fenetre *fenetre, Window fen_X);
static void     FEN_AdapterGadgets(Fenetre *fenetre, int dx, int dy);

#ifdef AVEC_OPEN_GL_
/* -------- Choisit le contexte OpenGL (GLX) le plus aproprié pour la fenêtre indiquée
   en fonction du mode de dessin utilisé : VRAI pour du dessin en direct, FAUX sinon.

   Renvoie le mode de dessin que l'on a pu retenir (le direct marche toujours)

   Actuellement, utilise aussi BI_GL_direct mais en fait, une version par fenêtre serait plus souple.
*/

boolean FEN_GL_choisit_contexte(Fenetre *fenetre, boolean mode)
{
  int x __attribute__((__unused__)), y __attribute__((__unused__)), retour;
  unsigned int larg_gl, haut_gl;
  Bool retourb;

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p mode=%d", (void *) fenetre, mode );

  if (fenetre == NULL) {
    SORTIE_FONCTION( __FUNCTION__, "%d [fenetre = NULL]", VRAI );
    return VRAI;
  }
  if (BI_OpenGL_actif == FAUX) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Open-GL inactif]", VRAI );
    return VRAI; /* De toute façon, pas de sens... */
  }
  if ( NULL == fenetre->open_gl ) {
    SORTIE_FONCTION( __FUNCTION__, "%d [Pas d'infos Open-GL sur la fenêtre !]", VRAI );
    return VRAI; /* De toute façon, pas de sens... */
  }

#ifdef _DEVELOPPE_
  (void) printf("%s,%s l.%u : Entrée... [mode = %d, direct = %d] [fenêtre X : handle = %lu]\n",
		__FILE__, __FUNCTION__, __LINE__, mode, BI_GL_direct, fenetre->handle );
#endif

  /* On récupère les dimensions de la fenêtre Open-GL */
  if ( BI_FENETRE_GL == fenetre->type ) {
    x = fenetre->w_x;
    y = fenetre->w_y;
    larg_gl = fenetre->w_larg;
    haut_gl = fenetre->w_haut;
  } else {
    (void) fprintf(stderr, "%s, %s l.%u : Appel pour une fenêtre de type incorrect [%d] : seul BI_FENETRE_GL [%d] est autorisé\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->type, BI_FENETRE_GL );
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return VRAI;
  }

  if (mode == VRAI) { /* Le dessin en mode direct marche forcément */
    /* Plus besoin de l'ancien contexte, ni du pixmap de fond */
    if ( fenetre->open_gl->image != None )
      glXDestroyGLXPixmap(BI_Display, fenetre->open_gl->image);
    if ( fenetre->open_gl->fond != None )
      XFreePixmap(BI_Display, fenetre->open_gl->fond);
    fenetre->open_gl->fond = None;
    fenetre->open_gl->image = None;

    if ( BI_GL_direct == FAUX ) { /* On ne s'occupe des contexte que si l'on change de mode
				     Sinon, on change le contexte à chaque changement de taille : gênant !*/
      if ( NULL != fenetre->open_gl->gl_contexte ) {
	glXDestroyContext(BI_Display, fenetre->open_gl->gl_contexte); /* On ôte l'ancien */
	fenetre->open_gl->gl_contexte = NULL;
      }
      BI_GL_direct = VRAI; /* On passe en mode direct */
    }
    if ( NULL == fenetre->open_gl->gl_contexte ) {
      fenetre->open_gl->gl_contexte = glXCreateContext(BI_Display, &XGL_Visual, NULL, True);
    }

    /* On crée un nouveau contexte, en mode direct, sur la fenêtre de fond */
    if ( glXMakeCurrent(BI_Display, fenetre->handle,
			fenetre->open_gl->gl_contexte) == False ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "FEN_GL_choisit_contexte : impossible de mettre un contexte en mode direct\n");
      (void) fprintf(stderr, "Fenêtre : %p (%s) - fenêtre X : %ld - GLX : %p\n",
		     (void *) fenetre, fenetre->titre, fenetre->handle,
		     (void *) fenetre->open_gl->gl_contexte);
#endif
    } else { /* On installe le viewport sur toute la fenêtre */
#ifdef _DEVELOPPE_
      printf( "%s : ViewPort direct, %ux%u [fenêtre : %u, %u, %ux%u (travail : %u, %u, %ux%u)\n", __FUNCTION__, larg_gl, haut_gl,
	      fenetre->x, fenetre->y, fenetre->larg, fenetre->haut, fenetre->w_x, fenetre->w_y, fenetre->w_larg, fenetre->w_haut );
#endif
      glViewport( (GLint) 0, (GLint) 0, (GLsizei) larg_gl, (GLsizei) haut_gl );
    }

    SORTIE_FONCTION( __FUNCTION__, "%d [mode direct]", VRAI );
    return VRAI;
  }

  DANS_FONCTION( "Affichage en mode indirect" );
  /* Si on est là : on demande un affichage en mode indirect, dans un pixmap. */
  /* On efface les éventuels anciens Pixmap et glXPixmap */
  if ( None != fenetre->open_gl->image ) {
    /* On commence par libérer le contexte */
    DANS_FONCTION( "Appel à glXMakeCurrent [display=%p, drawable=None, ctx=%p]", BI_Display, NULL );
    glXMakeCurrent( BI_Display, None, NULL );

    /* et on peut maintenant libérer le Pixmap GLX */
    DANS_FONCTION( "Appel à glXDestroyGLXPixmap" );
    glXDestroyGLXPixmap(BI_Display, fenetre->open_gl->image);
    fenetre->open_gl->image = None;
  }
  if ( None != fenetre->open_gl->fond ) {
    DANS_FONCTION( "Appel à XFreePixmap" );

    XFreePixmap(BI_Display, fenetre->open_gl->fond);
    fenetre->open_gl->fond = None;
  }

  /* On crée les nouveaux Pixmap, aux bonnes dimensions */
  DANS_FONCTION( "Appel à XCreatePixmap [Pixmap : %d * %d]", larg_gl, haut_gl );
  fenetre->open_gl->fond = XCreatePixmap(BI_Display,
					 fenetre->handle,
					 larg_gl, haut_gl, (unsigned) XGL_Visual.depth);
  if (None == fenetre->open_gl->fond) {
    (void) fprintf(stderr, "FEN_GL_choisit_contexte : impossible de créer le Pixmap pour le mode indirect\n");
    retour = FEN_GL_choisit_contexte(fenetre, VRAI); /* On passe en mode direct */

    SORTIE_FONCTION( __FUNCTION__, "%d [None == fenetre->open_gl->fond]", retour );
    return retour;
  }
  DANS_FONCTION( "Appel à glXCreateGLXPixmap" );
  fenetre->open_gl->image = glXCreateGLXPixmap(BI_Display, &XGL_Visual, fenetre->open_gl->fond);
  if (None == fenetre->open_gl->image) {
    (void) fprintf(stderr, "FEN_GL_choisit_contexte : impossible de créer le Pixmap GLX\n");
    retour = FEN_GL_choisit_contexte(fenetre, VRAI); /* On passe en mode direct */

    SORTIE_FONCTION( __FUNCTION__, "%d [None == fenetre->open_gl->image]", retour );
    return retour;
  }

  /* On travaille directement avec un contexte indirect.
     En effet, sur SGI et Linux/XFree4+DRI+Matrox, un essai avec un direct sur un pixmap donne une erreur X :-(
     On ne cherche à le créer que si on change de mode (on perd alors tout, dont les listes...)
  */
  if ( BI_GL_direct == VRAI ) {
    BI_GL_direct = FAUX;
    if ( NULL != fenetre->open_gl->gl_contexte ) {
      DANS_FONCTION( "Appel à glXDestroyContext" );
      glXDestroyContext(BI_Display, fenetre->open_gl->gl_contexte); /* On ôte l'ancien */
      fenetre->open_gl->gl_contexte = NULL;
    }
  }
  if ( NULL == fenetre->open_gl->gl_contexte ) { /* Pas encore de contexte */
    DANS_FONCTION( "Appel à glXCreateContext" );
    fenetre->open_gl->gl_contexte = glXCreateContext(BI_Display, &XGL_Visual, NULL, False);
  }
  if (NULL == fenetre->open_gl->gl_contexte) {
    (void) fprintf(stderr, "FEN_GL_choisit_contexte : impossible de créer un contexte indirect\n");
    retour = FEN_GL_choisit_contexte(fenetre, VRAI); /* On passe en mode direct */

    SORTIE_FONCTION( __FUNCTION__, "%d [NULL == fenetre->open_gl->gl_contexte]", retour );
    return retour;
  }

  DANS_FONCTION( "Appel à glXMakeCurrent [display=%p, drawable=%p, ctx=%p]",
		 BI_Display, fenetre->open_gl->image, fenetre->open_gl->gl_contexte );
  retourb = glXMakeCurrent(BI_Display, fenetre->open_gl->image,
			   fenetre->open_gl->gl_contexte);
  DANS_FONCTION( "... renvoie %d", retourb );
  if ( False == retourb ) {
    (void) fprintf(stderr, "FEN_GL_choisit_contexte : impossible d'utiliser le contexte indirect sur le pixmap GLX\n");
    retour = FEN_GL_choisit_contexte(fenetre, VRAI); /* On passe en mode direct */

    SORTIE_FONCTION( __FUNCTION__, "%d [False == glXMakeCurrent(BI_Display, fenetre->open_gl->image,fenetre->open_gl->gl_contexte)]", retour );
    return retour;
  }

  /* On installe la vue */
  DANS_FONCTION( "Appel à glViewport" );
  glViewport((GLint) 0, (GLint) 0, (GLsizei) larg_gl, (GLsizei) haut_gl);

  DANS_FONCTION( "Appel à XSetWindowBackgroundPixmap" );
  XSetWindowBackgroundPixmap(BI_Display, fenetre->handle, fenetre->open_gl->fond);

  SORTIE_FONCTION( __FUNCTION__, "%d [mode=FAUX, retomber en mode indirect]", FAUX );
  return FAUX; /* On est bien en mode indirect */
}
#endif

/* -------- Modifier la gestion des événements d'une partie Open-GL d'une fenêtre --------
    Attention, le redessin OpenGL proprement dit est modifié par FEN_AssocierEvenements()

  redessin : redessin X à faire après la partie Open-GL
  rect     : en cas de déplacement de la souris
  clic     : en cas de clic sur la fenêtre
  deplace  : quand on maintient le bouton enfoncé en bougeant la souris
  temps    : timer [animation éventuelle ?]
*/

void FEN_GL_evntsgl(Fenetre *fenetre,
		    void    (*redessin)(Fenetre *, int, int, unsigned int, unsigned int),
		    void    (*rect)    (Fenetre *, int, int, int, int),
		    boolean (*clic)    (Fenetre *, int, int, int, int, int),
		    boolean (*deplace) (Fenetre *, int, int, int, int),
		    boolean (*temps)   (Fenetre *) )
{
  if (fenetre == NULL) return;
  if ( fenetre->type != BI_FENETRE_GL ) {
    if ( BI_FENETRE_GL_ARBRE_SIMPLE == fenetre->type ) {
      /* Fenêtre triple : on transmet à la sous-fenêtre OpenGL, qui est la deuxième */
      if ( NULL == fenetre->ss_fenetres ) {
	(void) fprintf(stderr, "%s, %s l.%u : Fenêtre triple sans table de sous-fenêtres\n",
		       __FILE__, __FUNCTION__, __LINE__);
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
	return;
      }
      FEN_GL_evntsgl( fenetre->ss_fenetres[ 1 ], redessin, rect, clic, deplace, temps );
      return;
    }

    /* Autre cas : non géré actuellement */
    (void) fprintf(stderr, "FEN_GL_evntsgl : appel pour une fenêtre non Open-GL !\n");
    return;
  }

  if (fenetre->open_gl == NULL) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucune information OpenGL [%s]\n",
		   __FILE__, __FUNCTION__, __LINE__, fenetre->titre);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  if ( redessin != fenetre->open_gl->rds_post ) {
    fenetre->open_gl->rds_post = redessin;
    /* Puisque l'on a changé la fonction de redessin, on force le redessin ! */
    FEN_redessin( fenetre, BI_RDS_GL );
  }
  fenetre->rect     = rect;
  fenetre->clic     = clic;
  fenetre->open_gl->deplace  = deplace;
  fenetre->open_gl->temps    = temps;
}

/* -------------- Ouverture d'une fenêtre, pour voir -------------------- */

/* -------------- Procédure d'ouverture la plus simple -------------------\
 | Renvoie FAUX si l'ouverture n'a pas pu avoir lieu, VRAI sinon          |
 |                                                                        |
 | fenetre    : structure de la fenêtre à ouvrir                          |
 \----------------------------------------------------------------------- */

int FEN_Ouvrir(Fenetre *fenetre)
{
  if (fenetre == NULL) return FAUX;
  if (fenetre->drapeaux.ouverte == VRAI) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "FEN_Ouvrir : fenêtre déjà ouverte. Ouverture annulée\n");
#endif
    return FAUX;
  }

  /* Affichage de la fenetre pour X */
  XMapWindow( BI_Display, fenetre->handle );
  XMapSubwindows( BI_Display, fenetre->handle ); /* Rq : ATTENTION, cela ne le fait que sur UN niveau ! */

  fenetre->drapeaux.ouverte = VRAI; /* La fenêtre est ouverte */
  if ( fenetre->arbre != NULL ) {
    fenetre->arbre->drapeaux.ouvert = VRAI; /* Cet arbre est ouvert, maintenant */
    BI_ChangerEditable( fenetre->arbre, fenetre->arbre->premier_edit );
  }

  /* Récursif pour les sous-fenêtres ! */
  if ( fenetre->ss_fenetres != NULL ) {
    unsigned int i;
    Fenetre **ss_fenetres;

    ss_fenetres = fenetre->ss_fenetres;
    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      FEN_Ouvrir( ss_fenetres[ i ] );
    }
  }

  /* Pour les onglets : on coupe les fenêtres des onglets, sauf celle de l'onglet actif */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    unsigned int i;
    Fenetre **ss_fenetres;
    struct fen_onglets *infos_onglets;

    infos_onglets = (struct fen_onglets *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

    ss_fenetres = fenetre->ss_fenetres;
    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      if ( i != infos_onglets->onglet_actif ) {
	XUnmapWindow( BI_Display, ss_fenetres[ i ]->handle );
      }
    }
  }

/*  printf("FEN_Ouvrir : terminé\n"); */
  return VRAI;
}

/* ------------- Réouverture d'une fenêtre existante -------------------- */

int FEN_Reouvrir(Fenetre *fenetre)
{
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : fenetre = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1;
  }

  if ( fenetre->handle == None ) { /* Il n'y a pas de fenêtre associée -> on la crée */
    int retour = FAUX;

    retour = FEN_CreerFenetre( fenetre, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "FEN_Reouvrir : erreur dans FEN_CreerFenetre() [code %d]\n", retour);
      return -1;
    }
  }

  if ( fenetre->taille ) { /* On a des infos pour la position et la taille */
    /* On les réassocie donc à la fenêtre */
    XSetWMNormalHints( BI_Display, fenetre->handle, fenetre->taille );
  }

  return FEN_Ouvrir( fenetre );
}

/* Cherche la fenêtre (au sens GEM) à laquelle appartient la fenêtre (X) indiquée */

static Fenetre *FEN_TrouverFenetre(Fenetre *fenetre, Window fen_X)
{
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s,%s l.%u : fenetre = NULL\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  /* S'agit-il de la fenêtre elle-même ? */
  if (fenetre->handle                == fen_X) return fenetre;

  /* S'agit-il d'un gadget de fenêtre ? */
#ifdef XGEM_AVEC_CLOSER_
  if (fenetre->closer                == fen_X) return fenetre;
#endif
  if (fenetre->barre_d_infos         == fen_X) return fenetre;
  if (fenetre->fleche_haute          == fen_X) return fenetre;
  if (fenetre->fleche_basse          == fen_X) return fenetre;
  if (fenetre->fond_asc_vertical     == fen_X) return fenetre;
  if (fenetre->bouton_asc_vertical   == fen_X) return fenetre;
  if (fenetre->fleche_gauche         == fen_X) return fenetre;
  if (fenetre->fleche_droite         == fen_X) return fenetre;
  if (fenetre->fond_asc_horizontal   == fen_X) return fenetre;
  if (fenetre->bouton_asc_horizontal == fen_X) return fenetre;

  /* S'agit-il d'un des objets de l'arbre ? */ 
  if ( fenetre->arbre != NULL ) {
    Arbre *arbre = fenetre->arbre;
    unsigned int i;

    if ( NULL == arbre->objets ) {
      (void) fprintf(stderr, "%s,%s l.%u : La fenêtre %s contient un arbre sans objets\n",
		     __FILE__, __FUNCTION__, __LINE__, fenetre->titre);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      for (i = 0 ; i < arbre->nbr_objets ; i++)
	if ( (arbre->objets)[i] == fen_X )
	  return fenetre;
    }
  }

  /* S'agit-il de la fenêtre OpenGL ? */
  if ( NULL != fenetre->open_gl ) {
/*     if (fenetre->open_gl->handle == fen_X) */
/*       return fenetre; */
  }

  /* S'agit-il d'un onglet ? */
  if ( BI_FENETRE_ONGLETS == fenetre->type ) {
    int numero_onglet;
    boolean est_actif;

    numero_onglet = BIX_QuelOnglet( fenetre, fen_X, &est_actif );
    if ( numero_onglet >= 0 ) {
      return fenetre; /* Onglet trouvé */
    }
  }

  /* S'agit-il d'une sous-fenêtre ? */
  if ( fenetre->ss_fenetres != NULL ) {
    Fenetre *resultat;
    unsigned int i;

    for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
      resultat = fenetre->ss_fenetres[ i ];
      if ( resultat != NULL ) { /* La sous-fenêtre existe bien ! */
/* 	printf( "%s : On cherche dans la sous-fenêtre %u [%u] de la fenêtre %u [%s]\n", */
/* 		__FUNCTION__, i, (unsigned int) resultat->handle, (unsigned int) fenetre->handle, fenetre->titre ); */
	resultat = FEN_TrouverFenetre( resultat , fen_X );
	if ( resultat != NULL )
	  return resultat; /* On a trouvé la fenêtre */
      }
    }
  }

  /* On n'a rien trouvé */
  return NULL;
}

Fenetre *BIX_QuelleFenetre(Window fen_X)
{
  Fenetre *resultat, *fenetre;

  if ( NULL == BI_premiere_fenetre ) return NULL; /* Aucune fenêtre... */

  /* On essaye toutes les fenêtres du chaînage */
  for ( fenetre = BI_premiere_fenetre ; fenetre != NULL ; fenetre = fenetre->suivante ) {
    resultat = FEN_TrouverFenetre( fenetre, fen_X );
    if ( resultat != NULL ) /* On a trouvé la fenêtre */
      return resultat;
  }

#ifdef _DEVELOPPE_
/*   (void) fprintf(stderr, "%s,%s l.%u : fenêtre non-trouvée [%lu]\n", */
/* 		 __FILE__, __FUNCTION__, __LINE__, (unsigned long) fen_X); */
#endif
  return NULL; /* Fenêtre non trouvée */
}

/* -------- Trouver l'onglet qui correspond à la fenêtre X indiquée -------- */

int BIX_QuelOnglet(Fenetre *fenetre, Window fen_X, boolean *est_actif)
{
  struct fen_onglets *infos_onglets;
  unsigned int i;

  if ( fenetre->type != BI_FENETRE_ONGLETS ) {
    AVERTISSEMENT( "Appel pour la fenêtre %s, sans onglets ! [type %d]", fenetre->titre, fenetre->type );
    return -1;
  }

  /* On récupère la table des onglets */
  infos_onglets = (struct fen_onglets *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos_onglets ) {
    AVERTISSEMENT( "Fenêtre %s : pas d'onglets !", fenetre->titre );
    return -2; /* Pas d'onglets */
  }

  /* Il y a une sous-fenêtre par onglet ==> le nombre d'onglets est celui de sous-fenêtres ! */
  *est_actif = FAUX;
  for ( i = 0 ; i < fenetre->nbr_ss_fenetres ; i++ ) {
    if ( fen_X == infos_onglets->onglets[ i ] ) {
      if ( i == infos_onglets->onglet_actif ) *est_actif = VRAI;
      return i;
    }
  }

  /* Onglet non-trouvé */
  return -10;
}

/* -------- Envoyer un message de changement de taille à une fenêtre --------\
 |    Renvoie VRAI si tout s'est bien passé, FAUX sinon                      |
 |                                                                           |
 |  x, y, larg, haut définissent la position et les dimensions de la fenêtre |
 |      TOTALE (pas de la zone de travail !)                                 |
 | si valeurs <0, exprimées en % de la dimension de l'écran                  |
 | si valeurs >0, exprimées en tailles absolues en pixels                    |
 \-------------------------------------------------------------------------- */

boolean FEN_dimensions(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  unsigned int lg_original, ht_original;

  if ( NULL == fenetre ) return FAUX;      /* Pas de fenêtre */

  /* Conversion éventuelle relatif -> écran */
  if (x < 0)
    x = (int) ( BI_larg_ecran * ( (float) -x) / 1000.0 );

  if (y < 0)
    y = (int) ( BI_haut_ecran * ( (float) -y) / 1000.0 );

  if (larg < 0)
    larg = (int) ( BI_larg_ecran * ( (float) -larg) / 1000.0 );

  if (haut < 0)
    haut = (int) ( BI_haut_ecran * ( (float) -haut) / 1000.0 );

  /* Vérifications. Si invalide, on garde les coordonnées existantes */
  if ( ( x < 0 ) || ( x > (int) BI_larg_ecran ) )
    x = fenetre->x;
  if ( ( y < 0 ) || ( y > (int) BI_haut_ecran ) )
    y = fenetre->y;
  if ( ( larg <= 0 ) || ( larg > (int) BI_larg_ecran ) )
    larg = fenetre->larg;
  if ( ( haut <= 0 ) || ( haut > (int) BI_haut_ecran ) )
    haut = fenetre->haut;

  /* On adapte la région de travail [X et Y toujours inchangés !]*/
#ifdef _DEVELOPPE_
  printf( "%s : nouveaux = %ux%u, anciens = %ux%u // Fenetre %p [%s]\n",
	  __FUNCTION__, larg, haut, fenetre->larg, fenetre->haut, (void *) fenetre, fenetre->titre );
#endif

  /* Dimensions originales de la fenêtre complète [conditionne la position des gadgets]... */
  lg_original = fenetre->larg;
  ht_original = fenetre->haut;

  /* On met les bonnes valeurs */
  fenetre->x = x;
  fenetre->y = y;

  /* On adapte les gadgets */
  (void) FEN_AdapterGadgets( fenetre, fenetre->larg - lg_original, fenetre->haut - ht_original );

  /* Changement de taille effectif */
  if ( None == fenetre->handle ) { /* Pas de fenêtre X */
    /* On met les dimensions, car pas d'appel X => pas d'appel à change_taille_fenetre */
    fenetre->larg = larg;
    fenetre->haut = haut;

    /* Idem pour la région de travail */
    fenetre->w_larg += (larg - lg_original);
    fenetre->w_haut += (haut - ht_original);
  } else {
    /* Vérifications pour éviter une erreur X */
    if ( ( larg <= 0 ) || ( haut <= 0 ) ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problème de dimensions ! X=%d, Y=%d, L=%d [fenêtre : %u], H=%d [fenêtre : %u]\n",
		     __FILE__, __FUNCTION__, __LINE__, x, y, larg, (unsigned int) larg, haut, (unsigned int) haut);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      return FAUX;
    }

    /* On change la fenêtre contenante X-Window */
    if ( NULL == fenetre->taille ) {
      fenetre->taille = XAllocSizeHints();
    }
    if ( fenetre->taille != NULL ) {
      fenetre->taille->flags = PPosition;
      fenetre->taille->x = x;
      fenetre->taille->x = y;
      XSetWMNormalHints( BI_Display, fenetre->handle, fenetre->taille );
    }
    XMoveResizeWindow( BI_Display, fenetre->handle, x, y, (unsigned int) larg, (unsigned int) haut );

    /* S'il s'agit d'une fenêtre Open-GL & Open-GL est actif, il faut aussi mettre à jour le contexte ! */
    if ( ( BI_FENETRE_GL == fenetre->type ) && ( VRAI == BI_OpenGL_actif ) ) {
#ifdef AVEC_OPEN_GL_
      FEN_GL_choisit_contexte( fenetre, BI_GL_direct );
      if ( FAUX == BI_GL_direct ) {
	/* On refait le dessin. Ne peut pas être fait dans FEN_GL_... car aussi appelé à l'ouverture,
	   or à ce moment-là les préparatifs (listes,...) n'ont pas été faits */
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	if ( fenetre->redessin != ( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL )
	  (fenetre->redessin)( fenetre, 0, 0, fenetre->w_larg, fenetre->w_haut );
	glFlush();
	if ( VRAI == BI_OpenGL_dBuffer ) {
	  glXSwapBuffers( BI_Display, fenetre->open_gl->image );
	}
      
	/* On force l'affichage de ce qui a été dessiné */
	XClearWindow( BI_Display, fenetre->handle );
      }
#else
	/* Sans Open-GL, il n'y a vraiment rien à faire en changeant de taille ! */
#endif
    }

    /* S'il y a des sous-fenetres : on adapte leur taille */
  }

  return VRAI;
}

/* -------- Changer la palette associée à une fenêtre -------- */

boolean FEN_palette(Fenetre *fenetre, Palette palette)
{
  if (fenetre == NULL) return FAUX;
  if (palette == None) {
    fenetre->drapeaux.palette = FAUX;
    return FAUX;
  }

  fenetre->drapeaux.palette = VRAI;
  /* On installe la palette de la fenêtre */
  XSetWindowColormap(BI_Display, fenetre->handle, palette);
  /* On informe le WM qu'elle a sa propre palette */
  if (XSetWMColormapWindows(BI_Display, fenetre->handle, &(fenetre->handle), 1) == False) {
    fprintf(stderr, "Fenêtre %s : impossible d'indiquer la ColorMap\n", fenetre->titre);
  }

  return VRAI;
}

/* -------- Redimensionnement d'une fenêtre -> modif des éléments associés -------- */

int change_taille_fenetre(Fenetre *fenetre, int nv_larg, int nv_haut)
{
  int retour;
  int delta_x, delta_y;
  unsigned int vx_larg __attribute__((__unused__)), vx_haut __attribute__((__unused__));

  ENTREE_FONCTION( __FUNCTION__, "fenetre=%p, nv_larg=%d, nv_haut=%d", fenetre, nv_larg, nv_haut );

  delta_x = nv_larg - fenetre->larg;
  delta_y = nv_haut - fenetre->haut;

#ifdef _DEVELOPPE_
  printf("%s : fenetre = %s, delta_x = %d, delta_y = %d\n", __FUNCTION__, fenetre->titre, delta_x, delta_y);
#endif
  if ( ( 0 == delta_x ) && ( 0 == delta_y ) ) {
    SORTIE_FONCTION ( __FUNCTION__, "%d [taille inchangée]", 0 );
    return 0;
  }

  /* On s'occupe de tout le décor de la fenêtre */
  (void) FEN_AdapterGadgets( fenetre, delta_x, delta_y );

  /* On enregistre le changement de taille */
  fenetre->larg = nv_larg;
  fenetre->haut = nv_haut;
  fenetre->w_larg += delta_x;
  fenetre->w_haut += delta_y;

  /* On fait les opérations nécessaires, en fonction du type de fenêtre */
  switch ( fenetre->type )
  {
  case BI_FENETRE_SIMPLE:
    /* Rien de spécial à faire, en-dehors des gadgets ! */
    break;

  case BI_FENETRE_ARBRE:
    /* La racine doit toujours être de la taille de la fenêtre ! */
    OBJ_taille( fenetre->numero_arbre, ROOT, fenetre->w_larg, fenetre->w_haut, NULL, NULL);
    break;

  case BI_FENETRE_IMAGE:
    OPTION_A_FAIRE( "Fenêtre BI_FENETRE_IMAGE" );
    break;

  case BI_FENETRE_GL:
#ifdef AVEC_OPEN_GL_
    /* On redéfinit le contexte. De toute façon, s'occupe des Pixmap et autres trucs de ce genre */
    /*    printf("On va appeler FEN_GL_choisit_contexte depuis change_taille_fenetre\n"); */
    (void) FEN_GL_choisit_contexte( fenetre, BI_GL_direct );
    if ( FAUX == BI_GL_direct ) {
      /* On refait le dessin. Ne peut pas être fait dans FEN_GL_... car aussi appelé à l'ouverture,
	 or à ce moment-là les préparatifs (listes,...) n'ont pas été faits */
      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
      if ( fenetre->redessin != ( void (*)(Fenetre *, int, int, unsigned int, unsigned int) ) NULL )
	(fenetre->redessin)( fenetre, 0, 0, fenetre->w_larg, fenetre->w_haut );
      glFlush();
      if ( VRAI == BI_OpenGL_dBuffer ) {
	glXSwapBuffers( BI_Display, fenetre->open_gl->image );
      }
      
      /* On force l'affichage de ce qui a été dessiné */
      XClearWindow( BI_Display, fenetre->handle );
    }
#endif /* AVEC_OPEN_GL_ */
    break;

  case BI_FENETRE_MULTIPLE:
    /* On adapte les sous-fenêtres à cette nouvelle dimension */
    retour = FEN_OptimiserDimensionsInternes( fenetre );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ATTENTION, l'initialisation des dimensions internes a échoué\n",
		     __FILE__, __FUNCTION__, __LINE__);
      (void) fprintf(stderr, "change_taille_fenetre : \n");
    }
    break;

  case BI_FENETRE_SIMPLE_ARBRE:
  case BI_FENETRE_GL_ARBRE:
    /* On adapte l'arbre */
    FEN_AdapterArbre( fenetre, fenetre->ss_fenetres[ 0 ] );

    /* On adapte la fenêtre utilisateur (ou Open GL !) */
    vx_larg = fenetre->ss_fenetres[ 1 ]->larg;
    vx_haut = fenetre->ss_fenetres[ 1 ]->haut;
    FEN_AdapterSousFenetre( fenetre, fenetre->ss_fenetres[ 0 ], fenetre->ss_fenetres[ 1 ] );
/*     (void) FEN_AdapterGadgets( fenetre->ss_fenetres[ 1 ], fenetre->ss_fenetres[ 1 ]->larg - vx_larg, fenetre->ss_fenetres[ 1 ]->haut - vx_haut ); */
    break;

  case BI_FENETRE_GL_SIMPLE:
    /* On adapte la fenêtre Open-GL */
    FEN_AdapterOpenGL( fenetre, fenetre->ss_fenetres[ 0 ], 500 );

    /* On adapte la fenêtre utilisateur (ou Open GL !) */
    vx_larg = fenetre->ss_fenetres[ 1 ]->larg;
    vx_haut = fenetre->ss_fenetres[ 1 ]->haut;
    FEN_AdapterSousFenetre( fenetre, fenetre->ss_fenetres[ 0 ], fenetre->ss_fenetres[ 1 ] );
    break;

  case BI_FENETRE_GL_ARBRE_SIMPLE:
    /* On adapte l'arbre */
    FEN_AdapterArbre( fenetre, fenetre->ss_fenetres[ 0 ] );

    /* On adapte la partie Open GL */
    FEN_AdapterSousFenetre( fenetre, fenetre->ss_fenetres[ 0 ], fenetre->ss_fenetres[ 1 ] );

    /* On adapte la partie utilisateur */
    FEN_AdapterSousFenetre( fenetre, fenetre->ss_fenetres[ 0 ], fenetre->ss_fenetres[ 2 ] );
    break;
  }

  /* Si fenêtre demande un message de redessin en cas de changement de taille, on le fait */
  if ( fenetre->drapeaux.redessin == VRAI ) {
#ifdef _DEVELOPPE_
    (void) printf("On appelle FEN_redessin depuis change_taille_fenetre\n");
#endif
    FEN_redessin(fenetre, BI_RDS_TOUT);
  }

  SORTIE_FONCTION ( __FUNCTION__, "%d [normal]", 1 );
  return 1; /* Pas de problème */
}

/* -------- Adapter la position & la taille des gadgets d'une fenêtre --------
   Renvoie    0 si tout s'est bien passé,
           != 0 en cas de problème

   fenetre : la fenêtre à adapter
   dx, dy  : la variation de largeur (dx) et de hauteur (dy)
 */

static void FEN_AdapterGadgets(Fenetre *fenetre, int dx, int dy)
{
  Window w;
  int x, y;
  unsigned int larg, haut, j;

  if ( fenetre->barre_d_infos ) {
    DANS_FONCTION( "Barre d'infos" );

    XGetGeometry ( BI_Display, fenetre->barre_d_infos, &w, &x, &y, &larg, &haut, &j, &j );
    if ( ( larg + dx ) > 0 ) {
      XResizeWindow( BI_Display, fenetre->barre_d_infos, dx + larg, haut );
    }
  }

  if ( fenetre->fleche_haute ) {
    DANS_FONCTION( "Flèche haute" );

    XGetGeometry( BI_Display, fenetre->fleche_haute, &w, &x, &y, &larg, &haut, &j, &j );
    XMoveWindow ( BI_Display, fenetre->fleche_haute, dx + x, y );
  }

  if ( fenetre->fond_asc_vertical ) {
    int x_asc, y_asc;
    unsigned int l_asc, h_asc;
    float facteur_pos, facteur_taille;

    DANS_FONCTION( "Fond ascenseur vertical" );

    XGetGeometry( BI_Display, fenetre->fond_asc_vertical, &w, &x, &y, &larg, &haut, &j, &j );
    if ( ( haut + dy ) > 0 ) {
      /* La fenêtre resterait visible */
      XGetGeometry( BI_Display, fenetre->bouton_asc_vertical, &w, &x_asc, &y_asc, &l_asc, &h_asc, &j, &j );

      XMoveResizeWindow( BI_Display, fenetre->fond_asc_vertical, dx + x, y, larg, haut + dy );

      facteur_taille = ( (float) h_asc ) / (float) haut;
      if ( haut != h_asc ) { /* Poussoir plus petit que l'ascenseur */
	facteur_pos  = 1.0 - facteur_taille;
	facteur_pos /= (float) (haut - h_asc);
	facteur_pos *= (float) (haut + dy);
      } else {
	facteur_pos = 1; /* Pas de raison de changer la position */
      }

      /* La nouvelle taille du poussoir */
      h_asc = (unsigned int) ( (haut + dy) * facteur_taille );
      if ( 0 == h_asc ) {
	/* poussoir trop petit : bloqué à 1 */
	h_asc = 1;
      }

      XMoveResizeWindow( BI_Display, fenetre->bouton_asc_vertical,
			 x_asc, (int) (y_asc * facteur_pos), l_asc, h_asc );
    } else {
      XMoveResizeWindow( BI_Display, fenetre->fond_asc_vertical, dx + x, y, larg, 1 );
    }
  }

  if ( fenetre->fleche_basse ) {
    DANS_FONCTION( "Flèche basse" );

    XGetGeometry( BI_Display, fenetre->fleche_basse, &w, &x, &y, &larg, &haut, &j, &j );
    XMoveWindow ( BI_Display, fenetre->fleche_basse, dx + x, y + dy );
  }

  if ( fenetre->fleche_gauche ) {
    DANS_FONCTION( "Flèche gauche" );

    XGetGeometry( BI_Display, fenetre->fleche_gauche, &w, &x, &y, &larg, &haut, &j, &j );
    XMoveWindow ( BI_Display, fenetre->fleche_gauche, x, y + dy );
  }

  if ( fenetre->fond_asc_horizontal ) {
    int x_asc, y_asc;
    unsigned int l_asc, h_asc;
    float facteur_pos, facteur_taille;

    DANS_FONCTION( "Fond ascenseur horizontal" );

    XGetGeometry( BI_Display, fenetre->fond_asc_horizontal, &w, &x, &y, &larg, &haut, &j, &j );
    if ( ( larg + dx ) > 0 ) {
      XGetGeometry( BI_Display, fenetre->bouton_asc_horizontal, &w,
		    &x_asc, &y_asc, &l_asc, &h_asc, &j, &j);
      XMoveResizeWindow( BI_Display, fenetre->fond_asc_horizontal,
			 x, y + dy, larg + dx, haut );

      facteur_taille = ( (float) l_asc ) / (float) larg;
      if ( larg != l_asc ) { /* Poussoir plus petit que l'ascenseur */
	facteur_pos  = 1 - facteur_taille;
	facteur_pos /= (float) (larg - l_asc);
	facteur_pos *= (float) (larg + dx);
      } else {
	facteur_pos = 1;
      }

      /* Calcul de la nouvelle taille du poussoir horizontal */
      l_asc = (unsigned int) ( (larg + dx) * facteur_taille );
      if ( 0 == l_asc ) {
	l_asc = 1;
      }

      XMoveResizeWindow( BI_Display, fenetre->bouton_asc_horizontal,
			 (int) (x_asc * facteur_pos), y_asc, l_asc, h_asc );
    } else {
      XMoveResizeWindow( BI_Display, fenetre->fond_asc_horizontal, x, y + dy, 1, haut );
    }
  }

  if ( fenetre->fleche_droite ) {
    DANS_FONCTION( "Flèche droite" );

    XGetGeometry( BI_Display, fenetre->fleche_droite, &w, &x, &y, &larg, &haut, &j, &j );
    XMoveWindow ( BI_Display, fenetre->fleche_droite, dx + x, y + dy );
  }
}
