/* Bibliothque GEM pour X-Windows
   (C) 1998, Emmanuel CURIS

   Bibliothque de ressources (RSRC lib)
   Attention : structure RSHDR dfinie dans X_ressources.h

   10 avril 1998
     * Fonction rsrc_load() fonctionne (aux limitations de X_ressources.c prs)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "Bib_X.h"

static Ressource *rsrc_ressource = NULL; /* Le ressource, unique, charg par rsrc_load */

/* --------------- rsrc_load() : charge un fichier *.RSC ---------------- *
 *   Renvoie FAUX si le chargement a chou                               *
 *           VRAI sinon                                                   *
 *                                                                        *
 * Principe de l'mulation : on charge le ressource, un seul autoris     *
 *   et on fait la correspondance arbre GEM <-> arbre "X"                 *
 * ---------------------------------------------------------------------- */

int rsrc_load(const char *nom_fichier)
{
  extern OBJECT *adresse_selecteur; /* Dans X_fichiers.c */
  Arbre *nv_arbres, ex_alerte, ex_selecteur;
  int i;

  if (nom_fichier == NULL)
  {
    fprintf(stderr, "rsrc_load : pas de nom de fichier\n");
    return FAUX;
  }

  if (rsrc_ressource) /* Dj un fichier charg par rsrc_load() */
  {
    printf("On ne peut charger qu'une seule ressource  la fois\n");
    printf("avec les fonctions rsrc_...().\n");
    printf("Pour utiliser davantage de ressources, utiliser\n");
    printf("les fonctions internes - cf. X_ressources.h\n");
    return FAUX;
  }

  /* On cre une structure pour charger le ressource */

  rsrc_ressource = RSC_creer();
  if (rsrc_ressource == NULL) /* Cration a rat */
    return FAUX;

  if ( RSC_charge(nom_fichier, rsrc_ressource) == FAUX ) /* Erreur  la lecture */
  {
    fprintf(stderr,"rsrc_load : erreur au chargement\n");
    return FAUX;
  }

  /* On mmorise les arbres internes dj prpars */
  ex_alerte = arbres[_nbr_arbres];
  ex_selecteur = arbres[_nbr_arbres + 1];

  /* Maintenant, on initialise les arbres */

  nv_arbres = realloc(arbres, sizeof(Arbre) * (rsrc_ressource->nombre_arbres + 3));
  if (nv_arbres == NULL)
  {
#ifdef _DEVELOPPE_
    fprintf(stderr, "rsrc_load : mmoire insuffisante (nv_arbres)\n");
#endif
    rsrc_free();
    return FAUX;
  }
  arbres = nv_arbres;
  _nbr_arbres = rsrc_ressource->nombre_arbres;

  /* Les arbres  usage interne : bote d'alerte, slecteur de fichiers, sous-menu droulant */

  _ARB_alertes = arbres + _nbr_arbres;
  _ARB_alertes->adresse = Boite_d_alerte;
  arbres[_nbr_arbres] = ex_alerte;

  /* Prparation du slecteur de fichiers */

  _ARB_fselect = arbres + _nbr_arbres + 1;
  _ARB_fselect->adresse = adresse_selecteur;
  arbres[_nbr_arbres + 1] = ex_selecteur;

  /* Prparation des arbres du RSC */

  for (i = 0 ; i < _nbr_arbres ; i++)
  {
    if ( RSC_lit_adresse(rsrc_ressource, R_TREE, i, &(arbres[i].adresse) ) )
    {
#ifdef _DEVELOPPE_
      fprintf(stderr, "rsrc_load : RSC_lit_adresse a chou\n");
#endif
      return FAUX;
    }
    ARB_init(i, FAUX);
  }

#ifdef _DEVELOPPE_
  printf("rsrc_load : fini, tout va bien\n");
#endif
  return VRAI; /* Fonctionne bien */
}

/* --------------- rsrc_free() : libre le ressource charg par rsrc_load -------- */

int rsrc_free(void)
{
  if (rsrc_ressource == NULL)
  {
#ifdef _DEVELOPPE_
    fprintf(stderr, "rsrc_free : aucun fichier RSC charg par rsrc_load()");
#endif
    return -1; /* Pas de ressource charg */
  }

  RSC_effacer(rsrc_ressource); /* On efface le ressource */

  rsrc_ressource = NULL; /* Maintenant, possible d'en charger un autre */
  return 0; /* Pas d'erreur */
}

/* --------------- rsrc_gaddr() : cherche une adresse dans le ressource charg par rsrc_load */

int rsrc_gaddr(int type, int j, void *adresse)
{
  OBJECT *obj;
  unsigned long i;

  if (rsrc_ressource == NULL)
  {
#ifdef _DEVELOPPE_
    fprintf(stderr, "rsrc_gaddr : aucun fichier RSC charg par rsrc_load()");
#endif
    return -1; /* Pas de ressource charg */
  }

  return RSC_lit_adresse(rsrc_ressource, type, j, adresse);
}

int rsrc_saddr(int rstype, int rsid, void *lngval);

/* --------------- rsrc_obfix() : on convertit les coords. car. en coords. cran ----
   ATTENTION : c'est pour un objet -> valable quel que soit le ressource...           */

int rsrc_obfix(OBJECT *adresse, unsigned int objet)
{
  unsigned int arbre;

  arbre = quel_arbre( adresse );
  RSC_CarVEcran( arbre, objet );
  return VRAI;
}
