/* Bibliothque X-Window pour les interfaces
	    Compatible avec gem_gene.c (version GEM : rfrence)
	    Routines VDI avances
   (C) 1998, Emmanuel CURIS

 2 avril 1998
   * Cration du fichier
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "Bib_X.h"

static Station_VDI *premiere_station = NULL;
static unsigned int nbr_stations = 0;

static int VDI_prem_handle(void);

/* ---------- Ouverture d'une station VDI ----------\
 | Renvoie la strucure prte  l'emploi             |
 |                                                  |
 | type : le type de station  ouvrir               |
 \------------------------------------------------- */

Station_VDI *VDI_ouvre(int type)
{
  XGCValues valeurs;
  unsigned long masque;
  Station_VDI *station;
  Window w;
  int i, x, y, handle;
  unsigned int larg, haut, bord, plans;

  if (type > 9) { /* Autre que l'cran */
    (void) fprintf(stderr, "VDI_ouvre : pour avoir un affichage sur autre chose que l'cran,\n");
    (void) fprintf(stderr, "            il faut utiliser la version TOS (atari) du programme.\n");
    return NULL;
  }

  station = (Station_VDI *) malloc( sizeof(Station_VDI) );
  if (station == NULL) {
    (void) fprintf(stderr, "VDI_ouvre : mmoire insuffisante (station : taille %lu)\n", (unsigned long) sizeof(Station_VDI) );
    return NULL;
  }

  if ( NULL == premiere_station ) { /* Aucune station */
    premiere_station = station;
    handle = 1; /* Premire station possible */
  } else {
    Station_VDI *derniere_station;

    handle = VDI_prem_handle();
    if ( handle < 0 ) {
      (void) fprintf(stderr, "VDI_ouvre : plus de station VDI disponible\n");
      free(station);
      return NULL;
    }

    /* On se place sur la dernire */
    derniere_station = premiere_station;
    while ( derniere_station->suivante != NULL ) {
      derniere_station = derniere_station->suivante;
    }

    /* Et on met la nouvelle  la suite */
    derniere_station->suivante = station;
  }

  station->handle = handle;
  station->suivante = NULL;
  nbr_stations++;
  
  masque = GCFunction | GCForeground | GCBackground | GCLineWidth |
    GCLineStyle | GCCapStyle | GCJoinStyle | GCFillStyle | GCFillRule | GCFont;

  valeurs.function   = GXcopy; /* quivalent du mode remplacement */
  valeurs.foreground = BI_Couleurs[ NOIR  ].pixel;
  valeurs.background = BI_Couleurs[ BLANC ].pixel;
  valeurs.line_width = 0;
  valeurs.line_style = LineSolid; /* Ligne continue */
  valeurs.cap_style = CapProjecting;	/* On trace _tous_ les points */
  valeurs.join_style =  JoinMiter; /* On relie les points */
  valeurs.fill_style = FillSolid;
  valeurs.fill_rule  = WindingRule;
  valeurs.font       = BI_Fonte->fid;

  station->type        = type;
  if (_PRG_station != NULL) /* Avec cette modif, affichage de la grille correct sous WindowMaker et twm
			       Apparemment, le dessin avec deux GC simultans dans la mme fentre a des rsultats bizarres... */
    station->gr_contexte = _PRG_station->gr_contexte;
  else
    station->gr_contexte = XCreateGC(BI_Display, BI_Fen_racine, masque, &valeurs);

  XGetGeometry(BI_Display, BI_Fen_racine, &w, &x, &y, &larg, &haut, &bord, &plans);
  station->larg = larg;
  station->haut = haut;
  station->n_plans = plans;

  station->x_pixel = ( (float) WidthMMOfScreen(BI_Ecran))  / ( (float) WidthOfScreen(BI_Ecran)) ; /* [mm / pixel] */
  station->y_pixel = (float) HeightMMOfScreen(BI_Ecran) / (float) HeightOfScreen(BI_Ecran); /* [mm / pixel] */

  if (gdos != -2) {
  } else
    station->nbr_fontes = 0;

  station->fonte       = BI_Fonte; /* Fonte systme par dfaut */

  station->lgn_couleur   = NOIR;
  station->lgn_style     = SOLID;
  station->lgn_epaisseur = 1;

  station->txt_angle   = 0;
  station->txt_couleur = NOIR;
  station->txt_just    = BI_TEXTE_FER_A_GAUCHE;
  station->txt_vert    = BI_TEXTE_FER_EN_BAS;

  station->mk_couleur  = NOIR;
  station->mk_taille   = 10; /* 10 pixels de haut  */
  station->mk_type     =  0; /* Points par dfaut */

  /* Les couleurs : seules celles de 1  16 sont dfinies */
  for (i = 0 ; i < 15 ; i++) {
    station->couleurs[i] = BI_Couleurs[i];
  }
  for ( ; i < 256 ; i++) { /* Le reste est noir par dfaut */
    station->couleurs[i] = BI_Couleurs[NOIR];
  }

  return station;
}

/* --- Fonction interne : donne le premier handle libre... ------- */

static int VDI_prem_handle(void)
{
  Station_VDI *station = premiere_station;
  int handle = nbr_stations;
  boolean existe;

  if (station == NULL)
    return 1;

  do {
    handle++;
    existe = FAUX; /* On suppose qu'elle n'existe pas dj */
    for (station = premiere_station ; station ; station = station->suivante) {
      if ( station->handle == handle ) { /* La station existe dj */
	existe = VRAI;
	break; /* donc on passe de suite  la suivante */
      }
    }
  } while (existe == VRAI);

  return handle; /* Handle libre [pas forcment le premier...] */
}

Station_VDI *quelle_station(int handle)
{
  Station_VDI *station;

  if (premiere_station == NULL)
    return NULL;

  for (station = premiere_station ; station ; station = station->suivante) {
    if (station->handle == handle)
      return station;
  }

  (void) fprintf(stderr, "quelle_station : aucune station n'a le handle indiqu [%d]\n", handle);
  return NULL;
}

/* -------------- Fermeture d'une station VDI -------------------- */

int VDI_ferme(Station_VDI *station)
{
  Station_VDI *avant;

  if (station == NULL)
    return -1;
  
  XFreeGC(BI_Display, station->gr_contexte); /* Plus besoin du GC */

  if (premiere_station == station) {
    premiere_station = station->suivante;
  } else {
    for (avant = premiere_station ; avant->suivante != station ; avant = avant->suivante);
    avant->suivante = station->suivante; /* On la sort du chanage */
  }

  nbr_stations--;
  free(station); /* Ni de la station */

  return 0; /* Pas d'erreur */
}

/* -------- Cration d'une nouvelle palette de couleur
   -------- */

Palette VDI_creer_palette(void)
{
  Colormap palette;

  /* Remarque : si l'on est sur du PseudoColor par dfaut, a risque de faire mal...
     Il faudrait donc contrler tout a, mais dans un premier temps... */
  palette = XCreateColormap(BI_Display, BI_Fen_racine, BI_Visual, AllocNone);

  return (Palette) palette;
}

/* -------- Cration d'un dgrad de couleurs comme palette de station VDI

   Renvoie le nombre de couleurs qui n'ont PAS pu tre alloues

 -------- */

int VDI_creer_degrade(Station_VDI *station, int couleur, Palette palette)
{
  int i, nbr_echecs = 0;
  Status retour;

  if (palette == None) /* On utilise la Colormap par dfaut pour chercher les couleurs */
    palette = BI_Palette;

  if (couleur < 0) {
    /* Essai de dgrad noir -> bleu -> jaune -> rouge */
    station->couleurs[0].red = 65535;
    station->couleurs[0].green = 65535;
    station->couleurs[0].blue = 65535;
    for (i = 0 ; i < 255 / 3 ; i++) { /* noir -> bleu */
      station->couleurs[i].red = 0;
      station->couleurs[i].green = 0;
      station->couleurs[i].blue = (short int) (65535.0 * (i / 85.0));
    }
    for ( ; i < 2 * 85 ; i++) { /* bleu -> jaune */
      station->couleurs[i].red = (short int) (65535.0 * ((i - 85.0) / 85.0));
      station->couleurs[i].green = 0;
      station->couleurs[i].blue = 65535;
    }
    for ( ; i < 256 ; i++) { /* jaune -> rouge */
      station->couleurs[i].red = 65535;
      station->couleurs[i].green = 0;
      station->couleurs[i].blue = (short int) (65535.0 * ((255.0 - i) / 85.0));
    }
  }

  if ( BLANC == couleur ) { /* On met la palette par dfaut */
    for (i = 0 ; i < 256 ; i++) {
      station->couleurs[ i ] = BI_Couleurs[ i ];
    }
  }

  for (i = 0 ; i < 256 ; i++) {
    int valeur = i * 257;

    switch (couleur) {
    case NOIR: case GRIS: case GRIS_FONCE:
      station->couleurs[i].red   = valeur;
      station->couleurs[i].green = valeur;
      station->couleurs[i].blue  = valeur;
      break;

    case ROUGE: case ROUGE_FONCE:
      station->couleurs[i].red   = valeur;
      station->couleurs[i].green = 0;
      station->couleurs[i].blue  = 0;
      break;

    case VERT: case VERT_FONCE:
      station->couleurs[i].red   = 0;
      station->couleurs[i].green = valeur;
      station->couleurs[i].blue  = 0;
      break;

    case BLEU: case BLEU_FONCE:
      station->couleurs[i].red   = 0;
      station->couleurs[i].green = 0;
      station->couleurs[i].blue  = valeur;
      break;
    }

    retour = XAllocColor(BI_Display, palette, station->couleurs + i);
    if (retour == FAUX) {
      (void) fprintf(stderr, "creer_degrade : impossible d'allouer la couleur %d\n", i);
      nbr_echecs++;
    }
  }

  return nbr_echecs;
}

/* -------- Sauvegarde de la palette d'une station de travail -------- */

int VDI_sauver_palette(Station_VDI *station)
{
  VDI_XPalette *palette;
  unsigned int i;

  if (station == NULL) return FAUX;

  palette = (VDI_XPalette *) malloc(sizeof(VDI_XPalette));
  if (palette == NULL) {
    (void) fprintf(stderr, "VDI_sauver_palette : mmoire insuffisante [%lu octets]\n", (unsigned long) sizeof(VDI_XPalette) );
    return FAUX;
  }

  palette->ancienne = station->ancienne;
  station->ancienne = palette;
  for (i = 0 ; i < 256 ; i++) {
    palette->couleurs[i] = station->couleurs[i];
  }

  return VRAI;
}

int VDI_rappelle_palette(Station_VDI *station)
{
  VDI_XPalette *palette;
  unsigned int i;

  if (station == NULL) return FAUX;
  if (station->ancienne == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "VDI_rappelle_palette : appel sans palette sauvegarde !\n");
#endif
    return FAUX;
  }

  palette = station->ancienne;
  for (i = 0 ; i < 256 ; i++) {
    station->couleurs[i] = palette->couleurs[i];
  }
  station->ancienne = palette->ancienne;

  free(palette);
  return VRAI;
}

