/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1998
   Cration du fichier   : 17 novembre 1998
   Dernire modification : 24 novembre 1998

   Module d'extraction des oscillations EXAFS
   Deuxime tape de l'extraction [selon la mthode utilise dans EDA]
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"

#include "extrait.h"
#include "interface.h"
#include "lissage.h"
#include "tracer.h"

#include "exa_etape2.txt"

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static int  GER_mu02(unsigned int num_objet, int x, int y);
static int  CLV_mu02(Fenetre *fenetre, unsigned int touche, int etat);
static int  MSG_mu02(Fenetre *fenetre, Message *message);
static void RDS_mu02(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut);

static int  prepare_arbre_mu0_2(unsigned int);

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */

/* ~~~~~~~~ Calculs effectifs ~~~~~~~~~~~~~~~~~~~~ */

/* -------- Calcul du polynome servant  modliser au mieux mu_0 -----------------\
 |  ne renvoie rien                                                               |
 |                                                                                |
 | brut  : le spectre brut dont on veut extraire l'EXAFS                          |
 | infos : les informations ncessaires  la modlisation de mu_0                 |
 \------------------------------------------------------------------------------- */

void MU2_calc(Graphe *brut, MU2_infos *infos)
{
  if ( infos->methode == ELIM2_RAPIDE )    /* Mthode rapide     */
  {
    MC_Polynome_SurPartieGraphe(brut, infos->degre, 0, NULL, infos->coefs);
  }
  else if ( infos->methode == ELIM2_PRECISE ) /* Mthode plus lente */
  {
    double bornes[ 2 ];
    bornes[ 0 ] = brut->x_min; bornes[ 1 ] = brut->x_max;
	
    MC_Polynome_SurPartieGraphe_Juris(brut, infos->degre, 1, bornes, infos->coefs);
  }
}

/* -------- Extraction proprement dite du spectre E.X.A.F.S. -------------------- */

void MU2_extrait(Graphe *brut, MU2_infos *infos)
{
  register double *brut_x, *brut_y, *exafs_y, *mu0_y, *mu_ref_y = NULL;
  unsigned long i;

  brut_x  = brut->x; brut_y = brut->y;
  exafs_y = infos->exafs->y; mu0_y = infos->mu_0->y;
  if (infos->mu_0_ref)
    mu_ref_y = infos->mu_0_ref->y;
	
  for (i = 0 ; i < brut->nombre_point ; i++) {
    double x, y, mu_0 = 0;
    unsigned int j;

    x = *(brut_x);
    y = infos->coefs[0];
    for (j = 1 ; j <= infos->degre ; j++) {
      y += x * infos->coefs[j];
      x *= *(brut_x);
    }
    if (infos->mu_0_ref) { /* Polynme de rfrence */
      mu_0 = *mu_ref_y;
      mu_ref_y++;
    }
    *mu0_y   = y + mu_0;
    *exafs_y = *brut_y - y;
    brut_x++; brut_y++; exafs_y++; mu0_y++;
  }

  /* Recherche des bornes */
  GRF_bornes(infos->exafs, ORDONNE, DESORDONNE); /* Le graphe est normalement tri */ 
  GRF_bornes(infos->mu_0 , ORDONNE, DESORDONNE); /* Le graphe est normalement tri */
}

/* -------- Routine relle d'extraction avec adaptation, calcul d'erreurs,... -------- */

Graphe *MU2_ExtraitEXAFS(Graphe *brut, MU2_infos *infos, boolean recalcule)
{
  char *titre;
  int retour;

  if ( (NULL == brut) || (NULL == infos) ) return NULL;

  if ( GR_NORMAL != brut->genre_donnees ) return NULL;

  if ( NULL == infos->exafs ) {
    infos->exafs = GRF_Copier(brut, NULL, FAUX);
    if ( NULL == infos->exafs )
      return NULL;
  } else {
    if ( infos->exafs->nombre_point != brut->nombre_point ) {
      if ( FAUX == GRF_AdapteTaille(infos->exafs, brut->nombre_point) )
	return NULL;
      memcpy(infos->exafs->x, brut->x, sizeof(double) * infos->exafs->nombre_point);
      infos->exafs->x_min = brut->x_min;
      infos->exafs->x_max = brut->x_max;
    }
    memcpy(infos->exafs->y, brut->y, sizeof(double) * infos->exafs->nombre_point);
    infos->exafs->y_min = brut->y_min;
    infos->exafs->y_max = brut->y_max;    

    /* On propage les erreurs : inchanges par le traitement */
    infos->exafs->erreur_x = GRF_Copier( brut->erreur_x, NULL, VRAI );
    infos->exafs->erreur_y = GRF_Copier( brut->erreur_y, NULL, VRAI );
  }

  if ( NULL == infos->mu_0 ) {
    infos->mu_0 = GRF_nouveau( NULL, brut->nombre_point, GR_MU0, GR_NORMAL, MU"0,II" );
    if ( NULL == infos->mu_0 ) {
      GRF_efface(infos->exafs, VRAI, VRAI);
      return NULL;
    }
  } else {
    if ( infos->mu_0->nombre_point != brut->nombre_point ) {
      if ( FAUX == GRF_AdapteTaille(infos->mu_0, brut->nombre_point) ) {
	GRF_efface(infos->exafs, VRAI, VRAI);
	return NULL;
      }
    }
  }
  memcpy(infos->mu_0->x, brut->x, sizeof(double) * infos->mu_0->nombre_point);
  infos->mu_0->x_min = brut->x_min;
  infos->mu_0->x_max = brut->x_max;

  /* Adaptation titres, couleurs,... */
  titre = TXT_genere_chaine("%s - "MU"0,II", brut->nom);
  GRF_titre(infos->exafs, titre);
  free( titre ); titre = NULL;
  if ( NULL != infos->mu_0_ref )
    titre = TXT_genere_chaine(MU"0,II + %s", infos->mu_0_ref->nom); /* Contiendra la somme  la fin... */
  else
    titre = TXT_genere_chaine(MU"0,II "o_CIRC"t"e_AIGU" "a_GRAVE" %s", brut->nom);
  GRF_titre(infos->mu_0, titre);
  free( titre ); titre = NULL;

  if ( brut->affichage != NULL ) {
    /* L'opration ne change ni les units en X, ni les units en Y [donc ni les lgendes].
       On garde les mmes couleurs... */
    GRF_CopierAffichage( infos->exafs, brut, GRF_CP_AFF_LEGENDE_X | GRF_CP_AFF_UNITE_X | GRF_CP_AFF_LEGENDE_Y | GRF_CP_AFF_UNITE_Y | GRF_CP_AFF_COULEUR | GRF_CP_AFF_STYLE );
  }

  /* On refait toute l'extraction, pour propager les erreurs */

  /* Polynme  ter */
  if ( VRAI == recalcule )
    MU2_calc(brut, infos);
  GRF_GenerePolynome(infos->mu_0, FAUX, infos->degre, infos->coefs, NULL);

  /* Soustraction proprement dite -- s'occupe aussi des erreurs
     NULL :  remplacer par les termes de covariance... */
  retour = GRF_SoustraireGraphe(infos->exafs, infos->mu_0, VRAI, NULL);
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans GRF_SoustraireGraphe() [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);

    GRF_efface(infos->mu_0, VRAI, VRAI);
    GRF_efface(infos->exafs, VRAI, VRAI);
    return NULL;
  }

  /* Somme des 0 pour avoir le modle total */
  GRF_AjouterGraphe(infos->mu_0, infos->mu_0_ref, VRAI, NULL);

  /* Log dans l'historique */
  HIST_Titre(1, "Deuxi"e_GRAVE"me "e_AIGU"tape de l'extraction de l'EXAFS", "MU0_II");
  HIST_Texte(1, 0, "Signal brut   : %s\n", brut->nom);
  HIST_Texte(1, 0, "EXAFS extrait : %s\n", infos->exafs->nom);
  HIST_Texte(1, 0, "Mod"e_GRAVE"le de "MU"0  : %s\n", infos->mu_0->nom);
  HIST_Texte(2, 0, "M"e_AIGU"thode : Soustraction d'un polyn"o_CIRC"me");
  HIST_Texte(3, 0, " de degr"e_AIGU" %u", (unsigned int) infos->degre);
  switch ( infos->methode ) {
  case ELIM2_RAPIDE:
    HIST_Texte(4, 0, " (moindres carr"e_AIGU"s simples)");
    break;

  case ELIM2_PRECISE:
    HIST_Texte(4, 0, " (moindres carr"e_AIGU"s pond"e_AIGU"r"e_AIGU"s par (x(i) - x(i-1)))");
    break;
  }
  HIST_Texte(2, 0, "\n"); /* Pour finir la ligne quel que soit le niveau de l'historique */
  HIST_Polynome(4, 5, 2, infos->coefs, infos->degre, 'a');

  /* Termin */
  return infos->exafs;
}

/* ~~~~~~~~ Gestion de l'interface ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Ouverture de la fentre lie  cette opration --------------\
 |   Renvoie le graphe contenant l'exafs corrig aprs seconde extraction         |
 |                                                                                |
 | brut   : le graphe qui va subir la soustraction                                |
 | mu_0   : le polynme soustrait utilis  l'tape prcdente (NULL, en crer un)|
 | chaine : VRAI si dans une procdure chane                                    |
 | mu0_ex : le 0 corrig tenant compte de cette deuxime tape                   |
 \------------------------------------------------------------------------------- */

Graphe *MU2_ouvre(Graphe *brut, Graphe *mu_0, boolean chaine, Graphe **mu0_ex)
{
  Fenetre *fenetre;
  MU2_infos *infos = NULL;
  char *titre = NULL;

  if ( brut == NULL ) return NULL; /* Vraiment rien  faire */

  if ( arbres[ELIM_MUO_DEUX].drapeaux.ouvert == VRAI ) { /* Arbre dj ouvert */
    Graphe *vieux; /* Ancien spectre travaill dans ce module */

    fenetre = ARB_QuelleFenetre( &(arbres[ELIM_MUO_DEUX]), FAUX );
    vieux = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
    infos = (MU2_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

    if ( vieux != NULL ) {
      int reponse;

      reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

      if ( ( reponse == 1 ) || ( vieux == brut ) ) { /* On active seulement */
	FEN_premierplan(fenetre); /* Fentre au premier plan */
	if ( mu0_ex != NULL ) *mu0_ex = infos->mu_0;
	return vieux; /* Rien d'autre  faire */
      } else if ( reponse == 2 ) { /* On rappelle, pour le nouveau graphe */
	/* On dlie les graphes de la fentre */
	FEN_GRF_LierGraphe( fenetre, NULL );
	vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */
	FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

	/* On efface les graphes rsultat. Comme les infos sont dlies de la fentre, pas de souci normalement ! */
	GRF_efface( infos->exafs, VRAI, VRAI );
	infos->exafs = NULL;

	GRF_efface( infos->mu_0, VRAI, VRAI );
	infos->mu_0 = NULL;
      
	FEN_premierplan( fenetre ); /* Fentre au premier plan */
      } else { /* On annule l'appel */
	return NULL;
      }
    }
  } else { /* Arbre non ouvert : on l'ouvre */
    fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, titre_fenetre, infos_fenetre,
				  ELIM_MUO_DEUX, BI_FEN_ARBRE_HAUT, prepare_arbre_mu0_2,
				  RDS_mu02, CLV_mu02,
				  ( int  (*)(Fenetre *, int, int, int, int, int) ) NULL,
				  MSG_mu02,
				  ( void (*)(Fenetre *, int, int, int, int) ) NULL,
				  ( int  (*)(Fenetre *) ) NULL);
    if (fenetre == NULL) return NULL; /* Rien  faire... */

    if ( I_XAS_CHAINE_XAS == chaine )
      FEN_dimensions(fenetre, 0, -500, -500, -500); /* Un quart d'cran, bas gauche */
    else if ( I_XAS_CHAINE_EXAFS == chaine )
      FEN_dimensions(fenetre, -500, 0, -500, -500); /* Un quart d'cran, haut droit */

    /* On ouvre la fentre */
    FEN_Ouvrir( fenetre );
  }

  if ( NULL == infos ) { /* Aucune info n'existe dj */
    infos = (MU2_infos *) malloc( (size_t) sizeof(MU2_infos) );
    if ( infos == NULL ) {
      FEN_Detruire( fenetre );
      SYS_ERREUR(MANQUE_MEMOIRE, "exafs.c", "MU2_ouvre", "infos");
      return NULL;
    }
  }

  /* On lie les complments  la fentre */
  FEN_GRF_LierGraphe( fenetre, brut );  /* Graphe trait */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos ); /* Infos d'extraction de mu_0 */

  /* On initialise pour tre tranquille si un message d'erreur survient */
  infos->exafs = NULL;
  infos->mu_0_ref = NULL;
  infos->mu_0 = NULL;
  infos->coefs[ 0 ] = 0.0; /* On met  zro les coefficients */
  infos->coefs[ 1 ] = 0.0; infos->coefs[  2 ] = 0.0; infos->coefs[ 3 ] = 0.0; infos->coefs[ 4 ] = 0.0;
  infos->coefs[ 5 ] = 0.0; infos->coefs[  6 ] = 0.0; infos->coefs[ 7 ] = 0.0; infos->coefs[ 8 ] = 0.0;
  infos->coefs[ 9 ] = 0.0; infos->coefs[ 10 ] = 0.0;
  infos->degre = OBJ_LireEntier_U( ELIM_MUO_DEUX, ELMU2_VALEUR );

  titre = TXT_genere_chaine( "%s - "MU"0,II", brut->nom );
  infos->exafs = GRF_nouveau( brut, brut->nombre_point, GR_EXAFS2, GR_NORMAL, titre );
  free( titre ); titre = NULL;
  if ( infos->exafs == NULL ) {
    /* On dlie les graphes de la fentre */
    FEN_GRF_LierGraphe( fenetre, NULL );
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );
    free( infos ); infos = NULL;
    FEN_Detruire( fenetre );
    return NULL;
  }

  if ( mu_0 != NULL ) { /* On a envoy un polynme */
    if ( mu_0->nombre_point != brut->nombre_point ) {
      EXF_ERREUR(NBRPTS_INCORRECT);
      mu_0 = NULL;
    }
  }

  infos->mu_0_ref = mu_0; /* Le graphe mu_0 de rfrence */

  titre = TXT_genere_chaine( MU"0,II pour %s", brut->nom );
  infos->mu_0 = GRF_nouveau( NULL, brut->nombre_point, GR_MU0, GR_NORMAL, titre );
  free( titre ); titre = NULL;
  if (infos->mu_0 == NULL) {
    /* On dlie les graphes de la fentre */
    FEN_GRF_LierGraphe( fenetre, NULL );
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

    /* Pas de risque de boucle en cas de fermeture/effacement, puisque l'on a dli de la fentre */
    GRF_efface( infos->exafs, VRAI, VRAI );
    infos->exafs = NULL; 

    free( infos ); infos = NULL;
    FEN_Detruire( fenetre );
    return NULL;
  }

  brut->drapeaux.protege = VRAI; /* Calcul en cours sur ce spectre -> rien de faisable */
  if ( OBJ_choisit(ELIM_MUO_DEUX, ELIM2_RAPIDE , QUESTION) == VRAI )
    infos->methode = ELIM2_RAPIDE;
  if ( OBJ_choisit(ELIM_MUO_DEUX, ELIM2_PRECISE, QUESTION) == VRAI )
    infos->methode = ELIM2_PRECISE;

  /* On prpare les spectres pour gagner du temps : ils ont les mme X */
  memcpy(infos->mu_0->x , brut->x, sizeof(double) * brut->nombre_point);
  memcpy(infos->exafs->x, brut->x, sizeof(double) * brut->nombre_point);

  infos->mu_0->x_min  = brut->x_min; infos->mu_0->x_max  = brut->x_max;
  infos->exafs->x_min = brut->x_min; infos->exafs->x_max = brut->x_max;
  
  /* On calcule la fonction  soustraire */
  infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
  infos->mu_0->drapeaux.protege  = VRAI; /* On protge le graphe de toute btise */

/*   graf_mouse(S_ABEILLE, NULL); */ /* Souris en abeille  */
  MU2_calc(brut, infos);       /* Calcul du polynme */
  MU2_extrait(brut, infos);    /* Extraction         */
/*   graf_mouse(S_FLECHE, NULL); */  /* Souris normale     */

  fenetre->drapeaux.redessin    = VRAI;
  fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_mu02 */

  if ( mu0_ex != NULL ) *mu0_ex = infos->mu_0;
  return infos->exafs;
}

/* -------- Gestion du clavier
   r = mthode "rapide", l = mthode "lente", espace = quitter, M = choisir 0,II
   +,- = choix du degr du polynme
 */

static int CLV_mu02(Fenetre *fenetre, unsigned int touche, int etat)
{
  if ( NULL == fenetre ) return BI_EVENEMENT_ERREUR;
  if ( etat & K_CTRL ) return BI_EVENEMENT_NON_TRAITE; /* Pas de raccourci avec control ! */

  switch ( touche)
  {
  case '-':
#ifdef XK_KP_Subtract
  case XK_KP_Subtract:
#endif
    return ARB_SimulerClic( ELIM_MUO_DEUX, ELMU2_MOINS, 1 );

  case '+':
#ifdef XK_KP_Add
  case XK_KP_Add:
#endif
    return ARB_SimulerClic( ELIM_MUO_DEUX, ELMU2_PLUS, 1 );

  case 'r':
    return ARB_SimulerClic( ELIM_MUO_DEUX, ELIM2_RAPIDE, 1);

  case 'l':
    return ARB_SimulerClic( ELIM_MUO_DEUX, ELIM2_PRECISE, 1 );

  case 'M':
    return ARB_SimulerClic( ELIM_MUO_DEUX, ELMU2_CHOIX, 1);
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* ------------- Redessin de la fentre d'limination de mu_0, deuxime tape -------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |
 | fenetre    : structure associe  la fentre considre
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \---------------------------------------------------------------------------------------- */

static void RDS_mu02(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  Graphe *graphe;
  InfosRegion2D region;
  MU2_infos *infos;
  int retour;

  /* Prparation du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  graphe = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (MU2_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  if ( NULL == graphe ) {
    TRC_Message( fenetre, txt_brut_nul );
    return;
  }
  if ( ( graphe->x_max == graphe->x_min ) || ( graphe->y_max == graphe->y_min ) ) {
    TRC_Message( fenetre, txt_brut_nul );
    return;
  }

  /* On prpare la rgion 2D : aide pour les dessins aprs en fait */
  retour = TRC_Z2D_Initialiser( &region, fenetre, graphe );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en initialisant la rgion 2D [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* On trace le graphe dans la rgion 2D et on rcupre les chelles au passage */
  TRC_GrapheDansRegion2D( fenetre, &region, 0, graphe, NOIR, BI_CONTINU, BI_MARQUE_AUCUNE, 0, NULL, NULL );

  /* On trace la courbe  soustraire - en bleu */
  TRC_Polynome( fenetre, &region, BLEU, BI_CONTINU, 0, infos->degre, infos->coefs, graphe->nombre_point, graphe->x );

  /* On trace le graphe EXAFS extrait - en fin, car chelle diffrente */
  if ( infos->exafs->y_max != infos->exafs->y_min ) {
    int y_axe;
    double echelle_Y;

    /* On adapte la rgion 2D */
    retour = TRC_Z2D_Initialiser( &region, NULL, infos->exafs );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Erreur en initialisant la rgion 2D [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }

    /* On trace le signal extrait dans la rgion 2D et on rcupre les chelles au passage */
    TRC_GrapheDansRegion2D( fenetre, &region, 0, infos->exafs, GRIS_FONCE, BI_CONTINU, BI_MARQUE_AUCUNE, 0, NULL, &echelle_Y );

    /* On trace l'axe des X associ, en violet */
    y_axe = TRC_ConvertitY_PvE( 0.0, infos->exafs->y_min, fenetre->w_haut, echelle_Y );
    TRC_Couleur( VIOLET );
    TRC_LigneH( fenetre, 0, y_axe, fenetre->w_larg );
  } else {
    TRC_Message( fenetre, txt_exafs_nul );
  }
}

/* -------------- Messages reus par la fentre d'limination de mu_0, 2e tape ---------- */

static int MSG_mu02(Fenetre *fenetre, Message *message)
{
  int retour;
  Graphe *brut, *exafs, *mu_0, *mu_ref;
  MU2_infos *infos;
  
  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos = (MU2_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  if ( message->type == MSG_GRAPHE_MODIFIE ) { /* Le brut a t modifi, par consquent, le 0 aussi
						  On ne reoit de message que pour le brut */
    Graphe *nv_brut = (Graphe *) (message->divers.pointeur);
    Graphe *extrait = infos->exafs;

    if ( NULL == nv_brut ) {
      /* On demande en fait de fermer la fentre, car il n'y a plus de brut... */

      message->type = WM_CLOSED;
      message->fenetre.haut = -1; /* On force la fermeture de cette fentre */
      return MSG_mu02( fenetre, message );
    }

    if ( nv_brut->type == GR_MU0 ) { /* On change de 0 - devrait jamais arriver ! En effet,
					dans l'tape d'avant, on travaille  coup de realloc sur (x, y)
					donc sans toucher au graphe directement

					Si a arrive, normalement le brut a dj t chang ->
					c'est juste pour mettre  jour la structure d'infos */
#ifdef _DEVELOPPE_
      AVERTISSEMENT( "0 de rfrence modifi [%s], comportement anormal !", nv_brut->nom);
#endif
      if ( nv_brut->nombre_point != brut->nombre_point ) {
	(void) fprintf(stderr, "%s,%s l.%u : 0 modifi, en dsaccord avec l'exafs extrait !\n\n",
		       __FILE__, __FUNCTION__, __LINE__);
	fprintf(stderr, "           comportement ultrieur du programme alatoire...\n");
	FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
      }
      infos->mu_0_ref = nv_brut;
      return BI_EVENEMENT_TRAITE;
    }

    if ( message->divers.drapeau == VRAI ) { /* Pas autant de point -> on joue sur realloc */
      if ( ( GRF_AdapteTaille( extrait, nv_brut->nombre_point ) == FAUX ) ||
	   ( GRF_AdapteTaille( infos->mu_0, nv_brut->nombre_point ) == FAUX ) ) {
	message->type = WM_CLOSED;
	message->fenetre.haut = -1; /* On force la fermeture de cette fentre */
	return MSG_mu02( fenetre, message );
      }
    }

    /* Note : les X peuvent avoir chang, si l'on a jou sur E0
       Or, la suite du calcul ne modifie _pas_ les X -> le faire ici */
    memcpy(extrait->x, nv_brut->x, sizeof(double) * nv_brut->nombre_point);
    memcpy(infos->mu_0->x, nv_brut->x, sizeof(double) * nv_brut->nombre_point);

    /* Ici, on a fait ce qu'il fallait pour s'adapter -> on refait les calculs */
    brut = nv_brut;
    FEN_GRF_LierGraphe( fenetre, nv_brut );
/*     graf_mouse(S_ABEILLE, NULL); */
    MU2_calc   (brut, infos); /* On recalcule  */
    MU2_extrait(brut, infos); /* Et on extrait */
    GRF_modifier(infos->exafs, infos->exafs, message->divers.drapeau);
/*     graf_mouse(S_FLECHE, NULL); */
    FEN_redessin(fenetre, BI_RDS_UTIL); /* Puisqu'il y a eu modif */

    return BI_EVENEMENT_TRAITE;
  }

  if ( message->type == WM_CLOSED ) { /* Message de fermeture */
    if ( ( brut == NULL ) || ( message->fenetre.haut == -1 ) ) { /* Fermeture force */
      retour = 2;
    } else {
      retour = EXF_ERREUR(FIN_MODULE);
      if ( retour == 3 ) { /* On annule la fermeture */
	return BI_EVENEMENT_TRAITE;
      }
    }

    exafs  = infos->exafs;
    mu_0   = infos->mu_0;
    mu_ref = infos->mu_0_ref;

    /* On dlie les informations de la fentre */
    FEN_GRF_LierGraphe( fenetre, NULL );
    FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );

    /* De toute faon, on ferme */
    FEN_Detruire( fenetre );

    if ( mu_ref != NULL ) /* Il y a un graphe mu_0 de rfrence */
      mu_ref->drapeaux.protege = FAUX;
    if (brut != NULL) /* NULL si fermeture force pour cause d'effacement */
      brut->drapeaux.protege   = FAUX;
    
    if ( 2 == retour ) { /* On ferme sans soustraire */
      GRF_efface( exafs, VRAI, VRAI );
      GRF_efface( mu_0 , VRAI, VRAI );
      free( infos ); infos = NULL;
      return BI_EVENEMENT_FEN_FERMEE;
    }

    /* On veut extraire le spectre */

    /* On installe les valeurs dans l'arbre de paramtrage de l'exafs */
    OBJ_MettreEntier_U( EXAFS_GENERAL, EXF_DEGRE_MU0_II, infos->degre );
    OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, (infos->methode == ELIM2_RAPIDE) );
    OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_LENT,  (infos->methode == ELIM2_PRECISE) );

    exafs = MU2_ExtraitEXAFS( brut, infos, FAUX );
    free( infos ); infos = NULL;

    exafs->drapeaux.protege  = FAUX;
    mu_0->drapeaux.protege   = FAUX;
    exafs->drapeaux.sauve = FAUX;   /* Graphe non sauv */
    mu_0->drapeaux.sauve  = FAUX;   /* Graphe non sauv */
    GRF_aller( exafs ); /* On va  ce graphe */

    return BI_EVENEMENT_FEN_FERMEE;
  }

  AVERTISSEMENT( "Message non trait [%d]", message->type );
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion de l'arbre dans la fentre, extraction 2e tape -------- */

static int GER_mu02(unsigned int num_objet, int x, int y)
{
  MU2_infos *infos;
  Graphe *brut;
  Fenetre *fenetre;

  /* On rcupre les informations */
  fenetre = ARB_QuelleFenetre(arbres + ELIM_MUO_DEUX, FAUX); /* Fentre actuelle */
  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos = (MU2_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  /* On rcupre la fentre utilisateur,  redessiner ! */
  fenetre = FEN_FenetreMere( fenetre );	/* On prend la racine ! */
  if ( fenetre != NULL ) {
    if ( NULL == fenetre->ss_fenetres ) {
      AVERTISSEMENT( "Pas de partie utilisateur, fentre pour l'arbre %u [EXAFS tape 2]", ELIM_MUO_DEUX );
      return BI_EVENEMENT_ERREUR;
    }
    /* On rcupre donc la fentre utilisateur */
    fenetre = fenetre->ss_fenetres[ 1 ];
  }

  switch (num_objet)
  {
  case ELIM2_RAPIDE: /* Mthode utilise pour approcher l'absorption atomique */
  case ELIM2_PRECISE:
    if ( num_objet != (unsigned int) infos->methode ) { /* On change de mthode */
      OBJ_choisit(ELIM_MUO_DEUX, num_objet, FORCE_OUI);
      infos->methode = num_objet;
      MU2_calc   (brut, infos); /* On recalcule  */
      MU2_extrait(brut, infos); /* Et on extrait */
      GRF_modifier(infos->exafs, infos->exafs, FAUX); /* Nbr de points n'a pas chang */
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case ELMU2_MOINS:
  case ELMU2_FOND:
  case ELMU2_VALEUR:
  case ELMU2_PLUS:
    infos->degre = gere_ascenseur(ELIM_MUO_DEUX, ELMU2_MOINS, ELMU2_FOND, ELMU2_VALEUR, ELMU2_PLUS, ASC_HORIZONTAL,
				  1, 9, 1, infos->degre, num_objet, x, y);
    MU2_calc   (brut, infos); /* On recalcule    */
    MU2_extrait(brut, infos); /* Et on soustrait */
    GRF_modifier(infos->exafs, infos->exafs, FAUX); /* Nombre de points n'a pas chang */
    FEN_redessin(fenetre, BI_RDS_UTIL);
    return BI_EVENEMENT_TRAITE;

  case ELMU2_CHOIX: /* Choix du 0 de rfrence */
    {
      Graphe *nv_mu0 = GRF_choisir(GR_MU0, VRAI, VRAI, NULL);
      if (nv_mu0 == NULL) return VRAI;
			
      if (nv_mu0->nombre_point != brut->nombre_point) {
	EXF_ERREUR(NBRPTS_INCORRECT);
	return BI_EVENEMENT_TRAITE;
      }
      if (nv_mu0->drapeaux.protege == VRAI) {
	EXF_ERREUR(GRF_PROTEGE);
	return BI_EVENEMENT_TRAITE;
      }

      infos->mu_0_ref->drapeaux.protege = FAUX;
      infos->mu_0_ref = nv_mu0;
      nv_mu0->drapeaux.protege = VRAI;
    }
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre (y compris traductions) -------- */

static int prepare_arbre_mu0_2(unsigned int numero_arbre __attribute__((__unused__)))
{
  if ( FAUX == arbres[ ELIM_MUO_DEUX ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( ELIM_MUO_DEUX, "menu_exafs.html#etape_2", GER_mu02, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  }

  OBJ_agrandit(ELIM_MUO_DEUX, ROOT, 0, +1);

  ARB_EnjoliverAscenseur( ELIM_MUO_DEUX, ELMU2_MOINS, ELMU2_FOND, ELMU2_VALEUR, ELMU2_PLUS, ASC_HORIZONTAL, 0);
  OBJ_deplace (ELIM_MUO_DEUX, ELIM2_PRECISE, +1, 0, DEPLACE_PIXELS);

#ifdef _ENGLISH_ /* Traduction en anglais */
  met_chaine(ELIM_MUO_DEUX, EM2_TITRE_DEGRE, 0, "Degree of the polynom:", TXT_REMPLACE);
  met_chaine(ELIM_MUO_DEUX, ELIM2_RAPIDE , 0, "Fast", TXT_REMPLACE);
  met_chaine(ELIM_MUO_DEUX, ELIM2_PRECISE, 0, "Precise", TXT_REMPLACE);
  met_chaine(ELIM_MUO_DEUX, ELMU2_CHOIX, 0, "Choose "MU"0,I", TXT_REMPLACE);
#endif

  arbres[ELIM_MUO_DEUX].drapeaux.prepare = VRAI;

  return 0;
}
