/* Traitement de l'E.X.A.F.S. sur Atari
   Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   (c) 1996 - 1999, Emmanuel CURIS
   Cration : 27 mai 1998

   Fichiers multilments : somme de tous les lments
   Fichiers multispectres : somme de tous les spectres _par lment_,
                            le traitement par lment est fait _avant_

   faire : - interrompre proprement si I0 = 0 et utilisateur demande de sauter
            - cas de la somme de spectres de multi-lments si : > somme / moyenne des lments
                                                                 > sparation des lments
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"

#include "import_xas.h"

#include "ext_sommer.txt"

/* ~~~~~~~~ Ralisation pratique ~~~~~~~~ */

int EXTRAIT_AnalyseLigneSommer(unsigned int nbr_valeurs, double *valeurs, Intervalle_I *filtre, unsigned long nbr_filtres,
			       int *erreurs, int methode, double i0, unsigned long point,
			       Graphe **spectres, Graphe **i1, const char *nom_fichier, Infos_Extraction *infos)
{
  unsigned int j, nbr_detecteurs =0;
  double somme_i1 = 0.0, somme_mu = 0.0;
  const BI_Boite_Alerte *message;

  if ( NULL == filtre ) return 0; /* Ne nous intresse pas...*/
  if ( NULL == spectres[ methode ] ) {
    (void) fprintf(stderr, "EXTRAIT_AnalyseLigneSommer : mthode retenue, mais pas de graphe !\n");
    return 0; /* Pareil... */
  }

  /* Prparation dpendant de la mthode */
  switch ( methode )
  {
  case DETECT_TRANSMISSION:
    message = &ERR_iTransmissionAbsurde;
    break;

  case DETECT_FLUORESCENCE:
    message = &ERR_iFluorescenceAbsurde;
    break;

  case DETECT_ELECTRONS:
    message = &ERR_iDetectionAbsurde;
    break;

  default:
    (void) fprintf(stderr, "EXTRAIT_AnalyseLigneSommer : Mthode inconnue [%d]\n", methode );
    message = NULL;
    break;
  }

  for ( j = 0 ; j < nbr_valeurs ; j++ ) {
    if ( VRAI == TXT_est_dans_intervalle( j + 1, filtre, nbr_filtres ) ) {
      double val_i1, mu = 0.0;

      nbr_detecteurs++;	
      val_i1 = valeurs[ j ];
      /* Si on est en fluorescence : correction de temps mort, si demande */
      if ( ( DETECT_FLUORESCENCE == methode ) && ( infos->post_traitement & DETECT_POST_CORRECT ) ) {
	double tps_mort = 1.0, tps = infos->temps_mort.temps_mesure;

	if ( ( infos->temps_mort.col_tpsmort > 0 ) && ( infos->temps_mort.col_tpsmort <= nbr_valeurs ) )
	  tps_mort = valeurs[ infos->temps_mort.col_tpsmort - 1 ];
	if ( ( infos->temps_mort.col_temps > 0 ) && ( infos->temps_mort.col_temps <= nbr_valeurs ) )
	  tps = valeurs[ infos->temps_mort.col_temps - 1 ];

	val_i1 = EXTRAIT_CorrigeTempsMort( val_i1, infos->temps_mort.constante, tps, tps_mort );
      }

      /* Si on veut garder le signal, contrle sur I1 */
      if ( spectres != NULL ) {
	if ( i0 <= 0.0 ) {
	  mu = 0.0; /* Signal arbitrairement nul si pas de I0 ! */
	} else if ( ( val_i1 < 0.0 ) || /* Ce qui suit : hack sauvage pour la transmission, sale mais... */
		    ( ( DETECT_TRANSMISSION == methode ) && ( val_i1 == 0.0 ) ) ) {
	  if ( ( NULL == erreurs) || ( FAUX == erreurs[ j ] ) ) {
	    int reponse;
	      
	    reponse = BI_ERR_alerte( NULL, message, VRAI, j + 1, nom_fichier );
	    if ( ANNULER_CHARGEMENT == reponse ) return ANNULER_CHARGEMENT;
	    if ( ARRETER_CHARGEMENT == reponse ) return ARRETER_CHARGEMENT;
	  }
	  erreurs[ j ] = VRAI;
	} else {
	  switch ( methode )
	  {
	  case DETECT_TRANSMISSION:
	    mu = log( i0 / val_i1 );
	    break;

	  case DETECT_FLUORESCENCE:
	  case DETECT_ELECTRONS:
	    mu = val_i1 / i0;
	    break;
	  }
	}
	/* Somme des mu */
	somme_mu += mu;
      }

      /* Somme des intensits */
      somme_i1 += val_i1;
    }
  }

  /* Calcul des valeurs moyennes */
  if ( 0 == nbr_detecteurs ) {
    (void) fprintf(stderr, "EXTRAIT_AnalyseLigneSommer : Aucun dtecteur conserv !\n");
    return -1;
  }
  somme_i1 /= (double) nbr_detecteurs;
  somme_mu /= (double) nbr_detecteurs;

  if ( i1 != NULL ) /* On garde le I moyen */
    i1[ methode ]->y[ point ] = somme_i1;
  if ( spectres != NULL ) {
    spectres[ methode ]->y[ point ] = somme_mu; /* Il faudrait tenir compte du choix f(moyenne) ou moyenne(f) */
  }

  OPTION_A_FAIRE( "Estimer les incertitudes aprs moyenne" );

  /* Tout s'est bien pass */
  return 0;
}
