/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1999, Emmanuel CURIS
   Ce fichier : cration le 12 janvier 1999

   Cration d'un modle thorique pour FEFF

   Gestion des couches de rflexion simple (commande SS)
*/

#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"
#include "exafs.h"

#include "feff.h"

#include "feff_couches.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* -------- Ralisation pratique ----- */

/* ---------------------- Cration d'une nouvelle couche FEFF ---------------------------\
 |   Crer une couche de rflexion simple (SS) FEFF                                      |
 |   Renvoi l'adresse de cette couche - NULL si erreur                                   |
 |                                                                                       |
 | numero         : le numro du fichier FEFFnnnn.DAT associ  cette couche             |
 | potentiel      : le potentiel des atomes de la couche                                 |
 | degenerescence : le nombre d'atomes dans cette couche (= dgnrescence du chemin)    |
 | distance       : distance de cette couche  l'atome central (en )                    |
 \-------------------------------------------------------------------------------------- */

FEFF_SS *FEFF_AjouterCouche(Modele_FEFF *modele, unsigned int numero, int potentiel, int degenerescence, double distance)
{
  FEFF_SS *couche;

  if ( NULL == modele ) return NULL;

  if (0 == numero) {  /* Pas de sens */
    (void) BI_ERR_alerte(NULL, &ERR_IndexIncorrect , VRAI);
    return NULL;
  }

  if ( (potentiel < 1) || (potentiel > 7) ) {
    (void) BI_ERR_alerte(NULL, &ERR_PotentielIncorrect , VRAI, potentiel);
    return NULL;
  }

  if (degenerescence < 1) { /* Impossible aussi */
    (void) BI_ERR_alerte(NULL, &ERR_NIncorrect , VRAI);
    return NULL;
  }

  if (distance <= 0.1) { /* Pas trs raliste non plus */
    (void) BI_ERR_alerte(NULL, &ERR_DistanceIncorrecte , VRAI, distance);
    return NULL;
  }

  if ( modele->parametres.prem_ss != NULL ) { /* Il y a dj des couches */
    for (couche = modele->parametres.prem_ss ; couche != NULL ; couche = couche->suivante) {
      if ( numero == couche->index ) {
	(void) BI_ERR_alerte(NULL, &ERR_CoucheExiste, VRAI, numero);
	return NULL;
      }
    }
  }
  
  couche = (FEFF_SS *) malloc( sizeof(FEFF_SS) );
  if (couche == NULL) {
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "f_struct.c", "SS_creer", "couche");
    return NULL;
  }

  /* On initialise la couche */
  couche->index = numero;
  couche->potentiel = potentiel;
  couche->degenerescence = degenerescence;
  couche->distance = (float) distance;

  couche->suivante = modele->parametres.prem_ss; /* On l'insre en dbut de liste */
  modele->parametres.prem_ss = couche;
  modele->parametres.nbr_ss++; /* Une couche de plus */

  return couche;
}

/* -------- Effacer une couche de rflexion simple -------- */

boolean FEFF_EffacerCouche(Modele_FEFF *modele, FEFF_SS *couche, boolean forcer)
{
  FEFF_SS *avant;

  if ( (NULL == modele) || (NULL == couche) ) return FAUX;

  if ( FAUX == forcer ) {
    int reponse;

    reponse = BI_ERR_alerte(NULL, &Q_EffacerCouche, VRAI,
			    couche->index, couche->degenerescence, couche->potentiel, couche->distance);
    if (2 == reponse) /* On annule l'effacement */
      return FAUX;
  }

  avant = modele->parametres.prem_ss;
  if ( couche == avant ) { /* On efface la premire couche */
    modele->parametres.prem_ss = avant->suivante;
  } else {
    while ( avant->suivante != couche ) {
      avant = avant->suivante;
      if ( NULL == avant ) {
	(void) fprintf(stderr, "FEFF_EffacerCouche : la couche n'est pas dans la liste du modle indiqu !\n");
	return FAUX;
      }
    }

    avant->suivante = couche->suivante; /* On reforme la chane */
  }

  modele->parametres.nbr_ss--; /* Une couche de moins */
  free( couche );

  return VRAI;
}
