/* LASE : Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 1996 - 1998
   Cration du fichier : 6 juillet 1999

   Filtration du signal EXAFS

   Reproduction d'une filtration par TF/TFI avec les paramtres dj connus
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "filtrage.h"

#include "extrait.h" /* Pour EXF_chparams */
#include "operations.h" /* Pour tronquer un graphe */

#include "repro_tf.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_parametres = NULL;

/* Utilise aussi dans Fourier.c et quivalents */
struct infos_reproduit_filtre infos_reproduire = { NULL, NULL, FEN_BESSEL, FEN_HANNING, FAUX, FAUX };

#define LGMAX_TXT_FENETRE   36
#define LGMAX_TXT_AMPLITUDE 36
#define LGMAX_TXT_PHASE     36

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

int GER_reproduit_filtre(unsigned int objet, int x, int y); /* Utilise dans Filtrage/Fourier.c */
static void prepare_arbre(void);

static int TF_svparams(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

void TF_reproduit(Graphe *spectre)
{
  boolean etat, amp_existe, phase_existe;
  int retour;
  extern const char *txt_aucun_choisi, *txt_aucun_graphe; /* dans variables.c */

  if (spectre == NULL) return;

  if ( arbres[TF_GENERAL].drapeaux.prepare == FAUX )
    prepare_arbre();

  etat = (spectre->type == GR_TF);
  OBJ_active(TF_GENERAL, RF_FAIRE_TF, !etat);
  OBJ_choisit(TF_GENERAL, RF_FAIRE_TF, !etat);

  amp_existe   = GRF_type_existe(GR_EXAFSAMP, FAUX, NULL);
  phase_existe = GRF_type_existe(GR_EXAFSPHA, FAUX, NULL);

  OBJ_active(TF_GENERAL, RF_CORR_AMP, amp_existe);
  OBJ_active(TF_GENERAL, RF_CORR_PHASE, phase_existe);
  if (amp_existe)
    met_chaine(TF_GENERAL, RF_CORR_AMP, LGMAX_TXT_AMPLITUDE, txt_aucun_choisi, TXT_COPIER);
  else
    met_chaine(TF_GENERAL, RF_CORR_AMP, LGMAX_TXT_AMPLITUDE, txt_aucun_graphe, TXT_COPIER);
  if (phase_existe)
    met_chaine(TF_GENERAL, RF_CORR_PHASE, LGMAX_TXT_PHASE, txt_aucun_choisi, TXT_COPIER);
  else
    met_chaine(TF_GENERAL, RF_CORR_PHASE, LGMAX_TXT_PHASE, txt_aucun_graphe, TXT_COPIER);

  /*  refaire car a pu tre modifi depuis l'appel prcdent, si on a fait une TF "classique" p. ex. */
  met_chaine(TF_GENERAL, RF_FEN_TF, LGMAX_TXT_FENETRE,
	     Types_fenetres[infos_reproduire.type_fen_tf], TXT_COPIER);
  etat = Autorise_Symetrique( infos_reproduire.type_fen_tf );
  OBJ_active(TF_GENERAL, RF_TF_SYMETRIQUE, etat);
  OBJ_active(TF_GENERAL, RF_VAL_K1, etat);
  OBJ_active(TF_GENERAL, RF_VAL_K2, etat && (infos_reproduire.tf_symetrique == FAUX));
  OBJ_active(TF_GENERAL, TFG_TF_PARAM, Avec_Parametre( infos_reproduire.type_fen_tf ));
  if (etat)
    OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, infos_reproduire.tf_symetrique);
  else
    OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, FORCE_NON);

  /* Idem pour la TFI */
  met_chaine(TF_GENERAL, RF_TFI_FENETRE, LGMAX_TXT_FENETRE,
	     Types_fenetres[infos_reproduire.type_fen_tfi], TXT_COPIER);
  etat = Autorise_Symetrique( infos_reproduire.type_fen_tfi );
  OBJ_active(TF_GENERAL, RF_TFI_SYMETRIK, etat);
  OBJ_active(TF_GENERAL, RF_VAL_R1, etat);
  OBJ_active(TF_GENERAL, RF_VAL_R2, etat && (infos_reproduire.tfi_symetrique == FAUX));
  OBJ_active(TF_GENERAL, TFG_TFI_PARAM, Avec_Parametre( infos_reproduire.type_fen_tfi ));
  if (etat)
    OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, infos_reproduire.tfi_symetrique);
  else
    OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, FORCE_NON);

  do {
    retour = ARB_Bloquant( TF_GENERAL, titre_arbre_reproduire, principale, 0 );
    if (retour == RF_ANNULER)
      return;
  } while (retour != RF_CALC);

  if ( OBJ_choisit(TF_GENERAL, RF_FAIRE_TF, QUESTION) == VRAI ) { /* On fait la TF */
    TF_infos infos;
    int methode, retour;
    double r_min, r_max, pas;
    Graphe *fourier = NULL;

    /* Les spectres traits */
    infos.apercu  = NULL;
    infos.fenetre = NULL;
    infos.graphe = spectre;

    /* La fonction d'apodisation */
    OBJ_LireIntervalle( TF_GENERAL, RF_VAL_KMIN, RF_VAL_KMAX, 0, 0, &(infos.apodisation.x_min), &(infos.apodisation.x_max), NULL, NULL );
    infos.apodisation.parametre = OBJ_LireNombre( TF_GENERAL, TFG_TF_PARAM, NULL );
    infos.apodisation.type = infos_reproduire.type_fen_tf;
    if ( Autorise_Symetrique( infos.apodisation.type ) ) {
      OBJ_LireIntervalle( TF_GENERAL, RF_VAL_K1, RF_VAL_K2, 0, 0, &(infos.apodisation.x_1), &(infos.apodisation.x_2), NULL, NULL );
      infos.apodisation.symetrique = OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, QUESTION);
    } else {
      infos.apodisation.x_1 = infos.apodisation.x_min;
      infos.apodisation.x_2 = infos.apodisation.x_max;
      infos.apodisation.symetrique = FAUX;
    }

    /* Corrections amplitude / phase ? */
    infos.amplitude = infos_reproduire.corr_amplitude;
    infos.phase = infos_reproduire.corr_phase;

    retour = OBJ_LireIntervalle( TF_GENERAL, RF_TF_RMIN, RF_TF_RMAX, TF_RF_PAS, 0, &r_min, &r_max, &pas, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Valeurs de l'intervalle de calcul absurdes (TF)\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }

    /* On fait le calcul */
    if ( OBJ_choisit(TF_GENERAL, RF_TF_RAPIDE, QUESTION) == VRAI ) /* Calcul rapide */
      methode = TF_CALC_RAPIDE;
    else
      methode = TF_CALC_NORMAL;

    fourier = TF_CalculDirect(&infos, r_min, r_max, pas, methode,
			      OBJ_choisit( TF_GENERAL, TFG_TF_ERR, QUESTION ), OBJ_choisit( TF_GENERAL, TFG_TF_COVAR, QUESTION ));
    if (fourier == NULL) { /* Erreur survenue  l'init. */
      return;
    }
    spectre = fourier; /* Sert de point de dpart  l'tape suivante, si ncessaire */
  }

  /* ~-~-~-~-~-~-~-~- Calcul de la TF inverse  partir des paramtres indiqus ~-~-~-~-~-~-~-~- */

  if ( OBJ_choisit(TF_GENERAL, RF_FAIRE_TFI, QUESTION) == VRAI ) { /* On fait la TFI */
    TFI_infos infos;
    double k_min, k_max, pas;
    int methode;
    Graphe *fourier = NULL, *exafs = NULL;
    char *titre;

    /* On prpare les informations ncessaires */
    infos.fenetre = NULL;
    infos.graphe = spectre;

    OBJ_LireIntervalle( TF_GENERAL, RF_VAL_RMIN, RF_VAL_RMAX, 0, 0, &(infos.filtre.x_min), &(infos.filtre.x_max), NULL, NULL );
    OBJ_LireIntervalle( TF_GENERAL, RF_VAL_R1, RF_VAL_R2, 0, 0, &(infos.filtre.x_1), &(infos.filtre.x_2), NULL, NULL );
    infos.filtre.symetrique = OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, QUESTION);
    infos.filtre.parametre = OBJ_LireNombre( TF_GENERAL, TFG_TFI_PARAM, NULL );
    infos.filtre.type = infos_reproduire.type_fen_tfi;

    retour = OBJ_LireIntervalle( TF_GENERAL, TFG_TFI_KMIN, TFG_TFI_KMAX, TFG_TFI_PAS, 0, &k_min, &k_max, &pas, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : Valeurs de l'intervalle de calcul absurdes (TFI)\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }

    if ( OBJ_choisit(TF_GENERAL, RF_TFI_RAPIDE, QUESTION) == VRAI ) {
      methode = TF_CALC_RAPIDE;
    } else {
      methode = TF_CALC_NORMAL;
    }

    fourier = TFI_CalculInverse(&infos, k_min, k_max, pas, methode,
				OBJ_choisit(TF_GENERAL, TFG_TFI_ERREUR, QUESTION), OBJ_choisit(TF_GENERAL, TFG_TFI_COVAR, QUESTION) );
    if ( NULL == fourier ) {
      return;
    }

    /* On rcupre le spectre EXAFS proprement dit == la partie relle du spectre transform... */
    exafs = CMP_reelle( fourier );
    if ( exafs != NULL ) {
      titre = TXT_genere_chaine(titre_grf_exafsfiltre, spectre->nom);
      GRF_titre(exafs, titre);
      free(titre);
    }

    /* On peut supposer que l'on a fini ici... */
    fourier->drapeaux.protege = FAUX; /* On a fini de calculer sur ces graphes */
    spectre->drapeaux.protege  = FAUX;
    GRF_aller(fourier); /* On met ce graphe en graphe courant */
  }
}

/* ~~~~~~~~ Sauvegarde des infos pour la TF ~~~~~~~~ */

/* -------- Sauvegarde -------- (chargement avec EXF_chparams) */

static int TF_svparams(void)
{
  FILE *fichier;
  char *nom_fichier;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    0, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) titre_sv_selecteur;
  infos_selecteur.chemin = chemin_parametres;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, ARB_QuelleFenetre( arbres + TF_GENERAL, FAUX ) );
  /* On mmorise les chemins */
  chemin_parametres = infos_selecteur.chemin;

  if (nom_fichier == NULL) return FAUX;

  fichier = fopen(nom_fichier, "w");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, NULL, NULL);
    free(nom_fichier);
    return FAUX;
  }

  if (fprintf(fichier, "#LASE-EXT ("COPYRIGHT" 1999-2002 E. Curis)\n") <= 0) { /* En-tte pour reconnatre */
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, fichier, NULL);
    (void) fclose(fichier);
    free(nom_fichier);
    return FAUX;
  }

  fprintf(fichier, "#Param"e_GRAVE"tres pour le calcul direct\n");
  fprintf(fichier, "-TFD-\n");
  fprintf(fichier, "kmin : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_KMIN, NULL ));
  fprintf(fichier, "kmax : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_KMAX, NULL ));
  fprintf(fichier, "apod : %d (%s)\n", infos_reproduire.type_fen_tf,
	  Types_fenetres[infos_reproduire.type_fen_tf]);
  if ( Autorise_Symetrique( infos_reproduire.type_fen_tf ) ) { /* Si non, inutile de sauver */
    fprintf(fichier, "k1   : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_K1, NULL ));
    if ( OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, QUESTION) == FAUX )
      fprintf(fichier, "k2   : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_K2, NULL ));
    /* Si symtrique, inutile de sauver */
  }
  if ( Avec_Parametre( infos_reproduire.type_fen_tf ) ) { /* Si non, inutile de sauver */
    fprintf(fichier, "p    : %.20g\n", OBJ_LireNombre(TF_GENERAL, TFG_TF_PARAM, NULL ));
  }
  fprintf(fichier, "Rmin : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_TF_RMIN, NULL ));
  fprintf(fichier, "Rmax : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_TF_RMAX, NULL ));
  fprintf(fichier, "pas  : %.20g\n", OBJ_LireNombre(TF_GENERAL, TF_RF_PAS, NULL ));
  fprintf(fichier, "Met. : ");
  if ( OBJ_choisit(TF_GENERAL, RF_TF_RAPIDE, QUESTION) == VRAI )
    fprintf(fichier, "rapide\n");
  else
    fprintf(fichier, "normale\n");

  fprintf(fichier, "#Param"e_GRAVE"tres pour le calcul inverse\n");
  fprintf(fichier, "-TFI-\n");
  fprintf(fichier, "Rmin : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_RMIN, NULL ));
  fprintf(fichier, "Rmax : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_RMAX, NULL ));
  fprintf(fichier, "filt : %d (%s)\n", infos_reproduire.type_fen_tfi,
	  Types_fenetres[infos_reproduire.type_fen_tfi]);
  if ( Autorise_Symetrique( infos_reproduire.type_fen_tfi ) ) { /* Si non, inutile de sauver */
    fprintf(fichier, "R1   : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_R1, NULL ));
    if ( OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, QUESTION) == FAUX )
      fprintf(fichier, "R2   : %.20g\n", OBJ_LireNombre(TF_GENERAL, RF_VAL_R2, NULL ));
    /* Si symtrique, inutile de sauver */
  }
  fprintf(fichier, "kmin : %.20g\n", OBJ_LireNombre(TF_GENERAL, TFG_TFI_KMIN, NULL ));
  fprintf(fichier, "kmax : %.20g\n", OBJ_LireNombre(TF_GENERAL, TFG_TFI_KMAX, NULL ));
  fprintf(fichier, "pas  : %.20g\n", OBJ_LireNombre(TF_GENERAL, TFG_TFI_PAS, NULL ));
  fprintf(fichier, "Met. : ");
  if ( OBJ_choisit(TF_GENERAL, RF_TFI_RAPIDE, QUESTION) == VRAI )
    fprintf(fichier, "rapide\n");
  else
    fprintf(fichier, "normale\n");

  (void) fclose(fichier);
  free(nom_fichier);
  return VRAI; /* Tout va bien... */
}

/* ~~~~~~~~ Gestion de l'interface ~~~~~~~~ */

int GER_reproduit_filtre(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  boolean etat;
  int nv_fen;

  switch (objet)
  {
  case RF_FAIRE_TF:
    etat = OBJ_choisit(TF_GENERAL, RF_FAIRE_TF, ECHANGE);
    return VRAI;

  case RF_FEN_TF:
    nv_fen = SS_menu_texte(Types_fenetres, NOMBRE_FENETRES, titre_ssmenu_fenetre, TF_GENERAL, RF_FEN_TF);
    if ( (nv_fen >= 0) && (nv_fen != infos_reproduire.type_fen_tf) ) {
      infos_reproduire.type_fen_tf = nv_fen;
      met_chaine(TF_GENERAL, RF_FEN_TF, LGMAX_TXT_FENETRE, Types_fenetres[nv_fen], TXT_COPIER);
      etat = Autorise_Symetrique(nv_fen);
      OBJ_active(TF_GENERAL, RF_TF_SYMETRIQUE, etat);
      OBJ_active(TF_GENERAL, RF_VAL_K1, etat);
      OBJ_active(TF_GENERAL, RF_VAL_K2, etat && (infos_reproduire.tf_symetrique == FAUX));
      OBJ_active(TF_GENERAL, TFG_TF_PARAM, Avec_Parametre(nv_fen));
    }
    return VRAI;

  case RF_TF_SYMETRIQUE:
    etat = OBJ_choisit(TF_GENERAL, RF_TF_SYMETRIQUE, ECHANGE);
    OBJ_active(TF_GENERAL, RF_VAL_K2, !etat);
    return VRAI;

  case RF_TF_RAPIDE:
    OBJ_choisit(TF_GENERAL, RF_TF_RAPIDE, FORCE_OUI);
    OBJ_active(TF_GENERAL, RF_TF_RMIN, FORCE_NON);
    OBJ_active(TF_GENERAL, TFG_TF_ERR, FORCE_NON);
    OBJ_active(TF_GENERAL, TFG_TF_COVAR, FORCE_NON);
    return VRAI;

  case RF_TF_NORMALE:
    OBJ_choisit(TF_GENERAL, RF_TF_NORMALE, FORCE_OUI);
    OBJ_active(TF_GENERAL, RF_TF_RMIN, FORCE_OUI);
    OBJ_active(TF_GENERAL, TFG_TF_ERR, FORCE_OUI);
    OBJ_active(TF_GENERAL, TFG_TF_COVAR, FORCE_OUI);
    return VRAI;

  case RF_FAIRE_TFI:
    etat = OBJ_choisit(TF_GENERAL, RF_FAIRE_TFI, ECHANGE);
    return VRAI;

  case RF_TFI_FENETRE:
    nv_fen = SS_menu_texte(Types_fenetres, NOMBRE_FENETRES, titre_ssmenu_fenetre2, TF_GENERAL, RF_TFI_FENETRE);
    if ( (nv_fen >= 0) && (nv_fen != infos_reproduire.type_fen_tfi) ) {
      infos_reproduire.type_fen_tfi = nv_fen;
      met_chaine(TF_GENERAL, RF_TFI_FENETRE, LGMAX_TXT_FENETRE, Types_fenetres[nv_fen], TXT_COPIER);
      etat = Autorise_Symetrique(nv_fen);
      OBJ_active(TF_GENERAL, RF_TFI_SYMETRIK, etat);
      OBJ_active(TF_GENERAL, RF_VAL_R1, etat);
      OBJ_active(TF_GENERAL, RF_VAL_R2, etat && (infos_reproduire.tfi_symetrique == FAUX));
      OBJ_active(TF_GENERAL, TFG_TFI_PARAM, Avec_Parametre(nv_fen));
    }
    return VRAI;

  case RF_TFI_SYMETRIK:
    etat = OBJ_choisit(TF_GENERAL, RF_TFI_SYMETRIK, ECHANGE);
    OBJ_active(TF_GENERAL, RF_VAL_R2, !etat);
    return VRAI;

  case RF_TFI_RAPIDE:
    OBJ_choisit(TF_GENERAL, RF_TFI_RAPIDE, FORCE_OUI);
    OBJ_active(TF_GENERAL, TFG_TFI_ERREUR, FORCE_NON);
    OBJ_active(TF_GENERAL, TFG_TFI_COVAR, FORCE_NON);
    return VRAI;

  case RF_TFI_NORMALE:
    OBJ_choisit(TF_GENERAL, RF_TFI_NORMALE, FORCE_OUI);
    OBJ_active(TF_GENERAL, TFG_TFI_ERREUR, FORCE_OUI);
    OBJ_active(TF_GENERAL, TFG_TFI_COVAR, FORCE_OUI);
    return VRAI;

  case TFG_TF_COVAR:
  case TFG_TF_ERR:
  case TFG_TFI_COVAR:
  case TFG_TFI_ERREUR:
    OBJ_active(TF_GENERAL, objet, FORCE_OUI);
    return VRAI;

  case RF_CHARGER:
    (void) EXF_chparams(&infos_reproduire);
    return VRAI;

  case RF_SAUVER:
    (void) TF_svparams();
    return VRAI;
  }

  return FAUX;
}

static void prepare_arbre(void)
{
  if ( FAUX == arbres[ TF_GENERAL ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( TF_GENERAL, "menu_filtre.html#repro_tf", GER_reproduit_filtre, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }
  OBJ_deplace(TF_GENERAL, RF_VAL_KMIN , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_KMAX , 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, TFG_TF_PARAM, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_K1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_K2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_TF_SYMETRIQUE, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_CORR_PHASE, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(TF_GENERAL, RF_VAL_RMIN , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_RMAX , 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, TFG_TFI_PARAM, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_R1, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_VAL_R2, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(TF_GENERAL, RF_TFI_SYMETRIK, 0, +3, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_
  OBJ_active(TF_GENERAL, RF_CORR_AMP, FORCE_NON);
  OBJ_active(TF_GENERAL, RF_CORR_PHASE, FORCE_NON);
  OBJ_active(TF_GENERAL, TFG_TF_ERR, FORCE_NON);
  OBJ_active(TF_GENERAL, TFG_TF_COVAR, FORCE_NON);
  OBJ_active(TF_GENERAL, TFG_TFI_ERREUR, FORCE_NON);
  OBJ_active(TF_GENERAL, TFG_TFI_COVAR, FORCE_NON);
  OBJ_active(TF_GENERAL, RF_CHARGER, FORCE_NON);
#endif

  arbres[TF_GENERAL].drapeaux.prepare = VRAI;
}
