/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Cration d'un nouveau graphe
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "interface.h"		/*  SUPPRIMER  TERME ! */

#include "serie.h"
#include "matrices.h"

#include "grf_creer.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static Graphe *reserve_graphe(unsigned long nbr_pts, int type, int mode);

void ASC_graphe_fen_principale(Serie_Graphes *serie); /* Utilise dans l'effacement -> pas static */

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Le noyau : rservation mmoire & initialisation -------- */

static Graphe *reserve_graphe(unsigned long nbr_pts, int type, int mode)
{
  Graphe *graphe;
  boolean err_memoire = FAUX;

  if ( 0 == nbr_pts ) {
    (void) fprintf(stderr, "reserve_graphe : trop peu de points [%lu : type = %d, mode = %d]\n", nbr_pts, type, mode);
    (void) BI_ERR_alerte(NULL, &ERR_TropPeuPoints, VRAI);
    return NULL;
  }

  /* La structure */
  graphe = (Graphe *) calloc( 1, sizeof(Graphe) ); /* calloc => tout est  0 au dbut */
  if (graphe == NULL) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de rserver la structure graphe\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Graphes/creer.c", "reserve_graphe", "graphe");
    return NULL;
  }

  /* Initialisation des valeurs non-nulles */
  graphe->nom = NULL;
  graphe->nombre_point = nbr_pts;
  graphe->drapeaux.isole = VRAI; /* Il n'appartient  aucun chanage pour l'instant */
  graphe->affichage = NULL;
  graphe->nom_dossier = NULL;
  graphe->nom_fichier = NULL;

  /* En fonction du genre de donnes, les tables */
  graphe->genre_donnees = mode;
  switch ( mode )
  {
  case GRF_FCT_REELLE:   /* Deux tables : X & Y, tables "simples" */
  case GRF_COURBE_PLANE:
    graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
    if (graphe->x == NULL) {
      err_memoire = VRAI;
    } else {
      graphe->y = (double *) calloc( nbr_pts,  sizeof(double) );
      if ( NULL == graphe->y ) {
	err_memoire = VRAI;
	free( graphe->x );
      }
    }
    graphe->drapeaux.superpose = VRAI;
    break;

  case GRF_FCT_COMPLEXE:
    graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
    if (graphe->x == NULL) {
      err_memoire = VRAI;
    } else {
      Complexe *y;

      y = (Complexe *) calloc( nbr_pts,  sizeof(Complexe) );
      if ( NULL == y ) {
	err_memoire = VRAI;
	free( graphe->x );
      } else {
	graphe->y = (double *) y; /* Ce n'est pas trs beau, mais bon... */
      }
    }
    graphe->drapeaux.superpose = VRAI;
    break;

  case GRF_SERIE: /* Une seule table : les Y, "simple" */
    graphe->y = (double *) calloc( nbr_pts,  sizeof(double) );
    if ( NULL == graphe->y ) {
      err_memoire = VRAI;
    }
    graphe->x_min = 0.0; graphe->x_max = (double) (graphe->nombre_point - 1);
    graphe->drapeaux.superpose = VRAI;
    break;

  case GRF_COURBE_ESPACE:
  case GRF_SURFACE:
    graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
    if (graphe->x == NULL) {
      err_memoire = VRAI;
    } else {
      graphe->y = (double *) calloc( nbr_pts,  sizeof(double) );
      if ( NULL == graphe->y ) {
	err_memoire = VRAI;
	free( graphe->x );
      } else {
	graphe->z.z = (double *) calloc( nbr_pts,  sizeof(double) );
	if ( NULL == graphe->z.z ) {
	  err_memoire = VRAI;
	  free( graphe->y );
	  free( graphe->x );
	}
      }
    }
    break;

  case GRF_FAISCEAU:
    graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
    if (graphe->x == NULL) {
      err_memoire = VRAI;
    } else {
      graphe->z.graphes = (Graphe **) calloc( nbr_pts, sizeof( Graphe * ) );
      if ( NULL == graphe->z.graphes ) {
	err_memoire = VRAI;
	free( graphe->x );
      }
    }
    break;

  case GRF_HISTOGRAMME:
    graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
    if ( NULL == graphe->x ) {
      err_memoire = VRAI;
    } else {
      graphe->z.effectifs = (unsigned long *) calloc( nbr_pts + 1,  sizeof(unsigned long) ); /* Une valeur de plus : classe ]x_max, +infini[ */
      if ( NULL == graphe->z.effectifs ) {
	err_memoire = VRAI;
	free( graphe->x ); graphe->x = NULL;
      }
    }
    break;

  case GRF_MULTI_SERIE:
    (void) fprintf(stderr, "reserve_graphe :  FAIRE --- Graphes de type %d\n", mode);
    err_memoire = VRAI;
    break;

  case GRF_ERREUR_R:
    graphe->y = (double *) calloc( nbr_pts,  sizeof(double) );
    if ( NULL == graphe->y ) {
      err_memoire = VRAI;
    } else {
      graphe->z.ddl = (int *) calloc( nbr_pts, sizeof(int) );
      if ( NULL == graphe->z.ddl ) {
	err_memoire = VRAI;
	free( graphe->y ); graphe->y = NULL;
      }
    }
    break;

  case GRF_ERREUR_C:
    graphe->y = (double *) calloc( nbr_pts,  sizeof(Complexe) );
    if ( NULL == graphe->y ) {
      err_memoire = VRAI;
    } else {
      graphe->z.ddl = (int *) calloc( nbr_pts, sizeof(int) );
      if ( NULL == graphe->z.ddl ) {
	err_memoire = VRAI;
	free( graphe->y ); graphe->y = NULL;
      }
    }
    break;

  default:
    (void) fprintf(stderr, "reserve_graphe : Type de graphe inconnu [%d]\n", mode);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    err_memoire = VRAI;
  }

  /* La rservation a-t-elle t russie ? */
  if ( VRAI == err_memoire ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante (donnes : %lu valeurs)\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_pts);
#endif
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Graphes/creer.c", "reserve_graphe", "donnes");
    free( graphe );
    return NULL;
  }

  /* Rservations complmentaires ventuelles, en fonction du type de donnes */
  graphe->type = type;
  switch ( type )
  {
  case GR_INTERNE:
    graphe->drapeaux.superpose = FAUX;
    break;

  case GR_EXAFS1:
  case GR_EXAFS2:
  case GR_EXAFS3:
  case GR_EXAFS:
    graphe->drapeaux.exafs = VRAI; /* C'est un signal E.X.A.F.S.  */
    break;

  case GR_TFI:
    if ( GRF_FCT_REELLE == mode ) {
      graphe->drapeaux.exafs  = VRAI; /* C'est un signal E.X.A.F.S.  */
    }
    break;

  case GR_DEVIATION: /* Degrs de libert (supposs identiques pour Re et Im, si graphe complexe) */
    if ( NULL == graphe->z.ddl ) {
      graphe->z.ddl = (int *) calloc( nbr_pts, sizeof(int) );
      if ( NULL == graphe->z.ddl ) {
	(void) fprintf(stderr, "reserve_graphe : Mmoire insuffisante [ddl]\n");
	GRF_eff_isole( graphe );
	return NULL;
      }
    } else if ( ( GRF_ERREUR_R == graphe->genre_donnees ) || ( GRF_ERREUR_C == graphe->genre_donnees ) ) {
      /* DDL dj prpars, rien  faire */
    } else {
      (void) fprintf(stderr, "reserve_graphe : ATTENTION, GR_DEVIATION pour un graphe de type %d ==> table des ddl indisponible\n", mode);
    }

    graphe->drapeaux.superpose = FAUX;
    break;
  }

  return graphe;
}

/* -------- Le noyau : ralisation du chanage --------------------------\
 | Renvoie 0 si tout s'est bien pass, <0 en cas de problme             |
 |                                                                       |
 | Graphe = le graphe  insrer dans le chanage                         |
 | avant  = le graphe aprs lequel il faut l'insrer ( la fin si NULL)  |
 |          [ne peut tre const, car modifi pour faire le chanage !]   |
 | serie  = la srie dans laquelle ajouter le graphe (ERREUR si NULL)    |
 \---------------------------------------------------------------------- */

int GRF_ChainerGraphe(Graphe *graphe, Graphe *avant, Serie_Graphes *serie)
{
  if ( (graphe == NULL) || (serie == NULL) ) return -1;

  graphe->drapeaux.isole = FAUX;

  if (serie->premier_graphe == NULL) { /* Aucun graphe dans la srie */
    serie->nbr_graphes = 1;
    serie->premier_graphe = graphe;
    serie->dernier_graphe = graphe;

    if (serie_courante == serie) { /* Et en plus, c'est celle active ! */
      GRF_aller(graphe); /* Devient donc le graphe courant */
    }

    return 0; /* Tout va bien ! */
  }

  if (avant != NULL) {
    graphe->suivant = avant->suivant;
    graphe->precedent = avant;
    avant->suivant = graphe;

    if (graphe->suivant != NULL) { /* On n'est pas le dernier */
      graphe->suivant->precedent = graphe;
    } else {
      serie->dernier_graphe = graphe; /* Graphe cr aprs le dernier */
    }
  } else {
    graphe->precedent = serie->dernier_graphe;
    serie->dernier_graphe->suivant = graphe;
    serie->dernier_graphe = graphe;
  }
  serie->nbr_graphes++;

  return 0;
}

/* -------- Ajouter un graphe  une srie, pas forcment la courante -------- */

Graphe *Serie_nv_graphe(Serie_Graphes *serie, Graphe *avant, /* Pas const, car transmis  GRF_ChainerGraphe() qui modifie avant */
			unsigned long nbr_pts, int type, int mode, const char *nom)
{
  Graphe *graphe = NULL;
  int retour;

  if (serie == NULL) { /* C'est que l'on veut en crer une nouvelle */
    serie = SERIE_GRF_CreerSerie(NULL, NULL);
    if (serie == NULL) {
      (void) fprintf(stderr, "Serie_nv_graphe : erreur dans creer_serie\n");
      return NULL;
    }
  }
  if ( VRAI == serie->drapeaux.protegee ) { /* Srie protge : impossible d'y ajouter un graphe */
    (void) fprintf(stderr, "Serie_nv_graphe : srie protge, cration du graphe impossible.\n");
    return NULL;
  }

#ifdef _DEVELOPPE_
  if (avant != NULL) {
    graphe = serie->premier_graphe;
    while (graphe != avant) {
      if (graphe == NULL) break;
      graphe = graphe->suivant;
    }

    if (graphe == NULL) {
      (void) fprintf(stderr, "Serie_nv_graphe : ATTENTION, graphe prcdent pas dans la srie indique\n");
      (void) fprintf(stderr, "   graphe = %s\n", avant->nom);
      (void) fprintf(stderr, "   srie  = %s\n", serie->nom);
      avant = NULL;
    }
  }
#endif

  /* On rserve le graphe */
  graphe = reserve_graphe( nbr_pts, type, mode );
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "Serie_nv_graphe : impossible de rserver le graphe [erreur dans reserve_graphe].\n");
    return NULL;
  }

  /* On fait le chanage */
  if ( GRF_ChainerGraphe(graphe, avant, serie) < 0) {
    (void) fprintf(stderr, "Serie_nv_graphe : erreur dans GRF_ChainerGraphe.\n");
    /* Librer le graphe */
    printf("Serie_nv_graphe :  FAIRE, librer le graphe quand le chanage a chou\n");
    return NULL;
  }

  /* On rserve pour l'affichage */
  retour = GRF_CreerAffichage( graphe );
  if ( retour == 0 ) {
    /* On met le nom */
    GRF_titre(graphe, nom);

    /* Lgendes par dfaut --> le champ reste  NULL ! */
  }

  if (serie == serie_courante) { /* On modifie la srie courante */
    GRF_PreparerListe( VRAI );
    ASC_graphe_fen_principale( serie_courante );
  }

  return graphe;
}

/* -------- Crer un nouveau graphe juste aprs le graphe indiqu, donc dans la mme srie --------\
 |   En fait, c'est plus une surcouche pour les fichiers o l'on ne veut pas inclure serie.h...    |
 \-----------------------------------------------------------------------------------------------  */

Graphe *GRF_nouveau(Graphe *avant, unsigned long nbr_pts, int type, int mode, const char *nom)
{
  Graphe *resultat;
  Serie_Graphes *serie;

  if ( NULL == avant ) { /* On ajoute  la srie courante */
    serie = serie_courante;
  } else {
    serie = GRF_QuelleSerie(avant);
  }
  resultat = Serie_nv_graphe(serie, avant, nbr_pts, type, mode, nom);
  if (NULL == resultat) {
    (void) fprintf(stderr, "GRF_nouveau : erreur dans Serie_nv_graphe.\n");
  }
  return resultat;
}

/* -------- Crer un graphe isol (en gros, pour usage interne...) -------- */

Graphe *GRF_nv_isole(unsigned long nbr_pts, int type, int mode)
{
  Graphe *graphe;

  graphe = reserve_graphe(nbr_pts, type, mode);
  if (NULL == graphe) {
    (void) fprintf(stderr, "GRF_nv_isole : chec dans reserve_graphe()\n");
    return NULL;
  }

  graphe->drapeaux.isole = VRAI;
  return graphe;
}

/* ~-~-~-~-~-~-~-~-~ Crations plus subtiles ~-~-~-~-~-~-~-~-~ */

/* -------- Crer un graphe avec une grille de X impose (pas calcul) -------- */

Graphe *GRF_nv_GrilleXN(Graphe *avant, int type, int mode, const char *nom, boolean isole,
			double x_min, double x_max, unsigned long nbr_pts)
{
  Graphe *graphe;
  double *x, pas;
  unsigned long i;

  if ( x_max <= x_min ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXN : intervalle absurde [%g, %g]\n", x_min, x_max);
    return NULL;
  }
  if ( nbr_pts < 2 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Trop peu de points [%lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_pts);
    return NULL;
  }

  /* Calcul du pas */
  pas = ( x_max - x_min ) / (double) ( nbr_pts - 1 );
  if ( pas <= 0.0 ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXN : pas absurde [%g, DX = %g]\n", pas, x_max - x_min);
    return NULL;
  }

  /* On cre le graphe */
  if ( VRAI == isole ) {
    graphe = GRF_nv_isole( nbr_pts, type, mode );
  } else {
    graphe = GRF_nouveau( avant, nbr_pts, type, mode, nom );
  }
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXN : ERREUR dans GRF_nouveau()\n");
    return NULL;
  }

  /* On initialise la table des X */
  for ( i = 0, x = graphe->x ; i < graphe->nombre_point ; i++, x++ ) {
    *x = x_min + pas * (double) i;
  }
  graphe->x_min = x_min;
  graphe->x_max = graphe->x[ graphe->nombre_point - 1 ];

  /* On a fini, tout s'est bien pass */
  return graphe;
}

/* -------- Crer un graphe avec une grille de X impose (pas) -------- */

Graphe *GRF_nv_GrilleXP(Graphe *avant, int type, int mode, const char *nom, boolean isole,
			double x_min, double x_max, double pas)
{
  Graphe *graphe;
  double *x;
  unsigned long i, nbr_pts;

  if ( x_max <= x_min ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXP : intervalle absurde [%g, %g]\n", x_min, x_max);
    return NULL;
  }
  if ( ( pas <= 0.0 ) || ( pas >= ( x_max - x_min ) ) ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXP : pas absurde [%g, DX = %g]\n", pas, x_max - x_min);
    return NULL;
  }

  /* Combien de points faut-il crer ? */
  nbr_pts = 1 + (unsigned long) ( (x_max - x_min) / pas );

  /* On cre le graphe */
  if ( VRAI == isole ) {
    graphe = GRF_nv_isole( nbr_pts, type, mode );
  } else {
    graphe = GRF_nouveau( avant, nbr_pts, type, mode, nom );
  }
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "GRF_nv_GrilleXP : ERREUR dans GRF_nouveau()\n");
    return NULL;
  }

  /* On initialise la table des X */
  for ( i = 0, x = graphe->x ; i < graphe->nombre_point ; i++, x++ ) {
    *x = x_min + pas * (double) i;
  }
  graphe->x_min = x_min;
  graphe->x_max = graphe->x[ graphe->nombre_point - 1 ];

  /* On a fini, tout s'est bien pass */
  return graphe;
}

/* -------- Crer un graphe avec une grille de X impose (libre) -------- 
   Renvoie le nouveau graphe
           NULL en cas d'erreur

  avant   : le graphe aprs lequel crer le nouveau graphe (NULL : cf. GRF_nouveau())
  type    : type du nouveau graphe
  mode    : genre de donnes dans le nouveau graphe
  nom     : nom du nouveau graphe (ignor si graphe isol, sf en dveloppement)
  isole   : VRAI => le graphe cr est isol
  table_x : valeurs  utiliser en X
  nbr_pts : nombre de valeurs en X
  copier  : VRAI => les valeurs sont copies en X [alors, mode = GRF_FCT_*]
            FAUX => Si mode = GRF_SERIE, copie virtuelle des valeurs par pseudo-table
                    Sinon, on assigne directement la table qui NE DOIT PAS TRE LIBRE
*/

Graphe *GRF_nv_TableX(Graphe *avant, int type, int mode, const char *nom, boolean isole,
		      double *table_x, unsigned long nbr_pts, boolean copier)
{
  Graphe *graphe;

  if ( ( NULL == table_x ) || ( 0 == nbr_pts ) ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de table ou table vide\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }
  if ( GRF_SERIE == mode ) copier = FAUX; /* Une srie n'a pas de X ! On installe donc forcment la table. */

  /* On cre le graphe */
  if ( VRAI == isole ) {
    graphe = GRF_nv_isole( nbr_pts, type, mode );
    GRF_titre( graphe, nom ); /* nom utile pour le dboguage & si on chane ensuite le graphe ! */
  } else {
    graphe = GRF_nouveau( avant, nbr_pts, type, mode, nom );
  }
  if ( NULL == graphe ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer le nouveau graphe\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On initialise la table des X */
  if ( VRAI == copier ) {
    memcpy( graphe->x, table_x, sizeof( double ) * nbr_pts );
  } else {
    free( graphe->x );
    graphe->x = table_x;
#ifdef _DEVELOPPE_
    if ( mode != GRF_SERIE )
      (void) fprintf(stderr, "GRF_nv_TableX : Il faudrait viter de rserver les X pour rien...\n");
#endif
  }

  /* On met les bornes en X */
  GRF_bornes( graphe, DESORDONNE, NE_PAS_FAIRE );

  /* On a fini, tout s'est bien pass */
  return graphe;
}


/* -------- Crer un graphe  partir d'une double liste de valeurs textuelle --------
   Le graphe ainsi obtenu est isol, ncessairement de type fct_reelle
 */

Graphe *GRF_nv_DoubleListe(int type, const char *liste_valeurs_X, const char *liste_valeurs_Y)
{
  Graphe *resultat = NULL, *erreurs_sur_X = NULL, *erreurs_sur_Y = NULL;
  char **valeurs_X = NULL,  **valeurs_Y = NULL;
  double x, y, erreur_x, erreur_y;
  unsigned long i, nombre_valeurs, nbr_X, nbr_Y;

  /* Analyse de la liste des X */
  valeurs_X = TXT_DecomposerChaine( liste_valeurs_X, ';', &nbr_X );
  if ( ( NULL == valeurs_X ) || ( 0 == nbr_X ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en analysant la chane [X] %s\n",
		   __FILE__, __FUNCTION__, __LINE__, liste_valeurs_X );
    return NULL;
  }

  /* Analyse de la liste des Y */
  valeurs_Y = TXT_DecomposerChaine( liste_valeurs_Y, ';', &nbr_Y );
  if ( ( NULL == valeurs_Y ) || ( 0 == nbr_Y ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en analysant la chane [Y] %s\n",
		   __FILE__, __FUNCTION__, __LINE__, liste_valeurs_Y );
    free( valeurs_X[ 0 ] ); valeurs_X[ 0 ] = NULL;
    free( valeurs_X ); valeurs_X = NULL;
    return NULL;
  }

  /* Combien de points ? */
  if ( nbr_X > nbr_Y ) {
    AVERTISSEMENT( "Il manque des valeurs en Y [%lu, contre %lu valeurs en X]", nbr_Y, nbr_X );
    nombre_valeurs = nbr_X;
  } else if ( nbr_X < nbr_Y ) {
    AVERTISSEMENT( "Il manque des valeurs en X [%lu, contre %lu valeurs en Y]", nbr_X, nbr_Y );
    nombre_valeurs = nbr_Y;
  } else {
    /* galit : cas normal */
    nombre_valeurs = nbr_Y;
  }

  /* On cre le graphe */
  resultat = GRF_nv_isole( nombre_valeurs, type, GRF_FCT_REELLE );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer le graphe rsultat\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( valeurs_Y[ 0 ] ); valeurs_Y[ 0 ] = NULL;
    free( valeurs_Y ); valeurs_Y = NULL;

    free( valeurs_X[ 0 ] ); valeurs_X[ 0 ] = NULL;
    free( valeurs_X ); valeurs_X = NULL;
    return NULL;
  }

  /* On le remplit */
  for ( i = 0 ; i < resultat->nombre_point ; i++ ) {
    /* On convertit les valeurs */
    if ( i < nbr_X ) {
      x = TXT_CNV_vNombre_D( valeurs_X[ i ], NULL, &erreur_x );
    } else {
      x = 0.0;
      erreur_x = -1.0;
    }

    if ( i < nbr_Y ) {
      y = TXT_CNV_vNombre_D( valeurs_Y[ i ], NULL, &erreur_y );
    } else {
      y = 0.0;
      erreur_y = -1.0;
    }

    /* On place les valeurs dans le graphe */
    resultat->x[ i ] = x;
    resultat->y[ i ] = y;

    /* On s'occupe des erreurs, si ncessaire */
    if ( erreur_x > 0.0 ) {
      if ( NULL == resultat->erreur_x ) {
	/* On cre les incertitudes en Y */
	erreurs_sur_X = GRF_creer_erreurs( resultat, AXE_X );
      }
      /* Contrle en deux temps pour ne pas tre gn en cas d'chec ! */
      if ( erreurs_sur_X != NULL ) {
	erreurs_sur_X->y[ i ] = erreur_x;
      }
    }

    if ( erreur_y > 0.0 ) {
      if ( NULL == resultat->erreur_y ) {
	/* On cre les incertitudes en Y */
	erreurs_sur_Y = GRF_creer_erreurs( resultat, AXE_Y );
      }
      /* Contrle en deux temps pour ne pas tre gn en cas d'chec ! */
      if ( erreurs_sur_Y != NULL ) {
	erreurs_sur_Y->y[ i ] = erreur_y;
      }
    }
  }

  /* On calcule ses bornes */
  GRF_bornes( resultat, DESORDONNE, DESORDONNE );

  /* On libre ce qui ne sert plus */
  free( valeurs_X[ 0 ] ); valeurs_X[ 0 ] = NULL;
  free( valeurs_X ); valeurs_X = NULL;

  free( valeurs_Y[ 0 ] ); valeurs_Y[ 0 ] = NULL;
  free( valeurs_Y ); valeurs_Y = NULL;

  /* Termin : le reste [nom, lgendes, units,... ; insertion dans une srie] est  la charge de l'appelant */
  return resultat;
}


/* -------- Crer un graphe  partir d'une liste de valeurs textuelle --------
   Le graphe ainsi obtenu est isol, ncessairement de type srie
 */

Graphe *GRF_nv_ListeValeurs(int type, const char *liste_valeurs)
{
  Graphe *resultat = NULL, *erreurs_sur_Y = NULL;
  char **valeurs = NULL;
  double erreur;
  unsigned long i, nombre_valeurs;

  /* On analyse la liste [sparateur : ;] pour savoir combien il y a de valeurs */
  valeurs = TXT_DecomposerChaine( liste_valeurs, ';', &nombre_valeurs );
  if ( ( NULL == valeurs ) || ( 0 == nombre_valeurs ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en analysant la chane %s\n",
		   __FILE__, __FUNCTION__, __LINE__, liste_valeurs);
    return NULL;
  }

  /* On cre un graphe isol qui contiendra les valeurs ainsi obtenues */
  resultat = GRF_nv_isole( nombre_valeurs, type, GRF_SERIE );
  if ( NULL == resultat ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer le graphe rsultat\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free( valeurs[ 0 ] ); valeurs[ 0 ] = NULL;
    free( valeurs ); valeurs = NULL;
    return NULL;
  }

  /* On remplit ce graphe */
  for ( i = 0 ; i < resultat->nombre_point ; i++ ) {
    resultat->y[ i ] = TXT_CNV_vNombre_D( valeurs[ i ], NULL, &erreur );
    if ( erreur > 0.0 ) {
      if ( NULL == resultat->erreur_y ) {
	/* On cre les incertitudes en Y */
	erreurs_sur_Y = GRF_creer_erreurs( resultat, AXE_Y );
      }
      /* Contrle en deux temps pour ne pas tre gn en cas d'chec ! */
      if ( erreurs_sur_Y != NULL ) {
	erreurs_sur_Y->y[ i ] = erreur;
      }
    }
  }

  /* On calcule ses bornes */
  GRF_bornes( resultat, NE_PAS_FAIRE, DESORDONNE );

  /* On libre ce qui ne sert plus */
  free( valeurs[ 0 ] ); valeurs[ 0 ] = NULL;
  free( valeurs ); valeurs = NULL;

  /* Termin : le reste [nom, lgendes, units,... ; insertion dans une srie] est  la charge de l'appelant */
  return resultat;
}

/* -------- Crer la matrice de covariance associe  un graphe -------- */

int GRF_CreerCovariance(Graphe *graphe)
{
  Matrice *matrice;

  if ( NULL == graphe ) return FAUX;
  if ( graphe->covariance != NULL ) {
    return FAUX;
  }

  if ( GRF_FCT_REELLE == graphe->genre_donnees ) {
    matrice = creer_matrice( graphe->nombre_point, graphe->nombre_point, MATRICE_SYMETRIQUE);
  } else if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) {
    matrice = creer_matrice( 2 * graphe->nombre_point, 2 * graphe->nombre_point, MATRICE_SYMETRIQUE);
  } else {
    matrice = NULL;
  }

  if ( NULL == matrice ) {
    return FAUX;
  }

  graphe->covariance = matrice;
  return VRAI;
}

/* --------------- Cration d'un nouveau graphe, aprs le graphe courant -----\
 |      Renvoie l'adresse de ce graphe (NULL si erreur)                       |
 |      Les valeurs du graphe sont mises  zro par calloc                    |
 |                                                                            |
 | nbr_pts : nombre de points dans le graphe                                  |
 | type    : le type de graphe cr (GR_...)                                  |
 | mode    : la nature du graphe (GR_NORMAL, GR_IM_MOD, ...)                  |
 | nom     : le nom  donner au graphe                                        |
 | leg_x   : la lgende en X                                                  |
 | leg_y   : la lgende en Y                                                  |
 | unite_x : unit de l'axe des abscisses                                     |
 | unite_y : unit de l'axe des ordonnes                                     |
 \--------------------------------------------------------------------------- */

Graphe *GRF_creer(unsigned long nbr_pts, int type, int mode, const char *nom, 
		  char *leg_x, char *leg_y, char *unite_x, char *unite_y)
{
  Graphe *graphe;

  ENTREE_FONCTION("GRF_creer", "nbr_pts=%lu type=%d mode=%d nom=%s leg_x=%s leg_y=%s unit_x=%d unit_y=%d",
		  nbr_pts, type, mode, nom, leg_x, leg_y, unite_x, unite_y);

  if (nom == NULL) nom = "Sans nom";

  if ( (serie_courante == NULL) || (serie_courante->drapeaux.protegee == VRAI ) ) { /* Srie protge */
    Serie_Graphes *serie;

    serie = SERIE_GRF_CreerSerie(serie_courante, "Nouveaux graphes");
    if (NULL == serie) {
      SORTIE_FONCTION("GRF_creer", "ERREUR --- %p", NULL);
      return NULL;
    }
    I_SERIE_GRF_Aller( serie );
  }

  graphe = Serie_nv_graphe(serie_courante, graphe_courant, nbr_pts, type, mode, nom);
  if (graphe == NULL) {
    SORTIE_FONCTION("GRF_creer", "ERREUR --- %p", NULL);
    return NULL;
  }

  /* Les lgendes et units */
  GRF_ChangerLegende( graphe, AXE_X, leg_x, VRAI ); /* Lgende en X  */
  GRF_ChangerUnite( graphe, AXE_X, unite_x );
  GRF_ChangerLegende( graphe, AXE_Y, leg_y, VRAI ); /* Lgende en Y  */
  GRF_ChangerUnite( graphe, AXE_Y, unite_y );

  SORTIE_FONCTION("GRF_creer", "%p", graphe);
  return graphe; /* On a fini */
}

void ASC_graphe_fen_principale(Serie_Graphes *serie)
{
  int pos_ascenseur, taille_ascenseur;
  unsigned long num_graphe;
  Graphe *g;

  if (serie == NULL) {
    FEN_ascenseur(principale, ASC_VERTICAL, 0, 1000);
    return;
  }

  if ( serie_courante->nbr_graphes < 2 ) { /* 0 ou 1 */
    FEN_ascenseur( principale, ASC_VERTICAL, 0, 1000 );
    return;
  }

  /* On repre o on est dans la chane */
  for ( num_graphe = 0, g = serie->premier_graphe ; g != graphe_courant ; num_graphe++, g = g->suivant );

  pos_ascenseur = (int) ( (float) ( 1000.0 * num_graphe ) / (float) ( serie_courante->nbr_graphes - 1 ) );
  taille_ascenseur = (int) ( 1000.0 / (float) (serie_courante->nbr_graphes) );
  FEN_ascenseur(principale, ASC_VERTICAL, pos_ascenseur, taille_ascenseur);
}
