/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Dplacer un graphe dans une srie, ou entre sries
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

#include "grf_deplacer.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Isoler un graphe d'une srie -----------\
 |  Renvoie 0 si tout s'est bien pass, != 0 sinon  |
 |                                                  |
 | serie  : la srie dont on veut isoler le graphe  |
 | graphe : le graphe  isoler                      |
 \------------------------------------------------- */

int GRF_IsolerGraphe(Serie_Graphes *serie, Graphe *graphe)
{
  if ( (NULL == serie) || (NULL == graphe) ) return 0;

  if ( VRAI == serie->drapeaux.protegee ) {
    (void) fprintf(stderr, "GRF_IsolerGraphe : srie protge, isolement impossible !\n");
    (void) BI_ERR_alerte(principale, &ERR_SerieProtegee, VRAI);
    return -1;
  }

  /* On l'te de la srie */
  GRF_DefaireChainage(serie, graphe);

  /* Les erreurs sont des graphes isols : rien  faire de particulier */
  return 0;
}

/* -------- Insrer un graphe dans une srie dj existante -------- */

void ASC_graphe_fen_principale(Serie_Graphes *serie); /* Utilise dans l'effacement -> pas static */

int GRF_InsererDansSerie(Graphe *const graphe, Serie_Graphes *serie, Graphe *const avant)
{
  if ( NULL == graphe ) return -1;
  if ( NULL == serie ) {
    if ( NULL == serie_courante ) { /* C'est donc que l'on veut en profiter pour crer la premire srie */
      AVERTISSEMENT( "Insertion dans une srie, sans srie cre ! [graphe %s]", graphe->nom );
      serie = SERIE_GRF_CreerSerie( NULL, "Srie cre au vol [GRF_InsererDansSerie]" );
    } else if ( avant != NULL ) { /* On insre dans la srie du spectre indiqu ! */
      serie = GRF_QuelleSerie( avant );
    }
    if ( NULL == serie ) return -2;
  }

  if ( VRAI == serie->drapeaux.protegee ) {
    (void) fprintf(stderr, "GRF_InsererDansSerie : srie protge, insertion impossible !\n");
    (void) BI_ERR_alerte(principale, &ERR_SerieProtegee, VRAI);
    return -1;
  }

  if ( ( graphe->precedent != NULL ) || ( graphe->suivant != NULL ) ) {
    AVERTISSEMENT( "Le graphe %s n'est pas isol", graphe->nom );
    return -1;
  }

  /* On l'insre dans la nouvelle srie, aprs le graphe indiqu */
  GRF_ChainerGraphe(graphe, avant, serie);

  /* On lui attribue des infos d'affichage, s'il n'en a pas */
  if ( NULL == graphe->affichage ) {
    GRF_CreerAffichage( graphe );
  }

  if ( serie == serie_courante ) {
    /* On a insr dans la srie courante : remettre  jour la liste des graphes & l'ascenseur de la fentre principale ! */
    GRF_PreparerListe( VRAI );
    ASC_graphe_fen_principale( serie_courante );
  }

  /* Tout s'est bien pass */
  return 0;
}

/* ~-~-~-~-~-~-~-~- Passage des graphes d'une srie  une autre ~-~-~-~-~-~-~-~- */

int GRF_ChangeSerie(Graphe *graphe, Serie_Graphes *nv_serie, Graphe *nv_avant)
{
  Serie_Graphes *ex_serie;

  if ( (NULL == graphe) || ( NULL == nv_serie) ) return FAUX;

  ex_serie = GRF_QuelleSerie(graphe);
  if ( ex_serie == nv_serie ) return VRAI; /* Dj dans la bonne srie... */

  if ( VRAI == nv_serie->drapeaux.protegee ) {
    (void) fprintf(stderr, "GRF_ChangeSerie : srie d'arrive protge, dplacement impossible !\n");
    (void) BI_ERR_alerte(principale, &ERR_SerieProtegee, VRAI);
    return FAUX;
  }

  if ( ex_serie != NULL ) { /* NULL si graphe isol seulement, normalement */
    /* On extrait le graphe de la srie */

    if ( 0 != GRF_IsolerGraphe(ex_serie, graphe) ) {
      (void) fprintf(stderr, "GRF_ChangeSerie : erreur dans GRF_IsolerGraphe() !\n");
      return FAUX;
    }
  }

  /* On l'insre dans la nouvelle srie, aprs le graphe indiqu */
  if ( 0 != GRF_InsererDansSerie(graphe, nv_serie, nv_avant) ) {
    (void) fprintf(stderr, "GRF_ChangeSerie : erreur dans GRF_InsererDansSerie() !\n");
    return FAUX;
  }

  return VRAI;
}
