/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 29 juin 2002

   Gestion gnrique des graphes

   Filtres sur les series pour ne retenir que certains graphes
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static unsigned int GRF_RemplirTable_Taille(struct serie *serie, Graphe **table, unsigned long nombre_points);

/* ~~~~~~~~ Realisation pratique ~~~~~ */

/* -------- Filtrage en fonction du type de graphe -------- */

Graphe **GRF_ChoisirType(struct serie *serie, int type, boolean strict, unsigned long *nombre)
{
  Graphe **selection, *graphe;
  unsigned long i, total;

  if (NULL == serie) return NULL;

  total = GRF_compter_type(type, strict, serie);
  *nombre = total;
  if (0 == total) {
    return NULL;
  }

  selection = (Graphe **) calloc( total, sizeof(Graphe *));
  if (NULL == selection) {
    (void) fprintf(stderr, "GRF_ChoisirType : memoire insuffisante pour une table de %lu graphes\n", total);
    return NULL;
  }

  for (i = 0, graphe = serie->premier_graphe ; graphe != NULL ; graphe = graphe->suivant) {
    if ( ( graphe->type == type ) || ( (FAUX == strict) && ( GR_SSTYPE == graphe->type ) ) ||
	 ( GR_TOUS == type ) ) {
      selection[i] = graphe;
      i++;
    }
  }

  return selection;
}

/* -------- Filtrage en fonction du nombre de points du graphe -------- */

Graphe **GRF_ChoisirTaille(struct serie *serie, unsigned long nombre_points, unsigned long *nombre)
{
  unsigned long nombre_graphes = 0;
  Graphe **table_graphes = NULL;

  if ( NULL == nombre ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de variable pour mmoriser le nombre de graphes trouv\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }
  *nombre = 0;
  if ( 0 == nombre_points ) return NULL;

  nombre_graphes = GRF_compter_taille( nombre_points, serie );
  if ( 0 == nombre_graphes ) {
    (void) fprintf(stderr, "%s, %s l.%u : Aucun graphe avec le bon nombre de points trouv\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On rserve la table des graphes */
  table_graphes = (Graphe **) calloc( nombre_graphes, sizeof( Graphe * ) );
  if ( NULL == table_graphes ) {
    (void) fprintf(stderr, "%s, %s l.%u : Mmoire insuffisante pour crer la table de graphes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* On la remplit */
  if ( NULL == serie ) {
    unsigned long decalage = 0;
    struct serie *idx_serie;

    for ( idx_serie = premiere_serie ; idx_serie != NULL ; idx_serie = idx_serie->suivante ) {
      decalage += GRF_RemplirTable_Taille( idx_serie, table_graphes + decalage, nombre_points );
    }
  } else {
    (void) GRF_RemplirTable_Taille( serie, table_graphes, nombre_points );
  }

  /* On a fini */
  *nombre = nombre_graphes;
  return table_graphes;
}

static unsigned int GRF_RemplirTable_Taille(struct serie *serie, Graphe **table, unsigned long nombre_points)
{
  Graphe *idx_graphe;
  unsigned int nombre = 0;

  for ( idx_graphe = serie->premier_graphe ; idx_graphe != NULL ; idx_graphe = idx_graphe->suivant ) {
    if ( nombre_points == idx_graphe->nombre_point ) {
      table[ nombre ] = idx_graphe;
      nombre++;
    }
  }

  /* Renvoie le nombre de graphes ajouts */
  return nombre;
}

/* -------- Filtre extrieur : utilise une table indiquant les graphes  garder --------\
 |   Renvoie une table contenant les divers graphes retenus -- NULL en cas d'erreur     |
 |                                                                                      |
 | serie  = la srie  filtrer (NULL : srie courante)                                  |
 | table  = pour chaque graphe de la srie, VRAI si on veut le garder                   |
 |    ATTENTION, pas de vrification de taille possible...                              |
 | nombre = le nombre de graphes finalement retenus                                     |
 \------------------------------------------------------------------------------------- */

Graphe **GRF_FiltreSerie(Serie_Graphes *serie, boolean *table, unsigned long *nombre)
{
  Graphe **selection, *graphe;
  unsigned long i, j = 0;

  if (NULL == nombre) return NULL;
  if (serie == NULL) serie = serie_courante;
  if (NULL == serie) return NULL; /* Aucune srie en mmoire */

  /* On compte le nombre de graphes */
  *nombre = 0;
  for (i = 0, graphe = serie->premier_graphe ; graphe != NULL ; i++, graphe = graphe->suivant) {
    if ( VRAI == table[i] ) (*nombre)++;
  }

  /* On rserve la table */
  selection = (Graphe **) malloc( sizeof(Graphe *) * *nombre);
  if (NULL == selection) {
    return NULL;
  }

  /* On installe les graphes slectionns dans la table */
  for (i = 0, graphe = serie->premier_graphe ; graphe != NULL ; i++, graphe = graphe->suivant) {
    if ( VRAI == table[i] ) {
      selection[j] = graphe;
      j++;
      if (j >= *nombre) break; /* On sait que l'on a fini */
    }
  }

  return selection;
}
