/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1999-2002, Emmanuel CURIS

   Quotient de deux graphes (grande varit de cas...)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "graphes.h"
#include "matrices.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int Quotient_CC(Graphe *num, Graphe *den, boolean traite_erreurs);
static int Quotient_CR(Graphe *num, Graphe *den, boolean traite_erreurs);
static int Quotient_RC(Graphe *num, Graphe *den, boolean traite_erreurs);
static int Quotient_RR(Graphe *num, Graphe *den, boolean traite_erreurs);

/* ~~~~~~~~ Ralisation ~~~~~~~~~~~~~~ */

/* -------- Quotient d'un graphe complexe par un graphe rel, indpendants -------- */

static int Quotient_CR(Graphe *num, Graphe *den, boolean traite_erreurs)
{
  unsigned long i;
  double *y_den;
  Complexe *y_num;
  int retour = 0;

  if ( (NULL == num) || (NULL == den) ) return ERR_GRF_NULL;

  if ( VRAI == traite_erreurs ) {
    /* On commence par les matrices de covariance */
    if ( num->covariance != NULL ) {
      if ( den->covariance != NULL ) {
	fprintf(stderr, "Quotient_CR, covar/covar :  FAIRE\n");
      } else if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CR, covar/erreurs :  FAIRE\n");
      } else {
	unsigned long i;
	double *y_den;

	/* Division par une constante ==> cas simple ! */
	for (i = 0, y_den = den->y ; i < den->nombre_point ; i++, y_den++) {
	  if ( *y_den != 0.0 ) {
	    unsigned long j;
	    unsigned i_reel = 2*i;

	    /* On multiplie les lignes 2*i et 2*i+1 de la matrice : partie relle et imaginaire */
	    /* Moiti infrieure */
	    for (j = 0 ; j <= i_reel ; j++) {
	      num->covariance->valeurs[ MAT_SYM_element(i_reel, j) ] /= *y_den;
	    }
	    /* Moiti suprieure, par symtrie revient  remplir les colonnes 2*i et 2*i+1 */
	    for (j = i_reel ; j < 2*(den->nombre_point) ; j += 2 ) { /* Dmarre  i pour traiter 2 fois la diagonale ! */
	      num->covariance->valeurs[ MAT_SYM_element(j, i_reel) ] /= *y_den;
	    }
	  }
	}
      }
    } else if ( den->covariance != NULL ) {
      if ( num->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CR, erreur/covar :  FAIRE\n");
      } else {
	fprintf(stderr, "Quotient_CR, rien/covar :  FAIRE\n");
      }
    }

    /* On s'occupe ensuite des barres d'erreur */
    if ( num->erreur_y != NULL ) {
      if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CR, erreur/erreur :  FAIRE\n");	
      } else {
	unsigned long i;
	register double *y_den;
	register Complexe *y_err;

	/* Division par une constante : cas simple ! */
	for (i = 0, y_den = den->y, y_err = (Complexe *) num->erreur_y->y ; i < num->nombre_point ; i++, y_den++, y_err++) {
	  if ( *y_den != 0.0 ) {
	    y_err->re /= fabs(*y_den);
	    y_err->im /= fabs(*y_den);
	  }
	}
      }

      GRF_bornes(num->erreur_y, NE_PAS_FAIRE, DESORDONNE);
    } else if ( den->covariance != NULL ) {
      fprintf(stderr, "Quotient_CR, rien/erreur :  FAIRE\n");	
    }
  }

  /* Maintenant que les erreurs sont  jour, on s'occupe du graphe proprement dit */
  for (i = 0, y_num = (Complexe *) num->y, y_den = den->y ; i < den->nombre_point ; i++, y_den++, y_num++) {
    if ( *y_den == 0.0 ) {
      retour = ERR_GRF_DIV_ZERO;
      y_num->re = 0.0; /* Arbitraire ! */
      y_num->im = 0.0;
    } else {
      y_num->re /= *y_den;
      y_num->im /= *y_den;
    }
  }

  return retour;
}

/* -------- Quotient de deux graphes complexes indpendants -------- */

static int Quotient_CC(Graphe *num, Graphe *den, boolean traite_erreurs)
{
  unsigned long i;
  Complexe *y_den;
  Complexe *y_num;
  int retour = 0;

  if ( (NULL == num) || (NULL == den) ) return ERR_GRF_NULL;

  if ( VRAI == traite_erreurs ) {
    /* On commence par les matrices de covariance */
    if ( num->covariance != NULL ) {
      if ( den->covariance != NULL ) {
	fprintf(stderr, "Quotient_CC, covar/covar :  FAIRE\n");
      } else if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CC, covar/erreurs :  FAIRE\n");
      } else {
	fprintf(stderr, "Quotient_CC, covar/constante :  FAIRE\n");
      }
    } else if ( den->covariance != NULL ) {
      if ( num->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CC, erreur/covar :  FAIRE\n");
      } else {
	fprintf(stderr, "Quotient_CC, constante/covar :  FAIRE\n");
      }
    }

    /* On s'occupe ensuite des barres d'erreur */
    if ( num->erreur_y != NULL ) {
      if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_CC, erreur/erreur :  FAIRE\n");	
      } else {
	fprintf(stderr, "Quotient_CC, erreur/constante :  FAIRE\n");
      }
    } else if ( den->covariance != NULL ) {
      fprintf(stderr, "Quotient_CC, constante/erreur :  FAIRE\n");	
    }
  }

  /* Maintenant que les erreurs sont  jour, on s'occupe du graphe proprement dit */
  for (i = 0, y_num = (Complexe *) num->y, y_den = (Complexe *) den->y ; i < den->nombre_point ; i++, y_den++, y_num++) {
    if ( y_den->re == 0.0 ) {
      if ( y_den->im == 0.0 ) {
	retour = ERR_GRF_DIV_ZERO;
	y_num->re = 0.0; /* Arbitraire ! */
	y_num->im = 0.0;
      } else { /* Division par un imaginaire pur */
	double nv_re;

	nv_re = y_num->im / y_den->im;
	y_num->im = - y_num->re / y_den->im ;
	y_num->re = nv_re;
      }
    } else if ( 0.0 == y_den->re) { /* Division par un rel */
      y_num->re /= y_den->re;
      y_num->im /= y_den->re;
    } else {
      Complexe tmp;
      double module;

      module = y_den->re * y_den->re + y_den->im * y_den->im;
      tmp.re = y_num->re * y_den->re + y_num->im * y_den->im;
      tmp.im = y_num->im * y_den->re - y_num->re * y_den->im;

      y_num->re = tmp.re / module;
      y_den->im = tmp.im / module;
    }
  }

  return retour;
}

/* -------- Quotient d'un graphe  valeurs relles par un graphe  valeurs complexes, indpendants -------- */

static int Quotient_RC(Graphe *num, Graphe *den, boolean traite_erreurs)
{
  int retour = 0; /* Tout va bien... */
  unsigned long i;
  register Complexe *y_num, *y_den;

  if ( (NULL == num) || (NULL == den) ) return ERR_GRF_NULL;
  if ( VRAI == traite_erreurs ) {
    /* On commence par les matrices de covariance */
    if ( num->covariance != NULL ) {
      if ( den->covariance != NULL ) {
	fprintf(stderr, "Quotient_RC, covar/covar :  FAIRE\n");
      } else if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_RC, covar/erreurs :  FAIRE\n");
      } else {
	fprintf(stderr, "Quotient_RC, covar/constante :  FAIRE\n");
      }
    } else if ( den->covariance != NULL ) {
      if ( num->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_RC, erreur/covar :  FAIRE\n");
      } else {
	fprintf(stderr, "Quotient_RC, constante/covar :  FAIRE\n");
      }
    }

    /* On s'occupe ensuite des barres d'erreur */
    if ( num->erreur_y != NULL ) {
      if ( den->erreur_y != NULL ) {
	fprintf(stderr, "Quotient_RC, erreur/erreur :  FAIRE\n");	
      } else {
	fprintf(stderr, "Quotient_RC, erreur/constante :  FAIRE\n");
      }
    } else if ( den->covariance != NULL ) {
      fprintf(stderr, "Quotient_RC, constante/erreur :  FAIRE\n");	
    }
  }

  /* Maintenant que les erreurs sont  jour, on s'occupe du graphe proprement dit */
  for (i = 0, y_num = (Complexe *) num->y, y_den = (Complexe *) den->y ; i < den->nombre_point ; i++, y_den++, y_num++) {
    if ( y_den->re == 0.0 ) {
      if ( y_den->im == 0.0 ) {
	retour = ERR_GRF_DIV_ZERO;
	y_num->re = 0.0; /* Arbitraire ! */
	y_num->im = 0.0;
      } else { /* Division par un imaginaire pur */
	y_num->im = - y_num->re / y_den->im;
	y_num->re = 0.0;
      }
    } else if ( 0.0 == y_den->re) { /* Division par un rel */
      y_num->re /= y_den->re;
    } else {
      double module;

      module = y_den->re * y_den->re + y_den->im * y_den->im;
      y_num->im  = - y_num->re * y_den->im / module;
      y_num->re *=   y_den->re / module;
    }
  }

  return retour;
}

/* -------- Quotient de deux graphes  valeurs relles, indpendants -------- */

static int Quotient_RR(Graphe *num, Graphe *den, boolean traite_erreurs)
{
  int retour = 0; /* Tout va bien... */
  unsigned long i;
  register double *y_n, *y_d;

  if ( (NULL == num) || (NULL == den) ) return ERR_GRF_NULL;

  /* Graphes  valeurs relles de mme nombre de point, X postuls cohrents */
  for (i = 0, y_n = num->y, y_d = den->y ; i < num->nombre_point ; i++, y_n++, y_d++) {
    if ( 0.0 == *y_d ) { /* Division par zro : on ne fait pas la division... */
      retour = ERR_GRF_DIV_ZERO;
    } else {
      *y_n /= *y_d;
    }
  }

  GRF_bornes(num, NE_PAS_FAIRE, DESORDONNE);
  if ( FAUX == traite_erreurs ) return retour; /* On a fini */

  if ( num->erreur_y != NULL ) { /* Erreurs sur le numrateur */
    Graphe *erreurs;
    register double *y_q;

    erreurs = num->erreur_y;
    if ( NULL == erreurs ) {
      return retour;
    }

    if ( NULL == den->erreur_y ) { /* Pas d'erreur sur le dnominateur : erreurs divises par abs(y_d) */
      for ( i = 0, y_n = erreurs->y, y_d = den->y ; i < erreurs->nombre_point ; i++, y_n++, y_d++ ) {
	if ( *y_d != 0.0 ) { /* Cas de la division par zro dj voqu */
	  *y_n /= fabs( *y_d );
	}
      }

      GRF_bornes(erreurs, NE_PAS_FAIRE, DESORDONNE);
      return retour;
    }

    /* Erreurs sur le dnominateur ET le numrateur.
       Thorme de propagation des erreurs : E(X/Y) ~ E(X)/E(Y)___________________
                                                       1      / 2      E(X)  2   \
                                             s(X/Y) ~ ----\  / s (X) + ----- s (Y)
                                                      E(Y) \/          E(Y)     
       Sous rserve que Y et X aient une faible probabilit de changer de signe
       Les essais montrent que la densit du quotient a un pic en m1/m2,
         est bien symtrique comme la gaussienne (au moins dans les cas rencontrs en EXAFS)
       [C'est faux en gnral, en particulier si m2 = 0, m1 != 0 : densit bimodale]

       En toute rigueur, le quotient n'a ni moyenne, ni variance si p(Y=0) > 0 ou f_Y(0) > 0
    */

    for (i = 0, y_q = num->y, y_n = erreurs->y, y_d = den->erreur_y->y ; i < num->nombre_point ; i++, y_q++, y_n++, y_d++) {
      double sigma, y_den;

      y_den = den->y[i]; /* La division ayant t faite... */
      if ( 0.0 != y_den ) {
	sigma  = *y_d * *y_q;
	sigma *= sigma;
	sigma += (*y_n) * *(y_n);
	sigma  = sqrt(sigma);
	sigma /= y_den;
      } else {
	sigma = -1.0; /* Erreur indfinie... */
      }

      *y_n = sigma; /* Nouvelle valeur de l'erreur */
    }

    GRF_bornes(erreurs, NE_PAS_FAIRE, DESORDONNE);
    return retour;
  }

  /* Seul le dnominateur a des erreurs */
  if ( den->erreur_y != NULL ) {
    Graphe *erreurs;
    double *y_q;

    erreurs = GRF_creer_erreurs(num, AXE_X);
    if (NULL == erreurs) return retour;

    /* Thorme de propagation des erreurs : E(X/Y) ~ E(X)/E(Y)
                                                       1   E(X)  
                                             s(X/Y) ~ ---- ---- s(Y)
                                                      E(Y) E(Y)     
       Sous rserve que Y ait une faible probabilit de s'annuler ou changer de signe
    */
    for (i = 0, y_q = num->y, y_n = erreurs->y, y_d = den->erreur_y->y ; i < num->nombre_point ; i++, y_q++, y_n++, y_d++) {
      double sigma, y_den;

      y_den = den->y[i];
      if ( 0.0 != y_den ) {
	sigma  = *y_d * *y_q;
	sigma /= y_den;
      } else {
	sigma = -1.0; /* Erreur indfinie... */
      }

      *y_n = sigma; /* Nouvelle valeur de l'erreur */
    }

    GRF_bornes(erreurs, NE_PAS_FAIRE, DESORDONNE);
    return retour;
  }

  return retour; /* Aucun des graphes n'a d'erreurs : le quotient non plus... */
}

/* ~-~-~-~-~-~-~-~- Interface vue de l'extrieur ~-~-~-~-~-~-~-~- */

/* -------- Quotient de deux graphes ----------------------------------------------\
 |  Renvoie le graphe cr, NULL si erreur                                         |
 | SUPPOSE que les X des deux graphes sont cohrents                               |
 |                                                                                 |
 | num : valeurs pour le numrateur                                                |
 | den : valeurs pour le dnominateur                                              |
 | en_place       : VRAI, num/den est stock dans num ;                            |
 |                  FAUX : un nouveau graphe contient le quotient                  |
 | traite_erreurs : VRAI = on essaye d'estimer les erreurs sur le quotient aussi   |
 | covariance     : termes de liaisons entre les deux graphes (NULL si aucun)      |
 | code_erreur    : raison de l'erreur                                             |
 \-------------------------------------------------------------------------------- */

Graphe *GRF_QuotientGraphes(Graphe *num, Graphe *den, boolean en_place, boolean traite_erreurs, Matrice *covariance, int *code_erreur)
{
  int retour;

  /* Pas d'erreur par dfaut */
  if ( code_erreur != NULL ) *code_erreur = 0;

  /* Controles prliminaires */
  if ( (NULL == num) || (NULL == den) ) {
    (void) fprintf(stderr, "%s,%s l.%u : un des graphes NULL [num=%p, den=%p]\n",
		   __FILE__, __FUNCTION__, __LINE__, (void *) num, (void *) den);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return NULL;
  }

  if ( num->nombre_point != den->nombre_point ) {
    if ( code_erreur != NULL ) *code_erreur = ERR_GRF_TAILLE_DIFF;
    return NULL;
  }

  if ( FAUX == en_place ) { /* on veut un nouveau graphe */
    num = GRF_Copier( num, NULL , FAUX );
    if ( NULL == num ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de prparer le graphe quotient\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return NULL;
    }
  }

  /* En fonction des circonstances, on appelle telle ou telle fonction */
  if ( GRF_EstFonctionReelle( num ) == VRAI ) {
    if ( GRF_EstFonctionReelle( den ) == VRAI ) {
      if ( NULL == covariance )
	retour = Quotient_RR(num, den, traite_erreurs);
      else {
	fprintf(stderr, "GRF_QuotientGraphes : [Rel / Rel] avec matrice de covariance  FAIRE\n");
      }
    } else if ( GRF_FCT_COMPLEXE == den->genre_donnees ) {
      /* Le rsultat sera complexe... */
      retour = GRF_ChangerGenre( num, GRF_FCT_COMPLEXE );
      if ( 0 != retour ) {
	(void) fprintf(stderr, "%s,%s l.%u : R/C, impossible de convertir le dnominateur [code %d]\n",
		       __FILE__, __FUNCTION__, __LINE__, retour);
	if ( FAUX == en_place ) {
	  GRF_efface(num, VRAI, VRAI); /* Plus besoin ! */
	}
	return NULL;
      }
      retour = Quotient_RC(num, den, traite_erreurs);
    } else {
      fprintf(stderr, "GRF_QuotientGraphes : R / xxx  FAIRE\n");
    }
  } else if ( GRF_FCT_COMPLEXE == num->genre_donnees ) {
    if ( GRF_FCT_REELLE == den->genre_donnees ) {
      if ( NULL == covariance )
	retour = Quotient_CR(num, den, traite_erreurs);
      else {
	fprintf(stderr, "GRF_QuotientGraphes : [Complexe / Rel] avec matrice de covariance  FAIRE\n");
      }
    } else if ( GRF_FCT_COMPLEXE == den->genre_donnees ) {
      retour = Quotient_CC(num, den, traite_erreurs);
    }
  } else {
    fprintf(stderr, "GRF_QuotientGraphes : Autre / xxx  FAIRE\n");
  }

  return num;
}
