/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 23 avril 2002

   Imports/export, des fichiers exprimentaux entre autres
   Import d'un fichier contenant une carto en PIXE ralise  Bordeaux
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "fichiers.h"
#include "images.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Chargement d'un fichier "fc" ----------------\
 |   Renvoie l'image ainsi gnre, NULL en cas d'erreur |
 |   Format dduit du programme envoy par Ph. Moretto   |
 |                                                       |
 | nom          = le nom du fichier  charger            |
 | pas_x, pas_y = dimensions de la grille de balayage    |
 \------------------------------------------------------ */

Image *PIXE_Bordeaux_fc(const char *nom, double pas_x, double pas_y)
{
  FILE *fichier;
  Image *image;
  char *titre;
  double *valeur;
  unsigned long x, y;

  if (NULL == nom) return NULL;

  fichier = fopen(nom, "rb"); /* Fichier en binaire */
  if ( NULL == fichier ) {
    (void) fprintf(stderr, "PIXE_Bordeaux_fc : impossible d'ouvrir le fichier %s\n", nom);
    (void) SYS_ERREUR(ERREUR_LECTURE, nom, NULL, NULL);
    return NULL;
  }

  /* On cre l'image : 256256 de toute faon */
  image = IMG_CreerImage( NULL, NULL, IMG_GENRE_REELLE, IMG_TYPE_CARTOGRAPHIE, 128, 128);
  if (NULL == image) {
    (void) fclose(fichier);
    return NULL;
  }
  titre = TXT_genere_chaine("PIXE, Bordeaux : %s", nom);
  IMG_Titre(image, titre);
  free(titre);

  /* On charge l'image : stocke par lignes, en binaire, 2 octets par entier */
  valeur = image->valeurs.R;
  for ( x = 0 ; x < 127 ; x++, valeur++ ) {
    for ( y = 0 ; y < 127 ; y++, valeur++ ) {
#ifdef PETITBOUTIEN_ /* Mme ordre des octets que sur PC (type Intel) */
      short int i;

      (void) fread(&i, 2, 1, fichier);
      *valeur = (float) i;
#else                /* Ordre inverse, type Motorola */
      char c[2];

      (void) fread(c, 1, 2, fichier);
      *valeur = (float) ( (c[0] << 7) + c[1] );
#endif
    }
  }

  /* Chargement termin */
  (void) fclose(fichier);

  /* Les bornes et apparentes */
  IMG_bornes(image);
  image->l_reel = (float) pas_x;
  image->h_reel = (float) pas_y;

  return image;
}

/* ************************************************** *
 !                 Interface graphique                !
 * ************************************************** */

Image *PIXE_Bordeaux(void)
{
  Image *retour = NULL;
  char **noms;
  unsigned int nbr_fichiers = 0, i;

  noms = CH_selecteur( "*.fc,*.fb", "Cartographie PIXE", 0, NULL, &nbr_fichiers, principale );
  if (NULL == noms) return NULL;

  for (i = 0 ; i < nbr_fichiers ; i++) {
    Image *nv_retour;

    nv_retour = PIXE_Bordeaux_fc( noms[i], 1.0, 1.0 );
    if (NULL == retour) retour = nv_retour;
    free( noms[i] );
  }
  free( noms );

  IMG_AllerImage( retour ); /* On va sur la premire a avoir t charge */

  /* On a fini : on renvoie la premire image charge */
  return retour;
}
