/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2001, Emmanuel CURIS
   Cration : 11 avril 2001

   Imports/export, des fichiers exprimentaux entre autres
   Import d'un fichier contenant une carto ralise sur ID22  l'ESRF
   Contient aussi la selection des colonnes pour une mesure type "ascan"
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h> /* Pour stat */
#include <unistd.h>

#include "txt_utils.h"

#include "systeme.h"

#include "exafs.h"

#include "esrf.h"
#include "fichiers.h"
#include "images.h"
#include "interface.h"
#include "serie.h"

#include "id22.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_TXT_COLONNE     10 /* Dans l'arbre de choix des colonnes */
#define LGMAX_TXT_TYPE_GRAPHE 25
#define LGMAX_TXT_COLONNE_X   25
#define LGMAX_NOM_COLONNE     25

static char **titres_colonnes = NULL;
static boolean *colonnes_choisies = NULL;
static unsigned int nbr_colonnes = 0, colonne_des_x = 0;
static unsigned long prem_colonne_affichee = 0;

static int type_graphe_import = GR_AUTRETYPE;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static boolean init_colonnes(FILE *fichier);
static boolean lit_colonnes (FILE *fichier);

static int GER_colonnes          (unsigned int objet, int x, int y);
static void    prepare_arbre_colonnes(void);

static int    mesh_analyse_commande(FILE *fichier, float *x_min, float *x_max, unsigned int *n_x,
				    float *y_min, float *y_max, unsigned int *n_y, double *temps, boolean *x_puis_y);
static Image *mesh_post_traitement (Image **images, unsigned int nbr_images, boolean miroir_x, boolean miroir_y);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Lecture du fichier indiqu : chargement de tous les spectres disponibles -------- */

/* -------- Lecture d'un scan  un seul mouvement, type ascan
   Renvoie NULL en cas de problme, quel qu'il soit, le graphe cr sinon

   DANS LE CAS D'UN PROBLME UNIQUEMENT
   Si ok = FAUX, c'est un problme grave (fin de fichier intempestive)
       le fichier est alors FERM
   Sinon, c'est un problme  mineur  (scan annul avec trop peu de points)
       le fichier n'est alors PAS ferm, on peut continuer
*/

Graphe *id22_lit_scan(FILE *fichier, int mode, int numero_scan, char *mouvement, const char *nom_fichier, boolean *ok)
{
  int retour, c;
  long pos_debut;
  unsigned int nbr_choisis;
  unsigned long nbr_points = 0, i;
  double *les_x = NULL, *zone_x;
  Graphe *graphe = NULL, **liste_graphes = NULL;

  *ok = FAUX; /* De toute faon, n'a d'intrt qu'en cas de problme - souvent majeur ! */
  /* On saute les lignes de commentaires */
  ignore_ligne(fichier);
  do {
    int c = fgetc(fichier);
    if (c == '#') {/* Commentaire */
      c = fgetc(fichier);
      if (c == 'N') {/* Suivi du nombre de colonnes */
	if (init_colonnes(fichier) == FAUX) {
	  (void) fclose(fichier);
	  return NULL;
	}
      } else if (c == 'L') {
	if (lit_colonnes(fichier) == FAUX) {
	  (void) fclose(fichier);
	  return NULL;
	}
      }
      ignore_ligne(fichier);
    } else {
      ungetc(c, fichier);
      break;
    }
  } while ( ! feof(fichier) );

  /* On compte le nombre de lignes : ncessaire pour prparer les graphes */
  pos_debut = ftell(fichier); /* Premire ligne utile */
  c = fgetc(fichier);
  while ( (c != '#') && (c != EOF) ) /* Commentaire -> le scan suivant commence */
  {
    nbr_points++; /* Une ligne de plus... */
    if (ignore_ligne(fichier) == EOF) /* On l'ignore */
      c = EOF;
    else
      c = fgetc(fichier); /* Et on passe  la suivante */
  }
  fseek(fichier, pos_debut, SEEK_SET); /* On se remet au dbut des donnes */

  if (nbr_points <= 2) {
    printf("Scan %d : trop peu de points... Lecture annule !\n", numero_scan);
    *ok = VRAI; /* Problme mineur... On peut s'intresser  la suite du fichier */
    return NULL;
  }

  /* On demande  l'utilisateur ce qu'il veut */
  if ( arbres[IMPORT_ESRF_2].drapeaux.prepare == FAUX ) {
    OBJ_choisit(IMPORT_ESRF_2, IE2_AUTRETYPE, FORCE_OUI);
    prepare_arbre_colonnes();
  }

  ARB_PreparerDefileur_T(IMPORT_ESRF_2, IE2_PREM_COLONNE, IE2_DER_COLONNE, 0, IE2_AVANT, IE2_APRES, IE2_FOND_ASC, IE2_BOUTON_ASC,
			 nbr_colonnes, (const char **) titres_colonnes, colonnes_choisies, 0, prem_colonne_affichee);
  for (i = 0, nbr_choisis = 0 ; i < nbr_colonnes; i++) {
    if (colonnes_choisies[i] == VRAI) {
      nbr_choisis++;
    }
  }
  OBJ_active(IMPORT_ESRF_2, IE2_CHARGER, nbr_choisis > 0);
  OBJ_MettreEntier_U(IMPORT_ESRF_2, IE2_NUMERO_SCAN, numero_scan);
  met_chaine(IMPORT_ESRF_2, IE2_COMMANDE, 0, "scan", TXT_COPIER);
  met_chaine(IMPORT_ESRF_2, IE2_COLONNE_X, LGMAX_TXT_COLONNE_X, titres_colonnes[colonne_des_x], TXT_COPIER);

  OBJ_active (IMPORT_ESRF_2, IE2_AUTRETYPE, FORCE_OUI);
  OBJ_choisit(IMPORT_ESRF_2, IE2_AUTRETYPE, type_graphe_import == GR_AUTRETYPE);
  OBJ_active (IMPORT_ESRF_2, IE2_TYPE, type_graphe_import != GR_AUTRETYPE);

  retour = ARB_Bloquant( IMPORT_ESRF_2, "Choix des donn"e_AIGU"es", principale, 0 );
  if (retour == IE2_ANNULER) { /* On annule la lecture => fermeture fichier, "problme grave" */
    fin_colonnes();
    (void) fclose(fichier);
    return NULL;
  } else if (retour == IE2_SAUTER) { /* On saute ce scan => problme "mineur", la lecture continue */
    fin_colonnes();
    *ok = VRAI;
    return NULL;
  }

  /* On cre les graphes souhaits */
  liste_graphes = (Graphe **) calloc(nbr_colonnes, sizeof(Graphe *));
  if (liste_graphes == NULL) { /* Erreur grave, de toute faon la suite serait certainement compromise */
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "id22_lit_scan", "liste_graphes");
    fin_colonnes();
    (void) fclose(fichier);
    return NULL;
  }
  for (i = 0 ; i < nbr_colonnes; i++) {
    if (colonnes_choisies[i] == VRAI) { /* On garde le spectre */
      char *titre_graphe;

      titre_graphe = TXT_genere_chaine("%F - %s/%s, scan %d, colonne %s", nom_fichier,
				       (mode == SCAN_ABSOLU) ? "ascan" : "dscan", mouvement, numero_scan, titres_colonnes[i]);

      graphe = GRF_nouveau( NULL, nbr_points, type_graphe_import, GR_NORMAL, titre_graphe );
      free(titre_graphe);
      if (graphe == NULL) {
	free(liste_graphes);
	(void) fclose(fichier);
	return NULL;
      }
      if (les_x == NULL)
	les_x = graphe->x;
    } else
      graphe = NULL;
    liste_graphes[i] = graphe;
  }

  /* Ici, on commence  lire le vrai fichier */
  for (i = 0, zone_x = les_x ; i < nbr_points ; i++, zone_x++) {
    unsigned int j;
	
    for (j = 0 ; j < nbr_colonnes ; j++) {
      if (j == colonne_des_x) { /* Contient les abscisses */
	fscanf(fichier, "%lf", zone_x);
	if ( VRAI == colonnes_choisies[j] ) { /* Graphe Y=X, mais bon... */
	  *(liste_graphes[j]->y + i) = *zone_x;
	}
      } else if (colonnes_choisies[j] == VRAI) {
	double y;
	fscanf(fichier, "%lf", &y);
	/*	  printf(" - Valeur : %lf\n", I); */

	*(liste_graphes[j]->y + i) = y;
      } else
	fscanf(fichier, "%*f");
    }
  }

  for (i = 0, graphe = NULL ; i < nbr_colonnes ; i++) {
    if ( liste_graphes[i] != NULL ) {
      if ( liste_graphes[i]->x != les_x ) { /* On recopie les X */
	memcpy(liste_graphes[i]->x, les_x, (size_t) (sizeof(double) * (liste_graphes[i]->nombre_point) ) );
      }
      if ( GR_AUTRETYPE == type_graphe_import ) { /* On met en lgende les noms des colonnes */
	GRF_ChangerLegende( liste_graphes[i], AXE_X, titres_colonnes[ colonne_des_x ], VRAI);
	GRF_ChangerLegende( liste_graphes[i], AXE_Y, titres_colonnes[ i ], VRAI);
      }
      GRF_bornes(liste_graphes[i], DESORDONNE, DESORDONNE);
      graphe = liste_graphes[i];
    }
  }

  free(liste_graphes);
  return graphe;
}

/* -------- Lecture d'une carte mesh : cas des ROI directement dans le fichier -------- */

Image *id22_lit_mesh(FILE *fichier, int numero_scan, const char *nom_fichier)
{
  Image **liste_images;
  Image *image = NULL;
  boolean grille_hor = FAUX;
  int retour;
  float dep_x, fin_x, dep_y, fin_y;
  double tps_compte;
  unsigned int nbr_x, nbr_y, i, nbr_choisis;

  /* Lecture de la premire ligne : infos sur la grille */
  retour = mesh_analyse_commande(fichier, &dep_x, &fin_x, &nbr_x, &dep_y, &fin_y, &nbr_y, &tps_compte, &grille_hor);
  if (retour != 0) {
    (void) fprintf(stderr, "id22_lit_mesh(%s, scan %d) : erreur dans mesh_analyse_commande [code retour : %d]\n",
		   nom_fichier, numero_scan, retour);
    return NULL;
  }
  nbr_x++; nbr_y++;
#ifdef _DEVELOPPE_
  printf(" Grille horizontale : %u points de %f  %f\n", nbr_x, dep_x, fin_x);
  printf(" Grille verticale   : %u points de %f  %f\n", nbr_y, dep_y, fin_y);
  printf(" Acquisition : %f secondes par point\n", tps_compte);
#endif

  /* On saute les lignes de commentaires */
  (void) ignore_ligne(fichier);
  do {
    int c;

    c = fgetc(fichier);
    if (c == '#') {/* Commentaire */
      c = fgetc(fichier);
      if (c == 'N') {/* Suivi du nombre de colonnes */
	if (init_colonnes(fichier) == FAUX) {
	  (void) fprintf(stderr, "id22_lit_mesh : erreur dans init_colonnes\n");
	  return NULL;
	}
      } else if (c == 'L') {
	if (lit_colonnes(fichier) == FAUX) {
	  (void) fprintf(stderr, "id22_lit_mesh : erreur dans lit_colonnes\n");
	  return NULL;
	}
      }
      (void) ignore_ligne(fichier);
    } else {
      ungetc(c, fichier);
      break;
    }
  } while ( ! feof(fichier) );

  /* On demande  l'utilisateur ce qu'il veut */
  if ( arbres[ IMPORT_ESRF_2 ].drapeaux.prepare == FAUX )
    prepare_arbre_colonnes();

  ARB_PreparerDefileur_T( IMPORT_ESRF_2, IE2_PREM_COLONNE, IE2_DER_COLONNE, LGMAX_NOM_COLONNE, IE2_AVANT, IE2_FOND_ASC, IE2_BOUTON_ASC, IE2_APRES,
 			  nbr_colonnes, (const char **) titres_colonnes, colonnes_choisies, 0, prem_colonne_affichee );
  for ( i = 0, nbr_choisis = 0 ; i < nbr_colonnes; i++ ) {
    if ( colonnes_choisies[i] == VRAI ) {
      nbr_choisis++;
    }
  }
  OBJ_active( IMPORT_ESRF_2, IE2_CHARGER, nbr_choisis > 0 );
  OBJ_MettreEntier_U( IMPORT_ESRF_2, IE2_NUMERO_SCAN, numero_scan );
  met_chaine( IMPORT_ESRF_2, IE2_COMMANDE, 0, "mesh", TXT_COPIER );

  retour = ARB_Bloquant( IMPORT_ESRF_2, "Choix des donn"e_AIGU"es", principale, 0 );
  if ( retour == IE2_ANNULER ) {
    fin_colonnes();
    return NULL;
  }

  /* On cre les images souhaites */
  liste_images = (Image **) calloc(nbr_colonnes, sizeof(Image *));
  if (liste_images == NULL) {
    (void) fprintf(stderr, "id22_lit_mesh : impossible de crer la liste d''image\n");
    (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "id22_lit_mesh", "liste_images");
    fin_colonnes();
    return NULL;
  }
  for (i = 0 ; i < nbr_colonnes; i++) {
    if (colonnes_choisies[i] == VRAI) { /* On garde l'image */
      char *titre_image;

      image = IMG_CreerImage( NULL, NULL, IMG_GENRE_REELLE, IMG_TYPE_CARTOGRAPHIE, nbr_x, nbr_y);
      if ( NULL == image ) {
	unsigned int j;

	for (j = 0 ; j < i ; j++)
	  IMG_EffacerImage( liste_images[ j ], VRAI );
	free(liste_images);
	(void) fprintf(stderr, "id22_lit_mesh : impossible de crer l'image %u\n", i + 1);
	return NULL;
      }

      image->l_reel  = fabs(fin_x - dep_x)/(nbr_x - 1); /* Largeur en unites relles d'un pixel*/
      image->l_reel = MM_V_INTERNE(image->l_reel); /* Conversion dans l'unit interne */
      image->h_reel = fabs(fin_y - dep_y)/(nbr_y - 1);
      image->h_reel = MM_V_INTERNE(image->h_reel); /* Conversion dans l'unit interne */

      /* On place le centre pour que les coordonnes soient celles du mesh */
      if ( dep_x > fin_x ) {
#ifdef _DEVELOPPE_
	printf("id22_lit_mesh : carte inverse : par X dcroissants !\n");
#endif
	image->x_min = MM_V_INTERNE(fin_x);
      } else {
	image->x_min = MM_V_INTERNE(dep_x); /* Coordonnes en mm sur ID-22 */
      }
      if ( dep_y > fin_y ) {
#ifdef _DEVELOPPE_
	printf("id22_lit_mesh : carte inverse : par Y dcroissants !\n");
#endif
	image->y_min = MM_V_INTERNE(fin_y);
      } else {
	image->y_min = MM_V_INTERNE(dep_y);
      }

      titre_image = TXT_genere_chaine("%F - mesh, scan %d, colonne %s", nom_fichier, numero_scan, titres_colonnes[i]);
      IMG_Titre(image, titre_image);
      free(titre_image);
    } else
      image = NULL;
    liste_images[ i ] = image;
  }

  /* Ici, on commence  lire le vrai fichier
     Apparemment, mouvement h en premier puis v */
  if (grille_hor == VRAI) {/* Les X bougent d'abord */
    unsigned int x, y;
    boolean message = FAUX;

    for (y = 0 ; y < nbr_y ; y++)
      for (x = 0 ; x < nbr_x ; x++) {
	float pos_x, pos_y;
	int n_lu;
	
	n_lu = fscanf(fichier, "%f %f", &pos_x, &pos_y);
	while (n_lu != 2) {
	  int c;

	  c = fgetc(fichier);
	  while ('#' == c) {
	    c = fgetc(fichier);
	    if (EOF == c) {
	      (void) fprintf(stderr, "id22_lit_mesh : carte interrompue, fin de fichier atteinte\n");
	      return mesh_post_traitement(liste_images, nbr_colonnes,  dep_x > fin_x, dep_y > fin_y);
	    } else if (c != 'C') {
	      (void) fprintf(stderr, "id22_lit_mesh : carte interrompue, commande SPEC inconnue : #%c [%d]\n", c, c);
	      return mesh_post_traitement(liste_images, nbr_colonnes,  dep_x > fin_x, dep_y > fin_y);
	    } else { /* Commentaire : interruption */
	      if (FAUX == message) {
		(void) BI_ERR_alerte(principale, &INF_SpectreInterrompu, VRAI, numero_scan);
		message =VRAI;
	      }
	    }
	    c = ignore_ligne(fichier);
	  }

	  n_lu = fscanf(fichier, "%f %f", &pos_x, &pos_y);
	}
	/*      printf("Position (%f, %f)", pos_x, pos_y); */
	
	for (i = 2 ; i < nbr_colonnes ; i++) {
	  if (colonnes_choisies[i] == VRAI) {
	    double I;
	    unsigned long n_case;

	    fscanf(fichier, "%lf", &I);
	    /*	  printf(" - Valeur : %lf\n", I); */

	    IMG_IJ_v_N( liste_images[ i ], x, y, &n_case );
	    liste_images[ i ]->valeurs.R[ n_case ] = I;
	  }
	  else
	    fscanf(fichier, "%*f");
	}
      }
  } else { /* Les Y bougent d'abord */
    unsigned int x, y;

    for (x = 0 ; x < nbr_x ; x++)
      for (y = 0 ; y < nbr_y ; y++) {
	float pos_x, pos_y;
	
	fscanf(fichier, "%f %f", &pos_x, &pos_y);
	/*      printf("Position (%f, %f)", pos_x, pos_y); */
	
	for (i = 2 ; i < nbr_colonnes ; i++) {
	  if (colonnes_choisies[i] == VRAI) {
	    double I;
	    unsigned long n_case;

	    fscanf(fichier, "%lf", &I);
	    /*	  printf(" - Valeur : %lf\n", I); */

	    IMG_IJ_v_N( liste_images[ i ], x, y, &n_case );
	    liste_images[ i ]->valeurs.R[ n_case ] = I;
	  }
	  else
	    fscanf(fichier, "%*f");
	}
      }
  }

  return mesh_post_traitement(liste_images, nbr_colonnes,  dep_x > fin_x, dep_y > fin_y);
}

/* -------- Pranalyse d'une commande mesh -------- */

static int mesh_analyse_commande(FILE *fichier, float *x_min, float *x_max, unsigned int *n_x,
				 float *y_min, float *y_max, unsigned int *n_y, double *temps, boolean *x_puis_y)
{
  char *mvmt_1;
  int n_lu;

  if (NULL == fichier) return -1;

  /* Analyse du premier mouvement de la carte */
  mvmt_1 = lit_commande(fichier);
  if (NULL == mvmt_1) {
    (void) fprintf(stderr, "mesh_analyse_commande: impossible de lire le premier mouvement !\n");
    return -2;
  }
  if ( strcmp(mvmt_1, "samh") == 0 ) {
    *x_puis_y = VRAI;
    n_lu = fscanf(fichier, "%f %f %u %*s %f %f %u %lf",
		  x_min, x_max, n_x, y_min, y_max, n_y, temps);
  } else {
    *x_puis_y = FAUX;
    n_lu = fscanf(fichier, "%f %f %u %*s %f %f %u %lf",
		  y_min, y_max, n_y, x_min, x_max, n_x, temps);
  }
  free(mvmt_1);

  if (n_lu != 7) {
    (void) fprintf(stderr, "mesh_analyse_commande : erreur dans la commande mesh [n_lu = %d]\n", n_lu);
    (void) SYS_ERREUR(ERREUR_LECTURE, "ESRF/ID 22, commande mesh", fichier, NULL);
    return -3;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Traitement en fin de chargement -------- */

static Image *mesh_post_traitement(Image **images, unsigned int nbr_images, boolean miroir_x, boolean miroir_y)
{
  unsigned int i;
  Image *retour = NULL;

  if (NULL == images) return NULL;

  for (i = 0 ; i < nbr_images ; i++) {
    if (VRAI == miroir_x) { /* Image inverse : miroir en X */
      (void) IMG_PseudoMiroir( images[i], IMG_MIROIR_VERTICAL ); /* donc symtrie verticale */
    }

    if (VRAI == miroir_y) {
      (void) IMG_PseudoMiroir( images[i], IMG_MIROIR_HORIZONTAL ); /* donc symtrie verticale */
    }

    IMG_bornes(images[i]);
    if ( images[i] != NULL ) /* La dernire image cre est renvoye */
      retour = images[i];
  }
  free(images);

  return retour;
}

/* -------- Chargement d'une carte avec les spectres de fluo complets -------- */

Image *id22_lit_mesh_MCA(FILE *fichier, int numero_scan, const char *nom_fichier)
{
  Image *image;
  boolean grille_hor;
  float x_debut, x_fin, y_debut, y_fin;
  int c, retour;
  unsigned int nbr_x, nbr_y;
  unsigned long nbr_points;
  double temps_comptage;
  static char *dossier = NULL;
  char *nv_dossier;
  unsigned long lg_nom_fichier;
  char *nom_fichier_mca;
  Serie_Graphes *serie_mca;

  if (NULL == fichier) return NULL;

  /* On analyse la commande mesh */
  retour = mesh_analyse_commande(fichier, &x_debut, &x_fin, &nbr_x, &y_debut, &y_fin, &nbr_y, &temps_comptage, &grille_hor);
  if (retour != 0) {
    (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : erreur dans mesh_analyse_commande [code retour : %d]\n",
		   nom_fichier, numero_scan, retour);
    return NULL;
  }
  nbr_x++; nbr_y++;
  nbr_points = nbr_x * nbr_y;

  /* On saute les lignes de commentaire */
  c = ignore_ligne(fichier);
  while ('#' == c) {
    c = ignore_ligne(fichier);
  }

  /* On demande  l'utilisateur dans quel dossier se trouvent les fichiers MCA */
  nv_dossier = BI_Demande_Texte(principale, &Q_QuelDossierMCA, dossier, numero_scan, nom_fichier);
  free(dossier);
  dossier = nv_dossier; /* change pour ne pas avoir de fuite de mmoire... */
  if (NULL == dossier) {
    return NULL;
  }

  /* On prpare l'image */
  image = IMG_CreerImage( NULL, NULL, IMG_GENRE_GRAPHE, IMG_TYPE_CARTOGRAPHIE, nbr_x, nbr_y);
  if (NULL == image) {
    (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de crer l'image\n", nom_fichier, numero_scan);
    return NULL;
  }

  /* On prpare pour le nom des fichiers MCA */
  lg_nom_fichier = strlen(dossier) + 1; /* prfixe + _*/
  lg_nom_fichier += 3; /* numro de scan */
  if (numero_scan > 999) {
    printf("id22_lit_meshMCA :  FAIRE, numro de scan > 999\n");
  }
  lg_nom_fichier++; /* _ intermdiaire */
  if (nbr_points < 1000)
    lg_nom_fichier += 3;
  else
    lg_nom_fichier += TXT_nbr_car_UL(nbr_points);
  lg_nom_fichier += 5; /* .mca + 0 final */

  nom_fichier_mca = (char *) malloc( lg_nom_fichier );
  if ( NULL == nom_fichier_mca ) {
    (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de prparer le nom des fichiers MCA\n",
		   nom_fichier, numero_scan);
    IMG_EffacerImage( image, VRAI );
    return NULL;
  }

  /* On prpare la srie des fichiers MCA */
  serie_mca = SERIE_GRF_CreerSerie(NULL, NULL);
  if (NULL == serie_mca) {
    (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de crer la srie des spectres fluo\n",
		   nom_fichier, numero_scan);
    IMG_EffacerImage( image, VRAI );
    free(nom_fichier_mca);
    return NULL;
  }

  /* On charge les fichiers MCA un par un */
  if ( VRAI == grille_hor ) { /* Mouvement en X en premier */
    unsigned int x, y;
    unsigned long n_fichier = 0;
    unsigned long n_case;

    for (y = 0 ; y < nbr_y ; y++) {
      for (x = 0 ; x < nbr_x ; x++, n_fichier++) {
	Graphe *spectre;
	FILE *fichier_mca;

	sprintf(nom_fichier_mca, "%s_%03u_%03lu.mca", dossier, numero_scan, n_fichier);
	fichier_mca = fopen(nom_fichier_mca, "rb");
	if (NULL == fichier_mca) {
	  (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de lire le fichier MCA %s\n",
			nom_fichier, numero_scan, nom_fichier_mca);
	  (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier_mca, NULL, NULL);
	  IMG_EffacerImage( image, VRAI );
	  return NULL;
	}

	spectre = id22_lit_mca(fichier_mca, numero_scan, nom_fichier_mca, serie_mca);
	(void) fclose(fichier_mca); /* Plus besoin de lui */
	if (NULL == spectre) {
	  (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de crer le graphe MCA %s\n",
			 nom_fichier, numero_scan, nom_fichier_mca);
	}

	IMG_IJ_v_N( image, x, y, &n_case );
	image->valeurs.G[ n_case ] = spectre;
      }
    }
  } else {
    unsigned int x, y;
    unsigned long n_fichier = 0;
    unsigned long n_case;

    for (x = 0 ; x < nbr_x ; x++) {
      for (y = 0 ; y < nbr_y ; y++, n_fichier++) {
	Graphe *spectre;
	FILE *fichier_mca;

	sprintf(nom_fichier_mca, "%s_%03u_%03lu.mca", dossier, numero_scan, n_fichier);
	fichier_mca = fopen(nom_fichier_mca, "rb");
	if (NULL == fichier_mca) {
	  (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de lire le fichier MCA %s\n",
			nom_fichier, numero_scan, nom_fichier_mca);
	  (void) SYS_ERREUR(ERREUR_LECTURE, nom_fichier_mca, NULL, NULL);
	  IMG_EffacerImage( image, VRAI );
	  return NULL;
	}

	spectre = id22_lit_mca(fichier_mca, numero_scan, nom_fichier_mca, serie_mca);
	(void) fclose(fichier_mca); /* Plus besoin de lui */
	if (NULL == spectre) {
	  (void) fprintf(stderr, "id22_lit_meshMCA(%s, scan %d) : impossible de crer le graphe MCA %s\n",
			 nom_fichier, numero_scan, nom_fichier_mca);
	}

	IMG_IJ_v_N( image, x, y, &n_case );
	image->valeurs.G[ n_case ] = spectre;
      }
    }
  }

  free(nom_fichier_mca); /* Plus besoin de lui */

  /* On prpare correctement l'image ainsi cre */
  image->l_reel = fabs(x_fin - x_debut)/(nbr_x - 1); /* Largeur en unites relles d'un pixel*/
  image->l_reel = MM_V_INTERNE(image->l_reel); /* Conversion dans l'unit interne */
  image->h_reel = fabs(y_fin - y_debut)/(nbr_y - 1);
  image->h_reel = MM_V_INTERNE(image->h_reel); /* Conversion dans l'unit interne */
  if (x_fin < x_debut) {
    image->x_min = MM_V_INTERNE(x_fin);
    (void) IMG_PseudoMiroir( image, IMG_MIROIR_VERTICAL ); /* donc symtrie verticale */
  } else {
    image->x_min = MM_V_INTERNE(x_debut);
  }
  if (y_fin < y_debut) {
    image->y_min = MM_V_INTERNE(y_fin);
    (void) IMG_PseudoMiroir( image, IMG_MIROIR_HORIZONTAL ); /* donc symtrie verticale */
  } else {
    image->y_min = MM_V_INTERNE(y_debut);
  }
  IMG_bornes(image);

  nom_fichier_mca = TXT_genere_chaine("%F - mesh, scan %d", nom_fichier, numero_scan);
  IMG_Titre(image, nom_fichier_mca);
  free(nom_fichier_mca);

  nv_dossier = TXT_genere_chaine("Spectres de fluorescence / image %s", image->nom);
  Serie_titre(serie_mca, nv_dossier);
  free(nv_dossier);
  serie_mca->drapeaux.protegee = VRAI; /* Pas de modification de cette srie ! */
  serie_mca->drapeaux.homogene = VRAI; /* Tous les graphes sont de mme type */
  serie_mca->drapeaux.commun   = VRAI; /* On traite tous les spectres simultanment */

  /* On a fini, tout s'est bien pass */
  return image;
}

/* -------- Lecture d'une commande mcaacq = spectre de fluo X complet -------- */

Graphe *id22_lit_mca(FILE *fichier, int numero_scan, const char *nom_fichier, struct serie *serie_mca)
{
  Graphe *spectre = NULL;
  char *nom_graphe = NULL;
  int c;
  unsigned int canaux_par_ligne = 16, i = 0; /* Valeur arbitraire */
  unsigned long nbr_canaux = 0, canal_deb = 0, canal_fin = 0, pas_canal = 0, num_canal;
  double nrj_a = 0.0, nrj_b = 0.0, nrj_c = 0.0;
  double *x, *y;

  if (fichier == NULL) return NULL;
  if (NULL == serie_mca)
    serie_mca = serie_courante;

  do {
    c = ignore_ligne(fichier);

    if (c == '#') {
      (void) fgetc(fichier); /* #, Car ignore par ignore_ligne */
      c = fgetc(fichier); /* Caractre suivant */
      if (c == '@') { /* #@ */
	c = fgetc(fichier);
	if (c == 'C') { /* #@C */
	  c = fgetc(fichier);
	  if (c == 'H') { /* #@CH = channels */
	    fscanf(fichier, "%*s %lu %lu %lu %lu", &nbr_canaux, &canal_deb, &canal_fin, &pas_canal);
	  } else if (c == 'A') { /* #@CA = etalonnage en energie */
	    fscanf(fichier, "%*s %lf %lf %lf", &nrj_a, &nrj_b, &nrj_c);
	  }
	} else if (c == 'M') { /* #@MCA = nombre de canaux par ligne */
	  fscanf(fichier, "%*s %%%u", &canaux_par_ligne);
	}
      }
    }
  } while (c != '@'); /* @ en debut de ligne => @A */

  nom_graphe = TXT_genere_chaine("%F - Spectre de fluo X, scan %d", nom_fichier, numero_scan);
  spectre = Serie_nv_graphe(serie_mca, NULL, nbr_canaux, GR_FLUOX, GR_NORMAL, nom_graphe);
  free(nom_graphe);
  if (spectre == NULL)
    return NULL;
  GRF_ChangerLegende(spectre, AXE_Y, "Nombre de coups", VRAI);
  GRF_ChangerUnite( spectre, AXE_Y, "coups" );

  (void) fgetc(fichier); /* On fait sauter le A */
  for (num_canal = 0, x = spectre->x, y = spectre->y ; num_canal < nbr_canaux ; num_canal++, x++, y++) {
    *x = (double) num_canal;
    if ( ( (nrj_b != 0.0) && (nrj_b != 1.0) ) || /* Sur ID22, on peut avoir les 3 nuls ! */
	 (nrj_a != 0.0) || (nrj_c != 0.0) ) { /* tallonage en nergie disponible */
      *x  = nrj_a + *x * (nrj_b + *x * nrj_c);
    }
    if (i == canaux_par_ligne) {
      do {
	c = fgetc(fichier);
      } while (c != '\\'); /* Lignes finies par \, trs casse-pied... */
      i = 0;
    }
    fscanf(fichier, "%lf", y);
    i++;
  }
  GRF_bornes(spectre, ORDONNE, DESORDONNE);

  return spectre;
}

/* -~-~-~-~-~-~-~-~ Slection des colonnes  conserver, colonnes informatives individuellement -~-~-~-~-~-~-~-~ */

/* -------- Prparation de la gestion des diffrentes colonnes -------- 
      Renvoie FAUX s'il y a eu un problme
      Suppose que tout a t correctement libr quand il n'y en a pas l'usage
*/

static boolean init_colonnes(FILE *fichier)
{
  unsigned int nbr_col, i;

  if (fichier == NULL) return FAUX;
  fscanf(fichier, "%u", &nbr_col);
/*  printf("Le fichier contient %u colonnes\n", nbr_col); */

  if (titres_colonnes == NULL) {
    nbr_colonnes = nbr_col;
    titres_colonnes = (char **) calloc( nbr_colonnes, sizeof(char *) );
    if (titres_colonnes == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "init_colonnes", "titres_colonnes");
      nbr_colonnes = 0;
      colonne_des_x = 0;
      return FAUX;
    }

    colonnes_choisies = (boolean *) calloc(nbr_colonnes, sizeof(boolean) );
    if (colonnes_choisies == NULL) {
      (void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "init_colonnes", "colonnes_choisies");
      free(titres_colonnes);
      titres_colonnes = NULL;
      nbr_colonnes = 0;
      colonne_des_x = 0;
      return FAUX;
    }
  } else {
    /* On libre les anciens titres de colonnes */
    for (i = 0 ; i < nbr_colonnes ; i++) {
      free(titres_colonnes[i]);
      titres_colonnes[i] = NULL;
    }

    if (nbr_col > nbr_colonnes) { /* Il faut augmenter la taille prvue */
      char **nv_titres;
      boolean *nv_cch;

/*      printf("On refait la rservation mmoire\n"); */
      nv_titres = (char **) realloc(titres_colonnes, nbr_col * sizeof(char *));
      if (nv_titres == NULL) {
 	(void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "init_colonnes", "realloc - titres_colonnes");
	free(titres_colonnes);
	titres_colonnes = NULL;
	free(colonnes_choisies);
	colonnes_choisies = NULL;
	nbr_colonnes = 0;
	return FAUX;
      }
      titres_colonnes = nv_titres;

      nv_cch = (boolean *) realloc(colonnes_choisies, nbr_col * sizeof(boolean));
      if (nv_cch == NULL) {
 	(void) SYS_ERREUR(MANQUE_MEMOIRE, "Import/id22_carto.c", "init_colonnes", "realloc - colonnes_choisies");
	free(titres_colonnes);
	titres_colonnes = NULL;
	free(colonnes_choisies);
	colonnes_choisies = NULL;
	nbr_colonnes = 0;
	return FAUX;
      }
      colonnes_choisies = nv_cch;
      /* On initialise les nouvelles colonnes -  FAUX par dfaut */
      for (i = nbr_colonnes ; i < nbr_col ; i++)
	colonnes_choisies[i] = FAUX;
    } else {
      if (colonne_des_x >= nbr_col) /* On essaye de garder la colonne des X */
	colonne_des_x = 0;
    }
    nbr_colonnes = nbr_col;
  }

  return VRAI; /* Tout s'est bien pass */
}

/* -------- Lecture des noms des colonnes, placs dans le tableau -------- */

static boolean lit_colonnes(FILE *fichier)
{
  unsigned int i;
  char *titre;

  if (fichier == NULL) return FAUX;
  if ( (titres_colonnes == NULL) || (colonnes_choisies == NULL) ) return FAUX;

  for (i = 0 ; i < nbr_colonnes ; i++) {
/*    printf("On va lire pour la colonne %u\n", i); */
    titre = lit_mot(fichier);
    if (titre == NULL) {
      unsigned int j;

      (void) SYS_ERREUR(ERREUR_LECTURE, "Fichier en cours - titres", fichier, NULL);
      for (j = 0 ; j < i ; j++)
	free(titres_colonnes[j]);
      free(titres_colonnes);
      titres_colonnes = NULL;
      free(colonnes_choisies);
      colonnes_choisies = NULL;
      colonne_des_x = 0;
      nbr_colonnes = 0;
      return FAUX;
    }
/*    printf("  Titre lu : %s\n", titre); */
    titres_colonnes[i] = titre;
  }

  return VRAI;
}

/* -------- Libration de tout ce qui est utilis pour grer les colonnes -------- */

void fin_colonnes(void)
{
  unsigned int i;

  for (i = 0 ; i < nbr_colonnes ; i++)
    free(titres_colonnes[i]);
  free(titres_colonnes);
  titres_colonnes = NULL;
  free(colonnes_choisies);
  colonnes_choisies = NULL;
  colonne_des_x = 0;
  nbr_colonnes = 0;
}


/* -------- Gestion de la bote de dialogue pour choisir les colonnes  garder -------- */

static int GER_colonnes(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  unsigned int nbr_choisis, i;

  if ( ( (objet >= IE2_PREM_COLONNE) && (objet <= IE2_DER_COLONNE) ) ||
       (objet == IE2_AVANT) || (objet == IE2_APRES) || (objet == IE2_FOND_ASC) || (objet == IE2_BOUTON_ASC) ) {
    ARB_GererDefileur_T( IMPORT_ESRF_2, IE2_PREM_COLONNE, IE2_DER_COLONNE, LGMAX_NOM_COLONNE, IE2_AVANT, IE2_FOND_ASC, IE2_BOUTON_ASC, IE2_APRES,
			 nbr_colonnes, (const char **) titres_colonnes, colonnes_choisies, NULL, &prem_colonne_affichee, objet, y);

    if ( (objet >= IE2_PREM_COLONNE) && (objet <= IE2_DER_COLONNE) ) {
      for (i = 0, nbr_choisis = 0 ; i < nbr_colonnes; i++) {
	if (colonnes_choisies[i] == VRAI) {
	  nbr_choisis++;
	}
      }
      OBJ_active(IMPORT_ESRF_2, IE2_CHARGER, nbr_choisis > 0);
    }

    return BI_EVENEMENT_TRAITE;
  } else if (objet == IE2_AUCUNE) {
    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_OUI);
    for (i = 0, nbr_choisis = 0 ; i < nbr_colonnes; i++) {
      colonnes_choisies[i] = FAUX;
    }
    OBJ_active(IMPORT_ESRF_2, IE2_CHARGER, FORCE_NON);
    ARB_PreparerDefileur_T( IMPORT_ESRF_2, IE2_PREM_COLONNE, IE2_DER_COLONNE, LGMAX_NOM_COLONNE, IE2_AVANT, IE2_FOND_ASC, IE2_BOUTON_ASC, IE2_APRES,
			    nbr_colonnes, (const char **) titres_colonnes, colonnes_choisies, 0, prem_colonne_affichee);
    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_NON);

    return BI_EVENEMENT_TRAITE;
  } else if (objet == IE2_TOUTES) {
    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_OUI);
    for (i = 0, nbr_choisis = 0 ; i < nbr_colonnes; i++) {
      colonnes_choisies[i] = VRAI;
    }
    OBJ_active(IMPORT_ESRF_2, IE2_CHARGER, FORCE_OUI);
    ARB_PreparerDefileur_T( IMPORT_ESRF_2, IE2_PREM_COLONNE, IE2_DER_COLONNE, LGMAX_NOM_COLONNE, IE2_AVANT, IE2_FOND_ASC, IE2_BOUTON_ASC, IE2_APRES,
			    nbr_colonnes, (const char **) titres_colonnes, colonnes_choisies, 0, prem_colonne_affichee);
    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_NON);

    return BI_EVENEMENT_TRAITE;
  } else if (IE2_COLONNE_X == objet) {
    int nv_colonne;

    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_OUI);
    nv_colonne = SS_menu_texte( (const char **) titres_colonnes, nbr_colonnes, "Colonne des X", IMPORT_ESRF_2, IE2_COLONNE_X);
    if ( (nv_colonne >= 0) && (nv_colonne != (int) colonne_des_x) ) {
      colonne_des_x = nv_colonne;
      met_chaine(IMPORT_ESRF_2, IE2_COLONNE_X, LGMAX_TXT_COLONNE_X, titres_colonnes[colonne_des_x], TXT_COPIER);
    }
    OBJ_choisit(IMPORT_ESRF_2, objet, FORCE_NON);

    return BI_EVENEMENT_TRAITE;
  } else if (IE2_AUTRETYPE == objet) {
    int etat = OBJ_choisit(IMPORT_ESRF_2, objet, ECHANGE);

    if (etat == VRAI) {
      type_graphe_import = GR_AUTRETYPE;
      OBJ_active(IMPORT_ESRF_2, IE2_TYPE, FORCE_NON);
    } else {
      type_graphe_import = GR_BRUT;
      OBJ_active(IMPORT_ESRF_2, IE2_TYPE, FORCE_OUI);
    }
    met_chaine(IMPORT_ESRF_2, IE2_TYPE, LGMAX_TXT_TYPE_GRAPHE, GRF_TexteTypeGraphe( type_graphe_import ), TXT_COPIER);

    return BI_EVENEMENT_TRAITE;
  } else if (IE2_TYPE == objet) {
    type_graphe_import = OBJ_ChoisirTypeGraphe( IMPORT_ESRF_2, IE2_TYPE, LGMAX_TXT_TYPE_GRAPHE, type_graphe_import, QUESTION, VRAI );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

static void prepare_arbre_colonnes(void)
{
  ARB_EnjoliverDefileur( IMPORT_ESRF_2, 0, IE2_PREM_COLONNE, IE2_DER_COLONNE,
			 IE2_AVANT, IE2_FOND_ASC, IE2_BOUTON_ASC, IE2_APRES, ARB_ASCENSEUR_DROITE );

  met_chaine(IMPORT_ESRF_2, IE2_TYPE, LGMAX_TXT_TYPE_GRAPHE, "Sans type", TXT_COPIER);
  met_chaine(IMPORT_ESRF_2, IE2_COLONNE_X, LGMAX_TXT_COLONNE_X, "", TXT_COPIER);

  arbres[IMPORT_ESRF_2].fonction.arbre = GER_colonnes;
  arbres[IMPORT_ESRF_2].drapeaux.prepare = VRAI;
}
