/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) Emmanuel CURIS, 2003

   Module de calculs thoriques divers
   Dfinitions pour les raies de fluorescence [isol car long et fastidieux]
*/

#ifndef FLUORESCENCE_H_
# define FLUORESCENCE_H_  /* Protection contre l'inclusion multiple */

/* -------- Raies K -------- */

#define RAIE_K_L1  0 /* 2s    -> 1s, Ka3  , Z >= Li, interdite */
#define RAIE_K_L2  1 /* 2p1/2 -> 1s, Ka2  , Z >= B */
#define RAIE_K_L3  2 /* 2p3/2 -> 1s, Ka1  , Z >= N */
#define RAIE_K_M1  3 /* 3s    -> 1s,      , Z >= Na, interdite */
#define RAIE_K_M2  4 /* 3p1/2 -> 1s, Kb3  , Z >= Al */
#define RAIE_K_M3  5 /* 3p3/2 -> 1s, Kb1  , Z >= P */
#define RAIE_K_M4  6 /* 3d3/2 -> 1s, Kb5II, Z >= Sc, interdite */
#define RAIE_K_M5  7 /* 3d5/2 -> 1s, Kb5I , Z >= Cr, interdite */
#define RAIE_K_N1  8 /* 4s    -> 1s,      , Z >= K , interdite */
#define RAIE_K_N2  9 /* 4p1/2 -> 1s, Kb2II, Z >= Ga */
#define RAIE_K_N3 10 /* 4p3/2 -> 1s, Kb2I , Z >= As */
#define RAIE_K_N4 11 /* 4d3/2 -> 1s, Kb4II, Z >= Y , interdite */
#define RAIE_K_N5 12 /* 4d5/2 -> 1s, Kb4I , Z >= Mo, interdite */
#define RAIE_K_N6 13 /* 4f5/2 -> 1s,      , Z >= La, interdite */
#define RAIE_K_N7 14 /* 4f7/2 -> 1s,      , Z >=   , interdite */
#define RAIE_K_O1 15 /* 5s    -> 1s,      , Z >= Rb, interdite */
#define RAIE_K_O2 16 /* 5p1/2 -> 1s,      , Z >= In */
#define RAIE_K_O3 17 /* 5p3/2 -> 1s,      , Z >= Sb */
#define RAIE_K_O4 18 /* 5d3/2 -> 1s,      , Z >= Lu, interdite */
#define RAIE_K_O5 19 /* 5d5/2 -> 1s,      , Z >= W , interdite */
#define RAIE_K_O6 20 /* 5f5/2 -> 1s,      , Z >= Ac, interdite */
#define RAIE_K_O7 21 /* 5f7/2 -> 1s,      , Z >=   , interdite */
#define RAIE_K_P1 22 /* 6s    -> 1s,      , Z >= Cs, interdite */
#define RAIE_K_P2 23 /* 6p1/2 -> 1s,      , Z >= Tl */
#define RAIE_K_P3 24 /* 6p3/2 -> 1s,      , Z >= Bi */
#define RAIE_K_Q1 25 /* 7s    -> 1s,      , Z >= Fr, interdite */

#define NOMBRE_RAIES_K 26
#if NOMBRE_RAIES_K != ( 1 + RAIE_K_Q1 )
# error Erreur dans le numrotage des raies de fluorescence / K
#endif

/* Quelques synonymes */

#define RAIE_K_ALPHA_1 RAIE_K_L3
#define RAIE_K_ALPHA_2 RAIE_K_L2
#define RAIE_K_ALPHA_3 RAIE_K_L1

#define RAIE_K_BETA_1   RAIE_K_M3
#define RAIE_K_BETA_2I  RAIE_K_N3
#define RAIE_K_BETA_2II RAIE_K_N2
#define RAIE_K_BETA_3   RAIE_K_M2
#define RAIE_K_BETA_4I  RAIE_K_N5
#define RAIE_K_BETA_4II RAIE_K_N4
#define RAIE_K_BETA_5I  RAIE_K_M5
#define RAIE_K_BETA_5II RAIE_K_M4

/* Groupes de lignes */

#define RAIE_K_BETA_2 -1
#define RAIE_K_BETA_4 -2
#define RAIE_K_BETA_5 -3
#define RAIE_K_BETA   -4
#define RAIE_K_ALPHA  -5

/* -------- Raies L1 -------- */

#define RAIE_L1_L2 26 /* 2p1/2 -> 2s,     , Z >= B */
#define RAIE_L1_L3 27 /* 2p3/2 -> 2s,     , Z >= N */
#define RAIE_L1_M1 28 /* 3s    -> 2s,     , Z >= Na, interdite */
#define RAIE_L1_M2 29 /* 3p1/2 -> 2s, Lb4 , Z >= Al */
#define RAIE_L1_M3 30 /* 3p3/2 -> 2s, Lb3 , Z >= P */
#define RAIE_L1_M4 31 /* 3d3/2 -> 2s, Lb10, Z >= Sc, interdite */
#define RAIE_L1_M5 32 /* 3d5/2 -> 2s, Lb9 , Z >= Cr, interdite */
#define RAIE_L1_N1 33 /* 4s    -> 2s,     , Z >= K , interdite */
#define RAIE_L1_N2 34 /* 4p1/2 -> 2s, Lg2 , Z >= Ga */
#define RAIE_L1_N3 35 /* 4p3/2 -> 2s, Lg3 , Z >= As */
#define RAIE_L1_N4 36 /* 4d3/2 -> 2s,     , Z >= Y , interdite */
#define RAIE_L1_N5 37 /* 4d5/2 -> 2s, Lg11, Z >= Mo, interdite */
#define RAIE_L1_N6 38 /* 4f5/2 -> 2s,     , Z >= La, interdite */
#define RAIE_L1_N7 39 /* 4f7/2 -> 2s,     , Z >=   , interdite */
#define RAIE_L1_O1 40 /* 5s    -> 2s,     , Z >= Rb, interdite */
#define RAIE_L1_O2 41 /* 5p1/2 -> 2s, Lg4', Z >= In */
#define RAIE_L1_O3 42 /* 5p3/2 -> 2s, Lg4 , Z >= Sb */
#define RAIE_L1_O4 43 /* 5d3/2 -> 2s,     , Z >= Lu, interdite */
#define RAIE_L1_O5 44 /* 5d5/2 -> 2s,     , Z >= W , interdite */
#define RAIE_L1_O6 45 /* 5f5/2 -> 2s,     , Z >= Ac, interdite */
#define RAIE_L1_O7 46 /* 5f7/2 -> 2s,     , Z >=   , interdite */
#define RAIE_L1_P1 47 /* 6s    -> 2s,     , Z >= Cs, interdite */
#define RAIE_L1_P2 48 /* 6p1/2 -> 2s, Lg13, Z >= Tl */
#define RAIE_L1_P3 49 /* 6p3/2 -> 2s, Lg13, Z >= Bi */
#define RAIE_L1_Q1 50 /* 7s    -> 2s,     , Z >= Fr, interdite */

#define NOMBRE_RAIES_L1 25

/* -------- Raies L2 -------- */

#define RAIE_L2_L3 51 /* 2p3/2 -> 2p1/2,     , Z >= N , interdite */
#define RAIE_L2_M1 52 /* 3s    -> 2p1/2, Ln  , Z >= Na */
#define RAIE_L2_M2 53 /* 3p1/2 -> 2p1/2,     , Z >= Al, interdite */
#define RAIE_L2_M3 54 /* 3p3/2 -> 2p1/2, Lb17, Z >= P , interdite */
#define RAIE_L2_M4 55 /* 3d3/2 -> 2p1/2, Lb1 , Z >= Sc */
#define RAIE_L2_M5 56 /* 3d5/2 -> 2p1/2,     , Z >= Cr */
#define RAIE_L2_N1 57 /* 4s    -> 2p1/2, Lg5 , Z >= K  */
#define RAIE_L2_N2 58 /* 4p1/2 -> 2p1/2,     , Z >= Ga, interdite */
#define RAIE_L2_N3 59 /* 4p3/2 -> 2p1/2,     , Z >= As, interdite */
#define RAIE_L2_N4 60 /* 4d3/2 -> 2p1/2, Lg1 , Z >= Y  */
#define RAIE_L2_N5 61 /* 4d5/2 -> 2p1/2,     , Z >= Mo */
#define RAIE_L2_N6 62 /* 4f5/2 -> 2p1/2, Lv  , Z >= La, interdite */
#define RAIE_L2_N7 63 /* 4f7/2 -> 2p1/2,     , Z >=   , interdite */
#define RAIE_L2_O1 64 /* 5s    -> 2p1/2, Lg8 , Z >= Rb */
#define RAIE_L2_O2 65 /* 5p1/2 -> 2p1/2,     , Z >= In, interdite */
#define RAIE_L2_O3 66 /* 5p3/2 -> 2p1/2,     , Z >= Sb, interdite */
#define RAIE_L2_O4 67 /* 5d3/2 -> 2p1/2, Lg6 , Z >= Lu */
#define RAIE_L2_O5 68 /* 5d5/2 -> 2p1/2,     , Z >= W  */
#define RAIE_L2_O6 69 /* 5f5/2 -> 2p1/2,     , Z >= Ac, interdite */
#define RAIE_L2_O7 70 /* 5f7/2 -> 2p1/2,     , Z >=   , interdite */
#define RAIE_L2_P1 71 /* 6s    -> 2p1/2,     , Z >= Cs */
#define RAIE_L2_P2 72 /* 6p1/2 -> 2p1/2,     , Z >= Tl, interdite */
#define RAIE_L2_P3 73 /* 6p3/2 -> 2p1/2,     , Z >= Bi, interdite */
#define RAIE_L2_Q1 74 /* 7s    -> 2p1/2,     , Z >= Fr */

#define NOMBRE_RAIES_L2 24

/* -------- Raies L3 -------- */

#define RAIE_L3_M1 75 /* 3s    -> 2p3/2, Ll  , Z >= Na */
#define RAIE_L3_M2 76 /* 3p1/2 -> 2p3/2, Lt  , Z >= Al, interdite */
#define RAIE_L3_M3 77 /* 3p3/2 -> 2p3/2, Ls  , Z >= P , interdite */
#define RAIE_L3_M4 78 /* 3d3/2 -> 2p3/2, La2 , Z >= Sc */
#define RAIE_L3_M5 79 /* 3d5/2 -> 2p3/2, La1 , Z >= Cr */
#define RAIE_L3_N1 80 /* 4s    -> 2p3/2, Lb6 , Z >= K  */
#define RAIE_L3_N2 81 /* 4p1/2 -> 2p3/2,     , Z >= Ga, interdite */
#define RAIE_L3_N3 82 /* 4p3/2 -> 2p3/2,     , Z >= As, interdite */
#define RAIE_L3_N4 83 /* 4d3/2 -> 2p3/2, Lb15, Z >= Y  */
#define RAIE_L3_N5 84 /* 4d5/2 -> 2p3/2, Lb2 , Z >= Mo */
#define RAIE_L3_N6 85 /* 4f5/2 -> 2p3/2, Lu' , Z >= La, interdite */
#define RAIE_L3_N7 86 /* 4f7/2 -> 2p3/2,     , Z >=   , interdite */
#define RAIE_L3_O1 87 /* 5s    -> 2p3/2, Lb7 , Z >= Rb */
#define RAIE_L3_O2 88 /* 5p1/2 -> 2p3/2,     , Z >= In, interdite */
#define RAIE_L3_O3 89 /* 5p3/2 -> 2p3/2,     , Z >= Sb, interdite */
#define RAIE_L3_O4 90 /* 5d3/2 -> 2p3/2, Lb5 , Z >= Lu */
#define RAIE_L3_O5 91 /* 5d5/2 -> 2p3/2, Lb5 , Z >= W  */
#define RAIE_L3_O6 92 /* 5f5/2 -> 2p3/2,     , Z >= Ac, interdite */
#define RAIE_L3_O7 93 /* 5f7/2 -> 2p3/2,     , Z >=   , interdite */
#define RAIE_L3_P1 94 /* 6s    -> 2p3/2,     , Z >= Cs */
#define RAIE_L3_P2 95 /* 6p1/2 -> 2p3/2,     , Z >= Tl, interdite */
#define RAIE_L3_P3 96 /* 6p3/2 -> 2p3/2,     , Z >= Bi, interdite */
#define RAIE_L3_Q1 97 /* 7s    -> 2p3/2,     , Z >= Fr */

#define NOMBRE_RAIES_L3 23
#define NOMBRE_RAIES_L ( NOMBRE_RAIES_L1 + NOMBRE_RAIES_L2 + NOMBRE_RAIES_L3 )
#if (NOMBRE_RAIES_K + NOMBRE_RAIES_L) != ( 1 + RAIE_L3_Q1 )
# error Erreur dans le numrotage des raies de fluorescence / L
#endif

/* -------- Raies M1 -------- */

#define RAIE_M1_N1  98 /* 4s    -> 3s,     , Z >= K , interdite */
#define RAIE_M1_N2  99 /* 4p1/2 -> 3s,     , Z >= Ga */
#define RAIE_M1_N3 100 /* 4p3/2 -> 3s,     , Z >= As */
#define RAIE_M1_N4 101 /* 4d3/2 -> 3s,     , Z >= Y , interdite */
#define RAIE_M1_N5 102 /* 4d5/2 -> 3s,     , Z >= Mo, interdite */
#define RAIE_M1_N6 103 /* 4f5/2 -> 3s,     , Z >= La, interdite */
#define RAIE_M1_N7 104 /* 4f7/2 -> 3s,     , Z >=   , interdite */
#define RAIE_M1_O1 105 /* 5s    -> 3s,     , Z >= Rb, interdite */
#define RAIE_M1_O2 106 /* 5p1/2 -> 3s,     , Z >= In */
#define RAIE_M1_O3 107 /* 5p3/2 -> 3s,     , Z >= Sb */
#define RAIE_M1_O4 108 /* 5d3/2 -> 3s,     , Z >= Lu, interdite */
#define RAIE_M1_O5 109 /* 5d5/2 -> 3s,     , Z >= W , interdite */
#define RAIE_M1_O6 110 /* 5f5/2 -> 3s,     , Z >= Ac, interdite */
#define RAIE_M1_O7 111 /* 5f7/2 -> 3s,     , Z >=   , interdite */
#define RAIE_M1_P1 112 /* 6s    -> 3s,     , Z >= Cs, interdite */
#define RAIE_M1_P2 113 /* 6p1/2 -> 3s,     , Z >= Tl */
#define RAIE_M1_P3 114 /* 6p3/2 -> 3s,     , Z >= Bi */
#define RAIE_M1_Q1 115 /* 7s    -> 3s,     , Z >= Fr, interdite */

/* -------- Raies M2 -------- */

#define RAIE_M2_N1 116 /* 4s    -> 3p1/2,     , Z >= K  */
#define RAIE_M2_N2 117 /* 4p1/2 -> 3p1/2,     , Z >= Ga */
#define RAIE_M2_N3 118 /* 4p3/2 -> 3p1/2,     , Z >= As */
#define RAIE_M2_N4 119 /* 4d3/2 -> 3p1/2,     , Z >= Y  */
#define RAIE_M2_N5 120 /* 4d5/2 -> 3p1/2,     , Z >= Mo */
#define RAIE_M2_N6 121 /* 4f5/2 -> 3p1/2,     , Z >= La */
#define RAIE_M2_N7 122 /* 4f7/2 -> 3p1/2,     , Z >=    */
#define RAIE_M2_O1 123 /* 5s    -> 3p1/2,     , Z >= Rb */
#define RAIE_M2_O2 124 /* 5p1/2 -> 3p1/2,     , Z >= In */
#define RAIE_M2_O3 125 /* 5p3/2 -> 3p1/2,     , Z >= Sb */
#define RAIE_M2_O4 126 /* 5d3/2 -> 3p1/2,     , Z >= Lu */
#define RAIE_M2_O5 127 /* 5d5/2 -> 3p1/2,     , Z >= W  */
#define RAIE_M2_O6 128 /* 5f5/2 -> 3p1/2,     , Z >= Ac */
#define RAIE_M2_O7 129 /* 5f7/2 -> 3p1/2,     , Z >=    */
#define RAIE_M2_P1 130 /* 6s    -> 3p1/2,     , Z >= Cs */
#define RAIE_M2_P2 131 /* 6p1/2 -> 3p1/2,     , Z >= Tl */
#define RAIE_M2_P3 132 /* 6p3/2 -> 3p1/2,     , Z >= Bi */
#define RAIE_M2_Q1 133 /* 7s    -> 3p1/2,     , Z >= Fr */

/* -------- Raies M3 -------- */

#define RAIE_M3_N1 134 /* 4s    -> 3p3/2,     , Z >= K  */
#define RAIE_M3_N2 135 /* 4p1/2 -> 3p3/2,     , Z >= Ga */
#define RAIE_M3_N3 136 /* 4p3/2 -> 3p3/2,     , Z >= As */
#define RAIE_M3_N4 137 /* 4d3/2 -> 3p3/2, Mg  , Z >= Y  */
#define RAIE_M3_N5 138 /* 4d5/2 -> 3p3/2, Mg  , Z >= Mo */
#define RAIE_M3_N6 139 /* 4f5/2 -> 3p3/2,     , Z >= La */
#define RAIE_M3_N7 140 /* 4f7/2 -> 3p3/2,     , Z >=    */
#define RAIE_M3_O1 141 /* 5s    -> 3p3/2,     , Z >= Rb */
#define RAIE_M3_O2 142 /* 5p1/2 -> 3p3/2,     , Z >= In */
#define RAIE_M3_O3 143 /* 5p3/2 -> 3p3/2,     , Z >= Sb */
#define RAIE_M3_O4 144 /* 5d3/2 -> 3p3/2,     , Z >= Lu */
#define RAIE_M3_O5 145 /* 5d5/2 -> 3p3/2,     , Z >= W  */
#define RAIE_M3_O6 146 /* 5f5/2 -> 3p3/2,     , Z >= Ac */
#define RAIE_M3_O7 147 /* 5f7/2 -> 3p3/2,     , Z >=    */
#define RAIE_M3_P1 148 /* 6s    -> 3p3/2,     , Z >= Cs */
#define RAIE_M3_P2 149 /* 6p1/2 -> 3p3/2,     , Z >= Tl */
#define RAIE_M3_P3 150 /* 6p3/2 -> 3p3/2,     , Z >= Bi */
#define RAIE_M3_Q1 151 /* 7s    -> 3p3/2,     , Z >= Fr */

/* -------- Raies M4 -------- */

#define RAIE_M4_N1 152 /* 4s    -> 3d3/2,     , Z >= K  */
#define RAIE_M4_N2 153 /* 4p1/2 -> 3d3/2, Mx2 , Z >= Ga */
#define RAIE_M4_N3 154 /* 4p3/2 -> 3d3/2, Mx2 , Z >= As */
#define RAIE_M4_N4 155 /* 4d3/2 -> 3d3/2,     , Z >= Y  */
#define RAIE_M4_N5 156 /* 4d5/2 -> 3d3/2,     , Z >= Mo */
#define RAIE_M4_N6 157 /* 4f5/2 -> 3d3/2, Mb  , Z >= La */
#define RAIE_M4_N7 158 /* 4f7/2 -> 3d3/2,     , Z >=    */
#define RAIE_M4_O1 159 /* 5s    -> 3d3/2,     , Z >= Rb */
#define RAIE_M4_O2 160 /* 5p1/2 -> 3d3/2,     , Z >= In */
#define RAIE_M4_O3 161 /* 5p3/2 -> 3d3/2,     , Z >= Sb */
#define RAIE_M4_O4 162 /* 5d3/2 -> 3d3/2,     , Z >= Lu */
#define RAIE_M4_O5 163 /* 5d5/2 -> 3d3/2,     , Z >= W  */
#define RAIE_M4_O6 164 /* 5f5/2 -> 3d3/2,     , Z >= Ac */
#define RAIE_M4_O7 165 /* 5f7/2 -> 3d3/2,     , Z >=    */
#define RAIE_M4_P1 166 /* 6s    -> 3d3/2,     , Z >= Cs */
#define RAIE_M4_P2 167 /* 6p1/2 -> 3d3/2,     , Z >= Tl */
#define RAIE_M4_P3 168 /* 6p3/2 -> 3d3/2,     , Z >= Bi */
#define RAIE_M4_Q1 169 /* 7s    -> 3d3/2,     , Z >= Fr */

/* -------- Raies M5 -------- */

#define RAIE_M5_N1 170 /* 4s    -> 3d5/2,     , Z >= K  */
#define RAIE_M5_N2 171 /* 4p1/2 -> 3d5/2,     , Z >= Ga */
#define RAIE_M5_N3 172 /* 4p3/2 -> 3d5/2, Mx  , Z >= As */
#define RAIE_M5_N4 173 /* 4d3/2 -> 3d5/2,     , Z >= Y  */
#define RAIE_M5_N5 174 /* 4d5/2 -> 3d5/2,     , Z >= Mo */
#define RAIE_M5_N6 175 /* 4f5/2 -> 3d5/2, Ma2 , Z >= La */
#define RAIE_M5_N7 176 /* 4f7/2 -> 3d5/2, Ma1 , Z >=    */
#define RAIE_M5_O1 177 /* 5s    -> 3d5/2,     , Z >= Rb */
#define RAIE_M5_O2 178 /* 5p1/2 -> 3d5/2,     , Z >= In */
#define RAIE_M5_O3 179 /* 5p3/2 -> 3d5/2,     , Z >= Sb */
#define RAIE_M5_O4 180 /* 5d3/2 -> 3d5/2,     , Z >= Lu */
#define RAIE_M5_O5 181 /* 5d5/2 -> 3d5/2,     , Z >= W  */
#define RAIE_M5_O6 182 /* 5f5/2 -> 3d5/2,     , Z >= Ac */
#define RAIE_M5_O7 183 /* 5f7/2 -> 3d5/2,     , Z >=    */
#define RAIE_M5_P1 184 /* 6s    -> 3d5/2,     , Z >= Cs */
#define RAIE_M5_P2 185 /* 6p1/2 -> 3d5/2,     , Z >= Tl */
#define RAIE_M5_P3 186 /* 6p3/2 -> 3d5/2,     , Z >= Bi */
#define RAIE_M5_Q1 187 /* 7s    -> 3d5/2,     , Z >= Fr */

#define NOMBRE_RAIES_M 18
#if (NOMBRE_RAIES_K + NOMBRE_RAIES_L + 5 * NOMBRE_RAIES_M) != ( 1 + RAIE_M5_Q1 )
# error Erreur dans le numrotage des raies de fluorescence / M
#endif

/* -------- Raies N1 -------- */

#define RAIE_N1_O1 188 /* 5s    -> 4s,     , Z >= Rb */
#define RAIE_N1_O2 189 /* 5p1/2 -> 4s,     , Z >= In */
#define RAIE_N1_O3 190 /* 5p3/2 -> 4s,     , Z >= Sb */
#define RAIE_N1_O4 191 /* 5d3/2 -> 4s,     , Z >= Lu */
#define RAIE_N1_O5 192 /* 5d5/2 -> 4s,     , Z >= W  */
#define RAIE_N1_O6 193 /* 5f5/2 -> 4s,     , Z >= Ac */
#define RAIE_N1_O7 194 /* 5f7/2 -> 4s,     , Z >=    */
#define RAIE_N1_P1 195 /* 6s    -> 4s,     , Z >= Cs */
#define RAIE_N1_P2 196 /* 6p1/2 -> 4s,     , Z >= Tl */
#define RAIE_N1_P3 197 /* 6p3/2 -> 4s,     , Z >= Bi */
#define RAIE_N1_Q1 198 /* 7s    -> 4s,     , Z >= Fr */

/* -------- Raies N2 -------- */

#define RAIE_N2_O1 199 /* 5s    -> 4p1/2,     , Z >= Rb */
#define RAIE_N2_O2 200 /* 5p1/2 -> 4p1/2,     , Z >= In */
#define RAIE_N2_O3 201 /* 5p3/2 -> 4p1/2,     , Z >= Sb */
#define RAIE_N2_O4 202 /* 5d3/2 -> 4p1/2,     , Z >= Lu */
#define RAIE_N2_O5 203 /* 5d5/2 -> 4p1/2,     , Z >= W  */
#define RAIE_N2_O6 204 /* 5f5/2 -> 4p1/2,     , Z >= Ac */
#define RAIE_N2_O7 205 /* 5f7/2 -> 4p1/2,     , Z >=    */
#define RAIE_N2_P1 206 /* 6s    -> 4p1/2,     , Z >= Cs */
#define RAIE_N2_P2 207 /* 6p1/2 -> 4p1/2,     , Z >= Tl */
#define RAIE_N2_P3 208 /* 6p3/2 -> 4p1/2,     , Z >= Bi */
#define RAIE_N2_Q1 209 /* 7s    -> 4p1/2,     , Z >= Fr */

/* -------- Raies N3 -------- */

#define RAIE_N3_O1 210 /* 5s    -> 4p3/2,     , Z >= Rb */
#define RAIE_N3_O2 211 /* 5p1/2 -> 4p3/2,     , Z >= In */
#define RAIE_N3_O3 212 /* 5p3/2 -> 4p3/2,     , Z >= Sb */
#define RAIE_N3_O4 213 /* 5d3/2 -> 4p3/2,     , Z >= Lu */
#define RAIE_N3_O5 214 /* 5d5/2 -> 4p3/2,     , Z >= W  */
#define RAIE_N3_O6 215 /* 5f5/2 -> 4p3/2,     , Z >= Ac */
#define RAIE_N3_O7 216 /* 5f7/2 -> 4p3/2,     , Z >=    */
#define RAIE_N3_P1 217 /* 6s    -> 4p3/2,     , Z >= Cs */
#define RAIE_N3_P2 218 /* 6p1/2 -> 4p3/2,     , Z >= Tl */
#define RAIE_N3_P3 219 /* 6p3/2 -> 4p3/2,     , Z >= Bi */
#define RAIE_N3_Q1 220 /* 7s    -> 4p3/2,     , Z >= Fr */

/* -------- Raies N4 -------- */

#define RAIE_N4_O1 221 /* 5s    -> 4d3/2,     , Z >= Rb */
#define RAIE_N4_O2 222 /* 5p1/2 -> 4d3/2,     , Z >= In */
#define RAIE_N4_O3 223 /* 5p3/2 -> 4d3/2,     , Z >= Sb */
#define RAIE_N4_O4 224 /* 5d3/2 -> 4d3/2,     , Z >= Lu */
#define RAIE_N4_O5 225 /* 5d5/2 -> 4d3/2,     , Z >= W  */
#define RAIE_N4_O6 226 /* 5f5/2 -> 4d3/2,     , Z >= Ac */
#define RAIE_N4_O7 227 /* 5f7/2 -> 4d3/2,     , Z >=    */
#define RAIE_N4_P1 228 /* 6s    -> 4d3/2,     , Z >= Cs */
#define RAIE_N4_P2 229 /* 6p1/2 -> 4d3/2,     , Z >= Tl */
#define RAIE_N4_P3 230 /* 6p3/2 -> 4d3/2,     , Z >= Bi */
#define RAIE_N4_Q1 231 /* 7s    -> 4d3/2,     , Z >= Fr */

/* -------- Raies N5 -------- */

#define RAIE_N5_O1 232 /* 5s    -> 4d5/2,     , Z >= Rb */
#define RAIE_N5_O2 233 /* 5p1/2 -> 4d5/2,     , Z >= In */
#define RAIE_N5_O3 234 /* 5p3/2 -> 4d5/2,     , Z >= Sb */
#define RAIE_N5_O4 235 /* 5d3/2 -> 4d5/2,     , Z >= Lu */
#define RAIE_N5_O5 236 /* 5d5/2 -> 4d5/2,     , Z >= W  */
#define RAIE_N5_O6 237 /* 5f5/2 -> 4d5/2,     , Z >= Ac */
#define RAIE_N5_O7 238 /* 5f7/2 -> 4d5/2,     , Z >=    */
#define RAIE_N5_P1 239 /* 6s    -> 4d5/2,     , Z >= Cs */
#define RAIE_N5_P2 240 /* 6p1/2 -> 4d5/2,     , Z >= Tl */
#define RAIE_N5_P3 241 /* 6p3/2 -> 4d5/2,     , Z >= Bi */
#define RAIE_N5_Q1 242 /* 7s    -> 4d5/2,     , Z >= Fr */

/* -------- Raies N6 -------- */

#define RAIE_N6_O1 243 /* 5s    -> 4f5/2,     , Z >= Rb */
#define RAIE_N6_O2 244 /* 5p1/2 -> 4f5/2,     , Z >= In */
#define RAIE_N6_O3 245 /* 5p3/2 -> 4f5/2,     , Z >= Sb */
#define RAIE_N6_O4 246 /* 5d3/2 -> 4f5/2,     , Z >= Lu */
#define RAIE_N6_O5 247 /* 5d5/2 -> 4f5/2,     , Z >= W  */
#define RAIE_N6_O6 248 /* 5f5/2 -> 4f5/2,     , Z >= Ac */
#define RAIE_N6_O7 249 /* 5f7/2 -> 4f5/2,     , Z >=    */
#define RAIE_N6_P1 250 /* 6s    -> 4f5/2,     , Z >= Cs */
#define RAIE_N6_P2 251 /* 6p1/2 -> 4f5/2,     , Z >= Tl */
#define RAIE_N6_P3 252 /* 6p3/2 -> 4f5/2,     , Z >= Bi */
#define RAIE_N6_Q1 253 /* 7s    -> 4f5/2,     , Z >= Fr */

/* -------- Raies N7 -------- */

#define RAIE_N7_O1 254 /* 5s    -> 4d7/2,     , Z >= Rb */
#define RAIE_N7_O2 255 /* 5p1/2 -> 4d7/2,     , Z >= In */
#define RAIE_N7_O3 256 /* 5p3/2 -> 4d7/2,     , Z >= Sb */
#define RAIE_N7_O4 257 /* 5d3/2 -> 4d7/2,     , Z >= Lu */
#define RAIE_N7_O5 258 /* 5d5/2 -> 4d7/2,     , Z >= W  */
#define RAIE_N7_O6 259 /* 5f5/2 -> 4d7/2,     , Z >= Ac */
#define RAIE_N7_O7 260 /* 5f7/2 -> 4d7/2,     , Z >=    */
#define RAIE_N7_P1 261 /* 6s    -> 4d7/2,     , Z >= Cs */
#define RAIE_N7_P2 262 /* 6p1/2 -> 4d7/2,     , Z >= Tl */
#define RAIE_N7_P3 263 /* 6p3/2 -> 4d7/2,     , Z >= Bi */
#define RAIE_N7_Q1 264 /* 7s    -> 4d7/2,     , Z >= Fr */

#define NOMBRE_RAIES_N 11

#if (NOMBRE_RAIES_K + NOMBRE_RAIES_L + 5 * NOMBRE_RAIES_M + 7 * NOMBRE_RAIES_N) != ( 1 + RAIE_N7_Q1 )
# error Erreur dans le numrotage des raies de fluorescence / N
#endif

/* -------- Raies O1 -------- */

#define RAIE_O1_P1 265 /* 6s    -> 5s,     , Z >= Cs */
#define RAIE_O1_P2 266 /* 6p1/2 -> 5s,     , Z >= Tl */
#define RAIE_O1_P3 267 /* 6p3/2 -> 5s,     , Z >= Bi */
#define RAIE_O1_Q1 268 /* 7s    -> 5s,     , Z >= Fr */

/* -------- Raies O2 -------- */

#define RAIE_O2_P1 269 /* 6s    -> 5p1/2,     , Z >= Cs */
#define RAIE_O2_P2 270 /* 6p1/2 -> 5p1/2,     , Z >= Tl */
#define RAIE_O2_P3 271 /* 6p3/2 -> 5p1/2,     , Z >= Bi */
#define RAIE_O2_Q1 272 /* 7s    -> 5p1/2,     , Z >= Fr */

/* -------- Raies O3 -------- */

#define RAIE_O3_P1 273 /* 6s    -> 5p3/2,     , Z >= Cs */
#define RAIE_O3_P2 274 /* 6p1/2 -> 5p3/2,     , Z >= Tl */
#define RAIE_O3_P3 275 /* 6p3/2 -> 5p3/2,     , Z >= Bi */
#define RAIE_O3_Q1 276 /* 7s    -> 5p3/2,     , Z >= Fr */

/* -------- Raies O4 -------- */

#define RAIE_O4_P1 277 /* 6s    -> 5d3/2,     , Z >= Cs */
#define RAIE_O4_P2 278 /* 6p1/2 -> 5d3/2,     , Z >= Tl */
#define RAIE_O4_P3 279 /* 6p3/2 -> 5d3/2,     , Z >= Bi */
#define RAIE_O4_Q1 280 /* 7s    -> 5d3/2,     , Z >= Fr */

/* -------- Raies O5 -------- */

#define RAIE_O5_P1 281 /* 6s    -> 5d5/2,     , Z >= Cs */
#define RAIE_O5_P2 282 /* 6p1/2 -> 5d5/2,     , Z >= Tl */
#define RAIE_O5_P3 283 /* 6p3/2 -> 5d5/2,     , Z >= Bi */
#define RAIE_O5_Q1 284 /* 7s    -> 5d5/2,     , Z >= Fr */

/* -------- Raies O6 -------- */

#define RAIE_O6_P1 285 /* 6s    -> 5f5/2,     , Z >= Cs */
#define RAIE_O6_P2 286 /* 6p1/2 -> 5f5/2,     , Z >= Tl */
#define RAIE_O6_P3 287 /* 6p3/2 -> 5f5/2,     , Z >= Bi */
#define RAIE_O6_Q1 288 /* 7s    -> 5f5/2,     , Z >= Fr */

/* -------- Raies O7 -------- */

#define RAIE_O7_P1 289 /* 6s    -> 5d7/2,     , Z >= Cs */
#define RAIE_O7_P2 290 /* 6p1/2 -> 5d7/2,     , Z >= Tl */
#define RAIE_O7_P3 291 /* 6p3/2 -> 5d7/2,     , Z >= Bi */
#define RAIE_O7_Q1 292 /* 7s    -> 5d7/2,     , Z >= Fr */

#define NOMBRE_RAIES_O 4

#if (NOMBRE_RAIES_K + NOMBRE_RAIES_L + 5 * NOMBRE_RAIES_M + 7 * NOMBRE_RAIES_N + 7 * NOMBRE_RAIES_O) != ( 1 + RAIE_O7_Q1 )
# error Erreur dans le numrotage des raies de fluorescence / O
#endif

/* -------- Raies P1 -------- */

#define RAIE_P1_Q1 293 /* 7s    -> 6s,     , Z >= Fr */

/* -------- Raies P2 -------- */

#define RAIE_P2_Q1 294 /* 7s    -> 6p1/2,     , Z >= Fr */

/* -------- Raies P3 -------- */

#define RAIE_P3_Q1 295 /* 7s    -> 6p3/2,     , Z >= Fr */

#define NOMBRE_RAIES_P 1

#ifdef AVEC_TRANSURANIENS_ /* Raies qui ne peuvent exister que dans les transuraniens artificiels -> inutiles a priori ! */
# define RAIE_K_P4  296 /* 6d3/2 -> 1s   ,     , Z >= Lr , interdite */
# define RAIE_K_P5  297 /* 6d5/2 -> 1s   ,     , Z >= Sg , interdite */
# define RAIE_K_Q2  298 /* 7p1/2 -> 1s   ,     , Z >= 113 */
# define RAIE_K_Q3  299 /* 7p3/2 -> 1s   ,     , Z >= 115 */

# define RAIE_L1_P4 300 /* 6d3/2 -> 2s   ,     , Z >= Lr, interdite */
# define RAIE_L1_P5 301 /* 6d5/2 -> 2s   ,     , Z >= Sg, interdite */
# define RAIE_L1_Q2 302 /* 7p1/2 -> 2s   ,     , Z >= 113 */
# define RAIE_L1_Q3 303 /* 7p3/2 -> 2s   ,     , Z >= 115 */

# define RAIE_L2_P4 304 /* 6d3/2 -> 2p1/2,     , Z >= Lr */
# define RAIE_L2_P5 305 /* 6d5/2 -> 2p1/2,     , Z >= Sg */
# define RAIE_L2_Q2 306 /* 7p1/2 -> 2p1/2,     , Z >= 113, interdite */
# define RAIE_L2_Q3 307 /* 7p3/2 -> 2p1/2,     , Z >= 115, interdite */

# define RAIE_L3_P4 308 /* 6d3/2 -> 2p3/2,     , Z >= Lr */
# define RAIE_L3_P5 309 /* 6d5/2 -> 2p3/2,     , Z >= Sg */
# define RAIE_L3_Q2 306 /* 7p1/2 -> 2p3/2,     , Z >= 113, interdite */
# define RAIE_L3_Q3 307 /* 7p3/2 -> 2p3/2,     , Z >= 115, interdite */

# define RAIE_M1_P4 116 /* 6d3/2 -> 3s   ,     , Z >= Lr, interdite */

# define RAIE_M2_P4 133 /* 6d3/2 -> 3p1/2,     , Z >= Lr */

# define RAIE_M3_P4 151 /* 6d3/2 -> 3p3/2,     , Z >= Lr */

# define RAIE_M4_P4 173 /* 6d3/2 -> 3d3/2,     , Z >= Lr */

# define RAIE_M5_P4 187 /* 6d3/2 -> 3d5/2,     , Z >= Lr */

# define RAIE_N1_P4 204 /* 6d3/2 -> 4s   ,     , Z >= Lr */

# define RAIE_N2_P4 209 /* 6d3/2 -> 4p1/2,     , Z >= Lr */

# define RAIE_N3_P4 220 /* 6d3/2 -> 4p3/2,     , Z >= Lr */

# define RAIE_N4_P4 240 /* 6d3/2 -> 4d3/2,     , Z >= Lr */

# define RAIE_N5_P4 252 /* 6d3/2 -> 4d5/2,     , Z >= Lr */

# define RAIE_N6_P4 253 /* 6d3/2 -> 4f5/2,     , Z >= Lr */

# define RAIE_N7_P4 276 /* 6d3/2 -> 4d7/2,     , Z >= Lr */

# define RAIE_O1_P4 281 /* 6d3/2 -> 5s   ,     , Z >= Lr */

# define RAIE_O2_P4 286 /* 6d3/2 -> 5p1/2,     , Z >= Lr */

# define RAIE_O3_P4 291 /* 6d3/2 -> 5p3/2,     , Z >= Lr */

# define RAIE_O4_P4 296 /* 6d3/2 -> 5d3/2,     , Z >= Lr */

# define RAIE_O5_P4 301 /* 6d3/2 -> 5d5/2,     , Z >= Lr */

# define RAIE_O6_P4 306 /* 6d3/2 -> 5f5/2,     , Z >= Lr */

# define RAIE_O7_P4 311 /* 6d3/2 -> 5d7/2,     , Z >= Lr */

# define RAIE_P4_Q1 316 /* 7s    -> 6d3/2,     , Z >= Fr */
 #else
# define DERNIERE_RAIE_FLUORESCENCE RAIE_P3_Q1
#endif

/* -------- Total -------- */

#define NOMBRE_RAIES_FLUO ( NOMBRE_RAIES_K + NOMBRE_RAIES_L + 5 * NOMBRE_RAIES_M + 7 * NOMBRE_RAIES_N + 7 * NOMBRE_RAIES_O + 3 * NOMBRE_RAIES_P )
#if NOMBRE_RAIES_FLUO != ( 1 + DERNIERE_RAIE_FLUORESCENCE )
# error Erreur dans le numrotage des raies de fluorescence
#endif

int FLUO_InitialiserTables(void);

#define FLUO_MODELE_LORENTZ 0
#define FLUO_MODELE_GAUSS   1
#define FLUO_MODELE_DIRAC   2

double FLUO_CosterKronig  (int Z, int seuil_depart, int seuil_arrivee);
double FLUO_QuelleEnergie (int Z, int raie, double *rendement, double *largeur);
int    FLUO_RaiesExtremes (int Z, int couche_lacune, int *raie_1, int *raie_n);
int    FLUO_SimulerRaie   (struct graphe *graphe, double energie, double intensite, double largeur, int methode);
int    FLUO_SimulerSpectre(struct graphe *graphe, double energie, int Z, struct graphe *mu_seuil, double f_massique);

/* -------- Transitions CosterKronig -------- */

#define CK_L1L2  0
#define CK_L1L3  1
#define CK_L2L3  2

#define CK_M1M2  3
#define CK_M1M3  4
#define CK_M1M4  5
#define CK_M1M5  6
#define CK_M2M3  7
#define CK_M2M4  8
#define CK_M2M5  9
#define CK_M3M4 10
#define CK_M3M5 11
#define CK_M4M5 12

#define CK_N1N2 13
#define CK_N1N3 14
#define CK_N1N4 15
#define CK_N1N5 16
#define CK_N1N6 17
#define CK_N1N7 18
#define CK_N2N3 19
#define CK_N2N4 20
#define CK_N2N5 21
#define CK_N2N6 22
#define CK_N2N7 23
#define CK_N3N4 24
#define CK_N3N5 25
#define CK_N3N6 26
#define CK_N3N7 27
#define CK_N4N5 28
#define CK_N4N6 29
#define CK_N4N7 30
#define CK_N5N6 31
#define CK_N5N7 32
#define CK_N6N7 33

#define CK_O1O2 34
#define CK_O1O3 35
#define CK_O1O4 36
#define CK_O1O5 37
#define CK_O1O6 38
#define CK_O1O7 39
#define CK_O2O3 40
#define CK_O2O4 41
#define CK_O2O5 42
#define CK_O2O6 43
#define CK_O2O7 44
#define CK_O3O4 45
#define CK_O3O5 46
#define CK_O3O6 47
#define CK_O3O7 48
#define CK_O4O5 49
#define CK_O4O6 50
#define CK_O4O7 51
#define CK_O5O6 52
#define CK_O5O7 53
#define CK_O6O7 54

#define CK_P1P2 55
#define CK_P1P3 56
#define CK_P2P3 57

#define NOMBRE_TRANSITIONS_CK 55 /* transitions intra-P non prises en compte actuellement */

#endif /* FLUORESCENCE_H_ */
