/* LASE - Logiciel d'Analyse des Spectres E.X.A.F.S.
   (c) 1996 - 2002 - Emmanuel CURIS

   Dfinitions pour les graphes (entit fondamentale de LASE !)
*/

#ifndef GRAPHES_H_
# define GRAPHES_H_  /* Protection contre l'inclusion multiple */

# include "txt_utils.h" /* Pour les titres, on en aura srement besoin... */
# include "complexe.h"

/* --------- Le graphe lui-mme ------------------------------------- */

struct infos_fenetre; /* Dfinie proprement dans fourier.h */
struct nyquist;  /* Idem */
struct matrice;  /* Dfinie proprement dans matrices.h*/
struct serie;    /* Dfinie dans serie.h */

typedef struct infos_3d
{
  double z_min, z_max;           /* Pour avoir la bote entourante */
  double x_vu, y_vu, z_vu;       /* Le point vis */
  double x_oeil, y_oeil, z_oeil; /* Position de l'oeil */
  double x_h, y_h, z_h;          /* Vecteur dfinissant le haut */
  double angle_h, angle_v;       /* Ouverture de la vue */
} Infos_3d;

/* Mthode d'affichage : courbe 1D fonctionnelle */
#define GRF_R_CARTESIEN 0 /* (X,Y) dans le repre cartsien normal */
#define GRF_R_POLAIRE   1 /* (X,Y) dans un repre polaire [X = r, Y = thta] */

/* Mthode d'affichage : graphe complexe [valeurs choisies pour compatibilit avec les anciennes versions */
#define GRF_C_CHEMIN 0 /* Affichage : X=Re, Y=Im (chemin dans le plan) */
#define GRF_C_IM_MOD 1 /* Affichage : partie imaginaire, module */
#define GRF_C_RE_MOD 2 /* Affichage : partie relle, module */
#define GRF_C_RE_IM  3 /* Affichage : partie relle, partie imaginaire */

/* Mthode d'affichage : srie */
#define GRF_S_NORMAL    0 /* Affichage des valeurs directement, comme un graphe "classique" */
#define GRF_S_BOXPLOT   1 /* Affichage d'un diagramme rsum de type box-plot */
#define GRF_S_TUKEY     2 /* Affichage en diagramme de Tukey (type particulier de bote  moustaches) */

/* Mthode d'affichage : histogramme */
#define GRF_H_EFFECTIF  0 /* Affichage de l'histogramme des effectifs */
#define GRF_H_DP        1 /* Affichage de l'histogramme des densits de frquence */
#define GRF_H_FREQUENCE 2 /* Affichage de l'histogramme des frquences */

struct grf_infos_affichage {
  int methode; /* Suivant le type du graphe : polaire, cartsien,... */
  int couleur; /* Couleur de trac */
  short int style;   /* Style de ligne pour le trac */
  short int marque;  /* Marque  utiliser pour le trac */
  char *axe_x; /* Lgende de l'axe des X */
  char *axe_y; /* Lgende de l'axe des Y */
  char *unite_x, *unite_y;        /* Units utilises sur les axes [NULL = sans unit]  */
  union {
    Infos_3d *infos_3d;             /* Comment afficher la vue 3d ? - Graphes 3D uniquement */
    struct boite_moustache *infos_bmoustache; /* Comment afficher la bote  moustaches ? */
  } complement;
};

const char *GRF_TXT_TypeAffichage(int genre, int affichage);

#define GRF_FCT_REELLE    0 /* Table (X, Y = f(X)) ; cas le plus frquent */
			    /* x : contient les X --- y : contient les Y --- z : NULL en gneral */
#define GRF_FCT_COMPLEXE  1 /* Table (X, Z = (Re(f(X)), Im(f(X))) */
			    /* x : contient les X --- y : contient les Z (Complexe *) --- z : NULL en gnral */
#define GRF_SERIE         2 /* Table des Y, les X sont implicites */
			    /* x : NULL --- y : les valeurs (dans l'ordre) --- z : NULL */
                            /* Remarque : on peut avoir x != NULL, mais alors a t rserv ailleurs => ne sera pas libr */
                            /*            dans ce cas, se comporte pour les calculs comme un graphe de type GRF_FCT_REELLE */
#define GRF_COURBE_PLANE  3 /* Table (X, Y) quelconque : courbe paramtrique plane */
			    /* x : contient les X --- y : contient les Y --- z : NULL */
#define GRF_COURBE_ESPACE 4 /* Table (X, Y, Z) quelconque : courbe paramtrique dans l'espace */
#define GRF_SURFACE       5 /* Table (X, Y, Z=f(X,Y)) : surface simple dans l'espace */
#define GRF_HISTOGRAMME   6 /* Histogramme : X = limites suprieures de classe, Y = NULL, Z = effectifs [une valeur de plus !] */
#define GRF_FAISCEAU      7 /* Faisceau de graphes = famille de graphes */
#define GRF_MULTI_SERIE   8 /* Plusieurs sries statistiques, X implicites */
#define GRF_ERREUR_R      9 /* Incertitudes pour un graphe  valeurs relles : X = NULL, Y = Reel, Z = ddl [I] */
#define GRF_ERREUR_C     10 /* Incertitudes pour un graphe  valeurs complexes : X = NULL, Y = Complexe, Z = ddl [I] */

#define GR_NORMAL GRF_FCT_REELLE /* Pour compatibilit */

const char *GRF_TXT_Genre(int genre);

typedef struct graphe
{
  struct graphe *precedent;                 /* Le graphe prcdent en mmoire  */
  struct graphe *suivant;                   /* Le graphe suivant en mmoire    */
  int            genre_donnees;             /* Genre de donnes (nombres,...)  */
  int            type;                      /* Type du graphe                  */
  unsigned long  nombre_point;              /* Nombre de points dans le graphe */
  double        *x;                         /* Position des x en mmoire       */
  double        *y;                         /* Position des y en mmoire       */
  union {
    int            *ddl;                    /* Pour GR_DEVIATION : nombre de degrs de libert (= nbr_mesures - 1)
					       Remarque : si > 0, c'est qu'on a l'cart-type dans la population
					                  si < 0, c'est qu'on a l'cart-type sur la moyenne
					       On s'arrange pour TOUJOURS garder le lien avec le nombre de mesures original,
                                                  mme en cas de combinaison de variances, pour pouvoir toujours faire les calculs */
    double         *z;                      /* Pour GRF_COURBE_ESPACE & GRF_SURFACE : position des z en mmoire */
    struct graphe **graphes;                /* Pour GRF_FAISCEAU : graphes impliqus */
    unsigned long  *effectifs;	            /* Pour GRF_HISTOGRAMME : effectifs dans chaque classe */
  } z;
  double         x_min,x_max;               /* Bornes de l'intervalle en X     */
  double         y_min,y_max;               /* Bornes de l'intervalle en Y     */
  char           *nom;                      /* Nom du graphe */
  struct
  {
    /* Divers drapeaux donnant des infos sur le graphe */
    unsigned sauve     : 1;  /* VRAI -> le spectre a t sauv   */
    unsigned exafs     : 1;  /* VRAI -> spectre de type EXAFS    */
    unsigned protege   : 1;  /* VRAI -> graphe protg, ineffaable */
    unsigned isole     : 1;  /* VRAI -> graphe hors de toute srie */
    unsigned X_serie   : 1;  /* VRAI -> les X sont communs  toute sa srie */

    unsigned X_tries   : 1;  /* VRAI -> les X sont tris par ordre croissant */
    unsigned X_uniques : 1;  /* VRAI -> les X sont uniques : aucun doublon n'existe */
    unsigned theorique : 1;  /* VRAI -> le graphe rsulte d'un calcul thorique */
    unsigned normalise : 1;  /* VRAI -> le graphe a t normalis */

    /* Contrle de ce qu'il faut dessiner */
    unsigned superpose : 1;  /* VRAI s'il apparat dans la superposition */
    unsigned erreur_x  : 1;  /* VRAI s'il faut tracer les erreurs en X */
    unsigned erreur_y  : 1;  /* VRAI s'il faut tracer les erreurs en Y */
  } drapeaux;
  union                                     /* Pointeur sur des infos supplmentaires */
  {
    struct infos_fenetre *infos_tf;         /*   = infos sur le calcul de la TF              - GR_TF        uniquement */
    struct nyquist  *nyquist;               /*   = infos sur le calcul de la TFI             - GR_TFI, GR_EXAFS filtr */
    struct profil   *profil;                /*   = infos sur le profil de concentration      - GR_PROFIL    uniquement */
    void *autre;
  } pointeur;
  struct graphe *erreur_x;                  /* Graphe contenant l'erreur sur les X */
  struct graphe *erreur_y;                  /* Graphe contenant l'erreur sur les Y */
  struct matrice *covariance;               /* Matrice de covariance */
  struct graphe *grf_lie;                   /* Autre graphe li, au cas o [ex : graphe moyen d'un faisceau] */

  char *nom_dossier;		/* Nom du dossier ou se trouve le fichier du graphe */
  char *nom_fichier;		/* Nom du fichier qui contient le graphe [ou dont il derive] */

  struct grf_infos_affichage *affichage; /* NULL pour un graphe isol ! */
} Graphe;

/* --------- Les types de graphe spciaux reconnus ------------------------------- *
 *  types pour des slections plus souples dans le slecteur de graphe ( < -100)   *
 *  types spciaux  usage interne ( < 0)                                          *
 *  types courants >= 0                                                            *
 * ------------------------------------------------------------------------------- */

#define GR_TOUS     -100 /* On choisit tous les graphes */
#define GR_TT_EXAFS -101 /* On choisit tous les graphes pour lesquels exafs == VRAI */
#define GR_MARQUES  -102 /* On choisit les graphes marqus ("visibles") */

#define GR_INTERNE   -5 /* Graphe  usage interne -> utilisateur peut rien faire... */
#define GR_AUCUN     -3 /* Aucun graphe ne convient  ce que l'on veut faire        */
#define GR_AUTRETYPE -2 /* Graphe d'un type donn, diffrent des types courants     */
#define GR_SSTYPE    -1 /* Graphe sans type (type inconnu en fait)                  */

#define GR_BRUT           0 /* Graphe brut, soit A = f(E) exprimental */
#define GR_MOINSMAT       1 /* Seuil d'absorption du seul atome considr */
#define GR_DERIVEE        2 /* Drive premire */
#define GR_DERIVEE2       3 /* Drive seconde */
#define GR_EXAFS1         4 /* Spectre EXAFS en cours d'extraction */
#define GR_EXAFS2         5 /* Spectre EXAFS en cours d'extraction */
#define GR_EXAFS          6 /* Spectre EXAFS normalis */
#define GR_TF             7 /* Transforme de Fourier */
#define GR_TFI            8 /* Transforme de Fourier inverse */
#define GR_EXAFSAMP       9 /* Amplitude d'un signal EXAFS,  totale  */
#define GR_EXAFSPHA      10 /* Phase d'un signal EXAFS,  totale  */
#define GR_AJUSTE        11 /* Spectre ajust */
#define GR_MU0           12 /* Absorption intrinsque aprs seuil d'un atome isol */
#define GR_ECART         13 /* Diffrence entre une exprience et son modle */
#define GR_RDF           14 /* Densit de probabilit de prsence radiale d'un ligand */
#define GR_COMPREEL      15 /* Partie relle d'un spectre complexe */
#define GR_COMPIMA       16 /* Partie imaginaire d'un spectre complexe */
#define GR_COMPMOD       17 /* Module d'un spectre complexe */
#define GR_COMPARG       18 /* Argument d'un spectre complexe */
#define GR_I0            19 /* Intensit mesure avant l'chantillon */
#define GR_I1            20 /* Intensit mesure aprs l'chantillon */
#define GR_LIBREP        21 /* Libre parcours moyen = f(k) */
#define GR_FENETRE       22 /* Fentre pour la TF = fonction d'apodisation */
#define GR_DEVIATION     23 /* Incertitudes : cart-type ( dviation standard ) */
#define GR_FAC_RED       24 /* Facteur de rduction polylectronique, S0 */
#define GR_HISTOGRAMME   25
#define GR_DENSITE       26 /* Densit de probabilit */
#define GR_REPARTITION   27 /* Fonction de rpartition */
#define GR_EPREUVE       28 /* Valeurs alatoires, rptitions d'une preuve */
#define GR_MUMATRICE     29
#define GR_PROFIL        30 /* Profil de concentration : C = f(x), variation spatiale */
#define GR_RESOL_NRJ     31 /* Type Rocking-curve d'un monochromateur */
#define GR_FLUOX         32 /* Type Spectre de fluorescence complet : intensit = f(E) ou f(n canal) */
#define GR_CINETIQUE     33 /* Variation de concentration avec le temps : C = f(t) */
#define GR_CONCENTRATION 34 /* Variation de concentration, cas gnral : C = f(pH), par exemple */
#define GR_EXAFS3        35 /* EXAFS, 3e tape de l'extraction des oscillations, non normalis  */
#define GR_SEC_EFFICACE  36 /* Section efficace : diffusion, p. ex. */
#define GR_REP_DETECTEUR 37 /* Rponse d'un dtecteur  un signal */
#define GR_DEGRE_SIGNIFICATION 38 /* Degr de signification d'un test fonctionnel */

#define NOMBRE_TYPES_GRAPHES 39

const char *GRF_TexteTypeGraphe(int type);
inline boolean GRF_EstComplexe      (const Graphe *graphe);
inline boolean GRF_EstFonction      (const Graphe *graphe);
inline boolean GRF_EstFonctionReelle(const Graphe *graphe);
inline boolean GRF_EstReel          (const Graphe *graphe);
inline boolean GRF_EstSpectreEXAFS  (const Graphe *graphe, boolean strict);

inline boolean GRF_EstIsole  (const Graphe *graphe);
inline boolean GRF_EstProtege(const Graphe *graphe);

/* ~~~~~~~~~~~ Les variables globales ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#ifndef VARIABLES_GENERALES_ /* Elles sont toutes dclares dans variables.c
				voir ce fichier pour leur rle */
extern const char *extensions[];

extern boolean TT_exafs;

extern Graphe *graphe_courant; /* Graphe sur lequel on travaille */
extern Graphe *mu0_soustrait;  /* mu0 approch en cours d'laboration */
#endif /* VARIABLES_GENERALES_ */

/* -------- Crer un nouveau graphe -------- */

Graphe  *GRF_nouveau    (Graphe *avant, unsigned long nombre_points, int type, int mode, const char *nom);
Graphe  *GRF_nv_GrilleXN(Graphe *avant, int type, int mode, const char *nom, boolean isole,
			 double x_min, double x_max, unsigned long nbr_points);
Graphe  *GRF_nv_GrilleXP(Graphe *avant, int type, int mode, const char *nom, boolean isole,
			 double x_min, double x_max, double pas);
Graphe  *GRF_nv_TableX  (Graphe *avant, int type, int mode, const char *nom, boolean isole,
			 double *table_x, unsigned long n_x, boolean copier);
Graphe  *GRF_nv_isole(unsigned long nbr_pts, int type, int mode);

Graphe  *GRF_nv_DoubleListe (int type, const char *liste_valeurs_X, const char *liste_valeurs_Y);
Graphe  *GRF_nv_ListeValeurs(int type, const char *liste_valeurs);

int      GRF_CreerAffichage  (Graphe *graphe);
int      GRF_LibererAffichage(Graphe *graphe);
int      GRF_OptimiserAffichage(Graphe *graphe, double *x_min, double *x_max, double *y_min, double *y_max);

Graphe  *GRF_Copier           (const Graphe *source, struct serie *serie, boolean isole);
int      GRF_CopierAffichage  (Graphe *copie, const Graphe *source, int elements);
void    *GRF_CopierComplements(Graphe *original);

#define GRF_CP_AFF_LEGENDE_X (1U << 0) /* On copie la lgende de l'axe des X */
#define GRF_CP_AFF_LEGENDE_Y (1U << 1) /* On copie la lgende de l'axe des Y */
#define GRF_CP_AFF_UNITE_X   (1U << 2) /* On copie l'unit de l'axe des X */
#define GRF_CP_AFF_UNITE_Y   (1U << 3) /* On copie l'unit de l'axe des Y */
#define GRF_CP_AFF_COULEUR   (1U << 4) /* On copie la couleur de trac */
#define GRF_CP_AFF_STYLE     (1U << 5) /* On copie le style de trac */
#define GRF_CP_AFF_MARQUE    (1U << 6) /* On copie la marque de trac */

#define GRF_CP_AFF_AXE_X  (GRF_CP_AFF_LEGENDE_X|GRF_CP_AFF_UNITE_X)
#define GRF_CP_AFF_AXE_Y  (GRF_CP_AFF_LEGENDE_Y|GRF_CP_AFF_UNITE_Y)
#define GRF_CP_AFF_DESSIN (GRF_CP_AFF_COULEUR|GRF_CP_AFF_STYLE|GRF_CP_AFF_MARQUE)

#define GRF_CP_AFF_TOUT (GRF_CP_AFF_AXE_X|GRF_CP_AFF_AXE_Y|GRF_CP_AFF_DESSIN)

void     GRF_AfficherErreurs(Graphe *graphe, int axe, boolean afficher);
int      GRF_AssignerErreurs(Graphe *graphe, Graphe *erreurs, boolean x_corrects, int axe);
Graphe  *GRF_ChoisirErreurs (Graphe *graphe, int axe);
Graphe  *GRF_creer_erreurs  (Graphe *graphe, int axe);
int      GRF_InsererErreurs (Graphe *graphe, Graphe *erreurs, int axe);
Graphe  *GRF_SortirErreurs  (Graphe *graphe, int axe);
int      GRF_SupprimerErreurs(Graphe *graphe, int axe);

int      GRF_CreerCovariance(Graphe *graphe);

int      GRF_AssocierFichier(Graphe *graphe, const char *chemin, boolean avec_dossier );

/* -------- Chargement, sauvegarde -------- */

Graphe  *GRF_Charger(FILE *fichier, int version, const char *nom_fichier, struct serie *serie);
int      GRF_Sauver (FILE *fichier, Graphe *graphe);
int      GRF_SauverIsole(const char *nom_fichier, Graphe *graphe);

/* -------- Effacer un/des graphe(s) -------- */

boolean  GRF_tteffacer(void); /* Effacer tous les graphes en mmoire [quand on quitte, par exemple...] */
boolean  GRF_efface   (Graphe *graphe, boolean force_sauve, boolean force_protege);
boolean  GRF_eff_isole(Graphe *graphe);

/* -------- Adaptation, lien avec les autres graphes,... -------- */

int FEN_GRF_LierGraphe(Fenetre *fenetre, const Graphe *graphe);

int GRF_ChangerType (Graphe *graphe, int nouveau_type);
int GRF_ChangerGenre(Graphe *graphe, int nouveau_genre);

void GRF_PreparerListe        (boolean mode); /* Gestion de la liste des graphes en fentre */
void GRF_EnregistrerParametres( void );       /* Enregistrement des modifs de titre,...     */

boolean  GRF_AdapteTaille(Graphe *graphe, unsigned long nv_nbr_pts);
void     GRF_MiseAJour   (Graphe *graphe, boolean remettre_a_l_echelle);
void     GRF_aller (Graphe *graphe); /* Mettre graphe en graphe courant */

boolean       GRF_DelierGraphe(struct serie *serie, Graphe *graphe);
inline void   GRF_DeprotegerGraphe(Graphe *graphe);
inline void   GRF_ProtegerGraphe  (Graphe *graphe);
unsigned long GRF_utilisation(Graphe *graphe);

#define MSG_GRAPHE_MODIFIE -1 /* Le graphe source a t modifi -> adapter...
			         Le message est de la forme :
				 int code = ce code ;
				 int drapeau = VRAI si le nombre de points a t modifi
			         void *pointeur = adresse du nouveau graphe  utiliser */

void    GRF_modifier(Graphe *ancien_graphe, Graphe *nv_graphe, boolean npt_change);
boolean GRF_Informations(Graphe *graphe);

/* -------- Slection d'un ou plusieurs graphes dans une liste -------- */

boolean       GRF_type_existe (int type, boolean strict, struct serie *serie);
unsigned long GRF_compter_taille(unsigned long nombre_points, struct serie *serie);
unsigned long GRF_compter_type(int type, boolean strict, struct serie *serie);

#define OPTIONS_CHOIX_MULTIPLE   1 /* Bit mis : on autorise la slection multiple */
#define OPTIONS_CHOIX_MULTISERIE 2 /* Bit mis : on peut changer la srie */

Graphe  *GRF_choisir (int type, boolean strict, boolean multi_serie, struct serie *serie);
Graphe **GRF_mchoisir(int type, unsigned int min_choix, unsigned int max_choix, unsigned long *nbr_choisis);
Graphe **GRF_ChoisirDansTable(Graphe **table, unsigned long nbr_ds_table, unsigned int options, unsigned long *nbr_choisis);

Graphe **GRF_ChoisirType(struct serie *serie, int type, boolean strict, unsigned long *nombre);
Graphe **GRF_FiltreSerie(struct serie *serie, boolean *table, unsigned long *nombre);
Graphe **GRF_ChoisirTaille(struct serie *serie, unsigned long nombre_points, unsigned long *nombre);

#define GRF_CHOIX_GENRE    (1)	/* Bit mis : le genre doit tre le mme que celui du masque */
#define GRF_CHOIX_TYPE     (2)	/* Bit mis : le type  doit tre le mme que celui du masque */
#define GRF_CHOIX_NOMBRE   (4)	/* Bit mis : le graphe doit avoir autant de points que le masque */
#define GRF_CHOIX_UNITE_X  (8)	/* Bit mis : l'unit en X doit tre la mme */
#define GRF_CHOIX_X       (16)	/* Bit mis : on doit avoir le mme domaine en X [min, max ; valeurs si strict demand] */
#define GRF_CHOIX_UNITE_Y (32)	/* Bit mis : l'unit en Y doit tre la mme */
#define GRF_CHOIX_Y       (64)	/* Bit mis : on doit avoir le mme domaine en Y [min, max ; valeurs si strict demand] */

Graphe **GRF_ChoisirMasque(const struct serie *const serie, const Graphe *const modele, unsigned long reference, unsigned long strict, unsigned long *const nombre);

boolean GRF_ControlerUnite(Graphe *graphe, const char *unite_attendue, int axe, boolean convertir);

/* -------- Contrles sur des familles de graphes -------- */

boolean GRF_ComparerX    (const Graphe *grf_1, const Graphe *grf_2);

#define GRF_CONTROLE_N_STRICT   0 /* Exactement le mme nombre de points      */
#define GRF_CONTROLE_N_MINIMAL  1 /* On prend le plus petit nombre (question) */
#define GRF_CONTROLE_N_MAXIMAL  2 /* On prend le plus grand nombre (question) */
#define GRF_CONTROLE_N_SOMME    3 /* Somme de tous les nombres / fusion       */
#define GRF_CONTROLE_X_INTER   10 /* Intersection des intervalles en X */
#define GRF_CONTROLE_X_UNION   11 /* Union des intervalles en X        */

boolean GRF_ControleGenre (Graphe **graphes, unsigned long nombre, int *genre);
boolean GRF_ControleTaille(Graphe **graphes, unsigned long nombre, int correction, unsigned long *resultat, boolean message);
boolean GRF_ControleType  (Graphe **graphes, unsigned long nombre, boolean strict, int *type);
boolean GRF_ControleX     (Graphe **graphes, unsigned long nombre, int etendue, double *x_min, double *x_max);

#define GRF_CONTROLE_TYPE       1 /* Bit mis : on contrle que les graphes sont de mme type */
#define GRF_CONTROLE_GENRE      2 /* Bit mis : on contrle que les graphes sont de mme genre */
#define GRF_CONTROLE_LEGENDE_X  4 /* Bit mis : on contrle que les graphes ont la mme lgende en X */
#define GRF_CONTROLE_UNITE_X    8 /* Bit mis : on contrle que les graphes ont la mme unit en X */
#define GRF_CONTROLE_LEGENDE_Y 16 /* Bit mis : on contrle que les graphes ont la mme lgende en Y */
#define GRF_CONTROLE_UNITE_Y   32 /* Bit mis : on contrle que les graphes ont la mme unit en Y */
#define GRF_CONTROLE_NBR_PTS   64 /* Bit mis : on contrle que les graphes ont le mme nombre de points */

unsigned int GRF_ControlerCompatibilite(Graphe **graphes, unsigned long nombre, unsigned int masque, Graphe *modele);

/* -------- Modifier les parties  annexes  -------- */

#define AXE_X        1  /* Lgende pour l'axe des X */
#define AXE_Y        2  /* Lgende pour l'axe des Y */
#define DEUX_AXES    (AXE_X|AXE_Y)  /* Concerne les deux axes */
#define AXE_Z        4  /* Lgende pour l'axe des Z [images] */

void    GRF_couleur(Graphe *graphe, int couleur, short int style, short int marque);
void    GRF_CouleurSuivante(Graphe *graphe, Graphe *modele);
int     GRF_titre  (Graphe *graphe, const char *titre);               /* Met un titre    */

int GRF_ChangerLegende(Graphe *graphe, unsigned int axe, char *texte, boolean copier);
int GRF_ChangerUnite  (Graphe *graphe, unsigned int axe, const char *unite);

const char *GRF_QuelleLegende (const Graphe *graphe, unsigned int axe);
const char *GRF_QuelleUnite   (const Graphe *graphe, unsigned int axe);
const char *GRF_UniteParDefaut(int type, unsigned int axe);

/* -------- Travail simple sur le contenu -------- */

#define NE_PAS_FAIRE 0
#define ORDONNE      1 /* Valeurs pour en_x et en_y */
#define DESORDONNE   2

void GRF_bornes(Graphe *graphe, int en_x, int en_y); /* Bornes du graphe */
int  GRF_QuelsXPourY(Graphe *graphe, double y_min, double y_max, double *x_min, double *x_max);
int  GRF_QuelsYPourX(Graphe *graphe, double x_min, double x_max, double *y_min, double *y_max);

int  GRF_OterPoint(Graphe *graphe, unsigned long i);

unsigned long GRF_OterValeursManquantes(Graphe *graphe);

double plus_proche   (Graphe *graphe, double valeur, boolean mode);

unsigned long GRF_CompterPoints(const Graphe *graphe, int axe, double x_min, double x_max, boolean dehors);
unsigned long GRF_intervalle   (const Graphe *graphe, double x_min, double x_max, unsigned long *premier);
unsigned long GRF_extrait_table(const Graphe *graphe, double x_min, double x_max, boolean dehors, double **x, double **y);

Graphe *GRF_ExtraitGraphe (Graphe *graphe, unsigned long premier, unsigned long nombre, boolean isole);
Graphe *GRF_ExtraitGrapheI(Graphe *graphe, double x_min, double x_max, boolean dehors, boolean isole);

int GRF_PermuterXY(Graphe *graphe);

void GRF_FacteurAdditif      (Graphe *graphe, double valeur, double err_valeur, int axe);
void GRF_FacteurMultiplicatif(Graphe *graphe, double valeur, int axe, const char *unite);
void GRF_MettreConstante(Graphe *graphe, double valeur, unsigned int axe);

int  GRF_AjouterGraphe   (Graphe *reference, Graphe *soustrait, boolean traite_erreurs, double *covariances);
int  GRF_SoustraireGraphe(Graphe *reference, Graphe *soustrait, boolean traite_erreurs, double *covariances);

Graphe *GRF_ProduitGraphes (Graphe *a, Graphe *b, boolean en_place, boolean traite_erreurs, struct matrice *covariance);
Graphe *GRF_QuotientGraphes(Graphe *num, Graphe *den, boolean en_place, boolean traite_erreurs, struct matrice *covariance, int *code_erreur);

#define GRF_NORM_MIN_MAX   0
#define GRF_NORM_MIN_UTIL  1
#define GRF_NORM_UTIL_MAX  2
#define GRF_NORM_UTIL_UTIL 3

int GRF_Normaliser(Graphe *graphe, int methode, double val_0, double val_1);

int GRF_BruiterProportionnel(Graphe *graphe, unsigned int loi, double erreur, int axe);
int GRF_BruiterSimple       (Graphe *graphe, unsigned int loi, double erreur, int axe);

/* ----------- Tri et apparents ---------------------------------------------------------- */

Graphe *moyenner(void); /* Moyenne - mme chose              */

/* ------ Fonctions de base --------------- */

int GRF_GenerePolynome(Graphe *graphe, boolean ajouter, unsigned int degre, double *coefficients, struct matrice *covariance);

/* ------ Fonctions sur les graphes "chantillons" -------- */

int GRF_Serie_Parametres_Moyenne(Graphe *graphe);

/* ------ Codes d'erreur gnriques ------- */

#define ERR_GRF_DOUBLONS     ( -1) /* Le graphe contenait des doublons */
#define ERR_GRF_MEMOIRE      ( -2) /* Pas assez de mmoire */
#define ERR_GRF_NON_TRIE     ( -3) /* Le graphe n'est pas tri par X croissants */
#define ERR_GRF_DIV_ZERO     ( -4) /* Division par zro dans le calcul */
#define ERR_GRF_MATHS        ( -5) /* Erreur mathmatique, type sqrt(-1), log(0),... */
#define ERR_GRF_PROTEGE      ( -6) /* Graphe protg */
#define ERR_GRF_DISJOINTS    ( -7) /* Pas de rgion en X commune */
#define ERR_GRF_INCOMPATIBLE ( -8) /* Types de donnes incompatibles */
#define ERR_GRF_NON_REEL     ( -9) /* Donnes pas du type (x,y) */
#define ERR_GRF_NON_COMPLEXE (-10) /* Donnes pas du type (x,z) */
#define ERR_GRF_TAILLE_DIFF  (-11) /* Nombre de points diffrent */
#define ERR_GRF_NULL         (-12) /* Un des graphes n'existe pas */

/* ---------- Gestion des graphes ------------------------------- *
 *    Tout ce qui est traitement mathmatique                     *
 *     (Module GRF_MATH.C)                                        *
 ******************************************************************/

/* ------------ Oprations mathmatiques sur un seul graphe ------------------------------- */

#if defined(_TURBO_C_) || defined(_PURE_C_)
# include "filtrage.h" /* Turbo C et Pure C n'aiment pas du tout
			  quand on utilise struct A sans dfinir le contenu de A...
			  C'est pourtant autoris en C ANSI :-(  */
# include "images.h"  /* Pour les profils */
# include "matrices.h" /* Pour la matrice de covariance */
#endif

#endif /* GRAPHES_H_ */
