/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1998-2003, Emmanuel CURIS

   Dclarations ncessaires pour la gnration de spectres d'absorption
    partir de fichiers exprimentaux en tout genre...
*/

#ifndef IMPORT_XAS_H_
# define IMPORT_XAS_H_ /* Pas d'inclusion multiple */

#include "txt_utils.h"   /* Intervalles... */
#include "ascii_utils.h" /* Fonctions gnriques de lecture en ASCII */

struct graphe;   /* Dfinie dans graphes.h */
struct serie;    /* Dfinie dans serie.h   */

/* -------- Les lignes exprimentales -------- */

#define LIGNE_LURE_AQUVISU  0 /* Les lignes exprimentales */
#define LIGNE_LURE_D21      1
#define LIGNE_HASYLAB_DORIS 2
#define LIGNE_NSLS_MXANES   3
#define LIGNE_ESRF_SNBL     4	/* BM01b */
#define LIGNE_ESRF_BM05     5
#define LIGNE_ESRF_ID20     6
#define LIGNE_ESRF_ID21     7
#define LIGNE_ESRF_ID22     8
#define LIGNE_BESSY2_       9
#define LIGNE_SOLEIL_SAMBA 10

#define NOMBRE_LIGNES_EXPERIMENTALES 11

#define LIGNE_ERREUR    21275	/* Code absurde pour indiquer une erreur */

/* -------- Les modes de dtection -------- */

#define DETECT_TRANSMISSION 0
#define DETECT_FLUORESCENCE 1
#define DETECT_ELECTRONS    3

/* -------- Que faut-il garder comme information ? --- */

#define DETECT_GARDER_I0      1 /* Garder le faisceau incident */
#define DETECT_GARDER_TRANS   2 /* Garder le faisceau transmis */
#define DETECT_GARDER_FLUO    4 /* Garder le faisceau fluoresc */
#define DETECT_GARDER_EL      8 /* Garder le courrant d'lectrons */
#define DETECT_GARDER_SIGNAL 16 /* Garder les signaux calculs */

/* -------- Que faut-il faire aprs avoir charg un spectre ? -------- */

#define DETECT_POST_NORMALISER 1 /* On veut normaliser aprs chargement */
#define DETECT_POST_TRIER      2 /* On veut trier les X aprs chargement */
#define DETECT_POST_DOUBLONS   4 /* On veut traiter les doublons aprs chargement */
#define DETECT_POST_CORRECT    8 /* On veut raliser des corrections aprs chargement */
#define DETECT_POST_X_KEV     16 /* Les nergies sont en keV, convertir en eV */

struct infos_corrige_tps_mort {
  double constante;         /* Conversion valeur lue -> temps */
  double temps_mesure;      /* Temps de mesure total, si non indiqu dans une colonne */
  unsigned int col_tpsmort; /* Colonne qui indique le temps mort [0 : temps mort constant] */
  unsigned int col_temps;   /* Colonne qui indique le temps de mesure [0 : utiliser le temps donn] */
};

/* -------- Que faut-il faire s'il y a plusieurs spectres dans le fichier ? -------- */

#define DETECT_MULTI_SEPARER   0 /* Plusieurs spectres : on les garde distincts */
#define DETECT_MULTI_FONDRE    1 /* Plusieurs spectres : on les fusionne */
#define DETECT_MULTI_MOYENNER  2 /* Plusieurs spectres : on les moyenne [moyenne des I, puis calcul] */
#define DETECT_MULTI_MOY_LOG   3 /* Plusieurs spectres : on les moyenne [calcul, puis moyenne] */

typedef struct infos_import_xas {
  unsigned int proprietes;	/* Masque de bits ; cf. dfinitions ci-dessous */
  unsigned long nombre_lignes_en_tete;  /* Nombres de lignes  sauter au dbut, hors commentaires */
  int commentaire;		/* Caractre utilis comme marque de commentaire */
  int indication;		/* Caractre utilis comme marque d'indication (titre,...) */
  unsigned int nbr_colonnes; /* Nombre de colonnes au total dans le fichier */
  unsigned int col_x;        /* Colonne avec les donnes en X */
  unsigned int col_i0;       /* Colonne avec les donnes de I0 */
  Intervalle_I *colonnes_trans; /* Colonnes  garder, avec un signal de transmission */
  unsigned long nbr_I_trans;    /* Nombres d'intervalles ci-dessus */
  Intervalle_I *colonnes_fluo;  /* Mme chose, signal de fluorescence */
  unsigned long nbr_I_fluo;     /* Nombres d'intervalles ci-dessus */
  Intervalle_I *colonnes_det;  /* Mme chose, signal de dtection d'lectrons */
  unsigned long nbr_I_det;      /* Nombres d'intervalles ci-dessus */
  unsigned int que_garder;     /* Que faut-il conserver (cf. masques ci-dessus) */
  unsigned int post_traitement;/* Que faut-il faire aprs chargement ? */
  int MX_traitement; /* S'il y a des spectres successifs, que faire ? */
  int MD_traitement; /* S'il y a plusieurs dtecteurs de mme type, que faire ? */
  unsigned long nbr_successifs;	/* Combien de spectres successifs dans le fichier ? */
  Intervalle_I *filtre_MX; /* S'il y a plusieurs spectres successifs, lesquels charger ? */
  unsigned long nbr_I_MX;
  struct infos_corrige_tps_mort temps_mort; /* Correction du temps mort en fluorescence */
} Infos_Extraction;

#define DETECT_FICHIER_BINAIRE    1  /* Bit 1 : Mis si le format est binaire */
#define DETECT_I0_SEPARE          2  /* Bit 2 : Mis si le I0 est dans un autre fichier */
#define DETECT_MULTI_SPECTRE      4  /* Bit 3 : Mis si le fichier contient plusieurs spectres */
#define DETECT_CONCATENES         8  /* Bit 4 : Mis si les spectres se suivent sans sparation */
#define DETECT_DEMANDER_SPECTRES 16  /* Bit 5 : Mis s'il faut demander quels spectres garder */
#define DETECT_DEMANDER_COLONNES 32  /* Bit 6 : Mis s'il faut demander quels colonnes utiliser */

typedef struct {
  double *x;
  struct graphe *i_0;
  unsigned int nbr_spectres; /* Peut varier pas mal... */
  struct graphe **spectres;
  struct graphe **i_1;
} EXT_ListeGraphes;

/* -------- Les fonctions pour appeler tout a -------- */

#define CONTINUER_CHARGEMENT 1 /* Comportements possibles en cas de coupure de faisceau */
#define ARRETER_CHARGEMENT   2
#define ANNULER_CHARGEMENT   3

int  EXTRAIT_ChargerFichier(const char *nom, Infos_Extraction *infos, struct serie *serie, int type_ligne);
int  EXTRAIT_Generique     (const char *nom, FILE *fichier, Infos_Extraction *infos, EXT_ListeGraphes *graphes, int ligne,
			    unsigned int nbr_spectres, struct serie *serie, unsigned int num_spectre);

int EXTRAIT_AdapterSpectres(int type_ligne, Infos_Extraction *infos, EXT_ListeGraphes *graphes, const char *nom_fichier);
int EXTRAIT_DetruireGraphes(Infos_Extraction *infos, EXT_ListeGraphes *graphes);
int EXTRAIT_PreparerGraphes(Infos_Extraction *infos, unsigned int nbr_spectres, unsigned long nbr_points, const char *nom_fichier,
			     EXT_ListeGraphes *graphes, struct serie *serie, int style, unsigned int passage);
int EXTRAIT_TrouverTableX  (Infos_Extraction *infos, EXT_ListeGraphes *graphes);

long EXTRAIT_ChargerSpectres(FILE *fichier, unsigned long nbr_lignes, Infos_Extraction *infos,
			     EXT_ListeGraphes *graphes, const char *nom_fichier, unsigned int passage);

int EXTRAIT_AnalyseLigneSeparer(unsigned int nbr_valeurs, double *valeurs, Intervalle_I *filtre, unsigned long nbr_filtres,
				int *erreurs, int methode, double i0, unsigned long point,
				struct graphe **graphes, struct graphe **i1, const char *nom_fichier, Infos_Extraction *infos);
int EXTRAIT_AnalyseLigneSommer (unsigned int nbr_valeurs, double *valeurs, Intervalle_I *filtre, unsigned long nbr_filtres,
				int *erreurs, int methode, double i0, unsigned long point,
				struct graphe **graphes, struct graphe **i1, const char *nom_fichier, Infos_Extraction *infos);

double EXTRAIT_CorrigeTempsMort(double Imesuree, double constante, double tps_mesure, double tps_mort);

/* -------- Les fonctions spcifiques des lignes -------- */

#ifdef AVEC_HDF_
int BESSY2_LireSpectre(const char *nom, Infos_Extraction *infos, EXT_ListeGraphes *lst_graphes, struct serie *serie, unsigned int num_spectre);
#else
# define BESSY2_LireSpectre(nom, infos, lst_graphes, serie, num_spectre) (-1) /* Chargement impossible ! */
#endif

int NSLS_LireSpectre(const char *nom, FILE *fichier, Infos_Extraction *infos, EXT_ListeGraphes *lst_graphes, struct serie *serie);

unsigned long ID21_CompterSpectres( FILE *fichier, const char *commande );
int           ID21_TrouverDebut   ( FILE *fichier, const char *commande );

unsigned long ID22_CompterSpectres( FILE *fichier, const char *commande );
int           ID22_TrouverDebut   ( FILE *fichier, const char *commande );

unsigned long SNBL_CompterLignes  ( FILE *fichier, unsigned int col_X );
unsigned long SNBL_CompterSpectres( FILE *fichier );

#endif /* IMPORT_XAS_H_ */
