/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS - Date de cration : 22 mai 1998

   Tout ce qui concerne le X.A.N.E.S. - fichier  inclure
*/

#ifndef XANES_H_

#define XANES_H_ /* Protection contre l'inclusion multiple */

struct automatiser;		/* Dfinie dans Include/automatise.h */

/* ------- Les mthodes de soustraction de mu_matrice ------- */

#ifdef _DEVELOPPE_
# define NBR_MUM_METHODES 8
#else
# define NBR_MUM_METHODES 5
#endif

#define MUM_CONSTANTE   0
#define MUM_DROITE      1
#define MUM_VICTORINE_1 2
#define MUM_VICTORINE_2 3
#define MUM_POLYNOME    4
#define MUM_GAUSSIENNE  5
#define MUM_THEORIQUE   6
#define MUM_VICTORINE_3 7

#define UN_POINT    0 /* On modlise entre graphe->x_min et e1 */
#define DEUX_POINTS 1 /* On modlise entre e1p et e1 */

#define MUM_DEGRE_MAXIMAL 5 /* du polynme pour la soustraction
                               au moins 4, pour coller avec victorine 2 */

typedef struct
{
  unsigned int methode;	       /* Type de fond continu  soustraire */
  int mode;	      /* UN_POINT ou DEUX_POINTS                    */
  double e_1;	      /* Valeur de E1                               */
  double e_1p;	      /* Valeur de E1 prime, si deux points         */
  double *coefficients;	       /* Valeur des coefficients du modle */
  short int degre;    /* Degr du polynme (1 ... 5)                */
  Graphe *fond_continu;	/* Modle calcul pour (matrice) [grf. isol]*/
  Graphe *mu_calc; /* Graphe contenant un mu thorique (Lansberger) */
  Graphe *reference; /* Rfrence que l'on va essayer de coller au mieux */
} MAT_infos;

struct graphe *XAN_SoustraireMatrice(struct graphe *brut, MAT_infos *infos, boolean recalcule);

int    XAN_MUM_Automatisation  (struct automatiser *lot);
int    XAN_MUM_CalculerFond    (struct graphe *brut, MAT_infos *infos);
double XAN_MUM_CalculerResidu  (const Graphe *brut, const Graphe *resultat, const MAT_infos *infos, unsigned long *ddl);
int    XAN_MUM_PostSoustraction(struct graphe *brut, struct graphe *resultat, MAT_infos *infos);
double XAN_MUM_ProposerE1      (struct graphe *brut);
int    XAN_MUM_SoustraireFond  (struct graphe *brut, struct graphe *resultat, MAT_infos *infos);

void    MAT_soustrait (Graphe *brut, Graphe *traite, MAT_infos *infos);

/* ---------- Normalisation en modlisant [mu]0 ------------------\
 |   Utilise donc la structure MU1_infos (param. identiques)      |
 \--------------------------------------------------------------- */

void    norm_xanes(Graphe *signal);

#include "extrait.h" /* Pour avoir MU1_infos */

#define XNORM_MU0     0
#define XNORM_1_PT    1
#define XNORM_2_PTS   2
#define XNORM_MIN_MAX 3

Graphe *xanes_normalise(Graphe *signal, MU1_infos *infos, int mode, boolean nouveau);

/* ---------- Intgration du prseuil --------------------------- */

boolean XAN_integre(Graphe *signal);

#endif /* XANES_H_ */
