/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2006, Emmanuel CURIS
   Cration : mars 2006

   Interface graphique avec l'utilisateur

   Intgration (graphe : rel, complexe)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "automatise.h"

#include "atomes.h"
#include "graphes.h"
#include "images.h"
#include "interface.h"

/* -------- Variables locales -------- */

#define I_AUTO_LGMAX_NOM_REFERENCE   40
#define I_AUTO_LGMAX_TEXTE_SELECTEUR 43

#include "int_automatise.txt"

/* -------- Fonctions locales -------- */

static int I_AUTO_PreparerArbre(struct automatiser *infos, unsigned int options);

static const char *I_AUTO_ObjetVersTexte(unsigned int arbre, unsigned long numero_objet);

/* -------- Ralisation pratique -------- */

int I_AUTOMATISE_SurListeConnue(struct automatiser *lot, unsigned int options)
{
  int retour;

  /* 0. Prparer la bote d'automatisation [teste aussi que lot != NULL] */
  retour = I_AUTO_PreparerArbre( lot, options );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme lors de la prparation !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }

  /* S'il y a une rfrence, on la met */
  if ( lot->reference.graphe != NULL ) {
    const char *texte_reference = NULL;

    switch ( lot->type_objet )
    {
    case AUTOMATISE_GRAPHE:
      texte_reference = lot->reference.graphe->nom;
      break;

    case AUTOMATISE_IMAGE:
      texte_reference = lot->reference.image->nom;
      break;

    case AUTOMATISE_MOLECULE:
      OPTION_A_FAIRE( "Mettre le nom d'une molcule" );
      break;
    }

    OBJ_CopierTexte( TRAITE_LOT, TL_GRF_REFERENCE, texte_reference, I_AUTO_LGMAX_NOM_REFERENCE );
  }

  /* 1. Afficher la bote d'automatisation */
  retour = ARB_Bloquant( TRAITE_LOT, "Automatiser le traitement", principale, 0 );
  ARB_MettreComplement( TRAITE_LOT, NULL ); /* Plus besoin des infos ! */
  if ( TL_ANNULER == retour ) {
    return +1; /* Annulation : pas d'erreur, juste changement d'avis de l'utilisateur */
  }

  /* 2. Mmoriser ce qui a pu changer */
  /* a. La mthode de calcul */
  lot->methode = OBJ_OptionMenu( TRAITE_LOT, TL_METHODE, 0, QUESTION );

  /* b. Srie des rsultats */
  if ( OBJ_choisit( TRAITE_LOT, TL_NV_SERIE, QUESTION ) ) {
    lot->nom_serie = lit_chaine( TRAITE_LOT, TL_NOM_NV_SERIE, 0, NULL, TXT_COPIER );
  } else {
    lot->nom_serie = NULL;
  }

  /* 3. Raliser l'automatisation */
  retour = AUTO_EffectuerCalculs( lot );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Problme dans la rptition des calculs ! [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -10;
  }

  /* 4. Termin : on renvoie les informations de l'automatisation (donc la liste des rsultats !) pour utilisation... */
  return 0;
}

/* ------ Prparation de l'arbre ------ */

static int I_AUTO_PreparerArbre(struct automatiser *lot, unsigned int options)
{
  if ( NULL == lot ) return -1;

  /* Si ncessaire, on initialise l'arbre */
  if ( FAUX == arbres[ TRAITE_LOT ].drapeaux.prepare ) {
    int retour;

    /* Initialisation du menu des mthodes de calcul (sans routine spciale) */
    OBJ_ChangerType( TRAITE_LOT, TL_METHODE, G_SSMENU );
    OBJ_MettreMenu ( TRAITE_LOT, TL_METHODE, AUTOMATISE_NOMBRE_METHODES, AUTO_menu_choix_methode, titre_methodes, NULL );

    retour = ARB_Initialiser( TRAITE_LOT, NULL, NULL, NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : Problme  l'initialisation [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return -1;
    }

    /* Dplacements esthtiques des objets */
    ARB_EnjoliverDefileur( TRAITE_LOT, TL_FND_GRAPHES, TL_PREMIER_GRAPH, TL_DERNIER_GRAPH,
			   TL_ASC_AVANT, TL_FOND_ASC, TL_ASC_BOUTON, TL_ASC_APRES, ARB_ASCENSEUR_DROITE );

    /* Protection pour ne pas le faire plusieurs fois ! */
    arbres[ TRAITE_LOT ].drapeaux.prepare = VRAI;
  }

  /* On s'assure que la mthode propose est la bonne ! */
  if ( lot->methode >= AUTOMATISE_NOMBRE_METHODES ) {
    lot->methode = AUTOMATISE_METHODE_ROBOT;  /* Mthode par dfaut ! */
  }
  OBJ_OptionMenu( TRAITE_LOT, TL_METHODE, lot->methode, METTRE );

  /* On associe les infos  l'arbre */
  ARB_MettreComplement( TRAITE_LOT, lot );

  /* On prpare le dfileur */
  ARB_PreparerDefileur_O( TRAITE_LOT, TL_PREMIER_GRAPH, TL_DERNIER_GRAPH, I_AUTO_LGMAX_TEXTE_SELECTEUR,
			  TL_ASC_AVANT, TL_FOND_ASC, TL_ASC_BOUTON, TL_ASC_APRES,
			  lot->nombre_objets, NULL, 0, 0, I_AUTO_ObjetVersTexte );

  /* On installe ce qui dpend du type de l'objet (textes et fonctions de gestion) */

  switch ( lot->type_objet )
  {
  case AUTOMATISE_GRAPHE:
    break;

  case AUTOMATISE_IMAGE:
    break;

  case AUTOMATISE_MOLECULE:
    break;

  default:
    (void) fprintf(stderr, "%s, %s l.%u : Type d'objet non automatis [%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, lot->type_objet);
    return -5;
  }

  /* En fonction des options : ce qui est dsactiv... */
  if ( options & I_AUTO_AJOUT_POSSIBLE ) {
    OBJ_active( TRAITE_LOT, TL_AJOUTER_GRF, FORCE_OUI );
    OBJ_active( TRAITE_LOT, TL_OTER_GRF   , FORCE_OUI );
    OBJ_active( TRAITE_LOT, TL_TT_OTER    , FORCE_OUI );

    OBJ_active( TRAITE_LOT, TL_FILTRE      , FORCE_OUI );
    if ( options & I_AUTO_MULTI_SERIE_POSSIBLE ) {
      OBJ_active( TRAITE_LOT, TL_FILTRE_SERIE, FORCE_OUI );
    } else {
      OBJ_active( TRAITE_LOT, TL_FILTRE_SERIE, FORCE_NON );
    }
  } else {
    OBJ_active( TRAITE_LOT, TL_AJOUTER_GRF, FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_OTER_GRF   , FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_TT_OTER    , FORCE_NON );

    OBJ_active( TRAITE_LOT, TL_FILTRE      , FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_FILTRE_SERIE, FORCE_NON );
  }

  if ( options & I_AUTO_CHOIX_FONCTION_POSSIBLE ) {
    OBJ_active( TRAITE_LOT, TL_OP_GENERIQUE , FORCE_OUI );
    OBJ_active( TRAITE_LOT, TL_CHOIX_GENERIK, FORCE_OUI );

    OBJ_active( TRAITE_LOT, TL_GENRE    , FORCE_OUI );
    OBJ_active( TRAITE_LOT, TL_OPT_GENRE, FORCE_OUI );

    OBJ_active( TRAITE_LOT, TL_TYPE    , FORCE_OUI );
    OBJ_active( TRAITE_LOT, TL_OPT_TYPE, FORCE_OUI );
  } else {
    OBJ_active( TRAITE_LOT, TL_OP_GENERIQUE , FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_CHOIX_GENERIK, FORCE_NON );

    OBJ_active( TRAITE_LOT, TL_GENRE    , FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_OPT_GENRE, FORCE_NON );

    OBJ_active( TRAITE_LOT, TL_TYPE    , FORCE_NON );
    OBJ_active( TRAITE_LOT, TL_OPT_TYPE, FORCE_NON );
  }

  if ( options & I_AUTO_REFERENCE_POSSIBLE ) {
    OBJ_active( TRAITE_LOT, TL_GRF_REFERENCE, FORCE_OUI );
  } else {
    OBJ_active( TRAITE_LOT, TL_GRF_REFERENCE, FORCE_NON );
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Affichages dans l'arbre -------- */

static const char *I_AUTO_ObjetVersTexte(unsigned int arbre, unsigned long numero_objet)
{
  struct automatiser *lot;
  const char *nom_objet = NULL;

  /* On rcupre les informations associes  l'arbre */
  lot = ARB_LireComplement( arbre );
  if ( NULL == lot ) return NULL;

  /* On vrifie que l'on est bien dans la liste */
  if ( numero_objet >= lot->nombre_objets ) return "Au-del !!";

  /* On renvoie le nom du graphe */
  switch ( lot->type_objet )
  {
  case AUTOMATISE_GRAPHE:
    {
      Graphe *graphe;

      graphe = lot->objets[ numero_objet ].graphe;
      if ( graphe != NULL ) {
	nom_objet = graphe->nom;
      }
    }
    break;

  case AUTOMATISE_IMAGE:
    {
      Image *image;

      image = lot->objets[ numero_objet ].image;
      if ( image != NULL ) {
	nom_objet = image->nom;
      }
    }
    break;

  case AUTOMATISE_MOLECULE:
    {
      Molecule *molecule;

      molecule = lot->objets[ numero_objet ].molecule;
      if ( molecule != NULL ) {
	nom_objet = molecule->nom;
      }
    }
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Type d'objet inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, lot->type_objet );
    break;
  }

  return nom_objet;
}
