/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) Emmanuel CURIS 1996 - 2011

   Interface pour la gestion de base des graphes
   (Affichages de valeurs, proprietes...)
*/

#include <stdio.h>
#include <stdlib.h>

#include "exafs.h"
#include "systeme.h"
#include "txt_utils.h"

#include "interface.h"
#include "serie.h"

#include "int_graphes_affichage.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* Afficher les valeurs limites d'un graphe --- en fonction de son genre */
static int I_GRF_AfficherLimites_R(Graphe *graphe);
/* static int I_GRF_AfficherLimites_C(Graphe *graphe); */
/* static int I_GRF_AfficherLimites_S(Graphe *graphe); */

/* ~~~~~~~~ Realisation pratique ~~~~~~~~ */

/* ~-~-~-~-~-~-~-~-~ Affichages de proprietes et valeurs ~-~-~-~-~-~-~-~-~ */

/* -------- Fonction d'appel generique -------- */

int I_GRF_AfficherLimites(Graphe *graphe)
{
  int retour = 0;

  if ( NULL == graphe ) return -1;

  /* On appelle la bonne fonction, en fonction du genre du graphe */
  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
    retour = I_GRF_AfficherLimites_R( graphe );
    break;

  default:
    AVERTISSEMENT( "Genre de graphe pas encore gere : %u", graphe->genre_donnees )
  }

  /* On renvoie le code de sortie... */
  return retour;
}

/* -------- Cas d'un graphe a valeurs reelles -------- */

static int I_GRF_AfficherLimites_R(Graphe *graphe)
{
  unsigned long x_min = 0, y_min = 0, x_max = 0, y_max = 0;
  unsigned long i, i_x_min = 0, i_x_max = 0, i_y_min = 0, i_y_max = 0;
  double *x, *y;

  HIST_Titre( 4, "Recherche des valeurs extr"e_CIRC"mes", "GRF" );
  HIST_Texte( 4, FAUX, "Graphe : %s\n", graphe->nom );

  for ( i = 0, x = graphe->x, y = graphe->y ;
	i < graphe->nombre_point ;
	i++, x++, y++ ) {
    if ( *x <= graphe->x_min ) {
      HIST_Texte( 4, FAUX, "X minimal [%g] : valeur no %lu, Y = %g\n", *x, i, *y );
      x_min++;
      i_x_min = i;
    }
    if ( *x >= graphe->x_max ) {
      HIST_Texte( 4, FAUX, "X maximal [%g] : valeur no %lu, Y = %g\n", *x, i, *y );
      x_max++;
      i_x_max = i;
    }

    if ( *y <= graphe->y_min ) {
      HIST_Texte( 4, FAUX, "Y minimal [%g] : valeur no %lu, en X = %g\n", *y, i, *x );
      y_min++;
      i_y_min = i;
    }
    if ( *y >= graphe->y_max ) {
      HIST_Texte( 4, FAUX, "Y maximal [%g] : valeur no %lu, en X = %g\n", *y, i, *x );
      y_max++;
      i_y_max = i;
    }
  }

  /* Plus qu'a faire l'affichage */
  (void) BI_ERR_alerte( principale, &MSG_LimitesGrapheReel, VRAI,
			graphe->x_min, GRF_QuelleUnite( graphe, AXE_X ), i_x_min, graphe->y[ i_x_min ], GRF_QuelleUnite( graphe, AXE_Y ), ( x_min > 1 ) ? '*' : 0,
			graphe->x_max, GRF_QuelleUnite( graphe, AXE_X ), i_x_max, graphe->y[ i_x_max ], GRF_QuelleUnite( graphe, AXE_Y ), ( x_max > 1 ) ? '*' : 0,
			graphe->y_min, GRF_QuelleUnite( graphe, AXE_Y ), i_y_min, graphe->x[ i_y_min ], GRF_QuelleUnite( graphe, AXE_X ), ( y_min > 1 ) ? '*' : 0,
			graphe->y_max, GRF_QuelleUnite( graphe, AXE_Y ), i_y_max, graphe->x[ i_y_max ], GRF_QuelleUnite( graphe, AXE_X ), ( y_max > 1 ) ? '*' : 0 );

  /* On a fini */
  return 0;
}
