/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 2003, Emmanuel CURIS
   Création de ce fichier : 28 août 2003

   Interface pour l'import / export de fichiers généraux au format ASCII
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"

#include "fichiers.h"
#include "interface.h"
#include "serie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_TYPE_IMPORT   42 /* Longueur maximale utilisable pour afficher le type de graphe */
#define LGMAX_FORMAT_IMPORT 20 /* Longueur maximale du nom de format prédéfini */

static int type_import = GR_SSTYPE; /* Quel type donner aux spectres importés ? */
static int format_import = ASCII_FORMAT_UNIVERSEL; /* Par défaut, import au format LASE ! */

/* static char *chemin_import = NULL; */ /* Dossier pour l'import ASCII */
static char *chemin_export = NULL; /* Dossier pour l'export ASCII */

/* Chemins par défaut : définies dans variables.c */
extern unsigned int nombre_chemins_chargement; extern char **chemins_chargement;
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

#include "int_impexp.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int IA_LireInfos(struct infos_import_grf *infos);

static const char *IA_Extension(int format);

extern boolean import_asc_3d(const char *nom, int nbr_colonnes, int x_colonne,
			     int y_colonne, int z_colonne,
			     unsigned int en_tete, char commentaire);

static int GER_ExportASCII(unsigned int objet, int x, int y);
static int GER_ImportASCII(unsigned int objet, int x, int y);

static void IEA_PreparerArbre(void);
static void IEA_LireNumerosColonnes(unsigned int *nombre, unsigned int *nc_X, unsigned int *nc_Re, unsigned int *nc_Im, unsigned int *nc_Z,
				    unsigned int *nc_sX, unsigned int *nc_sRe, unsigned int *nc_sIm);

static void IA_ActiverMultiGraphe(int sens);

static void EA_BaseFormat(unsigned int nbr_colonnes, unsigned int col_X, int separateur, int commentaire);
static void IA_BaseFormat(unsigned int nbr_colonnes, unsigned int col_X, int separateur, int commentaire, boolean multi);
static void EA_InstallerFormat(unsigned int arbre, unsigned int objet, unsigned long format);
static void IA_InstallerFormat(unsigned int arbre, unsigned int objet, unsigned long format);

inline static void EA_InitArbre(Graphe *graphe);
inline static void IA_InitArbre(void);

#define IA_ActiveObjet( objet )     { OBJ_choisit( IMPORT_ASCII, objet, FORCE_OUI ); OBJ_active( IMPORT_ASCII, objet, FORCE_OUI ); }
#define IA_DesactiveObjet( objet ) { OBJ_choisit( IMPORT_ASCII, objet, FORCE_NON ); OBJ_active( IMPORT_ASCII, objet, FORCE_NON ); }

/* ~-~-~-~-~-~-~-~-~ Interface graphique : Import ~-~-~-~-~-~-~-~-~ */

/* -------- Paramétrage de l'import en ASCII et sélection des fichiers -------- */

void GRF_ImportASCII(void)
{
  char **liste_noms;
  int retour;
  unsigned int i, nbr_fichiers;
  boolean sans_X __attribute__((__unused__)); /* VRAI si on a des valeurs des erreurs à charger / pas de X */
  boolean nlle_serie = FAUX;
  struct infos_import_grf infos_lecture;
  Serie_Graphes *serie_import = NULL;

  /* On initialise les boutons dans le sens d'un import */
  IEA_PreparerArbre();
  IA_InitArbre();
  IA_InstallerFormat( IMPORT_ASCII, IA_FORMAT, format_import );

  /* On appelle la boîte de dialogue */
  retour = ARB_Bloquant( IMPORT_ASCII, IA_titre_arbre, principale, 0 );
  if ( IA_ANNULER == retour ) /* On annule */
    return;

  /* On lit les informations */
  retour = IA_LireInfos( &infos_lecture );

  /* On demande la liste des fichiers */
  liste_noms = CH_selecteur( IA_Extension( format_import ), IA_titre_selecteur, 0, NULL, &nbr_fichiers, principale );
  if ( NULL == liste_noms ) {
    return;
  }

  /* A-t-on plus [ou moins] d'infos que les (X, Y) d'un graphe réel classique ? */
  sans_X = !OBJ_choisit( IMPORT_ASCII, IA_AV_X, QUESTION ); /* Il n'y a pas de X -> spectre de type série */

  /* Veut-on une nouvelle série ? */
  nlle_serie = OBJ_choisit(IMPORT_ASCII, IA_NV_SERIE, QUESTION);
  if ( FAUX == nlle_serie ) { /* Non : on travaille sur la courante si possible */
    serie_import = serie_courante;
    if ( NULL == serie_import ) { /* Pas de série courante -> on doit la créer ! */
      nlle_serie = VRAI;
    } else if ( VRAI == serie_import->drapeaux.protegee ) { /* La série étant protégée, il faut en créer une nouvelle ! */
      nlle_serie = VRAI;
    }
  }

  /* Doit-on en faire une nouvelle ? */
  if ( nlle_serie == VRAI ) {
    char *nom_serie, *squelette;

    squelette = lit_chaine( IMPORT_ASCII, IA_NOM_SERIE, 0, NULL, TXT_COPIER );
    if ( ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_SERIE_FICHIER, QUESTION) ) || /* Une série par fichier */
	 ( 1 == nbr_fichiers ) ) { /* Un seul fichier, donc une seule série */
      nom_serie = TXT_genere_chaine("%s [%F]", squelette, liste_noms[ 0 ]);
    } else {
      nom_serie = TXT_genere_chaine("%s [%u fichiers]", squelette, nbr_fichiers );
      nlle_serie = FAUX;	/* On n'aura plus à en créer une */
    }
    serie_import = SERIE_GRF_CreerSerie( serie_courante, nom_serie );
    free( nom_serie );

    if ( NULL == serie_import ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de créer la série -> chargement annulé \n",
		     __FILE__, __FUNCTION__, __LINE__);

      for ( i = 0 ; i < nbr_fichiers ; i++ ) {
	free( liste_noms[ i ] );
      }
      free( liste_noms );

      return;
    }
  }

  /* La série initiale : celle indiquée */
  infos_lecture.serie = serie_import;

  /* Le type de fichier à créer */
  infos_lecture.type_import = type_import;

  /* Pour chacun des fichiers de la liste */
  for ( i = 0 ; i < nbr_fichiers ; i++ ) {
    switch ( format_import )
    {
    case ASCII_FORMAT_SPEC:
      OPTION_A_FAIRE( "Import format SPEC ESRF" );
      break;

    default: /* Par défaut, considéré comme de l'ASCII générique */
      retour |= ASCII_Generique( liste_noms[ i ], format_import, &infos_lecture );
    }

    /* Doit-on faire une nouvelle série ? */
    if ( ( VRAI == nlle_serie ) && ( i < ( nbr_fichiers - 1 ) ) ) {
      /* Demandé et on n'était pas sur le dernier fichier */
      char *nom_serie, *squelette;

      squelette = lit_chaine( IMPORT_ASCII, IA_NOM_SERIE, 0, NULL, TXT_COPIER );
      /* Si on est là, c'est que l'on a demandé une série par fichier, forcément */
      nom_serie = TXT_genere_chaine("%s [%F]", squelette, liste_noms[ 0 ]);
      serie_import = SERIE_GRF_CreerSerie( serie_courante, nom_serie );
      free( nom_serie ); nom_serie = NULL;

      /* Cela devient la série à utiliser */
      infos_lecture.serie = serie_import;
    }

    free( liste_noms[ i ] ); liste_noms[ i ] = NULL;
  }

  /* Plus besoin de la table des noms */
  free( liste_noms ); liste_noms = NULL;
}

/* -------- Quelle extension, quel masque utiliser dans le sélecteur de fichiers ? -------- */

static const char *IA_Extension(int format)
{
  switch ( format )
  {
  case ASCII_FORMAT_EDA_XY:
  case ASCII_FORMAT_EDA_YX:
    return "*.dat";

  case ASCII_FORMAT_GNUPLOT:
    return "*.txt";

  case ASCII_FORMAT_LASE_R:
  case ASCII_FORMAT_LASE_C:
    return "*.asc,*.txt";

  case ASCII_FORMAT_FEFF_F:
    return "feff*.dat";

  case ASCII_FORMAT_FEFF_C:
    return "chi.dat";

  case ASCII_FORMAT_FEFF_Cp:
    return "chi*.dat";

  case ASCII_FORMAT_FEFF_X:
    return "xmu.dat";

  case ASCII_FORMAT_SPEC:
    return "*.spec,*.dat";

  case ASCII_FORMAT_HDF:
    return "*.hdf";

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Format inconnu [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, format);
  }

  /* Par défaut, on charge tout ! */
  return "*";
}

/* -------- Gestion de la boîte de dialogue -------- */

static int GER_ImportASCII(unsigned int objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{  
  boolean etat;

  switch (objet)
  {
  case IA_MULTI_GRAPHE:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    IA_ActiverMultiGraphe( etat );
    return BI_EVENEMENT_TRAITE;

  case IA_NV_SERIE:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    if ( VRAI == etat ) {
      OBJ_active( IMPORT_ASCII, IA_SERIE_FICHIER, FORCE_OUI );
      OBJ_active( IMPORT_ASCII, IA_NOM_SERIE, FORCE_OUI );
    } else {
      IA_DesactiveObjet( IA_SERIE_FICHIER );
      OBJ_active( IMPORT_ASCII, IA_NOM_SERIE, FORCE_NON );
    }
    return BI_EVENEMENT_TRAITE;

  case IA_AUTO:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_NBR_COLONNES, !etat );
    return BI_EVENEMENT_TRAITE;

  case IA_SEP_ESPACE:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, !etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_Z:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_Z, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_IMAG:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_IMAG, etat );
    OBJ_active( IMPORT_ASCII, IA_ERR_IM_Y, etat && OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_Y, QUESTION ) );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_X:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_X, etat );
    if ( VRAI == etat ) {
      OBJ_active( IMPORT_ASCII, IA_AV_ERR_X, FORCE_OUI );
    } else {
      IA_DesactiveObjet( IA_AV_ERR_X );
      OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
    }
    return BI_EVENEMENT_TRAITE;

  case IA_AV_ERR_X:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_ERREUR_X, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_ERR_Y:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, etat );
    OBJ_active( IMPORT_ASCII, IA_ERR_IM_Y, etat && OBJ_choisit( IMPORT_ASCII, IA_AV_IMAG, QUESTION ) );
    return BI_EVENEMENT_TRAITE;

  case IA_TYPE_GRAPHE: /* Quel type attribuer au graphe importé ? */
    type_import = OBJ_ChoisirTypeGraphe( IMPORT_ASCII, IA_TYPE_GRAPHE, LGMAX_TYPE_IMPORT, type_import, QUESTION, VRAI );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* ------- Mise à jour de la boîte dans le cas de plusieurs fichiers ------- */

static void IA_ActiverMultiGraphe( int sens )
{
  if ( VRAI == sens ) {
    IA_DesactiveObjet( IA_AV_Z ); OBJ_active( IMPORT_ASCII, IA_COL_Z, FORCE_NON );
    IA_DesactiveObjet( IA_AV_IMAG ); OBJ_active( IMPORT_ASCII, IA_COL_IMAG, FORCE_NON );
    IA_DesactiveObjet( IA_AV_ERR_X ); OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
    IA_DesactiveObjet( IA_AV_ERR_Y ); OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, FORCE_NON ); OBJ_active( IMPORT_ASCII, IA_ERR_IM_Y, FORCE_NON );
  } else {
    int etat;

    OBJ_active( IMPORT_ASCII, IA_AV_Z, FORCE_OUI )    ; OBJ_active( IMPORT_ASCII, IA_COL_Z, OBJ_choisit( IMPORT_ASCII, IA_AV_Z, QUESTION ) );
    OBJ_active( IMPORT_ASCII, IA_AV_IMAG, FORCE_OUI ) ; OBJ_active( IMPORT_ASCII, IA_COL_IMAG, OBJ_choisit( IMPORT_ASCII, IA_AV_IMAG, QUESTION ) );
    OBJ_active( IMPORT_ASCII, IA_AV_ERR_X, FORCE_OUI ); OBJ_active( IMPORT_ASCII, IA_ERREUR_X, OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_X, QUESTION ) );
    OBJ_active( IMPORT_ASCII, IA_AV_ERR_Y, FORCE_OUI );
    etat = OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_X, QUESTION );
    OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, etat ); OBJ_active( IMPORT_ASCII, IA_ERR_IM_Y, etat && OBJ_choisit( IMPORT_ASCII, IA_AV_IMAG, QUESTION ) );
  }
}

/* -------- Installation de la base du format -------- */

static void EA_BaseFormat(unsigned int nbr_colonnes, unsigned int col_X, int separateur, int commentaire)
{
  /* Pas besoin de toucher aux boutons : a déjà été fait */

  /* 1. Combien de colonnes au total ? */
  OBJ_MettreEntier_U( IMPORT_ASCII, IA_NBR_COLONNES, (unsigned long) nbr_colonnes );

  /* 2. Quelle colonne pour les X ? */
  OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_X, (unsigned long) col_X );

  /* 3. Quel séparateur ? */
  if ( 0 == separateur ) {
    OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, FORCE_OUI );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, FORCE_NON );
  } else {
    OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, FORCE_NON );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, FORCE_OUI );
    OBJ_MettreCaractere( IMPORT_ASCII, IA_SEPARATEUR, separateur );
  }

  /* 4. Commentaire */
  OBJ_MettreCaractere( IMPORT_ASCII, IA_COMMENTAIRE, commentaire );
}

static void IA_BaseFormat(unsigned int nbr_colonnes, unsigned int col_X, int separateur, int commentaire, boolean multi)
{
  /* 0. On désélectionne le maximum de boutons */
  IA_ActiverMultiGraphe( VRAI );

  /* 1. Combien de colonnes au total ? */
  OBJ_MettreEntier_U( IMPORT_ASCII, IA_NBR_COLONNES, (unsigned long) nbr_colonnes );
  if ( 0 == nbr_colonnes ) {
    OBJ_active( IMPORT_ASCII, IA_NBR_COLONNES, FORCE_NON );
    OBJ_choisit( IMPORT_ASCII, IA_AUTO, FORCE_OUI );
  } else {
    OBJ_active( IMPORT_ASCII, IA_NBR_COLONNES, FORCE_OUI );
    OBJ_choisit( IMPORT_ASCII, IA_AUTO, FORCE_NON );
  }

  /* 2. Quelle colonne pour les X ? */
  OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_X, (unsigned long) col_X );
  if ( 0 == col_X ) {
    OBJ_choisit( IMPORT_ASCII, IA_AV_X, FORCE_NON );
    OBJ_active( IMPORT_ASCII, IA_COL_X, FORCE_NON );
  } else {
    OBJ_choisit( IMPORT_ASCII, IA_AV_X, FORCE_OUI );
    OBJ_active( IMPORT_ASCII, IA_COL_X, FORCE_OUI );
  }

  /* 3. Quel séparateur ? */
  if ( 0 == separateur ) {
    OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, FORCE_OUI );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, FORCE_NON );
  } else {
    OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, FORCE_NON );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, FORCE_OUI );
    OBJ_MettreCaractere( IMPORT_ASCII, IA_SEPARATEUR, separateur );
  }

  /* 4. Commentaire */
  OBJ_MettreCaractere( IMPORT_ASCII, IA_COMMENTAIRE, commentaire );

  /* 5. Multi-graphes ? */
  if ( VRAI == multi ) {
    OBJ_choisit( IMPORT_ASCII, IA_MULTI_GRAPHE, FORCE_OUI );
  } else {
    OBJ_choisit( IMPORT_ASCII, IA_MULTI_GRAPHE, FORCE_NON );
    IA_ActiverMultiGraphe( FAUX );
  }
}

/* -------- Installation des caractéristiques du format d'import -------- */

static void IA_InstallerFormat(unsigned int arbre, unsigned int objet, unsigned long format)
{
  if ( format >= NOMBRE_FORMATS_ASCII ) {
    OBJ_RemplacerTexte( arbre, objet, IEA_format_libre, FAUX );
    return;
  } else {
    OBJ_RemplacerTexte( arbre, objet, NULL, FAUX ); /* Pour afficher le nom du format pris dans le menu ! */
  }

  switch( format )
  {
  case ASCII_FORMAT_UNIVERSEL:
    {
      boolean est_multi; /* On reste dans l'état précédent ! */

      est_multi = OBJ_choisit( IMPORT_ASCII, IA_MULTI_GRAPHE, QUESTION );
      IA_BaseFormat( 0, 1, 0, '#', est_multi );
      met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-", TXT_COPIER );
    }
    break;

  case ASCII_FORMAT_EDA_XY:
    IA_BaseFormat( 2, 1, 0, 0, FAUX );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y, 2 );
    OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_X, FORCE_NON );
    OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
    break;

  case ASCII_FORMAT_EDA_YX:
    IA_BaseFormat( 2, 2, 0, 0, FAUX );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y, 1 );
    OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_X, FORCE_NON );
    OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
    break;

  case ASCII_FORMAT_GNUPLOT:
    IA_BaseFormat( 0, 1, 0, '#', VRAI );
    met_chaine ( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-", TXT_COPIER ); /* Y réels, chaque colonne après la 1 */
    break;

  case ASCII_FORMAT_LASE_R:
    IA_BaseFormat( 0, 1, 0, '#', FAUX );
    IA_ActiverMultiGraphe( FAUX ); /* Un seul graphe... */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y   , 2 ); /* Y réels, en colonne 2 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_ERREUR, 3 ); /* s(Y), en colonne 3 */
    break;

  case ASCII_FORMAT_LASE_C:
    IA_BaseFormat( 0, 1, 0, '#', FAUX );
    IA_ActiverMultiGraphe( FAUX ); /* Un seul graphe... */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y   , 2 ); /* Re(Y) en colonne 2 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_IMAG  , 3 ); /* Im(Y) en colonne 3 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_ERREUR, 3 ); /* s(Re(Y)), colonne 4 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_ERR_IM_Y  , 5 ); /* s(Im(Y)), colonne 5 */
    GER_ImportASCII( IA_AV_IMAG, 0, 0 );
    break;

  case ASCII_FORMAT_FEFF_F: /* FEFF : feff*.dat */
    IA_BaseFormat( 7, 1, 0, 0, VRAI );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-7", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_C: /* FEFF : chi.dat */
    IA_BaseFormat( 4, 1, 0, 0, VRAI );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-4", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_Cp: /* FEFF : chip*.dat */
    IA_BaseFormat( 5, 1, 0, 0, VRAI );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-5", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_X: /* FEFF : xmu.dat */
    IA_BaseFormat( 6, 1, 0, 0, VRAI ); /* X : 1, 2 ou 3 ; énergies utilisées par défaut */
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "4-6", TXT_COPIER );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Format inconnu [%lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, format);
    return;
  }

/*   met_chaine( IMPORT_ASCII, IA_FORMAT, LGMAX_FORMAT_IMPORT, IEA_formats[ format ], TXT_COPIER );   */
}

/* -------- Préparer l'arbre -------- */

inline static void IA_InitArbre( void )
{
  /* On a une liste de formats pour l'import */
  OBJ_MettreMenu( IMPORT_ASCII, IA_FORMAT, NOMBRE_FORMATS_ASCII, IEA_formats, IEA_titre_formats, IA_InstallerFormat );

  /* Contrôle des séries */
  OBJ_active( IMPORT_ASCII, IA_NV_SERIE, VRAI );
  OBJ_active( IMPORT_ASCII, IA_SERIE_FICHIER, OBJ_choisit( IMPORT_ASCII, IA_NV_SERIE, QUESTION ) );
  OBJ_active( IMPORT_ASCII, IA_NOM_SERIE, OBJ_choisit( IMPORT_ASCII, IA_NV_SERIE, QUESTION ) );

  /* Le fichier d'import peut être multi-graphe */
  OBJ_active( IMPORT_ASCII, IA_MULTI_GRAPHE, VRAI );

  /* On peut faire une détection automatique du nombre de colonnes */
  OBJ_active( IMPORT_ASCII, IA_AUTO, VRAI );
  OBJ_active( IMPORT_ASCII, IA_NBR_COLONNES, ! OBJ_choisit( IMPORT_ASCII, IA_AUTO, QUESTION ) );

  /* On s'arrange pour avoir les bons champs accessibles */
  IA_ActiverMultiGraphe( OBJ_choisit( IMPORT_ASCII, IA_MULTI_GRAPHE, QUESTION ) );

  /* Les Y sont toujours actifs */
  OBJ_choisit( IMPORT_ASCII, IA_AV_Y, FORCE_OUI ); OBJ_active( IMPORT_ASCII, IA_CHOIX_Y, FORCE_OUI );

  /* On peut choisir le type des graphes */
  OBJ_active( IMPORT_ASCII, IA_TYPE_GRAPHE, VRAI );

  /* Texte pour le bouton de sortie */
  met_chaine(IMPORT_ASCII, IA_OK, 0, IA_texte_importer, TXT_REMPLACE);

  /* Fonction de gestion correcte */
  arbres[ IMPORT_ASCII ].fonction.arbre = GER_ImportASCII;
}

/* -------- Lire les valeurs dans l'arbre -------- */

static int IA_LireInfos(struct infos_import_grf *infos)
{
  /* Que doit-on faire ? */
  infos->proprietes = IA_AVEC_HISTORIQUE; /* Copie dans l'historique, de toute façon */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_ERR_INCLUSE, QUESTION ) ) {
    infos->proprietes |= IA_UTILISER_ERREURS; /* On utilise les erreurs incluses aux colonnes */
  }

  /* 1. En-tête & assimilables */
  infos->lg_en_tete = OBJ_LireEntier_U( IMPORT_ASCII, IA_LIGNE_DEBUT );
  infos->commentaire = OBJ_LireCaractere( IMPORT_ASCII, IA_COMMENTAIRE );
  if ( OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, QUESTION ) == VRAI )
    infos->separateur = 0; /* N'importe-quoi... */
  else
    infos->separateur = OBJ_LireCaractere( IMPORT_ASCII, IA_SEPARATEUR );

  /* 2. Organisation des colonnes */
  IEA_LireNumerosColonnes( &(infos->nbr_colonnes), &(infos->colonne_X), &(infos->colonne_Re), &(infos->colonne_Im), &(infos->colonne_Z), &(infos->colonne_sX), &(infos->colonne_sY), &(infos->colonne_sIm) );
  infos->colonne_Y = infos->colonne_Re;
  if ( 0 == infos->colonne_X ) { /* On ne charge pas les X -> pas les erreurs en X non-plus ! */
    infos->colonne_sX = 0;
  }
  if ( ( 0 == infos->colonne_Im ) || ( 0 == infos->colonne_sY ) ) {
    infos->colonne_sIm = 0;
  }

  /* Combien de colonnes dans le fichier ? */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AUTO, QUESTION ) ) {
    infos->nbr_colonnes = 0; /* Détermination automatique du nombre de colonnes */
  }

  /* 3. Quelles colonnes veut-on conserver ? */
  infos->charger_grf = NULL;
  if ( VRAI == OBJ_choisit(IMPORT_ASCII, IA_MULTI_GRAPHE, QUESTION) ) {
    infos->proprietes |= IA_MULTI_COLONNES;

    /* Comment gérer les informations ? */
    if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_ERR_INCLUSE, QUESTION ) )
      infos->proprietes |= IA_UTILISER_ERREURS;
    if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_SERIE_FICHIER, QUESTION ) )
      infos->proprietes |= IA_SERIE_X_COMMUNS; /* Si une série par fichier, on peut le faire sans danger */

    infos->filtre = lit_chaine(IMPORT_ASCII, IA_CHOIX_Y, 0, NULL, TXT_REMPLACE);
  } else {
    infos->filtre = NULL;
  }

  /* Initialisation réussie */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Interface graphique : Export ~-~-~-~-~-~-~-~-~ */

void GRF_ExportASCII(Graphe *graphe)
{
  FILE *fichier;
  char *nom_fichier;
/*   char *filtres[ 2 ] = { "*.asc", "*.txt" }; */
  int commentaire, separateur;
  int retour;
  unsigned int nbr_colonnes;
  unsigned int colonne_X, colonne_Re, colonne_Im, colonne_Z;
  unsigned int col_err_X, col_err_Re, col_err_Im;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    2, NULL,
    0, NULL,
    NULL
  };

  if ( NULL == graphe ) return;

  /* Préparation de l'arbre */
  IEA_PreparerArbre();
  EA_InitArbre( graphe );
  EA_InstallerFormat( IMPORT_ASCII, IA_FORMAT, -1 );

  /* Appel de la boîte de dialogue */
  retour = ARB_Bloquant( IMPORT_ASCII, EA_titre_arbre, principale, 0 );
  if ( IA_ANNULER == retour ) /* On annule */
    return;

  /* On repère le type du fichier */
  /* 1. Organisation des colonnes */
  IEA_LireNumerosColonnes( &nbr_colonnes, &colonne_X, &colonne_Re, &colonne_Im, &colonne_Z, &col_err_X, &col_err_Re, &col_err_Im );

  if ( nbr_colonnes < 1 ) {
    (void) fprintf(stderr, "%s,%s l.%u : pas assez de colonnes. Format d'export incorrect. Opération annulée\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  commentaire = *lit_chaine(IMPORT_ASCII, IA_COMMENTAIRE, 0, NULL, TXT_REMPLACE);
  if ( OBJ_choisit( IMPORT_ASCII, IA_SEP_ESPACE, QUESTION ) == VRAI )
    separateur = ' ';
  else
    separateur = *lit_chaine( IMPORT_ASCII, IA_SEPARATEUR, 0, NULL, TXT_REMPLACE );

  /* On prépare le sélecteur de fichiers */
  infos_selecteur.titre  = (char *) EA_titre_selecteur;
/*   infos_selecteur.filtres = &filtres; */
  infos_selecteur.chemin = chemin_export;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, principale );
  /* On mémorise les chemins */
  chemin_export = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant à la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_export );

  if ( nom_fichier == NULL ) {
    return;
  }

  fichier = fopen(nom_fichier, "wb");
  if (fichier == NULL) {
    (void) SYS_ERREUR(ERREUR_ECRITURE, nom_fichier, NULL, NULL);
    return;
  }

  /* Log dans l'historique */
  HIST_Titre(1, "Export en ASCII", "E/S");
  HIST_Texte(1, 0, "Graphe export"e_AIGU" : %s\n", graphe->nom);
  HIST_Texte(1, 0, "Fichier : %s\n", nom_fichier);
  if ( commentaire > ' ' )
    HIST_Texte(4, 0, "Commentaires introduits par %c (%d)\n", commentaire, (int) commentaire);
  HIST_Texte(4, 0, "Nombre de colonnes : %u\n", nbr_colonnes);
  HIST_Texte(4, 0, "Colonnes           :");
  if ( colonne_X > 0 ) HIST_Texte(4, 0, " X = %u\n", colonne_X);
  if ( (graphe->erreur_x != NULL) && ( col_err_X > 0 ) )
    HIST_Texte(4, 0, " "SIGMA"(X) = %u", col_err_X);
  if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) {
    if ( colonne_Re > 0 ) HIST_Texte(4, 0, " Re(Z) = %u", colonne_Re );
    if ( colonne_Im > 0 ) HIST_Texte(4, 0, " Im(Z) = %u", colonne_Im );
    if ( graphe->erreur_y != NULL ) {
      if ( col_err_Re > 0 )
	HIST_Texte(4, 0, " "SIGMA"(Re(Z)) = %u", col_err_Re );
      if ( col_err_Im > 0 )
	HIST_Texte(4, 0, " "SIGMA"(Im(Z)) = %u", col_err_Im );
    }
  } else {
    if ( colonne_Re > 0 ) HIST_Texte(4, 0, " Y = %u", colonne_Re );
    if ( graphe->erreur_y != NULL ) {
      if ( col_err_Re > 0 )
	HIST_Texte(4, 0, " "SIGMA"(Y) = %u", col_err_Re );
    }
  }

  free(nom_fichier);

  /* On met quelques commentaires, si on a le droit */
  if ( commentaire > ' ') {
    fprintf(fichier, "%c Export ASCII du programme LASE\r\n", commentaire);
    fprintf(fichier, "%c (C) Emmanuel CURIS, 2003\r\n", commentaire);
    fprintf(fichier, "%c Graphe : %s\r\n", commentaire, graphe->nom);
    fprintf(fichier, "%c Type   : %s\r\n", commentaire, GRF_TexteTypeGraphe( graphe->type ) );
    fprintf(fichier, "%c Colonnes : X = %u ("SIGMA" = %u) [%s]", commentaire, colonne_X, col_err_X,
	    GRF_QuelleUnite( graphe, AXE_X ) );

    if ( GRF_EstReel( graphe ) == VRAI ) {
      fprintf(fichier, " Y = %u ("SIGMA" = %u) [%s]", colonne_Re, col_err_Re, GRF_QuelleUnite( graphe, AXE_Y ) );
    } else if ( GRF_EstComplexe( graphe ) == VRAI ) {
      fprintf(fichier, " Z : Re = %u ("SIGMA" = %u), Im = %u ("SIGMA" = %u)", colonne_Re, col_err_Re, colonne_Im, col_err_Im );
    }
    fprintf(fichier, " (0 = donnée absente ; total : %u colonnes)\r\n", nbr_colonnes);
    fprintf(fichier, "%c\r\n", commentaire);
  }

  /* Export proprement dit */
  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
    retour = EA_GrapheReel( fichier, graphe, separateur, nbr_colonnes, colonne_X, colonne_Re, col_err_X, col_err_Re );
    break;

  case GRF_FCT_COMPLEXE:
    retour = EA_GrapheComplexe( fichier, graphe, separateur, nbr_colonnes, colonne_X, colonne_Re, colonne_Im, col_err_X, col_err_Re, col_err_Im );
    break;

  case GRF_SERIE:
    if ( NULL == graphe->x ) {
      retour = EA_GrapheSerie( fichier, graphe, separateur, nbr_colonnes, colonne_X, colonne_Re, col_err_Re );
    } else {
      retour = EA_GrapheReel( fichier, graphe, separateur, nbr_colonnes, colonne_X, colonne_Re, col_err_X, col_err_Re );
    }
    break;

  default:
    retour = -484;
    (void) fprintf(stderr, "%s,%s l.%u : Graphe de genre non-supporté [%d]\n",
		   __FILE__, __FUNCTION__, __LINE__, graphe->genre_donnees);
  }

  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR survenue dans l'écriture [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
  }

  (void) fclose(fichier); /* Terminé */
}

/* ------ Préparer l'arbre pour l'export ------ */

static void EA_InitArbre( Graphe *graphe )
{
  unsigned int nbr_colonnes = 0; /* Y */

  /* On a une liste de formats pour l'export */
  OBJ_MettreMenu( IMPORT_ASCII, IA_FORMAT, NOMBRE_FORMATS_ASCII, IEA_formats, IEA_titre_formats, EA_InstallerFormat );

  /* Ce qui ne sert jamais */
  OBJ_active( IMPORT_ASCII, IA_NV_SERIE     , FORCE_NON ); /* La série */
  OBJ_active( IMPORT_ASCII, IA_SERIE_FICHIER, FORCE_NON );
  OBJ_active( IMPORT_ASCII, IA_NOM_SERIE    , FORCE_NON );

  OBJ_active( IMPORT_ASCII, IA_TYPE_GRAPHE, FORCE_NON ); /* Le type de graphe */

  OBJ_active( IMPORT_ASCII, IA_AUTO, FORCE_NON ); /* On doit indiquer le nombre de colonnes */
  OBJ_active( IMPORT_ASCII, IA_NBR_COLONNES, FORCE_OUI );

  /* On désactive tout, pour partir tranquillement... */
  IA_ActiverMultiGraphe( VRAI );

  /* Qu'est-ce que l'on peut et veut sauver ? */
  if ( graphe->x != NULL ) {
    OBJ_choisit( IMPORT_ASCII, IA_AV_X, FORCE_OUI ); OBJ_active( IMPORT_ASCII, IA_COL_X, FORCE_OUI );
    nbr_colonnes++;
    if ( graphe->erreur_x != NULL ) {
      IA_ActiveObjet( IA_AV_ERR_X ); OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_OUI );
      nbr_colonnes++;
    } else {
      IA_DesactiveObjet( IA_AV_ERR_X ); OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
    }
  } else {
    OBJ_choisit( IMPORT_ASCII, IA_AV_X, FORCE_NON ); OBJ_active( IMPORT_ASCII, IA_COL_X, FORCE_NON );
    IA_DesactiveObjet( IA_AV_ERR_X ); OBJ_active( IMPORT_ASCII, IA_ERREUR_X, FORCE_NON );
  }

  switch ( graphe->genre_donnees )
  {
  case GRF_FCT_REELLE:
  case GRF_SERIE:
    nbr_colonnes++;
    if ( graphe->erreur_y != NULL ) {
      nbr_colonnes++;
      IA_ActiveObjet( IA_AV_ERR_Y ); OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, FORCE_OUI );
/*     } else { */
/*       IA_DesactiveObjet( IA_AV_ERR_Y ); OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, FORCE_NON ); */
    }
/*     IA_DesactiveObjet( IA_AV_IMAG ); OBJ_active( IMPORT_ASCII, IA_COL_IMAG, FORCE_NON ); OBJ_active( IMPORT_ASCII, IA_ERR_IM_Y, FORCE_NON ); */
    break;

  case GRF_FCT_COMPLEXE:
    nbr_colonnes += 2; /* Partie réelle & imaginaire */
    IA_ActiveObjet( IA_AV_IMAG ); OBJ_active( IMPORT_ASCII, IA_COL_IMAG, FORCE_OUI );
    if ( graphe->erreur_y != NULL ) {
      nbr_colonnes += 2;
      IA_ActiveObjet( IA_AV_ERR_Y ); OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, FORCE_OUI );
    } else {
/*       IA_DesactiveObjet( IA_AV_ERR_Y ); OBJ_active( IMPORT_ASCII, IA_COL_ERREUR, FORCE_NON ); */
    }
    break;
  }

  /* Texte pour le bouton de sortie */
  met_chaine(IMPORT_ASCII, IA_OK, 0, EA_texte_exporter, TXT_REMPLACE);

  /* Fonction de gestion correcte */
  arbres[ IMPORT_ASCII ].fonction.arbre = GER_ExportASCII;
}

static void EA_InstallerFormat(unsigned int arbre, unsigned int objet, unsigned long format)
{
  if ( format > 0 ) {
    OBJ_RemplacerTexte( arbre, objet, IEA_format_libre, FAUX );
    return;
  } else {
    OBJ_RemplacerTexte( arbre, objet, NULL, FAUX ); /* Pour afficher le nom du format pris dans le menu ! */
  }

  switch( format )
  {
  case ASCII_FORMAT_UNIVERSEL: /* On reste inchangé, en fait... */
    EA_BaseFormat( 0, 1, 0, '#' );
    break;

  case ASCII_FORMAT_EDA_XY:
    EA_BaseFormat( 2, 1, 0, 0 );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_X, 2U ); /* X en colonne 2 (!) */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y, 1U ); /* Y en colonne 1 (!) */
    break;

  case ASCII_FORMAT_EDA_YX:
    EA_BaseFormat( 2, 2, 0, 0 );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_X, 1U ); /* X en colonne 1 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y, 2U ); /* Y en colonne 1 */
    break;

  case ASCII_FORMAT_GNUPLOT:
    EA_BaseFormat( 0, 1, 0, '#' );
    met_chaine ( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-", TXT_COPIER ); /* Y réels, chaque colonne après la 1 */
    break;

  case ASCII_FORMAT_LASE_R:
    EA_BaseFormat( 0, 1, 0, '#' );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y   , 2 ); /* Y réels, en colonne 2 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_ERREUR, 3 ); /* s(Y), en colonne 3 */
    break;

  case ASCII_FORMAT_LASE_C:
    EA_BaseFormat( 0, 1, 0, '#' );
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_CHOIX_Y   , 2 ); /* Re(Y) en colonne 2 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_IMAG  , 3 ); /* Im(Y) en colonne 3 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_COL_ERREUR, 3 ); /* s(Re(Y)), colonne 4 */
    OBJ_MettreEntier_U( IMPORT_ASCII, IA_ERR_IM_Y  , 5 ); /* s(Im(Y)), colonne 5 */
    break;

  case ASCII_FORMAT_FEFF_F: /* FEFF : feff*.dat */
    EA_BaseFormat( 7, 1, 0, 0 );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-7", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_C: /* FEFF : chi.dat */
    EA_BaseFormat( 4, 1, 0, 0 );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-4", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_Cp: /* FEFF : chip*.dat */
    EA_BaseFormat( 5, 1, 0, 0 );
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "2-5", TXT_COPIER );
    break;

  case ASCII_FORMAT_FEFF_X: /* FEFF : xmu.dat */
    EA_BaseFormat( 6, 1, 0, 0 ); /* X : 1, 2 ou 3 ; énergies utilisées par défaut */
    met_chaine( IMPORT_ASCII, IA_CHOIX_Y, 0, "4-6", TXT_COPIER );
    break;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Format inconnu [%lu]\n",
		   __FILE__, __FUNCTION__, __LINE__, format);
    return;
  }

/*   met_chaine( IMPORT_ASCII, IA_FORMAT, LGMAX_FORMAT_IMPORT, IEA_formats[ format ], TXT_COPIER );   */
}

/* -------- Gestion de l'arbre -------- */

static int GER_ExportASCII( unsigned int objet, int x, int y )
{
  boolean etat;

  switch ( objet )
  {
/*   case IA_FORMAT: */
/*     { */
/*       int nv_format; */

/*       nv_format = SS_menu_texte( IEA_formats, NOMBRE_FORMATS_ASCII, IEA_titre_formats, IMPORT_ASCII, IA_FORMAT ); */
/*       EA_InstallerFormat( nv_format ); */
/*     } */
/*     return VRAI; */

  case IA_SEP_ESPACE:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_SEPARATEUR, !etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_X:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_X, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_Y:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_CHOIX_Y, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_IMAG:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_IMAG, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_Z:
    etat = OBJ_choisit( IMPORT_ASCII, objet, ECHANGE );
    OBJ_active( IMPORT_ASCII, IA_COL_Z, etat );
    return BI_EVENEMENT_TRAITE;

  case IA_AV_ERR_X: case IA_AV_ERR_Y:
    return GER_ImportASCII( objet, x, y );
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* ~-~-~-~-~-~-~-~-~ Interface graphique : Commune ~-~-~-~-~-~-~-~-~ */

/* -------- Lire les numéros de colonnes (0 = inutilisée) -------- */

static void IEA_LireNumerosColonnes(unsigned int *nombre, unsigned int *nc_X, unsigned int *nc_Re, unsigned int *nc_Im, unsigned int *nc_Z,
				    unsigned int *nc_sX, unsigned int *nc_sRe, unsigned int *nc_sIm)
{
  /* Nombre de colonnes [0 = automatique] */
  *nombre = OBJ_LireEntier_U( IMPORT_ASCII, IA_NBR_COLONNES );

  /* Les X */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_X, QUESTION ) ) { /* Il y a une p. imaginaire */
    *nc_X = OBJ_LireEntier_U( IMPORT_ASCII, IA_COL_X );
  } else {
    *nc_X = 0;
  }

  /* Les Y */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_Y, QUESTION ) ) {
    *nc_Re = OBJ_LireEntier_U( IMPORT_ASCII, IA_CHOIX_Y );
  } else {
    *nc_Re = 0;
  }

  /* Les Z */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_Z, QUESTION ) ) { /* Il y a des Z */
    *nc_Z = OBJ_LireEntier_U( IMPORT_ASCII, IA_COL_Z );
  } else {
    *nc_Z = 0;
  }

  /* Partie imaginaire */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_IMAG, QUESTION ) ) { /* Il y a une p. imaginaire */
    *nc_Im = OBJ_LireEntier_U( IMPORT_ASCII, IA_COL_IMAG );
  } else {
    *nc_Im = 0;
  }

  /* Les erreurs en X */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_X, QUESTION ) ) {
    *nc_sX = OBJ_LireEntier_U(IMPORT_ASCII, IA_ERREUR_X);
  } else {
    *nc_sX = 0;
  }

  /* Les erreurs en Y */
  if ( VRAI == OBJ_choisit( IMPORT_ASCII, IA_AV_ERR_Y, QUESTION ) ) {
    *nc_sRe = OBJ_LireEntier_U(IMPORT_ASCII, IA_COL_ERREUR );
    if ( *nc_Im > 0 ) {
      *nc_sIm = OBJ_LireEntier_U(IMPORT_ASCII, IA_ERR_IM_Y );
    } else {
      *nc_sIm = 0;
    }
  } else {
    *nc_sRe = 0;
    *nc_sIm = 0;
  }
}

/* -------- Préparation de la boîte de dialogue -------- */

void IEA_PreparerArbre(void)
{
  if ( VRAI == arbres[ IMPORT_ASCII ].drapeaux.prepare ) return;

  if ( FAUX == arbres[ CHG_BORNES ].drapeaux.init ) {
    int retour;

    OBJ_ChangerType( IMPORT_ASCII, IA_FORMAT, G_SSMENU );

    retour = ARB_Initialiser( IMPORT_ASCII, "menu_fichiers.html#import_ascii", NULL, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) return;
  }

  /* Améliorations graphiques */
  OBJ_deplace(IMPORT_ASCII, IA_TXT_PRL_AVANT, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_LIGNE_DEBUT  , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_TXT_PRL_APRES, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_COMMENTAIRE  , 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(IMPORT_ASCII, IA_AV_IMAG , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_COL_IMAG, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_TXT_IMAG, 0, +1, DEPLACE_PIXELS);

  OBJ_deplace(IMPORT_ASCII, IA_AV_ERR_X  , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_TXT_X_ERR , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_ERREUR_X  , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_AV_ERR_Y  , 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_TXT_ERR_RE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_COL_ERREUR, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_TXT_ERR_IM, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_ERR_IM_Y, 0, +2, DEPLACE_PIXELS);

  OBJ_deplace(IMPORT_ASCII, IA_TXT_NOM_SERIE, 0, +3, DEPLACE_PIXELS);
  OBJ_deplace(IMPORT_ASCII, IA_NOM_SERIE    , 0, +3, DEPLACE_PIXELS);

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_FORMAT   , texte_format );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_MULTI_GRF, texte_contient_ ); /* plusieurs graphes */
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_NV_SERIE , texte_nlle_serie );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_SER_FICH , texte_une_serie );

  OBJ_TraduireTexte( IMPORT_ASCII, IA_AUTO         , texte_auto );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_NBR_COL  , texte_colonnes_ ); /* séparées par */
  OBJ_TraduireTexte( IMPORT_ASCII, IA_SEP_ESPACE   , texte_tab_espace );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_PRL_AVANT, texte_ignorer );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_PRL_APRES, texte_lignes_ ); /* au début & celles commencées par */

  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_VALEURS, texte_valeurs );

  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_ERREURS, texte_erreurs );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_ERR_INC, texte_utiliser_ ); /* les valeurs incluses */

  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_TYPE_GRF, texte_type_graphe );
  OBJ_TraduireTexte( IMPORT_ASCII, IA_TXT_NOM_SERIE, texte_nom_serie );

  OBJ_TraduireTexte( IMPORT_ASCII, IA_ANNULER, texte_annuler );
#endif

  OBJ_ChoisirTypeGraphe( IMPORT_ASCII, IA_TYPE_GRAPHE, LGMAX_TYPE_IMPORT, type_import, METTRE, VRAI );

  arbres[IMPORT_ASCII].drapeaux.prepare = VRAI;
}
