/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion de la bote d'informations & changement de menu
*/

#include <stdlib.h>
#include <stdio.h>

#include "systeme.h"
#include "exafs.h"

#include "interface.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define NBR_LIGNES_D_INFOS 37 /* Lignes de texte  afficher */
static unsigned long premiere_ligne_affichee = 0;

#include "infos.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void ARB_PrepareInformations(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Appeler les informations -------- */

int ARB_AppelerInformations( void )
{
  Fenetre *fenetre;

  fenetre = ARB_Appeler( INFORMATIONS, NAME|CLOSER|MOVER, titre_arbre_infos, ARB_PrepareInformations);
  if ( NULL == fenetre ) {
    (void) fprintf(stderr, "%s : ERREUR dans ARB_Appeler()\n", __FUNCTION__ );
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return -1;
  }

  /* On remet le texte au dbut */
  premiere_ligne_affichee = 0;
  return 0;
}

/* -------- Grer la bote pour les informations -------- */

int GER_infos(unsigned int objet, int x_souris, int y_souris) /* utilise dans menus.c */
{
  unsigned long i = NBR_LIGNES_D_INFOS; /* Pour les infos */

  switch ( objet )
  {
  case TRAITE_SPECTRES:
    OBJ_choisit(INFORMATIONS, TRAITE_SPECTRES, FORCE_OUI);
    BI_InstallerMenu(MENU, titre_menu_LASE1, MEN_extrait);
    return BI_EVENEMENT_TRAITE;

  case EXF_AJUSTEMENTS:
    OBJ_choisit(INFORMATIONS, EXF_AJUSTEMENTS, FORCE_OUI);
    BI_InstallerMenu(MEN_AJUSTE, titre_menu_LASE2, MEN_ajuste);
    return BI_EVENEMENT_TRAITE;

  case CALC_THEORIQUES:
    OBJ_choisit(INFORMATIONS, CALC_THEORIQUES, FORCE_OUI);
    BI_InstallerMenu(MEN_THEORIE, titre_menu_LASE3, MEN_theorie);
    return BI_EVENEMENT_TRAITE;

  case STRUCTURE:
    OBJ_choisit(INFORMATIONS, STRUCTURE, FORCE_OUI);
    BI_InstallerMenu(MEN_STRUCTURE, titre_menu_LASE4, MEN_structure);
    return BI_EVENEMENT_TRAITE;

  case INFOS_BANQUE:
    OBJ_choisit(INFORMATIONS, INFOS_BANQUE, FORCE_OUI);
    BI_InstallerMenu(MEN_BANQUE, titre_menu_LASE5, MEN_banque);
    return BI_EVENEMENT_TRAITE;

  case INFOS_FLUOX:
    OBJ_choisit(INFORMATIONS, INFOS_FLUOX, FORCE_OUI);
    BI_InstallerMenu(MEN_FLUOX, titre_menu_LASE6, MEN_fluoX);
    return BI_EVENEMENT_TRAITE;

  case HISTORIQUE:
    if ( OBJ_choisit(INFORMATIONS, HISTORIQUE, ECHANGE) ) { /* On demande un historique */
      if ( FAUX == HIST_LancerHistorique("HISTOIRE.TXT", 2) ) {
	(void) SYS_ERREUR(ERREUR_ECRITURE, "HISTOIRE.TXT", NULL);
	OBJ_choisit(INFORMATIONS, HISTORIQUE, FORCE_NON);
      }
      HIST_ChangerNiveau( OBJ_LireEntier_U( INFORMATIONS, INF_HIST_BOUTON ) );
      HIST_Texte(1, VRAI, "LASE, version %.2f\n", VERSION / 10.0);
    } else { /* On ferme l'historique */
      HIST_ArreterHistorique();
    }
    return BI_EVENEMENT_TRAITE;

    /* ---- Conventions pour les niveaux de l'historique ----
        -1 = messages de dboguage, version de dveloppement uniquement...
         1 = l'essentiel : opration effectue (titre), graphe original, graphe rsultat
         2 = description sommaire de la mthode (surtout quand choix multiple de mthode)
             rsultat de l'opration
         3 = valeurs numriques essentielles pour le calcul
             valeurs numriques essentielles pour interprter le rsultat
         4 = informations dtailles sur la mthode utilise
         5 = valeurs numriques compltes pour la mthode
     */

  case INF_HIST_AVANT:
  case INF_HIST_APRES:
  case INF_HIST_FOND:
  case INF_HIST_BOUTON:
    (void) HIST_ChangerNiveau( (unsigned int) gere_ascenseur(INFORMATIONS, INF_HIST_AVANT, INF_HIST_FOND, INF_HIST_BOUTON, INF_HIST_APRES, ASC_HORIZONTAL,
							     1, 5, 1, 3, objet, x_souris, y_souris) );
    return BI_EVENEMENT_TRAITE;

  case INFOS_AVANT:
  case INFOS_APRES:
  case INFOS_FOND:
  case INFOS_FOND + 1:
    ARB_GererDefileur_T( INFORMATIONS, INFOS_PREM_TEXTE, INFOS_DERN_TEXTE, 0,
			 INFOS_AVANT, INFOS_FOND, INFOS_FOND + 1, INFOS_APRES, 
			 NBR_LIGNES_D_INFOS, TXT_informations, NULL, &i, &premiere_ligne_affichee, objet, y_souris );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparation de l'arbre -------- */

void ARB_PrepareInformations(void)
{
  int retour;

  if ( VRAI == arbres[INFORMATIONS].drapeaux.prepare )
    return;

  /* Initialisation */
  retour = ARB_Initialiser( INFORMATIONS, "menu_lase.html#infos", GER_infos, ( int (*)(unsigned int, int, int *) ) NULL );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR %d dans ARB_Initialiser() [arbre INFORMATIONS=%u]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour, INFORMATIONS);
    FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
    return;
  }

  ARB_MettreComplement( INFORMATIONS, NULL );
  arbres[ INFORMATIONS ].drapeaux.prepare = VRAI;

  ARB_EnjoliverDefileur(INFORMATIONS, INFOS_FOND_TEXTE, INFOS_PREM_TEXTE, INFOS_DERN_TEXTE,
			INFOS_AVANT, INFOS_FOND, ROOT, INFOS_APRES, ARB_ASCENSEUR_DROITE );

  ARB_PreparerDefileur_T(INFORMATIONS, INFOS_PREM_TEXTE, INFOS_DERN_TEXTE, 0,
			 INFOS_AVANT, INFOS_FOND, INFOS_FOND + 1, INFOS_APRES,
			 NBR_LIGNES_D_INFOS, TXT_informations, NULL, NBR_LIGNES_D_INFOS + 1, 0);

  OBJ_deplace(INFORMATIONS, CALC_THEORIQUES, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INFORMATIONS, EXF_AJUSTEMENTS, 0, +2, DEPLACE_PIXELS);
  OBJ_deplace(INFORMATIONS, INFOS_BANQUE, 0, +1, DEPLACE_PIXELS);
  OBJ_deplace(INFORMATIONS, INFOS_FLUOX, 0, +2, DEPLACE_PIXELS);

  ARB_EnjoliverAscenseur(INFORMATIONS, INF_HIST_AVANT, INF_HIST_FOND, INF_HIST_BOUTON, INF_HIST_APRES, ASC_HORIZONTAL, +1);
  OBJ_deplace(INFORMATIONS, INF_TXT_HISTO, 0, +1, DEPLACE_PIXELS);

#ifndef _DEVELOPPE_ /* Version de dveloppement uniquement */
  OBJ_active(INFORMATIONS, INFOS_BANQUE, FORCE_NON);
#endif

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte(INFORMATIONS, TRAITE_SPECTRES, texte_infos_extraction);
  OBJ_TraduireTexte(INFORMATIONS, CALC_THEORIQUES, texte_infos_theorie);
  OBJ_TraduireTexte(INFORMATIONS, EXF_AJUSTEMENTS, texte_infos_ajustement);
  OBJ_TraduireTexte(INFORMATIONS, STRUCTURE, texte_infos_structure);
  OBJ_TraduireTexte(INFORMATIONS, HISTORIQUE, texte_infos_historique);
#endif
}
