/* LASE - Logiciel d'Analyse des Spectres Exprimentaux
   (C) 1996-2002, Emmanuel Curis

   Interface graphique : routines gnrales indpendantes du systme

   Gestion de la bote de modification des lgendes d'un axe
*/

#include <stdlib.h>
#include <stdio.h>

#include "systeme.h"
#include "exafs.h"

#include "graphes.h"
#include "images.h"
#include "interface.h"
#include "tracer.h"

#define NOMBRE_TYPE_GRADUATIONS 5
#include "legendes.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_TYPE_GRADUATION 20

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int  GER_legende(unsigned int num_objet, int x_souris, int y_souris);

static void ActiverGrille(boolean activer);
static void LEG_InstallerAxe(struct infos_axe *axe);
static void LEG_LireParametres(struct infos_axe *axe, boolean complet);
static void LEG_PreparerArbre(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Fonction visible de l'extrieur : paramtrage complet
   Renvoie 0 si tout s'est bien pass, != 0 sinon

   axe         : l'axe  modifier
   orientation : AXE_X, AXE_Y ou AXE_Z
   complet     : VRAI = on peut modifier aussi la lgende et l'unit
   -------- */

int I_ParametrerLegendes(struct infos_axe *axe, int orientation, boolean complet, Fenetre *mere)
{
  int retour, ancien_type;

  if ( NULL == axe ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Aucun axe indiqu\n",
		   __FILE__, __FUNCTION__, __LINE__);
#endif
    return -1;
  }

  /* Initialisation de l'arbre */
  LEG_PreparerArbre();
  LEG_InstallerAxe( axe );
  OBJ_active( PARAM_LEGENDE, PL_UNITE, complet );
  OBJ_active( PARAM_LEGENDE, PL_TEXTE_LEGENDE, complet );

  ancien_type = axe->type_graduation;

  /* Appel de l'arbre : on choisit le bon titre */
  ARB_MettreComplement( PARAM_LEGENDE, axe );
  if ( AXE_X == orientation ) {
    retour = ARB_Bloquant( PARAM_LEGENDE, titre_arbre_legendes_X, mere, PARAM_LEGENDE );
  } else if ( AXE_Y == orientation ) {
    retour = ARB_Bloquant( PARAM_LEGENDE, titre_arbre_legendes_Y, mere, PARAM_LEGENDE );
  } else if ( AXE_Z == orientation ) {
    retour = ARB_Bloquant( PARAM_LEGENDE, titre_arbre_legendes_Z, mere, PARAM_LEGENDE );
  } else {
    AVERTISSEMENT( "Orientation inconnue [%d]", orientation);
    return -2;
  }
  ARB_MettreComplement( PARAM_LEGENDE, NULL );
  if ( PL_ANNULER == retour ) {
    axe->type_graduation = ancien_type;
    return +1; /* Annulation */
  }

  /* Lecture des informations dans l'arbre */
  LEG_LireParametres( axe, complet );

  return 0;/* Tout s'est bien pass */
}

/* -------- Installer les paramtres actuels -------- */

static void LEG_InstallerAxe(struct infos_axe *axe)
{
  /* Les graduations */
  if ( ( axe->type_graduation < 0 ) || ( axe->type_graduation >= NOMBRE_TYPE_GRADUATIONS ) ) {
    AVERTISSEMENT( "Type de graduation inconnu [%d] ==> aucune", axe->type_graduation );
    axe->type_graduation = GRADUATION_AUCUNE;
  }
  met_chaine( PARAM_LEGENDE, PL_TYPE_GRADUER, LGMAX_TYPE_GRADUATION, types_graduation[ axe->type_graduation ], TXT_REMPLACE );

  if ( NULL == axe->unite ) {
    met_chaine( PARAM_LEGENDE, PL_UNITE, 0, "", TXT_COPIER );
  } else {
    met_chaine( PARAM_LEGENDE, PL_UNITE, 0, axe->unite, TXT_COPIER );
  }

  OBJ_MettreEntier_U( PARAM_LEGENDE, PL_NIVEAU_GRADUA, axe->finesse );
  if ( axe->finesse > 0 ) {
    OBJ_active( PARAM_LEGENDE, PL_NIVEAU_GRADUA, FORCE_OUI );
    OBJ_choisit( PARAM_LEGENDE, PL_NIVEAU_AUTO, FORCE_NON );
  } else {
    OBJ_active( PARAM_LEGENDE, PL_NIVEAU_GRADUA, FORCE_NON );
    OBJ_choisit( PARAM_LEGENDE, PL_NIVEAU_AUTO, FORCE_OUI );
  }

  OBJ_MettreNombre( PARAM_LEGENDE, PL_DEPART_GRAD, axe->min, 0.0 );
  OBJ_MettreNombre( PARAM_LEGENDE, PL_PAS_GRADUATIO, axe->pas_graduation, 0.0 );
  if ( axe->pas_graduation <= 0.0 ) {
    OBJ_active( PARAM_LEGENDE, PL_PAS_GRADUATIO, FORCE_NON );
    OBJ_active( PARAM_LEGENDE, PL_DEPART_GRAD, FORCE_NON );
    OBJ_choisit( PARAM_LEGENDE, PL_DEPART_AUTO, FORCE_OUI );
  } else {
#ifndef _DEVELOPPE_
    OBJ_active( PARAM_LEGENDE, PL_DEPART_GRAD, FORCE_OUI );
#endif
    OBJ_active( PARAM_LEGENDE, PL_PAS_GRADUATIO, FORCE_OUI );
    OBJ_choisit( PARAM_LEGENDE, PL_DEPART_AUTO, FORCE_NON );
  }

  OPTION_A_FAIRE( "Dfinition du format des graduations" );

  /* Grille */
  if ( axe->proprietes & AXE_AVEC_GRILLE ) {
    OBJ_choisit( PARAM_LEGENDE, PL_GRILLE_VISIBL, FORCE_OUI );
    ActiverGrille( VRAI );
  } else {
    OBJ_choisit( PARAM_LEGENDE, PL_GRILLE_VISIBL, FORCE_NON );
    ActiverGrille( FAUX );
  }
  OBJ_couleur( PARAM_LEGENDE, PL_COULEUR_GRILL, axe->couleur_grille, COULEUR_FOND, METTRE );
  OBJ_MettreStyle( PARAM_LEGENDE, PL_STYLE_GRILLE, axe->style_grille );

  if ( axe->proprietes & AXE_VISIBLE ) {
    OBJ_choisit( PARAM_LEGENDE, PL_AXE_VISIBLE, FORCE_OUI );
  } else {
    OBJ_choisit( PARAM_LEGENDE, PL_AXE_VISIBLE, FORCE_NON );
  }

  /* Lgende */
  met_chaine( PARAM_LEGENDE, PL_TEXTE_LEGENDE, 0, axe->legende, TXT_COPIER );

  OBJ_couleur( PARAM_LEGENDE, PL_COULEUR_TEXTE, axe->couleur_legende, COULEUR_FOND, METTRE );

}

static void ActiverGrille(boolean activer)
{
  OBJ_active( PARAM_LEGENDE, PL_STYLE_GRILLE, activer );
  OBJ_active( PARAM_LEGENDE, PL_COULEUR_GRILL, activer );
}

/* -------- Lire les paramtres actuels -------- */

static void LEG_LireParametres(struct infos_axe *axe, boolean complet)
{
  /* Les graduations [le type est modifi au vol] */
  if ( VRAI == complet ) {
    AXE_MettreLegende( axe, lit_chaine( PARAM_LEGENDE, PL_TEXTE_LEGENDE, 0, NULL, TXT_REMPLACE ) );
    AXE_MettreUnite  ( axe, lit_chaine( PARAM_LEGENDE, PL_UNITE, 0, NULL, TXT_REMPLACE ) );
  }
  if ( VRAI == OBJ_choisit( PARAM_LEGENDE, PL_NIVEAU_AUTO, QUESTION ) ) {
    axe->finesse = 0;
  } else {
    axe->finesse = OBJ_LireEntier_U( PARAM_LEGENDE, PL_NIVEAU_GRADUA );
  }
  if ( VRAI == OBJ_choisit( PARAM_LEGENDE, PL_DEPART_AUTO, QUESTION ) ) {
    axe->pas_graduation = -1.0;
  } else {
    axe->pas_graduation = OBJ_LireNombre( PARAM_LEGENDE, PL_PAS_GRADUATIO, NULL ) ;
  }

  /* Grille */
  if ( VRAI == OBJ_choisit( PARAM_LEGENDE, PL_GRILLE_VISIBL, QUESTION ) ) {
    axe->proprietes |= AXE_AVEC_GRILLE;
    axe->couleur_grille = OBJ_couleur( PARAM_LEGENDE, PL_COULEUR_GRILL, 0, COULEUR_FOND, QUESTION );
    axe->style_grille = OBJ_LireStyle( PARAM_LEGENDE, PL_STYLE_GRILLE );
  } else {
    axe->proprietes &= ~AXE_AVEC_GRILLE;
  }

  if ( VRAI == OBJ_choisit( PARAM_LEGENDE, PL_AXE_VISIBLE, QUESTION ) ) {
    axe->proprietes |= AXE_VISIBLE;
  } else {
    axe->proprietes &= ~AXE_VISIBLE;
  }

  /* Texte */
  axe->couleur_legende = OBJ_couleur( PARAM_LEGENDE, PL_COULEUR_TEXTE, 0, COULEUR_FOND, QUESTION );
}

/* -------- Gestion de l'arbre -------- */

static int GER_legende(unsigned int num_objet, int x __attribute__((__unused__)), int y __attribute__((__unused__)))
{
  int nv_valeur;
  struct infos_axe *axe;

  axe = (struct infos_axe *) ARB_LireComplement( PARAM_LEGENDE );
  if ( NULL == axe ) {
    (void) fprintf(stderr, "%s,%s l.%u : Aucun axe associ  l'arbre\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE);
    return BI_EVENEMENT_ERREUR;
  }

  switch (num_objet)
  {
  case PL_TYPE_GRADUER:
    nv_valeur = SS_menu_texte( types_graduation, NOMBRE_TYPE_GRADUATIONS, "Type de graduation", PARAM_LEGENDE, PL_TYPE_GRADUER );
    if ( nv_valeur >= 0 ) axe->type_graduation = nv_valeur;
    met_chaine( PARAM_LEGENDE, PL_TYPE_GRADUER, LGMAX_TYPE_GRADUATION, types_graduation[ axe->type_graduation ], TXT_REMPLACE );
    return BI_EVENEMENT_TRAITE;

  case PL_NIVEAU_AUTO:
    nv_valeur = OBJ_choisit( PARAM_LEGENDE, num_objet, ECHANGE );
    OBJ_active( PARAM_LEGENDE, PL_NIVEAU_GRADUA, !nv_valeur );
    return BI_EVENEMENT_TRAITE;

  case PL_DEPART_AUTO:
    nv_valeur = OBJ_choisit( PARAM_LEGENDE, num_objet, ECHANGE );
#ifdef _DEVELOPPE_
    OBJ_active( PARAM_LEGENDE, PL_DEPART_GRAD, !nv_valeur );
#endif
    OBJ_active( PARAM_LEGENDE, PL_PAS_GRADUATIO, !nv_valeur );
    return BI_EVENEMENT_TRAITE;

  case PL_GRILLE_VISIBL:
    nv_valeur = OBJ_choisit( PARAM_LEGENDE, num_objet, ECHANGE );
    ActiverGrille( nv_valeur );
    return BI_EVENEMENT_TRAITE;

  case PL_COULEUR_GRILL:
  case PL_COULEUR_TEXTE:
    nv_valeur = OBJ_couleur( PARAM_LEGENDE, num_objet, 0, COULEUR_FOND, QUESTION );
    nv_valeur = GER_ssmenu ( CHX_COULEUR  , PARAM_LEGENDE, num_objet, nv_valeur - 1 );
    if ( nv_valeur > 0 ) {
      OBJ_couleur( PARAM_LEGENDE, num_objet, nv_valeur, COULEUR_FOND, FORCE_OUI );
    }
    return BI_EVENEMENT_TRAITE;

  case PL_STYLE_GRILLE:       /* Style de dessin de la grille */
/*     nv_valeur = (int) OBJ_utilisateur(PARAM_LEGENDE, PL_STYLE_GRILLE, ( int cdecl (*)(PARMBLK *) ) NULL, */
/* 				      0, QUESTION); */
    nv_valeur = BI_DemanderStyleLigne( PARAM_LEGENDE, PL_STYLE_GRILLE );
    if ( nv_valeur > 0 ) {
      OBJ_utilisateur( PARAM_LEGENDE, PL_STYLE_GRILLE, ( int cdecl (*)(PARMBLK *) ) NULL, nv_valeur, METTRE );
    }
    return BI_EVENEMENT_TRAITE;
  }

  /* Objet classique... */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Prparer l'aspect de l'arbre -------- */

static void LEG_PreparerArbre(void)
{
  if ( VRAI == arbres[ PARAM_LEGENDE ].drapeaux.prepare ) return;

  /* Objets utilisateurs */
  OBJ_ChangerType( PARAM_LEGENDE, PL_STYLE_GRILLE, G_STYLELIGNE );
/*   OBJ_utilisateur(PARAM_LEGENDE, PL_CHOIX_FONTE , USR_fonte, 0, METTRE); */ /* Fonte systme  */

  /* Init. de l'arbre */
  if ( FAUX == arbres[ PARAM_LEGENDE ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( PARAM_LEGENDE, NULL, GER_legende, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) return;
  }

  /* Placement des intertitres */
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_GRADUATIO, 0, 500, DEPLACE_RELATIF );
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_GRILLE   , 0, 500, DEPLACE_RELATIF );
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_LEGENDE  , 0, 500, DEPLACE_RELATIF );

  /* Amlioration graphique */
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_TYPEGRD  , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_TYPE_GRADUER , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_NIVEAU   , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_NIVEAU_GRADUA, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_NIVEAU_AUTO  , 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( PARAM_LEGENDE, PL_TXT_DEPART , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_DEPART_GRAD, 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_DEPART_AUTO, 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_FORMAT , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_FORMAT     , 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( PARAM_LEGENDE, PL_TXT_COUL_GRIL, 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_COULEUR_GRILL, 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( PARAM_LEGENDE, PL_TXT_TEXTE    , 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_TEXTE_LEGENDE, 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_TXT_FONTE    , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( PARAM_LEGENDE, PL_CHOIX_FONTE  , 0, +2, DEPLACE_PIXELS );

#ifndef _DEVELOPPE_
  OBJ_active( PARAM_LEGENDE, PL_FORMAT      , FORCE_NON );
  OBJ_active( PARAM_LEGENDE, PL_TAILLE_TEXTE, FORCE_NON );
  OBJ_active( PARAM_LEGENDE, PL_CHOIX_FONTE , FORCE_NON );
#endif

  /* Termin */
  arbres[ PARAM_LEGENDE ].drapeaux.prepare = VRAI;
}

/* ~-~-~-~-~-~-~-~-~ Modifier les units sur les axes ~-~-~-~-~-~-~-~-~ */

/* -------- Convertir les valeurs d'un axe en changeant d'unit ------------ *
     Renvoie 0    si la conversion a pu tre faite
             != 0 en cas de problme

   graphe : le graphe pour lequel on veut convertir l'unit
   axe    : l'axe concern
 * --------------------------------------------------------------------- */

int I_GRF_ConvertirUnites(Graphe *graphe, unsigned int axe, const char *nouvelle_unite)
{
  const char *ancienne_unite;
  double f_ancien_v_SI, f_nouveau_v_SI, facteur;
  int type_ancienne, type_nouvelle;
  int retour;

  /* Contrles */
  if ( NULL == graphe ) {
    AVERTISSEMENT( "Appel avec graphe=NULL, axe=%u, unite=%s", axe, nouvelle_unite );
    return -1;
  }
  if ( NULL == nouvelle_unite ) {
    AVERTISSEMENT( "Appel avec nouvelle_unite=NULL, axe=%u", axe );
  }
  if ( axe & AXE_X ) {
    if ( NULL == graphe->x ) {
      /* Pas de valeurs en X ==> conversion impossible */
      (void) fprintf(stderr, "%s,%s l.%u : Pas de X\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -5;
    }

    if ( GRF_SERIE == graphe->genre_donnees ) {
      OPTION_A_FAIRE( "Convertir les X d'un graphe \"srie\" : tous, perdre les X,... ?" );
      return +3;
    }
  }

  /* Quelle est l'unit d'origine ? [1 ancien = f_ancien_v_SI SI] */
  ancienne_unite = GRF_QuelleUnite( graphe, axe );
  f_ancien_v_SI  = TXT_NormaliserUnite( ancienne_unite, VRAI, &type_ancienne, NULL );

  /* Quelle est la nouvelle unit ? [1 nouveau = f_nouveau_v_SI SI] */
  f_nouveau_v_SI = TXT_NormaliserUnite( nouvelle_unite, VRAI, &type_nouvelle, NULL );

  /* Les deux units sont-elles compatibles ? */
  if ( type_ancienne != type_nouvelle ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &MSG_GrandeursDifferentes, VRAI,
			     ancienne_unite, TXT_LegendeParDefaut( type_ancienne ),
			     nouvelle_unite, TXT_LegendeParDefaut( type_nouvelle ) );
    if ( 2 == reponse ) {
      /* On annule la conversion */
      return +1;
    }
  }

  /* On calcule le facteur de conversion */
  if ( f_nouveau_v_SI != 0.0 ) {
    /* ancienne -> SI puis SI -> nouvelle donc 1 nouveau = f_nouveau_SI / f_ancien_v_SI ancien */
    facteur = f_ancien_v_SI / f_nouveau_v_SI;
  } else {
    facteur = 0.0;
  }

  if ( facteur != 0.0 ) {
    /* On demande confirmation */
    int reponse = 1;

    (void) fprintf(stderr, "%s,%s l.%u : Ancienne unit = %s, nouvelle = %s, facteur = %g\n",
		   __FILE__, __FUNCTION__, __LINE__, ancienne_unite, nouvelle_unite, facteur );

    reponse = BI_ERR_alerte( principale, &MSG_ValiderConversion, VRAI, ancienne_unite, nouvelle_unite, ancienne_unite, facteur, nouvelle_unite );
    if ( 2 == reponse ) {
      /* On annule la conversion */
      return +1;
    }

    (void) fprintf(stderr, "%s,%s l.%u : Ancienne unit = %s, nouvelle = %s, facteur = %g\n",
		   __FILE__, __FUNCTION__, __LINE__, ancienne_unite, nouvelle_unite, facteur );
    /* On installe la nouvelle unit
       AVANT le calcul, car le calcul met  jour => on rcupre l'ancienne unit
       si la nouvelle est le champ affichant l'unit dans la liste des graphes ! */
    retour = GRF_ChangerUnite( graphe, axe, nouvelle_unite );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit ! ATTENTION\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }

    /* On fait la conversion proprement dite */
    GRF_FacteurMultiplicatif( graphe, facteur, axe, NULL );
  } else {
    OPTION_A_FAIRE( "Conversion avec une formule indique par l'utilisateur" );
    return -10;
  }

  /* Conversion russie */
  return 0;
}

/* -------- Conversion des units pour une image -------- */

int I_IMG_ConvertirUnites(struct image *image, unsigned int axe, const char *nouvelle_unite)
{
  const char *ancienne_unite;
  double f_ancien_v_SI, f_nouveau_v_SI, facteur;
  int type_ancienne, type_nouvelle;
  int retour;

  /* Contrles */
  if ( NULL == image ) {
    AVERTISSEMENT( "Appel avec image=NULL, axe=%u, unite=%s", axe, nouvelle_unite );
    return -1;
  }
  if ( NULL == nouvelle_unite ) {
    AVERTISSEMENT( "Appel avec nouvelle_unite=NULL, axe=%u", axe );
  }

  /* Quelle est l'unit d'origine ? [1 ancien = f_ancien_v_SI SI] */
  ancienne_unite = IMG_QuelleUnite( image, axe );
  f_ancien_v_SI  = TXT_NormaliserUnite( ancienne_unite, VRAI, &type_ancienne, NULL );

  /* Quelle est la nouvelle unit ? [1 nouveau = f_nouveau_v_SI SI] */
  f_nouveau_v_SI = TXT_NormaliserUnite( nouvelle_unite, VRAI, &type_nouvelle, NULL );

  /* Les deux units sont-elles compatibles ? */
  if ( type_ancienne != type_nouvelle ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &MSG_GrandeursDifferentes, VRAI,
			     ancienne_unite, TXT_LegendeParDefaut( type_ancienne ),
			     nouvelle_unite, TXT_LegendeParDefaut( type_nouvelle ) );
    if ( 2 == reponse ) {
      /* On annule la conversion */
      return +1;
    }
  }

  /* On calcule le facteur de conversion */
  if ( f_nouveau_v_SI != 0.0 ) {
    /* ancienne -> SI puis SI -> nouvelle donc 1 nouveau = f_nouveau_SI / f_ancien_v_SI ancien */
    facteur = f_ancien_v_SI / f_nouveau_v_SI;
  } else {
    facteur = 0.0;
  }

  if ( facteur != 0.0 ) {
    /* On demande confirmation */
    int reponse = 1;

    (void) fprintf(stderr, "%s,%s l.%u : Ancienne unit = %s, nouvelle = %s, facteur = %g\n",
		   __FILE__, __FUNCTION__, __LINE__, ancienne_unite, nouvelle_unite, facteur );

    reponse = BI_ERR_alerte( principale, &MSG_ValiderConversion, VRAI, ancienne_unite, nouvelle_unite, ancienne_unite, facteur, nouvelle_unite );
    if ( 2 == reponse ) {
      /* On annule la conversion */
      return +1;
    }

    (void) fprintf(stderr, "%s,%s l.%u : Ancienne unit = %s, nouvelle = %s, facteur = %g\n",
		   __FILE__, __FUNCTION__, __LINE__, ancienne_unite, nouvelle_unite, facteur );
    /* On installe la nouvelle unit
       AVANT le calcul, car le calcul met  jour => on rcupre l'ancienne unit
       si la nouvelle est le champ affichant l'unit dans la liste des graphes ! */
    retour = IMG_ChangerUnite( image, axe, nouvelle_unite );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit ! ATTENTION\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -10;
    }

    /* On fait la conversion proprement dite */
    IMG_FacteurMultiplicatif( image, facteur, axe );
  } else {
    OPTION_A_FAIRE( "Conversion avec une formule indique par l'utilisateur" );
    return -10;
  }

  /* Conversion russie */
  return 0;
}

/* ~-~-~-~-~-~-~-~-~ Lgendes par dfaut associes aux units ~-~-~-~-~-~-~-~-~ */

const char *TXT_LegendeParDefaut(int type_grandeur)
{
  if ( ( type_grandeur >= 0 ) && ( type_grandeur < UNITE_EQUIVALENT_DOSE ) ) {
    return noms_grandeurs_physiques[ type_grandeur ];
  }

  return NULL;
}
