/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2006, Emmanuel CURIS
   Cration : 20 janvier 2003

   Gestion des cartographies obtenues par fluorescence X (y compris micro-XANES)

   Gestion de la liste des images
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "exafs.h"
#include "interface.h" /* Pour ARB_PrepareListeImages() */
#include "fichiers.h"
#include "fonctions.h"
#include "graphes.h" /* Pour accder au nom du graphe / profil */
#include "images.h"
#include "tracer.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LGMAX_NOM_SERIE     45	/* Caractres pour le nom de la srie  */
#define LGMAX_TXT_IMG_LISTE 32  /* Caractres dans la liste des images */
#define LGMAX_NOM_PROFIL    34  /* Caractres pour le nom du profil    */
#define LGMAX_TYPE_IMAGE    40	/* Caractres pour le type de l'image  */
#define LGMAX_GENRE_IMAGE   21	/* Caractres pour le genre de l'image */

#define NOMBRE_IMAGES_VISIBLES ( LI_DER_IMAGE - LI_PREM_IMAGE + 1 )

static unsigned long numero_prem_img_affichee = 0; /* Index de la premire image affiche */

/* Les options pour la cration d'images  */

#define LIMG_CREER_SERIE       0
#define LIMG_CREER_FONCTION    2 /* 1 : sparateur */
#define LIMG_CREER_IMAGE       3
#define LIMG_CREER_ALEATOIRE   4
#define LIMG_CREER_CONTINGENCE 5
#define LIMG_CREER_ANVA2F      6

#define LIMG_NOMBRE_OPT_CREATION 7

/* Les options valables pour les sries */

#define LIMG_SERIE_SAUVER    0
#define LIMG_SERIE_EXPORTER  1	/* 2 : sparateur */
#define LIMG_SERIE_EFFACER   3
#define LIMG_SERIE_RENOMMER  4	/* 5 : sparateur */
#define LIMG_SERIE_COPIER    6
#define LIMG_SERIE_DEPLACER  7
#define LIMG_SERIE_ISOLER    8	/* 9 : sparateur */
#define LIMG_SERIE_LOT      10

#define LIMG_NOMBRE_OPT_SERIE_IMG 11

#include "int_lst_images.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int LIMG_GererArbre(unsigned int numero_objet, int x, int y);

static void          LIMG_AfficheListe(void);
static void          LIMG_CouperOptions(boolean couper);
#ifdef _DEVELOPPE_
static unsigned long LIMG_NombreImages(Image *premiere);
#endif
static Image *       LIMG_TrouverImage(Image *premiere, unsigned long numero);

/* Les fonctions de gestion des sous-menus */

static void LIMG_GererMenuOpt_Creation (unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LIMG_GererMenuOpt_Generique(unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void LIMG_GererMenuOpt_Serie    (unsigned int num_arbre, unsigned int num_objet, unsigned long action);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Gestion proprement dite ~-~-~-~-~-~-~-~- */

/* -------- Gestion de l'arbre de la liste des images ------------ */

static int LIMG_GererArbre(unsigned int numero_objet, int x, int y)
{
  int etat, boutons;

  /* Bouton de cration : marche n'importe quand ! */
  if ( LI_NV_SERIE == numero_objet ) {
    /* Gestion du sous-menu : automatique -> on n'a pas encore trait ! */
    return BI_EVENEMENT_NON_TRAITE;
  }

  /* Tous les autres boutons n'ont de sens que s'il existe au moins une srie d'image */
  if ( NULL == IMG_serie_courante ) {
    return BI_EVENEMENT_TRAITE;
  }

  /* Plus d'infos : bouton cliqu, touches spciales,... */
  BI_EtatSouris( NULL, NULL, &boutons, NULL, &etat );

  if ( LI_SERIE == numero_objet ) {
    if ( 2 == boutons ) {
      /* On simule un clic sur "Renommer"... */
      LIMG_GererMenuOpt_Serie( 0, 0, LIMG_SERIE_RENOMMER );
    } else {
      Serie_Images *serie = NULL;

      serie = I_SERIE_IMG_Choisir( CHOIX_IMAGE, LI_SERIE, LGMAX_NOM_SERIE );
      if ( serie != NULL ) {
	IMG_AllerSerie( serie );
      }
    }
    return BI_EVENEMENT_TRAITE;
  }

  /* Tous les autres boutons n'ont de sens que s'il existe au moins une image dans la srie courante */
  if ( ( NULL == image_courante ) || ( NULL == IMG_serie_courante->premiere_image ) ) {
    if ( LI_OPTIONS_SERIE == numero_objet ) return BI_EVENEMENT_NON_TRAITE;
    return BI_EVENEMENT_TRAITE;
  }

  /* On enregistre les paramtres de l'image courante */
  LIMG_EnregistrerParametres();

  if ( ( numero_objet >= LI_PREM_IMAGE ) && ( numero_objet <= LI_DER_IMAGE ) ) {
    /* Gestion du dfileur d'images */
    Image *image;

    image = LIMG_TrouverImage( IMG_serie_courante->premiere_image, numero_prem_img_affichee + ( numero_objet - LI_PREM_IMAGE ) );
    if ( NULL == image ) {
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s, %s l.%u : impossible de trouver l'image clique...\n",
		     __FILE__, __FUNCTION__, __LINE__);
#endif
      return BI_EVENEMENT_TRAITE;
    }

    IMG_AllerImage( image );
    return BI_EVENEMENT_TRAITE;
  }

  switch ( numero_objet )
  {
  case CI_IMG_AVANT:
  case CI_ASC_FOND:
  case CI_ASC_BOUTON:
  case CI_IMG_APRES:
    numero_prem_img_affichee = ARB_ascenseur( CHOIX_IMAGE, CI_IMG_AVANT, CI_ASC_FOND, CI_ASC_BOUTON, CI_IMG_APRES, ASC_VERTICAL,
					      IMG_serie_courante->nbr_images, NOMBRE_IMAGES_VISIBLES, numero_prem_img_affichee,
					      numero_objet, x, y, ECHANGE );
    LIMG_AfficheListe();
    return BI_EVENEMENT_TRAITE;

  case LI_CONVERTIR_X:
    OBJ_choisit( CHOIX_IMAGE, numero_objet, FORCE_OUI );
    I_IMG_ConvertirUnites( image_courante, AXE_X,
			   lit_chaine( CHOIX_IMAGE, LI_UNITE_X, 0, NULL, TXT_REMPLACE ) );
    OBJ_choisit( CHOIX_IMAGE, numero_objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

  case LI_GENRE_IMAGE:
    OBJ_choisit( CHOIX_IMAGE, numero_objet, FORCE_OUI );
    I_IMG_ChangerGenre( image_courante );
    OBJ_choisit( CHOIX_IMAGE, numero_objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

/*   case LI_COPIE_IMAGE: /\* Copier l'image courante *\/ */
/*     IMG_AllerImage( IMG_CopierImage( image_courante, NULL, FAUX ) ); */
/*     return VRAI; */

/*   case LI_SANS_GRILLE: */
/*   case LI_GRILLE_LIGNES: */
/*   case LI_GRILLE_POINTS: */
/*     OBJ_choisit(CHOIX_IMAGE, objet, FORCE_OUI); */
/*     FEN_redessin(fen_images, BI_RDS_UTIL); */
/*     OPTION_A_FAIRE( "Mmoriser le type d'affichage dans l'image [?]" ); */
/*     return VRAI; */
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Mmoriser les paramtres modifiss -------- */

void LIMG_EnregistrerParametres(void)
{
  char *nv_nom, *nv_legende_X, *nv_unite_X, *nv_legende_Y, *nv_unite_Y, *nv_legende_Z, *nv_unite_Z;
  const char *ancien_texte;
  int retour = 0;

  if ( NULL == image_courante ) return;	 /* Pas d'image courante : pas de modification  enregistrer ! */

  if ( FAUX == arbres[ CHOIX_IMAGE ].drapeaux.ouvert ) {
    /* Si la bote est ferme, l'utilisateur n'a pas pu modifier les valeurs => on se dispense de les relire
       D'autant plus que si elle n'a jamais t ouverte, les FTEXT ne sont pas convertis en textes infinis
       ==> on aurait une troncature des noms, lgendes, units...
     */
    return; /* Rien  faire ! */
  }

  /* Le titre */
  /* 1. On rcupre les valeurs entres */
  nv_nom       = lit_chaine( CHOIX_IMAGE, LI_NOM_IMAGE, 0, NULL, TXT_REMPLACE );
  nv_legende_X = lit_chaine( CHOIX_IMAGE, LI_LEGENDE_X, 0, NULL, TXT_COPIER );
  nv_unite_X   = lit_chaine( CHOIX_IMAGE, LI_UNITE_X  , 0, NULL, TXT_COPIER );
  nv_legende_Y = lit_chaine( CHOIX_IMAGE, LI_LEGENDE_Y, 0, NULL, TXT_COPIER );
  nv_unite_Y   = lit_chaine( CHOIX_IMAGE, LI_UNITE_Y  , 0, NULL, TXT_COPIER );
  nv_legende_Z = lit_chaine( CHOIX_IMAGE, LI_LEGENDE_Z, 0, NULL, TXT_COPIER );
  nv_unite_Z   = lit_chaine( CHOIX_IMAGE, LI_UNITE_Z  , 0, NULL, TXT_COPIER );

  /* 2. Maintenant seulement, on modifie
     En effet, si un texte a chang, en l'enregistrant
     on remet les anciennes valeurs dans l'arbre ==> on perd les suivants !
     C'est pour cela aussi que l'on travaille avec une COPIE de ces chanes, qu'il faudra librer ensuite !
  */

  /* Le titre */
  if ( NULL == nv_nom  ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans le nouveau nom de l'image !\n",
		   __FILE__, __FUNCTION__, __LINE__);
  } else if ( NULL == image_courante->nom ) {
    /* L'image n'avait pas de nom : on met celui propos, de toute faon */
    IMG_Titre( image_courante, nv_nom );
  } else {
    if ( strcmp( nv_nom, image_courante->nom ) != 0 ) { /* Il y a des diffrences */
      IMG_Titre( image_courante, nv_nom );
    }
  }

  /* La lgende en X */
  if ( NULL == nv_legende_X ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet lgende X\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    ancien_texte = IMG_QuelleLegende( image_courante, AXE_X );
    if ( NULL == ancien_texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme grave dans IMG_QuelleLegende [renvoie NULL !]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      if ( strcmp( nv_legende_X, ancien_texte ) != 0 ) { /* Lgende en X change */
	retour = IMG_ChangerLegende( image_courante, AXE_X, nv_legende_X );
	if ( retour != 0 ) {
	  (void) BI_ERR_alerte( principale, &ERR_ChangementLegende, VRAI, 'X' );
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer la lgende en X [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
#endif
	}
      }
    }
  }
  free( nv_legende_X ); nv_legende_X = NULL;

  /* L'unit en X */
  if ( NULL == nv_unite_X ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet unit X\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    retour = 0;
    ancien_texte = IMG_QuelleUnite( image_courante, AXE_X );
    if ( NULL == ancien_texte ) {
      /* On attribue une unit, il n'y en avait pas avant : pas de message particulier ! */
      retour = IMG_ChangerUnite( image_courante, AXE_X, nv_unite_X );
    } else {
      if ( strcmp( nv_unite_X, ancien_texte ) != 0 ) { /* Unit en X change */
	int reponse;

	reponse = BI_ERR_alerte( fen_images, &MSG_ChangerUnite, VRAI, 'X', ancien_texte, nv_unite_X );
	if ( 1 == reponse ) {
	  retour = IMG_ChangerUnite( image_courante, AXE_X, nv_unite_X );
	}
      }
    }
    if ( retour != 0 ) {
      (void) BI_ERR_alerte( fen_images, &ERR_ChangementUnite, VRAI, 'X' );
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit en X [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    }
  }
  free( nv_unite_X ); nv_unite_X = NULL;

  /* La lgende en Y */
  if ( NULL == nv_legende_Y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet lgende Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    ancien_texte = IMG_QuelleLegende( image_courante, AXE_Y );
    if ( NULL == ancien_texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme grave dans IMG_QuelleLegende [renvoie NULL !]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      if ( strcmp( nv_legende_Y, ancien_texte ) != 0 ) { /* Lgende en Y change */
	retour = IMG_ChangerLegende( image_courante, AXE_Y, nv_legende_Y );
	if ( retour != 0 ) {
	  (void) BI_ERR_alerte( fen_images, &ERR_ChangementLegende, VRAI, 'Y' );
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer la lgende en Y [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
#endif
	}
      }
    }
  }
  free( nv_legende_Y ); nv_legende_Y = NULL;

  /* L'unit en Y */
  if ( NULL == nv_unite_Y ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet unit Y\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    retour = 0;
    ancien_texte = IMG_QuelleUnite( image_courante, AXE_Y );
    if ( NULL == ancien_texte ) {
      /* On attribue une unit, il n'y en avait pas avant : pas de message particulier ! */
      retour = IMG_ChangerUnite( image_courante, AXE_Y, nv_unite_Y );
    } else {
      if ( strcmp( nv_unite_Y, ancien_texte ) != 0 ) { /* Unit en Y change */
	int reponse;

	reponse = BI_ERR_alerte( fen_images, &MSG_ChangerUnite, VRAI, 'Y', ancien_texte, nv_unite_Y );
	if ( 1 == reponse ) {
	  retour = IMG_ChangerUnite( image_courante, AXE_Y, nv_unite_Y );
	}
      }
    }
    if ( retour != 0 ) {
      (void) BI_ERR_alerte( fen_images, &ERR_ChangementUnite, VRAI, 'Y' );
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit en Y [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    }
  }
  free( nv_unite_Y ); nv_unite_Y = NULL;

  /* La lgende en Z */
  if ( NULL == nv_legende_Z ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet lgende Z\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    ancien_texte = IMG_QuelleLegende( image_courante, AXE_Z );
    if ( NULL == ancien_texte ) {
      (void) fprintf(stderr, "%s,%s l.%u : Problme grave dans IMG_QuelleLegende [renvoie NULL !]\n",
		     __FILE__, __FUNCTION__, __LINE__);
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    } else {
      if ( strcmp( nv_legende_Z, ancien_texte ) != 0 ) { /* Lgende en Z change */
	retour = IMG_ChangerLegende( image_courante, AXE_Z, nv_legende_Z );
	if ( retour != 0 ) {
	  (void) BI_ERR_alerte( fen_images, &ERR_ChangementLegende, VRAI, 'Z' );
#ifdef _DEVELOPPE_
	  (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer la lgende en Z [code %d]\n",
			 __FILE__, __FUNCTION__, __LINE__, retour);
#endif
	}
      }
    }
  }
  free( nv_legende_Z ); nv_legende_Z = NULL;

  /* L'unit en Z */
  if ( NULL == nv_unite_Z ) {
    (void) fprintf(stderr, "%s,%s l.%u : Problme dans l'objet unit Z\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
  } else {
    retour = 0;
    ancien_texte = IMG_QuelleUnite( image_courante, AXE_Z );
    if ( NULL == ancien_texte ) {
      /* On attribue une unit, il n'y en avait pas avant : pas de message particulier ! */
      retour = IMG_ChangerUnite( image_courante, AXE_Z, nv_unite_Z );
    } else {
      if ( strcmp( nv_unite_Z, ancien_texte ) != 0 ) { /* Unit en Z change */
	int reponse;

	reponse = BI_ERR_alerte( fen_images, &MSG_ChangerUnite, VRAI, 'Z', ancien_texte, nv_unite_Z );
	if ( 1 == reponse ) {
	  retour = IMG_ChangerUnite( image_courante, AXE_Z, nv_unite_Z );
	}
      }
    }
    if ( retour != 0 ) {
      (void) BI_ERR_alerte( fen_images, &ERR_ChangementUnite, VRAI, 'Z' );
#ifdef _DEVELOPPE_
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR en voulant changer l'unit en Z [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
#endif
    }
  }
  free( nv_unite_Z ); nv_unite_Z = NULL;
}

/* -------- Affichage de la liste des images dans le dfileur -------- */

void LIMG_RafraichirListe(boolean active_a_change)
{

  if ( NULL == IMG_serie_courante ) {
    OBJ_CopierTexte( CHOIX_IMAGE, LI_SERIE, txt_err_aucune_serie, LGMAX_TXT_IMG_LISTE );
  } else {
    OBJ_CopierTexte( CHOIX_IMAGE, LI_SERIE, IMG_serie_courante->nom, LGMAX_TXT_IMG_LISTE );
  }

  /* On commence par affiche la liste proprement dite */
  LIMG_AfficheListe();

  /* Doit-on aussi mettre  jour les infos sur l'image active ? */
  if ( ( VRAI == active_a_change ) && ( image_courante != NULL ) ) {
    /* Dimensions d'un pixel de l'image */
    OBJ_MettreNombre( CHOIX_IMAGE, LI_LARGEUR_CASE, image_courante->l_reel, 0.0 );
    OBJ_MettreNombre( CHOIX_IMAGE, LI_HAUTEUR_CASE, image_courante->h_reel, 0.0 );

    /* Nom de l'image */
    OBJ_CopierTexte( CHOIX_IMAGE, LI_NOM_IMAGE, image_courante->nom, 0 );

    /* Type & genre */
    OBJ_RemplacerTexte( CHOIX_IMAGE, LI_TYPE_IMAGE , IMG_TXT_Type ( image_courante->type  ), VRAI );
    OBJ_RemplacerTexte( CHOIX_IMAGE, LI_GENRE_IMAGE, IMG_TXT_Genre( image_courante->genre ), VRAI );

    /* Coupe associe, si elle existe */
    if ( image_courante->infos_affichage != NULL) {
      /* Les lgendes & units des axes */
      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_X, image_courante->infos_affichage->legende_X, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_X  , image_courante->infos_affichage->unite_X  , 0 );

      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_Y, image_courante->infos_affichage->legende_Y, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_Y  , image_courante->infos_affichage->unite_Y  , 0 );

      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_Z, image_courante->infos_affichage->legende_Z, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_Z  , image_courante->infos_affichage->unite_Z  , 0 );

      /* Le profil */
      if ( image_courante->infos_affichage->coupe != NULL ) {
	OBJ_CopierTexte( CHOIX_IMAGE, LI_PROFIL, image_courante->infos_affichage->coupe->nom, LGMAX_NOM_PROFIL );
      } else {
	OBJ_CopierTexte( CHOIX_IMAGE, LI_PROFIL, txt_err_aucun_profil, LGMAX_NOM_PROFIL );
      }
    } else {
      OBJ_CopierTexte( CHOIX_IMAGE, LI_PROFIL, txt_err_aucun_profil, LGMAX_NOM_PROFIL );

      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_X, txt_sans_legende, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_Y, txt_sans_legende, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_LEGENDE_Z, txt_sans_legende, 0 );

      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_X, txt_sans_unite, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_Y, txt_sans_unite, 0 );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_UNITE_Z, txt_sans_unite, 0 );
    }
  }
}

static void LIMG_AfficheListe(void)
{
  Image *image;
  unsigned long i = 0, nbr_images = 0;

  if ( NULL == IMG_serie_courante ) {
    OBJ_active( CHOIX_IMAGE, LI_SERIE, FORCE_NON );

    LIMG_CouperOptions( VRAI );
    return;
  }

  /* Si l'on est l, c'est qu'il y a forcment au moins une srie d'images */
  OBJ_active( CHOIX_IMAGE, LI_SERIE, FORCE_OUI );
  LIMG_CouperOptions( FAUX );

#ifdef _DEVELOPPE_
  nbr_images = LIMG_NombreImages( IMG_serie_courante->premiere_image );
  if ( nbr_images != IMG_serie_courante->nbr_images ) {
    (void) fprintf(stderr, "%s,%s l.%u : Incohrence, la srie courante contient %lu images et dit en contenir %lu\n",
		   __FILE__, __FUNCTION__, __LINE__, nbr_images, IMG_serie_courante->nbr_images);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }
#endif

  /* On initialise l'ascenseur */
  numero_prem_img_affichee = ARB_ascenseur( CHOIX_IMAGE, CI_IMG_AVANT, CI_ASC_FOND, CI_ASC_BOUTON, CI_IMG_APRES, ASC_VERTICAL,
					    nbr_images, NOMBRE_IMAGES_VISIBLES, numero_prem_img_affichee, 0, 0, 0, METTRE );

  /* On se place sur la premire image affiche dans le dfileur */
  image = LIMG_TrouverImage( IMG_serie_courante->premiere_image, numero_prem_img_affichee );
  if ( NULL == image ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s, %s l.%u : problme, premire image affiche = %lu non trouve !\n",
		   __FILE__, __FUNCTION__, __LINE__, numero_prem_img_affichee );
#endif
    numero_prem_img_affichee = 0;
    image = IMG_serie_courante->premiere_image;
  }

  /* On affiche les textes */
  for ( i = 0 ; i < NOMBRE_IMAGES_VISIBLES ; i++ ) {
    if ( image != NULL ) {
      OBJ_active( CHOIX_IMAGE, LI_PREM_IMAGE + i, FORCE_OUI );
      OBJ_choisit( CHOIX_IMAGE, LI_PREM_IMAGE + i, image == image_courante );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_PREM_IMAGE + i, image->nom, LGMAX_TXT_IMG_LISTE );
      image = image->suivante;
    } else {
      OBJ_active( CHOIX_IMAGE, LI_PREM_IMAGE + i, FORCE_NON );
      OBJ_choisit( CHOIX_IMAGE, LI_PREM_IMAGE + i, FORCE_NON );
      OBJ_CopierTexte( CHOIX_IMAGE, LI_PREM_IMAGE + i, "", LGMAX_TXT_IMG_LISTE );
    }
  }
}

/* ~-~-~-~-~-~-~-~- Routines utilitaires pour la gestion ~-~-~-~-~-~-~-~- */

/* -------- Couper les boutons lorsqu'il n'y a pas d'image [ou les remettre] -------- */

static void LIMG_CouperOptions(boolean couper)
{
  unsigned int i;

  /* On coupe les options dans la srie */
  OBJ_active( CHOIX_IMAGE, LI_OPTIONS_SERIE, !(couper) );

  /* On coupe la liste d'images */
  (void) ARB_ascenseur( CHOIX_IMAGE, CI_IMG_AVANT, CI_ASC_FOND, CI_ASC_BOUTON, CI_IMG_APRES, ASC_VERTICAL,
			0, NOMBRE_IMAGES_VISIBLES, 0, 0, 0, 0, METTRE );
  for ( i = 0 ; i < NOMBRE_IMAGES_VISIBLES ; i++ ) {
    OBJ_active( CHOIX_IMAGE, LI_PREM_IMAGE + i, FORCE_NON );
    OBJ_choisit( CHOIX_IMAGE, LI_PREM_IMAGE + i, FORCE_NON );
    OBJ_CopierTexte( CHOIX_IMAGE, LI_PREM_IMAGE + i, "", LGMAX_TXT_IMG_LISTE );
  }

  /* On coupe les axes */
  OBJ_active( CHOIX_IMAGE, LI_LEGENDE_X   , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_UNITE_X     , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_CONVERTIR_X , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_LARGEUR_CASE, !(couper) );

  OBJ_active( CHOIX_IMAGE, LI_LEGENDE_Y   , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_UNITE_Y     , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_CONVERTIR_Y , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_HAUTEUR_CASE, !(couper) );

  OBJ_active( CHOIX_IMAGE, LI_LEGENDE_Z  , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_UNITE_Z    , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_CONVERTIR_Z, !(couper) );

  /* On coupe les paramtres d'affichage */
  OBJ_active( CHOIX_IMAGE, LI_COULEUR   , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_OPT_GRILLE, !(couper) );

  /* On coupe les menus de commandes */
  OBJ_active( CHOIX_IMAGE, LI_OPT_GENERIQUE, !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_OPT_PAR_GENRE, !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_OPT_PAR_TYPE , !(couper) );

  /* On coupe le profil */
  OBJ_active( CHOIX_IMAGE, LI_PROFIL, !(couper) );

  /* On coupe le nom, le type, le genre */
  OBJ_active( CHOIX_IMAGE, LI_NOM_IMAGE  , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_TYPE_IMAGE , !(couper) );
  OBJ_active( CHOIX_IMAGE, LI_GENRE_IMAGE, !(couper) );
}

#ifdef _DEVELOPPE_
/* -------- Compter le nombre d'images aprs celle indique (incluse) -------- */

static unsigned long LIMG_NombreImages(Image *premiere)
{
  unsigned long nbr_images = 0;

  /* On parcourt la liste */
  while ( premiere != NULL ) {
    nbr_images++;
    premiere = premiere->suivante;
  }

  /* On a fini... */
  return nbr_images;
}
#endif

/* -------- Se placer sur l'image x crans aprs la premire -------- */

static Image *LIMG_TrouverImage(Image *premiere, unsigned long numero)
{
  Image *image;
  unsigned long compte = 0;

  if ( NULL == premiere ) {
#ifdef _DEVELOPPE_
    if ( numero > 0UL ) {
      (void) fprintf(stderr, "IMG_TrouverImage : valeurs incorrectes [numero = %lu, image = NULL]\n", numero);
    }
#endif
    return NULL;
  }

  image = premiere;
  while ( compte < numero ) {
    compte++;
    image = image->suivante;
    if ( NULL == image ) {
      (void) fprintf(stderr, "IMG_TrouverImage : impossible d'aller %lu images aprs l'image = \"%s\"]\n", numero, image->nom);
      return NULL;
    }
  }

  /* On a fini... */
  return image;
}

/* -------- Prparation de l'arbre de la liste des images -------- */

void LIMG_PreparerArbre(void)
{
  const char **table_options = NULL;
  unsigned long nombre_options = 0;

  /* Le slecteur de srie reste un bouton : facilite la gestion !
     En effet, le sous-menu appel doit tre gnr au vol pour rester  jour avec la table de sries...
  */

  /* Les slecteurs d'options peuvent tre convertis en sous-menus */
  OBJ_ChangerType( CHOIX_IMAGE, LI_NV_SERIE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_IMAGE, LI_NV_SERIE, LIMG_NOMBRE_OPT_CREATION, options_creation, titre_options, LIMG_GererMenuOpt_Creation );
  OBJ_RemplacerTexte( CHOIX_IMAGE, LI_NV_SERIE, titre_options_creation, FAUX );

  OBJ_ChangerType( CHOIX_IMAGE, LI_OPTIONS_SERIE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_IMAGE, LI_OPTIONS_SERIE, LIMG_NOMBRE_OPT_SERIE_IMG, options_serie_images, titre_options, LIMG_GererMenuOpt_Serie );
  OBJ_RemplacerTexte( CHOIX_IMAGE, LI_OPTIONS_SERIE, titre_options, FAUX );

  table_options = FCT_ListeFonctions_IMG( FONCTION_EST_GLOBALE, 0, 0, &nombre_options );
  OBJ_ChangerType( CHOIX_IMAGE, LI_OPT_GENERIQUE, G_SSMENU );
  OBJ_MettreMenu ( CHOIX_IMAGE, LI_OPT_GENERIQUE, nombre_options, table_options, titre_options, LIMG_GererMenuOpt_Generique );
  OBJ_RemplacerTexte( CHOIX_IMAGE, LI_OPT_GENERIQUE, texte_opt_generique, FAUX );

  /* Le slecteur de mode d'affichage aussi */

  /* Ensuite, on initialise */
  if ( FAUX == arbres[ CHOIX_IMAGE ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( CHOIX_IMAGE, NULL, LIMG_GererArbre, ( int (*)(unsigned int, int, int *) ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s, %s l.%u : ERREUR  l'initialisation de l'arbre [code %d]\n",
		     __FILE__, __FUNCTION__, __LINE__, retour);
      return;
    }
  }

  /* On enjolive en plaant mieux les objets */
  ARB_EnjoliverDefileur( CHOIX_IMAGE, 0, LI_PREM_IMAGE, LI_DER_IMAGE,
			 CI_IMG_AVANT, CI_ASC_FOND, CI_ASC_BOUTON, CI_IMG_APRES, ARB_ASCENSEUR_DROITE );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_UNITE_X  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_UNITE_X      , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_CONVERTIR_X  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_TXT_ECHELLE_X, 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_LARGEUR_CASE , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_SYMB_UNITE_X , 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_UNITE_Y  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_UNITE_Y      , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_CONVERTIR_Y  , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_TXT_ECHELLE_Y, 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_HAUTEUR_CASE , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_SYM_UNITE_Y , 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_UNITE_Z, 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_UNITE_Z    , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_CONVERTIR_Z, 0, +1, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_PROFIL, 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_PROFIL    , 0, -2, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_TYPE   , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_TYPE_IMAGE , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_TXT_GENRE  , 0, +3, DEPLACE_PIXELS );
  OBJ_deplace( CHOIX_IMAGE, LI_GENRE_IMAGE, 0, +3, DEPLACE_PIXELS );

  OBJ_deplace( CHOIX_IMAGE, LI_TXT_AXE_X, 0, +500, DEPLACE_RELATIF );
  OBJ_deplace( CHOIX_IMAGE, LI_TXT_AXE_Y, 0, +500, DEPLACE_RELATIF );
  OBJ_deplace( CHOIX_IMAGE, LI_TXT_AXE_Z, 0, +500, DEPLACE_RELATIF );

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_COULEURS, txt_arbre_li_couleur);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_APPLIQUER, txt_arbre_li_appliquer);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_EFF_IMAGE, txt_arbre_li_effacer);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_COPIE_IMAGE, txt_arbre_li_copier);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_PROFIL, txt_arbre_li_profil);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_UNIT_X, txt_arbre_li_echelle_x);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_GRILLE, txt_arbre_li_grille);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_SANS_GRILLE, txt_arbre_li_aucune);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_GRILLE_LIGNES, txt_arbre_li_lignes);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_GRILLE_POINTS, txt_arbre_li_points);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_UNIT_Y, txt_arbre_li_echelle_y);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_TXT_PALETTE, txt_arbre_li_palette);
  OBJ_TraduireTexte(CHOIX_IMAGE, LI_EDIT_PALETTE, txt_arbre_li_editer);
#endif

  /* On fait la prparation de la 1re liste */
  LIMG_RafraichirListe( VRAI );

  arbres[ CHOIX_IMAGE ].drapeaux.prepare = VRAI;
}

/* ~-~-~-~-~-~-~-~- Routines d'appel des diffrentes options (via les sous-menus) */

static void LIMG_GererMenuOpt_Creation(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Image *nouveau = NULL;

  switch ( action )
  {
  case LIMG_CREER_SERIE:
    I_SERIE_IMG_Creer( IMG_serie_courante, NULL );
    break;

  case LIMG_CREER_FONCTION:
    OPTION_A_FAIRE( "Crer une fonction simple Z=f(X, Y)" );
    break;

  case LIMG_CREER_IMAGE:
    OPTION_A_FAIRE( "Gnrer une image quelconque" );
/*     nouveau = I_IMG_Generer( image_courante ); */
    break;

  case LIMG_CREER_ALEATOIRE:
    nouveau = I_IMG_CreerAleatoire( image_courante );
    break;

  case LIMG_CREER_CONTINGENCE:
    OPTION_A_FAIRE( "Crer une table de contingence  2 variables" );
    break;

  case LIMG_CREER_ANVA2F:
    OPTION_A_FAIRE( "Crer une table de donnes  deux facteurs type ANVA" );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( nouveau != NULL ) {
    /* Cette nouvelle image devient l'image courante */
    IMG_AllerImage( nouveau );
  }
}

/* -------- Grer le sous-menu proposant des actions sur la srie courante -------- */

static void LIMG_GererMenuOpt_Serie(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  char *nv_nom;

  switch ( action )
  {
  case LIMG_SERIE_SAUVER:
    (void) I_SERIE_IMG_Sauver( IMG_serie_courante );
    break;

  case LIMG_SERIE_EXPORTER:
/*     (void) EA_SerieComplete_IMG( serie_courante ); */
    break;

  case LIMG_SERIE_EFFACER:
    OPTION_A_FAIRE( "Effacer la srie d'images !" );
    break;

  case LIMG_SERIE_RENOMMER:
    nv_nom = BI_Demande_Texte( NULL, &Q_NouveauNomSerie, IMG_serie_courante->nom) ;
    if ( nv_nom != NULL ) {
      IMG_SERIE_Titre( IMG_serie_courante, nv_nom );
      free( nv_nom ); nv_nom = NULL;
      OBJ_CopierTexte( CHOIX_IMAGE, LI_SERIE, IMG_serie_courante->nom, LGMAX_NOM_SERIE );
    }
    break;

  case LIMG_SERIE_COPIER:
    I_IMG_Copier( image_courante, VRAI );
    break;

  case LIMG_SERIE_DEPLACER:
    I_IMG_Deplacer( image_courante, VRAI );
    break;

  case LIMG_SERIE_ISOLER:
    I_IMG_Isoler( image_courante, VRAI );
    break;

  case LIMG_SERIE_LOT:
    OPTION_A_FAIRE("Traitement par lot [arbre : TRAITE_LOT]" );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }
}

/* -------- Grer le sous-menu proposant des actions sur l'image courante, quel que soit son type/genre -------- */

static void LIMG_GererMenuOpt_Generique(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Image *resultat = NULL;

  switch ( action )
  {
  case FCT_IMG_SAUVER_IMAGE:
    I_IMG_Sauver( image_courante );
    break;

  case FCT_IMG_EXPORTER_IMAGE:
    OPTION_A_FAIRE( "Interface pour l'export ASCII d'une image" );
/*     I_EXPORT_IMG_ASCII( image_courante ); */
    break;

  case FCT_IMG_EXPORT_POSTSCRIPT:
    OPTION_A_FAIRE( "Interface pour l'export PostScript d'une image" );
/*     I_EXPORT_PS_ImageUnique( image_courante ); */
    break;

  case FCT_IMG_EFFACER_IMAGE:
    I_IMG_Effacer( image_courante, VRAI );
    break;

  case FCT_IMG_COPIER_IMAGE:
    I_IMG_Copier( image_courante, FAUX );
    break;

  case FCT_IMG_DEPLACER_IMAGE:
    I_IMG_Deplacer( image_courante, FAUX );
    break;

  case FCT_IMG_ISOLER_IMAGE:
    I_IMG_Isoler( image_courante, FAUX );
    break;

  case FCT_IMG_ERREUR_V_IMAGE:
    resultat = I_IMG_SortirErreurs( image_courante, AXE_Z );
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer une nouvelle image */
    if ( image_courante == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage de l'image courante" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* La nouvelle image devient l'image courante */
      IMG_AllerImage( resultat );
    }
  }
}
