/* LASE - Logiciel d'Analyse des spectres EXAFS
   (C) 1997 - 2007, Emmanuel CURIS
   Copie de Atomes/liste.c, qui disparatra  terme

   Interface
   Liste des molcules (modles structuraux, atomes,...)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h> /* Pour isspace */
#include <string.h>

#include "systeme.h"
#include "exafs.h"
#include "fonctions.h"

#include "utils_3d.h" /*  FAIRE SAUTER QUAND LES ATOMES UTILISERONT Vecteur3D POUR LES COORDONNES */

#include "atomes.h"
#include "couches.h"  /* Pour modele_feff() ==>  faire disparatre un jour */
#include "fichiers.h" /* Pour chargement/sauvegarde des molcules */
#include "interface.h"
#include "modeles.h"
#include "tracer.h"

/* static const char *titre_lst_molecules = "Mol"e_AIGU"cules en m"e_AIGU"moire"; */

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

#define LG_MAX_ABS_DS_LISTE 19
#define LG_MAX_ATM_DS_LISTE 10
#define LG_MAX_FRG_DS_LISTE 25
#define LG_MAX_MOL_DS_LISTE 30

#define NBR_ABS_VISIBLES (LM_DERNIER_ABS   - LM_PREMIER_ABS   + 1)
#define NBR_ATM_VISIBLES (LM_DERNIER_ATOME - LM_PREMIER_ATOME + 1)
#define NBR_FRG_VISIBLES (LM_DERNIER_FRAG  - LM_PREMIER_FRG   + 1)
#define NBR_MOL_VISIBLES (LM_DER_MOLECULE  - LM_PREM_MOLECULE + 1)

static unsigned long ABS_premier_visible  = 0; /* Index du premier absorbeur affich */
static unsigned long ATM_premier_visible  = 0; /* Index du premier atome affich (0...nombre_atomes - 1) */
static unsigned long FRG_premier_visible  = 0; /* Index du premier fragment affich */
static unsigned long MOL_premiere_visible = 0; /* Index de la premire molcule affiche */

static unsigned long ATM_numero_courant   = 0; /* Index de l'atome actif */
static unsigned long FRG_fragment_courant = 0; /* Index du fragment actif; */

#define IMOL_CRIS_COPIER_MAILLE    0  /* Crer une nouvelle maille en crant l'origine */
#define IMOL_CRIS_EFFACER_MAILLE   1  /* Effacer une maille */
#define IMOL_CRIS_COMPLETER_MAILLE 3  /* Complter une maille par les atomes sur les faces externes */

#define IMOL_NOMBRE_OPT_CRISTALLO 4

#define IMOL_FRAG_EFFACER_FRAGMENT 0  /* Effacer le fragment courant */

#define IMOL_AFF_TOUS_LES_ATOMES    0 /* Afficher tous les atomes de la molcule */
#define IMOL_AFF_ATOMES_DU_FRAGMENT 1 /* Afficher les atomes du fragment courant */
#define IMOL_AFF_ATOMES_DU_MODELE   2 /* Afficher les atomes impliqus dans le modle actif */
#define IMOL_AFF_PAR_Z_INDIQUE      4 /* Afficher les atomes du Z indiqu [3 : sparateur] */
#define IMOL_AFF_LIES_A_ABSORBEUR   5 /* Afficher les atomes lis  l'absorbeur */
#define IMOL_AFF_PROCHES_ABSORBEUR  6 /* Afficher les atomes  moins d'une certaine distance de l'absorbeur */
#define IMOL_AFF_LISTE_INVALIDE     7 /* La liste n'est plus  jour, dans le cas d'un filtre complexe */

#define IMOL_NOMBRE_OPT_FILTRE_ATOMES 8

static char texte_affichage_nom_atome[ LG_MAX_ATM_DS_LISTE + 1 ];  /* 1 caractres de plus : 0 final */
static unsigned int mode_affichage_atomes = IMOL_AFF_TOUS_LES_ATOMES;  /* Par dfaut, on affiche tous les atomes */
static unsigned int Z_affichage_atomes = SYMBOLE_CHIMIQUE_INCONNU;

#define IMOL_ABS_EFFACER_ABSORBEUR 0  /* Effacer l'absorbeur courant */
#define IMOL_ABS_CONTRIBUTION      1  /* Paramtrer la contribution de cet absorbeur au total de la molcule */
#define IMOL_ABS_NOMBRE_OPT_GENERIQUES 2

#include "int_lst_molecules.txt"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* static void enregistre_atome(void); */

static const char *I_MOL_AbsorbeurVersTexte(unsigned int numero_arbre, unsigned long numero_objet);
static const char *I_MOL_AtomeVersTexte    (unsigned int numero_arbre, unsigned long numero_objet);
static const char *I_MOL_FragmentVersTexte (unsigned int numero_arbre, unsigned long numero_objet);
static const char *I_MOL_MoleculeVersTexte (unsigned int numero_arbre, unsigned long numero_objet);

static Atome *I_MOL_NumeroVersAtome(unsigned long numero_atome);

static int  I_MOL_GererListeMolecules       (unsigned int objet, int x, int y);
static void I_MOL_GererMenuOpt_Affichage_ATM(unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void I_MOL_GererMenuOpt_Generique_ABS(unsigned int num_arbre, unsigned int num_objet, unsigned long action);
static void I_MOL_GererMenuOpt_Generique_MOL(unsigned int num_arbre, unsigned int num_objet, unsigned long action);

static void I_MOL_CouperOptionsAbsorbeurs(boolean couper);
static void I_MOL_CouperOptionsAtomes    (boolean couper);
static void I_MOL_CouperOptionsFragments (boolean couper);
static void I_MOL_CouperOptionsMolecule  (boolean couper);

static void I_MOL_AfficherFragmentActif(Fragment *fragment);

static Fragment *I_MOL_TrouverFragmentActif(void);

static unsigned long I_MOL_CompterAtomesAAfficher(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~-~-~-~-~-~-~-~- Liste des molcules ~-~-~-~-~-~-~-~- */

/* -------- Prparation de l'arbre --------
   Pas static, car appel en tout dbut de programme pour initialiser les sous-menus
 */

void LMOL_PreparerArbre(void)
{
  if ( VRAI == arbres[ LISTE_MOLECULES ].drapeaux.prepare ) return;

  /* On convertit les options en sous-menus */
  OBJ_ChangerType( LISTE_MOLECULES, LM_OPT_MOLECULE, G_SSMENU );
  OBJ_MettreMenu ( LISTE_MOLECULES, LM_OPT_MOLECULE, FCT_MOL_NOMBRE_OPT_GENERIQUES, options_molecule,
		   titre_options, I_MOL_GererMenuOpt_Generique_MOL );
  OBJ_RemplacerTexte( LISTE_MOLECULES, LM_OPT_MOLECULE, titre_options, FAUX );

  OBJ_ChangerType( LISTE_MOLECULES, LM_FILTRE_ATM, G_SSMENU );
  OBJ_MettreMenu ( LISTE_MOLECULES, LM_FILTRE_ATM, IMOL_NOMBRE_OPT_FILTRE_ATOMES, options_filtre_atome,
		   NULL, I_MOL_GererMenuOpt_Affichage_ATM );

  OBJ_ChangerType( LISTE_MOLECULES, LM_OPT_GERE_ABS, G_SSMENU );
  OBJ_MettreMenu ( LISTE_MOLECULES, LM_OPT_GERE_ABS, IMOL_ABS_NOMBRE_OPT_GENERIQUES, options_absorbeur,
		   titre_options, I_MOL_GererMenuOpt_Generique_ABS );
  OBJ_RemplacerTexte( LISTE_MOLECULES, LM_OPT_MOLECULE, titre_options, FAUX );

  /* On initialise */
  if ( FAUX == arbres[ LISTE_MOLECULES ].drapeaux.init ) {
    int retour;

    OPTION_A_FAIRE( "Documentation, liste des molcules disponibles" );
    retour = ARB_Initialiser( LISTE_MOLECULES, NULL, I_MOL_GererListeMolecules, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return;
    }
  }

  /* Amliorations esthtiques */
  ARB_EnjoliverDefileur( LISTE_MOLECULES, ROOT, LM_PREM_MOLECULE, LM_DER_MOLECULE,
			 LM_MOL_AVANT, LM_FOND_ASC_MOL, LM_BOUT_ASC_MOL, LM_MOL_APRES, ARB_ASCENSEUR_DROITE );
  ARB_EnjoliverDefileur( LISTE_MOLECULES, LM_FND_CHX_FRG, LM_PREMIER_FRG, LM_DERNIER_FRAG,
			 LM_FRG_AVANT, LM_FRG_FND_ASC, LM_FRG_BOUTON, LM_FRG_APRES, ARB_ASCENSEUR_DROITE );
  ARB_EnjoliverDefileur( LISTE_MOLECULES, LM_FND_LST_ATOME, LM_PREMIER_ATOME, LM_DERNIER_ATOME,
			 LM_ATOME_AVANT, LM_FND_ASC_ATOME, LM_BOUTON_ATOME, LM_ATOME_APRES, ARB_ASCENSEUR_DROITE );
  ARB_EnjoliverDefileur( LISTE_MOLECULES, ROOT, LM_PREMIER_ABS, LM_DERNIER_ABS,
			 LM_ABS_AVANT, LM_ABS_FOND_ASC, LM_ABS_BOUT_ASC, LM_ABS_APRES, ARB_ASCENSEUR_DROITE );

  OBJ_deplace( LISTE_MOLECULES, LM_TXT_FRAGMENTS, 0, +500, DEPLACE_RELATIF );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_ATOMES   , 0, +500, DEPLACE_RELATIF );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_ABSORBEUR, 0, +500, DEPLACE_RELATIF );

  OBJ_deplace( LISTE_MOLECULES, LM_NOM_ELEMENT  , 0, -3, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_NUM_ATM  , 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_NUMERO_ATOMIK, 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_CHARGE   , 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_CHARGE       , 0, -2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_X        , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_COORD_X      , 0, -1, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_Z        , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_COORD_Z      , 0, +1, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_TXT_RAYON    , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_RAYON_ATOME  , 0, +2, DEPLACE_PIXELS );
  OBJ_deplace( LISTE_MOLECULES, LM_PROPOSER_R   , 0, +2, DEPLACE_PIXELS );

  OBJ_deplace( LISTE_MOLECULES, LM_OPTIONS_ABS, 0, +2, DEPLACE_PIXELS );

#ifdef AVEC_TRADUCTION_
  OBJ_TraduireTexte( LISTE_MOLECULES, , txt_lst_mol_ );
#endif

  /* L'arbre est maintenant prs... */
  arbres[ LISTE_MOLECULES ].drapeaux.prepare = VRAI;

  /* On initialise brutalement le tout */
  MOL_PrepareListe( );
}

/* -------- Affichage de la liste des molcules & des options de la molcule courante -------- */

void MOL_PrepareListe(void)
{
  unsigned long nbr_molecules = 0, numero_molecule_courante = 0;
  Molecule *molecule;

  if ( NULL == premiere_molecule ) { /* Pas de molcule en mmoire */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREM_MOLECULE, LM_DER_MOLECULE, LG_MAX_MOL_DS_LISTE,
			    LM_MOL_AVANT, LM_FOND_ASC_MOL, LM_BOUT_ASC_MOL, LM_MOL_APRES,
			    0, NULL, 0, 0, I_MOL_MoleculeVersTexte );

    I_MOL_CouperOptionsMolecule( VRAI );
    return;
  }
  if ( NULL == mol_courante ) {
    /* Se produit quand on cre la 1re molcule...
       Fait ici pour tre fait en toute fin de cration, quand elle est pleine...
       Sinon, plus possible de mettre  jour facilement...
     */
    MOL_aller( premiere_molecule );
  }

  /* Il y a au moins une molcule : on peut y utiliser des options... */
  I_MOL_CouperOptionsMolecule( FAUX );
    
  /* On compte le nombre de molcules */
  for ( molecule = premiere_molecule ; molecule ; molecule = molecule->suivante, nbr_molecules++ ) {
    if ( mol_courante == molecule ) {
      /* Et on repre au passage l'index de la molcule courante */
      numero_molecule_courante = nbr_molecules;
    }
  }

  /* On s'en sert pour afficher le dfileur */
  ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREM_MOLECULE, LM_DER_MOLECULE, LG_MAX_MOL_DS_LISTE,
			  LM_MOL_AVANT, LM_FOND_ASC_MOL, LM_BOUT_ASC_MOL, LM_MOL_APRES,
			  nbr_molecules, NULL, numero_molecule_courante, MOL_premiere_visible, I_MOL_MoleculeVersTexte );

  /* Infos pour la molcule active */
  OBJ_CopierTexte( LISTE_MOLECULES, LM_NOM_MOLECULE, mol_courante->nom, 0 );

  /* Possde-t-elle des infos cristallographiques ? */
  if ( mol_courante->maille != NULL ) {
    OBJ_choisit( LISTE_MOLECULES, LM_EST_CRISTALLO, FORCE_OUI );
    OBJ_active( LISTE_MOLECULES, LM_OPT_CRISTAL , FORCE_OUI );
    OBJ_active( LISTE_MOLECULES, LM_PARAMS_CRIST, FORCE_OUI );
  } else {
    OBJ_choisit( LISTE_MOLECULES, LM_EST_CRISTALLO, FORCE_NON );
    OBJ_active( LISTE_MOLECULES, LM_OPT_CRISTAL , FORCE_NON );
    OBJ_active( LISTE_MOLECULES, LM_PARAMS_CRIST, FORCE_NON );
  }

  /* Possde-t-elle des atomes ? */
  if ( NULL == mol_courante->premier ) {
    I_MOL_CouperOptionsAtomes( VRAI );
    OBJ_active( LISTE_MOLECULES, LM_FILTRE_ATM, FORCE_NON );  /* Pas de filtre d'affichage ! */
  } else {
    ATM_PrepareListe();
    I_MOL_CouperOptionsAtomes( FAUX );
    OBJ_active( LISTE_MOLECULES, LM_FILTRE_ATM, FORCE_OUI );  /* Filtre d'affichage possible */
  }

  /* On fait la liste des fragments [adapte les options au passage] */
  I_MOL_RefaireListeFragments();

  /* Possde-t-elle des absorbeurs ? */
  if ( NULL == mol_courante->absorbeurs ) {
    I_MOL_CouperOptionsAbsorbeurs( VRAI );
  } else {
    ABS_PrepareListe();
    I_MOL_CouperOptionsAbsorbeurs( FAUX );
  }
}

/* Conversion molcule -> texte dans la liste */

static const char *I_MOL_MoleculeVersTexte(unsigned int numero_arbre __attribute__((__unused__)), unsigned long numero_objet)
{
  Molecule *molecule;
  unsigned long i = 0;

  if ( NULL == premiere_molecule ) return NULL;

  for ( i = 0, molecule = premiere_molecule ; i < numero_objet ; i++, molecule = molecule->suivante ) {
    if ( NULL == molecule ) {
      AVERTISSEMENT( "Pas de molcule n%lu", numero_objet );
      return NULL;
    }
  }

  return molecule->nom;
}

/* --- Macros de coupure/activation des objets --- */

static void I_MOL_CouperOptionsMolecule(boolean couper)
{
  /* Le bouton "crer" est toujours accessible. Les autres non en l'absence de molcule ! */
  /* Le dfileur est gr par la fonction gnrique : pas trait ici */

  OBJ_active( LISTE_MOLECULES, LM_NOM_MOLECULE , !couper ); /* Nom de la molcule courante */
  OBJ_active( LISTE_MOLECULES, LM_OPT_MOLECULE , !couper ); /* Options d'une molcule */
  OBJ_active( LISTE_MOLECULES, LM_TYPE_MOLECULE, !couper ); /* Options d'un type de molcule */
  OBJ_active( LISTE_MOLECULES, LM_EST_CRISTALLO, !couper ); /* Paramtres cristallins prsents ? */
  OBJ_active( LISTE_MOLECULES, LM_OPT_GERE_ATM , !couper ); /* Cration d'un nouvel atome */

  if ( VRAI == couper ) {
    /* Ceux-ci sont coups en l'absence de molcule, mais pas forcment actifs en prsence de molcule */
    OBJ_active( LISTE_MOLECULES, LM_PARAMS_CRIST, FORCE_NON );
    OBJ_active( LISTE_MOLECULES, LM_OPT_CRISTAL , FORCE_NON );
    OBJ_active( LISTE_MOLECULES, LM_OPT_TYPE_MOL, FORCE_NON );

    OBJ_active( LISTE_MOLECULES, LM_FILTRE_ATM, FORCE_NON );

    /* On coupe tout pour les fragments */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_FRG, LM_DERNIER_FRAG, LG_MAX_FRG_DS_LISTE,
			    LM_FRG_AVANT, LM_FRG_FND_ASC, LM_FRG_BOUTON, LM_FRG_APRES,
			    0, NULL, 0, 0, I_MOL_MoleculeVersTexte );
    OBJ_active( LISTE_MOLECULES, LM_OPT_FRG, FORCE_NON );
    I_MOL_CouperOptionsFragments( VRAI );

    /* On coupe tout pour les atomes */
    I_MOL_CouperOptionsAtomes( VRAI );
    OBJ_active( LISTE_MOLECULES, LM_FILTRE_ATM, FORCE_NON ); /* Le filtre n'a pas lieu d'tre ! */

    /* On coupe tout pour les absorbeurs */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_ABS, LM_DERNIER_ABS, LG_MAX_ABS_DS_LISTE,
			    LM_ABS_AVANT, LM_ABS_FOND_ASC, LM_ABS_BOUT_ASC, LM_ABS_APRES,
			    0, NULL, 0, 0, I_MOL_AbsorbeurVersTexte );
    OBJ_active( LISTE_MOLECULES, LM_OPT_GERE_ABS, FORCE_NON );
    I_MOL_CouperOptionsAbsorbeurs( VRAI );
  }
}

/* -------- Gestion de l'arbre -------- */

static int I_MOL_GererListeMolecules(unsigned int objet, int x __attribute__((__unused__)), int y)
{
  Molecule *molecule;
  unsigned long nbr_molecules = 0, n_mol_courante = 0;

  /* On peut toujours crer une molcule */
  if ( LM_OPT_GESTION == objet ) {
    molecule = I_MOL_CreerMolecule( mol_courante );
    MOL_aller( molecule );
    return BI_EVENEMENT_TRAITE;
  }

  /* Ensuite, si aucune molcule, rien  faire ! */
  if ( NULL == premiere_molecule ) {
    return BI_EVENEMENT_NON_TRAITE;
  }

  /* On compte les molcules et on trouve le numro de la courante */
  for ( molecule = premiere_molecule ; molecule != NULL ; molecule = molecule->suivante, nbr_molecules++ ) {
    if ( mol_courante == molecule ) {
      n_mol_courante = nbr_molecules;
    }
  }

  /* Le dfileur de molcules [dont changement de molcule courante ventuel...] */
  if ( ( ( objet >= LM_PREM_MOLECULE ) && ( objet <= LM_DER_MOLECULE ) ) ||
       ( LM_MOL_AVANT == objet ) || ( LM_FOND_ASC_MOL == objet ) || ( LM_BOUT_ASC_MOL == objet ) || ( LM_MOL_APRES == objet ) ) {
    unsigned int i = 0;

    /* On enregistre le nom */
    MOL_RenommerMolecule( mol_courante, lit_chaine( LISTE_MOLECULES, LM_NOM_MOLECULE, 0, NULL, TXT_REMPLACE ) );

    /* On gre le clic */
    ARB_GererDefileur_O( LISTE_MOLECULES, LM_PREM_MOLECULE, LM_DER_MOLECULE, LG_MAX_MOL_DS_LISTE,
			 LM_MOL_AVANT, LM_FOND_ASC_MOL, LM_BOUT_ASC_MOL, LM_MOL_APRES,
			 nbr_molecules, NULL, &n_mol_courante, &MOL_premiere_visible, I_MOL_MoleculeVersTexte, objet, y );

    /* On rcupre la nouvelle molcule courante */
    molecule = premiere_molecule;
    while ( i < n_mol_courante ) {
      molecule = molecule->suivante;
      i++;
    }
    if ( molecule != mol_courante ) {
      MOL_aller( molecule );
    }

    return BI_EVENEMENT_TRAITE;
  }

  /* Le dfileur des fragments [dont changement du fragment actif, s'il y en a un] */
  if ( ( ( objet >= LM_PREMIER_FRG ) && ( objet <= LM_DERNIER_FRAG ) ) ||
       ( LM_FRG_AVANT == objet ) || ( LM_FRG_FND_ASC == objet ) || ( LM_FRG_BOUTON == objet ) || ( LM_FRG_APRES == objet ) ) {
    Fragment *fragment_actif = NULL;

    /* On enregistre les paramtres du fragment actif */
    fragment_actif = I_MOL_TrouverFragmentActif();

    /* On gre le clic */
    ARB_GererDefileur_O( LISTE_MOLECULES, LM_PREMIER_FRG, LM_DERNIER_FRAG, LG_MAX_FRG_DS_LISTE,
			 LM_FRG_AVANT, LM_FRG_FND_ASC, LM_FRG_BOUTON, LM_FRG_APRES,
			 mol_courante->nbr_fragments, NULL, &(FRG_fragment_courant), &FRG_premier_visible, I_MOL_FragmentVersTexte,
			 objet, y );

    /* On affiche les paramtres du nouveau fragment courant */
    fragment_actif = I_MOL_TrouverFragmentActif();
    I_MOL_AfficherFragmentActif( fragment_actif );

    /* Dfileur trait... */
    return BI_EVENEMENT_TRAITE;
  }

  /* Le dfileur des absorbeurs [dont changement d'absorbeur actif ventuel] */
  if ( ( ( objet >= LM_PREMIER_ABS ) && ( objet <= LM_DERNIER_ABS ) ) ||
       ( LM_ABS_AVANT == objet ) || ( LM_ABS_FOND_ASC == objet ) || ( LM_ABS_BOUT_ASC == objet ) || ( LM_ABS_APRES == objet ) ) {
    Absorbeur *absorbeur;
    unsigned long numero_absorbeur;

    /* On enregistre les paramtres de l'ancien absorbeur actif */    
    absorbeur = mol_courante->absorbeurs + mol_courante->absorbeur_courant;
    MOL_RenommerAbsorbeur( absorbeur, lit_chaine( LISTE_MOLECULES, LM_NOM_ABSORBEUR, 0, NULL, TXT_REMPLACE) );

    /* On gre le clic */
    numero_absorbeur = mol_courante->absorbeur_courant;
    ARB_GererDefileur_O( LISTE_MOLECULES, LM_PREMIER_ABS, LM_DERNIER_ABS, LG_MAX_ABS_DS_LISTE,
			 LM_ABS_AVANT, LM_ABS_FOND_ASC, LM_ABS_BOUT_ASC, LM_ABS_APRES,
			 mol_courante->nbr_absorbeurs, NULL, &(numero_absorbeur), &ABS_premier_visible, I_MOL_AbsorbeurVersTexte, objet, y );

    /* On va au nouvel absorbeur */
    MOL_AllerAbsorbeur( mol_courante, numero_absorbeur );
    return BI_EVENEMENT_TRAITE;
  }

  /* Le dfileur des atomes */
  if ( ( ( objet >= LM_PREMIER_ATOME ) && ( objet <= LM_DERNIER_ATOME ) ) ||
       ( LM_ATOME_AVANT == objet ) || ( LM_FND_ASC_ATOME == objet ) || ( LM_BOUTON_ATOME == objet ) || ( LM_ATOME_APRES == objet ) ) {
    Atome *choisi;
    unsigned long nombre_atomes_visibles, numero_atome_choisi;

    /* On commence par enregistrer les modifications ventuelles */
/*     enregistre_atome(); */

    /* On compte combient d'atomes sont visibles */
    nombre_atomes_visibles = I_MOL_CompterAtomesAAfficher();
    if ( 0 == nombre_atomes_visibles ) return BI_EVENEMENT_TRAITE;

    /* On cherche le nouvel atome courant */
    ARB_GererDefileur_O( LISTE_MOLECULES, LM_PREMIER_ATOME, LM_DERNIER_ATOME, LG_MAX_ATM_DS_LISTE,
			 LM_ATOME_AVANT, LM_FND_ASC_ATOME, LM_BOUTON_ATOME, LM_ATOME_APRES,
			 nombre_atomes_visibles, NULL, &numero_atome_choisi, &ATM_premier_visible, I_MOL_AtomeVersTexte, objet, y );

    /* On dtecte l'atome ainsi cliqu */
    choisi = I_MOL_NumeroVersAtome( numero_atome_choisi );

    /* En fonction de la nature du clic, on fait ce qu'il faut */
    if ( BI_etat & ( K_LSHIFT | K_RSHIFT ) ) { /* shift-clic : visible/invisible */
      OBJ_coche( LISTE_MOLECULES, objet, ECHANGE );
      choisi->marques.visible = !(choisi->marques.visible);

      if ( mode_vue3d & V3D_VOIR_SELECTION ) {
	if ( fen_vue_3d != NULL ) { /* Vue en 3D ouverte */
#ifdef AVEC_OPEN_GL_ /* Affichage de la vue 3d seulement avec Open GL */
	  if ( liste_molecule != 0 ) {
	    glNewList( liste_molecule, GL_COMPILE );
	    dessine_molecule( mol_courante, mode_vue3d );
	    glEndList();
	  }
#endif
	  FEN_redessin( fen_vue_3d, BI_RDS_GL );
	}
      }
    } else { /* Slection de l'atome courant */
      ATM_numero_courant = numero_atome_choisi;
      ATM_aller( choisi );
    }

    return BI_EVENEMENT_TRAITE;
  }

#ifdef ancien
  /* La gestion des modles est pass dans le slecteur de modles ! */
  if ( (objet >= LM_PREM_MODELE) && (objet <= LM_DERN_MODELE) ) {
    unsigned int num_modele, i;
    struct modele_ajuste *modele;

    absorbeur = mol_courante->absorbeurs + mol_courante->absorbeur_courant;
    num_modele = MOD_premier_visible + (objet - LM_PREM_MODELE);
    for (i = 0, modele = absorbeur->modeles ; i < num_modele ; i++) {
      modele = modele->suivant;
    }
    if ( modele != absorbeur->modele_actif ) { /* On change de modle actif */
      absorbeur->modele_actif = modele;
      ABS_AllerModele(modele, VRAI);
      ABS_PrepareListe();
    }
    return VRAI;
  }
#endif

  OBJ_choisit( LISTE_MOLECULES, objet, FORCE_OUI );
  switch (objet)
  {
  case LM_AFF_MODELES:
    /* On passe en mode "absorbeurs" pour les modles */
    LMOD_ChangerFamille( LM_FAMILLE_ABSORBEUR );

    /* Et on affiche l'onglet correspondant */
    BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_MODELES );

    /* Termin */
    OBJ_choisit( LISTE_MOLECULES, objet, FORCE_NON );
    return BI_EVENEMENT_TRAITE;

#ifdef ancien
    /* Ces fonctions ont t regroupes en sous-menus ! */
  case MOL_NOUVEL_ABS: /* Ajouter un absorbeur */
    if (nouvel_absorbeur(mol_courante) != NULL)
      ABS_PrepareListe();
    OBJ_choisit(LISTE_MOLECULES, objet, FORCE_OUI);
    return VRAI;

  case MOL_EFFACER_ABS: /* Effacer l'absorbeur courant */
    {
      int reponse = 1;
      if ( !(BI_etat & K_CTRL) ) /*    v----------------------------v */
	reponse = form_alert(1, "[2][---- Effacer l'absorbeur ----"
			           "|   Voulez-vous vraiment perdre"
 			           "|cet atome absorbeur ?][ Oui | Non ]");
      if (reponse == 1)
	ABS_oter(mol_courante, mol_courante->absorbeur_courant, FAUX);
      ABS_PrepareListe();
    }
    OBJ_choisit(LISTE_MOLECULES, objet, FORCE_OUI);
    return VRAI;

  case MOL_MODIFIER_ABS: /* Modifier les paramtres de l'absorbeur courant */
    absorbeur = mol_courante->absorbeurs + mol_courante->absorbeur_courant;
    if ( modifier_absorbeur(absorbeur) == VRAI) { /* Changements possibles -> on remet */
      ABS_aller(absorbeur, VRAI); /* On va  cet absorbeur */
    }
    OBJ_choisit(LISTE_MOLECULES, objet, FORCE_OUI);
    return VRAI;

  case LM_NV_MODELE: /* Actuellement : chargement paths.dat */
    modele_feff(); /* MAIS SALE CAR PAS FORCMENT L'ABSORBEUR CONCERN ! */
    ABS_PrepareListe(); /* On met  jour la liste des modles */
    OBJ_choisit(LISTE_MOLECULES, objet, FORCE_OUI);
    return VRAI;
#endif
  }
  OBJ_choisit( LISTE_MOLECULES, objet, FORCE_NON );

  return BI_EVENEMENT_NON_TRAITE;
}

/* --- Grer le sous-menu proposant des actions sur la molcule courante --- */

static void I_MOL_GererMenuOpt_Generique_MOL(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{
  Molecule *resultat = NULL;

  switch ( action )
  {
  case FCT_MOL_SAUVER_MOLECULE:
    I_MOL_Sauver( mol_courante );
    break;

  case FCT_MOL_EXPORTER_MOLECULE:
    break;

  case FCT_MOL_EXPORT_POSTSCRIPT:
    break;

  case FCT_MOL_EFFACER_MOLECULE:
    I_MOL_EffacerMolecule( mol_courante );
    MOL_PrepareListe();		/* Si l'on a effac une molcule, il faut refaire la liste ! */
    break;

  case FCT_MOL_COPIER_MOLECULE:
    break;

  case FCT_MOL_CENTRER_MOLECULE:
    break;

  case FCT_MOL_TRANSLATER_MOLECULE:
    break;

  case FCT_MOL_TOURNER_MOLECULE:
    break;

  case FCT_MOL_COMPOSITION:
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }

  if ( resultat != NULL ) {
    /* L'action demande  conduit  crer une nouvelle molcule */
    if ( mol_courante == resultat ) {
      OPTION_A_FAIRE( "Mettre  jour l'affichage de la molcule courante" );
      /* Simple modification en place : on met  jour ! */
    } else {
      /* La nouvelle molcule devient la molcule courante */
      MOL_aller( resultat );
    }
  }
}

/* ~-~-~-~-~-~-~-~- Liste des fragments (de la molcule courante) ~-~-~-~-~-~-~-~- */

/* -------- Affichage de la liste des fragments -------- */

void I_MOL_RefaireListeFragments(void)
{
  Fragment *fragment_actif;

  if ( NULL == mol_courante ) {
    (void) fprintf(stderr, "%s, %s l.%u : On ne devrait jamais tre ici !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return;
  }

  if ( NULL == mol_courante->fragments ) {
    /* Aucun fragment, mais on peut en crer ! */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_FRG, LM_DERNIER_FRAG, LG_MAX_FRG_DS_LISTE,
			    LM_FRG_AVANT, LM_FRG_FND_ASC, LM_FRG_BOUTON, LM_FRG_APRES,
			    0, NULL, 0, FRG_premier_visible, I_MOL_FragmentVersTexte );
    I_MOL_CouperOptionsFragments( VRAI );
    return;
  }

  /* Il y a un fragment : on peut donc activer les options associes... */
  I_MOL_CouperOptionsFragments( FAUX );

  /* On affiche la liste des fragments */
  ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_FRG, LM_DERNIER_FRAG, LG_MAX_FRG_DS_LISTE,
			  LM_FRG_AVANT, LM_FRG_FND_ASC, LM_FRG_BOUTON, LM_FRG_APRES,
			  mol_courante->nbr_fragments, NULL, FRG_fragment_courant, FRG_premier_visible, I_MOL_FragmentVersTexte );

  /* On affiche le nom du fragment courant */
  fragment_actif = I_MOL_TrouverFragmentActif();
  I_MOL_AfficherFragmentActif( fragment_actif );
}

/* Conversion fragment -> texte dans la liste */

static const char *I_MOL_FragmentVersTexte(unsigned int numero_arbre __attribute__((__unused__)), unsigned long numero_objet)
{
  Fragment *fragment;
  unsigned long i;

  if ( NULL == mol_courante ) return NULL;
  if ( NULL == mol_courante->fragments ) return NULL;
  if ( numero_objet >= mol_courante->nbr_fragments ) return NULL;

  for ( i = 0, fragment = mol_courante->fragments ; i < numero_objet ; i++, fragment = fragment->suivant );

  if ( NULL == fragment->nom ) return "<Sans nom>";
  return fragment->nom;
}

/* --- Affichage des informations associes au fragment actif --- */

static void I_MOL_AfficherFragmentActif(Fragment *fragment)
{
  if ( NULL == fragment ) {
    OBJ_CopierTexte( LISTE_MOLECULES, LM_NOM_FRAGMENT, "", 0 );
    return;
  }

  /* Le nom du fragment */
  OBJ_CopierTexte( LISTE_MOLECULES, LM_NOM_FRAGMENT, fragment->nom, 0 );

  /* Les autres informations du fragment */
  OPTION_A_FAIRE( "Afficher le type, le code, le numro,..." );

  /* Si ncessaire, la liste de ses atomes */
  if ( IMOL_AFF_ATOMES_DU_FRAGMENT == mode_affichage_atomes ) {
    /* On remet  0 le dfileur */
    ATM_premier_visible = 0;
    ATM_numero_courant  = 0;

    /* On refait la liste */
    ATM_PrepareListe();
  }
}

/* --- Trouver le fragment courant,  partir de la liste --- */

static Fragment *I_MOL_TrouverFragmentActif(void)
{
  Fragment *fragment;
  unsigned long i;

  if ( NULL == mol_courante ) return NULL;
  if ( ( NULL == mol_courante->fragments ) || ( 0 == mol_courante->nbr_fragments ) ) {
    return NULL;
  }

  for ( i = 0, fragment = mol_courante->fragments ; i < FRG_fragment_courant ; i++, fragment = fragment->suivant ) {
    if ( NULL == fragment ) {
      (void) fprintf(stderr, "%s, %s l.%u : Incohrence, fragment %lu demand mais 0...%u possible !\n",
		     __FILE__, __FUNCTION__, __LINE__, FRG_fragment_courant, mol_courante->nbr_fragments );
      FIN_ANTICIPEE( FIN_ERREUR_MINEURE );
      return NULL;
    }
  }

  /* Ici, on a bien trouv le fragment actif */
  return fragment;
}

/* --- Macros de coupure/activation des objets --- */

static void I_MOL_CouperOptionsFragments(boolean couper)
{
  /* Le dfileur est gr par les options gnriques ! ; le bouton "crer" est quasi-toujours actif */
  OBJ_active( LISTE_MOLECULES, LM_NOM_FRAGMENT, !couper );
  OBJ_active( LISTE_MOLECULES, LM_OPT_GERE_FRG, !couper );
  OBJ_active( LISTE_MOLECULES, LM_TYPE_FRG    , !couper );
}

/* ~-~-~-~-~-~-~-~- Liste des atomes (de la molcule courante) ~-~-~-~-~-~-~-~- */

/* -------- Affichage de la liste des atomes & des coordonnes de l'atome courant -------- */

void ATM_PrepareListe(void)
{
  unsigned long /* i, */ nombre_atomes;
/*   Atome *atome; */

  /* On compte le nombre d'atomes  afficher */
  nombre_atomes = I_MOL_CompterAtomesAAfficher();

  if ( 0 == nombre_atomes ) { /* Pas d'atome  afficher : tout est inactif, aucun atome ! */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_ATOME, LM_DERNIER_ATOME, LG_MAX_ATM_DS_LISTE,
			    LM_ATOME_AVANT, LM_FND_ASC_ATOME, LM_BOUTON_ATOME, LM_ATOME_APRES,
			    0, NULL, 0, 0, I_MOL_AtomeVersTexte );

    /* Options qui n'ont pas de sens */
    I_MOL_CouperOptionsAtomes( VRAI );
    return;
  }

  if ( nombre_atomes > NBR_ATM_VISIBLES ) { /* Plus d'atomes que l'on ne peut en afficher */
    if ( ATM_premier_visible + NBR_ATM_VISIBLES > nombre_atomes ) {
      ATM_premier_visible = nombre_atomes - NBR_ATM_VISIBLES;
    }
  } else {
    ATM_premier_visible = 0;
  }

  ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_ATOME, LM_DERNIER_ATOME, LG_MAX_ATM_DS_LISTE,
			  LM_ATOME_AVANT, LM_FND_ASC_ATOME, LM_BOUTON_ATOME, LM_ATOME_APRES,
			  nombre_atomes, NULL, ATM_numero_courant, ATM_premier_visible, I_MOL_AtomeVersTexte );

  /* Infos pour l'atome choisi */
  if ( NULL == ATM_courant ) {
    I_MOL_CouperOptionsAtomes( VRAI );
    return; /* Rien d'autre  faire */
  }
  I_MOL_CouperOptionsAtomes( FAUX );

  OBJ_MettreEntier_U( LISTE_MOLECULES, LM_NUMERO_ATOMIK, ATM_courant->num_atomique );
  OBJ_MettreCoordonnees( LISTE_MOLECULES, LM_COORD_X, LM_COORD_Y, LM_COORD_Z, &(ATM_courant->position), ATM_courant->err_position );
  OBJ_MettreNombre( LISTE_MOLECULES, LM_RAYON_ATOME, ATM_courant->rayon, 0.0 ); /* Attention, il est en pm */
}

/* Reprer l'atome qui apparat avec le numro indiqu dans la liste [0 == le premier] */

static Atome *I_MOL_NumeroVersAtome(unsigned long numero_atome)
{
  unsigned long i;
  Atome *atome = NULL;
  Fragment *fragment_actif = NULL;

  if ( NULL == mol_courante ) return NULL;

  /* Tout dpend du mdoe d'affichage choisi */
  switch ( mode_affichage_atomes )
  {
  case IMOL_AFF_TOUS_LES_ATOMES:
    /* On affiche tous les atomes de la molcule : il suffit de reprer, dans l'ordre */
    for ( i = 0, atome = mol_courante->premier ; i < numero_atome ; i++, atome = atome->suivant ) {
      if ( NULL == atome ) break;
    }
    break;

  case IMOL_AFF_ATOMES_DU_FRAGMENT:
    /* On rcupre le fragment actif */
    fragment_actif = I_MOL_TrouverFragmentActif();
    if ( NULL == fragment_actif ) return NULL;

    /* On affiche les atomes dans l'ordre de ce fragment */
    if ( numero_atome >= fragment_actif->nbr_atomes ) {
      return NULL;
    }
    atome = fragment_actif->atomes[ numero_atome ];
    break;

  case IMOL_AFF_ATOMES_DU_MODELE:
    OPTION_A_FAIRE( "Dtection d'un atome / modle" );
    break;

  case IMOL_AFF_PAR_Z_INDIQUE:
    /* On se place sur le premier atome ayant le Z voulu */
    atome = mol_courante->premier;
    while ( atome->num_atomique != Z_affichage_atomes ) {
      atome = atome->suivant;
      if ( NULL == atome ) return NULL;
    }

    /* On repre ensuite celui qui a le bon numro */
    for ( i = 0 ; atome != NULL ; atome = atome->suivant ) {
      if ( Z_affichage_atomes == atome->num_atomique ) {
	if ( i == numero_atome ) {
	  /* On a trouv le bon atome */
	  return atome;
	}
	i++;			/* On en a trouv un de plus */
      }
    }
    break;

  case IMOL_AFF_PROCHES_ABSORBEUR:
    OPTION_A_FAIRE( "Dtection d'un atome / proche absorbeur" );
    break;

  case IMOL_AFF_LISTE_INVALIDE:
    break;
  }

  /* On a termin : on renvoie l'atome indiqu */
  return atome;
}

/* Conversion absorbeur -> texte dans la liste */

static const char *I_MOL_AtomeVersTexte(unsigned int numero_arbre __attribute__((__unused__)), unsigned long numero_objet)
{
  Atome *atome;
  char *nom;

  /* On rcupre l'atome qui a le numro indiqu */
  atome = I_MOL_NumeroVersAtome( numero_objet );
  if ( NULL == atome ) return NULL;

  /* On construit le texte */
  if ( atome->index > 0 ) {
    nom = TXT_genere_chaine( "  %s %lu", ATM_Z_vers_symbole( atome->num_atomique ), atome->index );
  } else {
    nom = TXT_genere_chaine( "  %s", ATM_Z_vers_symbole( atome->num_atomique ) );
  }

  /* On le copie dans le tableau, en le limitant  la bonne taille */
  strncpy( texte_affichage_nom_atome, nom, LG_MAX_ATM_DS_LISTE );

  /* Plus besoin du texte rserv */
  free( nom ); nom = NULL;

  /* On renvoie. Sera _copi_ [car on a indiqu une longueur maximale] : pas de souci pour le rutiliser ! */
  return texte_affichage_nom_atome;
}

/* --- Macros de coupure/activation des objets --- */

static void I_MOL_CouperOptionsAtomes(boolean couper)
{
  /* Le dfileur est gr par les options gnriques ! ; le bouton "crer" est quasi-toujours actif */

  OBJ_active( LISTE_MOLECULES, LM_NUMERO_ATOMIK, !couper );
  OBJ_active( LISTE_MOLECULES, LM_COORD_X      , !couper );
  OBJ_active( LISTE_MOLECULES, LM_COORD_Y      , !couper );
  OBJ_active( LISTE_MOLECULES, LM_COORD_Z      , !couper );
  OBJ_active( LISTE_MOLECULES, LM_CHARGE       , !couper );
  OBJ_active( LISTE_MOLECULES, LM_RAYON_ATOME  , !couper );
  OBJ_active( LISTE_MOLECULES, LM_PROPOSER_R   , !couper );
  OBJ_active( LISTE_MOLECULES, LM_OPT_ATOMES   , !couper );
  OBJ_active( LISTE_MOLECULES, LM_COORD_CRISTAL, !couper );

  if ( VRAI == couper ) {
    /* Peut tre inactif mme si un atome est prsent ! */
  }
}

/* -------- Choix du filtre d'affichage des atomes dans la liste des atomes -------- */

static void I_MOL_GererMenuOpt_Affichage_ATM(unsigned int num_arbre __attribute__((__unused__)), unsigned int num_objet __attribute__((__unused__)), unsigned long action)
{
  BI_Champ_Demande valeur = { 0, NULL, NULL, 0, 0.0 , 0.0, 0.0, 0, NULL, { 0 }, { 0 }, { 0 } };
  int reponse;
  unsigned int Z = SYMBOLE_CHIMIQUE_INCONNU;

  /* S'il n'y a pas de molcule, rien  faire ! */
  if ( NULL == mol_courante ) return;

  /* Si ncessaire, demande d'informations complmentaires */
  switch ( action )
  {
  case IMOL_AFF_TOUS_LES_ATOMES: break;	 /* RAS */

  case IMOL_AFF_ATOMES_DU_FRAGMENT:
    if ( NULL == mol_courante->fragments ) {
      /* Pas de fragment dans la molcule : option inutile ! */
      (void) BI_ERR_alerte( principale, &MSG_AucunFragment, VRAI );
      return;
    }
    break;

  case IMOL_AFF_ATOMES_DU_MODELE:
    OPTION_A_FAIRE( "Contrle de l'existence d'un modle !" );
    break;

  case IMOL_AFF_PAR_Z_INDIQUE:
    /* On prpare la demande */
    valeur.type  = BI_CHAMP_DEMANDE_CHAINE;
    valeur.texte = txt_demande_Z;
    valeur.valeur_initiale.chaine = NULL;

    /* On demande */
    reponse = BI_DemandeValeur( principale, &Q_NumeroAtomique, &valeur );
    if ( 2 == reponse ) return;	/* On a annul ! */

    /* On analyse la rponse */
    Z = ATM_num_atomique( valeur.valeur_donnee.chaine );
    if ( SYMBOLE_CHIMIQUE_INCONNU == Z ) {
      /* On essaye de lire une valeur numrique */
      Z = TXT_CNV_vNombre_L( valeur.valeur_donnee.chaine, NULL );
      if ( ( Z < HYDROGENE ) || ( Z > MEITNERIUM ) ) {
	(void) BI_ERR_alerte( principale, &ERR_ZInconnu, VRAI, valeur.valeur_donnee.chaine );
	Z = SYMBOLE_CHIMIQUE_INCONNU;
      }
    }

    /* Plus besoin des demandes */
    free( valeur.valeur_donnee.chaine ); valeur.valeur_donnee.chaine = NULL;

    /* On mmorise */
    if ( Z != SYMBOLE_CHIMIQUE_INCONNU ) {
      Z_affichage_atomes = Z;
    } else {
      return;
    }
    break;

  case IMOL_AFF_LIES_A_ABSORBEUR:
    OPTION_A_FAIRE( "Demander le nombre maximal de liaisons sparant et marquer en fonction les atomes" );
    break;

  case IMOL_AFF_PROCHES_ABSORBEUR:
    OPTION_A_FAIRE( "Demander la distance maximale sparant et marquer en fonction les atomes" );
    break;

  default:
    AVERTISSEMENT( "Mode de filtre inconnu : %lu", action );
    action = IMOL_AFF_TOUS_LES_ATOMES;
    break;
  }

  /* Le mode d'affichage devient celui slectionn [aprs, pour ne pas changer en cas de problme ou d'annulation !]*/
  mode_affichage_atomes = action;

  /* On recommence donc  afficher au premier atome, sans atome slectionn */
  ATM_premier_visible = 0;
  ATM_courant = NULL;

  /* Et on refait la liste en consquence */
  ATM_PrepareListe();
}

/* -------- Compter le nombre d'atomes  afficher -------- */

static unsigned long I_MOL_CompterAtomesAAfficher(void)
{
  unsigned long nombre_atomes = 0;
  Atome *atome;
  Fragment *fragment_actif;

  /* S'il n'y a pas de molcule, il n'y a pas d'atome  afficher ! */
  if ( NULL == mol_courante ) return 0;

  switch ( mode_affichage_atomes )
  {
  case IMOL_AFF_ATOMES_DU_FRAGMENT:
    fragment_actif = I_MOL_TrouverFragmentActif();
    if ( NULL == fragment_actif ) return 0;

    nombre_atomes = fragment_actif->nbr_atomes;
    break;

  case IMOL_AFF_ATOMES_DU_MODELE:
    break;

  case IMOL_AFF_PAR_Z_INDIQUE:
    for ( atome = mol_courante->premier ; atome != NULL ; atome = atome->suivant ) {
      if ( Z_affichage_atomes == atome->num_atomique ) {
	nombre_atomes++;
      }
    }
    break;

  case IMOL_AFF_LIES_A_ABSORBEUR:
  case IMOL_AFF_PROCHES_ABSORBEUR:
  case IMOL_AFF_LISTE_INVALIDE:
    OPTION_A_FAIRE( "Compter les atomes marqus" );
    break;

  case IMOL_AFF_TOUS_LES_ATOMES:
  default: /* Par dfaut, on affiche tous les atomes */
    nombre_atomes = mol_courante->nbr_atomes;
  }

  /* On renvoie le nombre d'atomes  afficher */
  return nombre_atomes;
}

/* ~-~-~-~-~-~-~-~- Liste des absorbeurs (de l'atome courant) ~-~-~-~-~-~-~-~- */

/* -------- Affichage de la liste des absorbeurs & des options de l'absorbeur courant -------- */

void ABS_PrepareListe(void)
{
  Absorbeur *absorbeur;

  if ( ( NULL == mol_courante ) || ( NULL == mol_courante->absorbeurs ) ) { /* Pas d'absorbeurs */
    ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_ABS, LM_DERNIER_ABS, LG_MAX_ABS_DS_LISTE,
			    LM_ABS_AVANT, LM_ABS_FOND_ASC, LM_ABS_BOUT_ASC, LM_ABS_APRES,
			    0, NULL, 0, 0, I_MOL_AbsorbeurVersTexte );

    /* Options qui n'ont pas de sens */
    I_MOL_CouperOptionsAbsorbeurs( VRAI );
    return;
  }

  /* On peut activer les options s'il y a un absorbeur */
  I_MOL_CouperOptionsAbsorbeurs( FAUX );

  ARB_PreparerDefileur_O( LISTE_MOLECULES, LM_PREMIER_ABS, LM_DERNIER_ABS, LG_MAX_ABS_DS_LISTE,
			  LM_ABS_AVANT, LM_ABS_FOND_ASC, LM_ABS_BOUT_ASC, LM_ABS_APRES,
			  mol_courante->nbr_absorbeurs, NULL, mol_courante->absorbeur_courant, ABS_premier_visible, I_MOL_AbsorbeurVersTexte );

  /* Informations sur l'absorbeur actif */
  absorbeur = mol_courante->absorbeurs + mol_courante->absorbeur_courant;

  OBJ_CopierTexte( LISTE_MOLECULES, LM_NOM_ABSORBEUR, absorbeur->nom, 0 );

  /* On s'occupe des modles */
  if ( NULL == absorbeur->modeles ) { /* Pas de modle */
    OBJ_active( LISTE_MOLECULES, LM_AFF_MODELES, FORCE_NON );
  } else {
    OBJ_active( LISTE_MOLECULES, LM_AFF_MODELES, FORCE_OUI );
  }
}

/* Conversion absorbeur -> texte dans la liste */

static const char *I_MOL_AbsorbeurVersTexte(unsigned int numero_arbre __attribute__((__unused__)), unsigned long numero_objet)
{
  Absorbeur *absorbeur;

  if ( NULL == mol_courante ) return NULL;
  if ( NULL == mol_courante->absorbeurs ) return NULL;
  if ( numero_objet >= mol_courante->nbr_absorbeurs ) return NULL;

  absorbeur = mol_courante->absorbeurs + numero_objet;
  return absorbeur->nom;
}

/* --- Macros de coupure/activation des objets --- */

static void I_MOL_CouperOptionsAbsorbeurs(boolean couper)
{
  /* Le dfileur est gr par les options gnriques ! ; le bouton "crer" est quasi-toujours actif */

  OBJ_active( LISTE_MOLECULES, LM_TYPE_ABS     , !couper );
  OBJ_active( LISTE_MOLECULES, LM_NOM_ABSORBEUR, !couper );

  if ( VRAI == couper ) {
    /* Peut tre inactif mme si un absorbeur est prsent ! */
    OBJ_active( LISTE_MOLECULES, LM_OPTIONS_ABS  , FORCE_NON );
    OBJ_active( LISTE_MOLECULES, LM_AFF_MODELES  , FORCE_NON );
  }
}

/* --- Grer le sous-menu proposant des actions sur l'absorbeur courant --- */

static void I_MOL_GererMenuOpt_Generique_ABS(unsigned int arbre __attribute__((__unused__)), unsigned int objet __attribute__((__unused__)), unsigned long action)
{

  switch ( action )
  {
  case IMOL_ABS_EFFACER_ABSORBEUR:
    I_MOL_EffacerAbsorbeur( mol_courante, mol_courante->absorbeur_courant );
    break;

  case IMOL_ABS_CONTRIBUTION:
    break;

  default:
    AVERTISSEMENT( "Option inconnue [%lu]", action );
    return;
  }
}

/* -------- Gestion effective -------- */

#ifdef ancien
/*  fusionner avec la gestion de la liste des molcules ! */
static boolean GER_liste_atomes(unsigned int objet, int x, int y)
{
  Atome *choisi;

  switch (objet)
  {
  case LST_ATM_AVANT: /* On fait monter la liste d'un cran */
  case LST_ATM_APRES: /* On fait descendre la liste d'un cran */
  case LST_ATM_FOND_ASC:
  case LST_ATM_BOUTON:
    ATM_premier_visible = ARB_ascenseur(LISTE_MOLECULES, LST_ATM_AVANT, LST_ATM_FOND_ASC, LST_ATM_BOUTON, LST_ATM_APRES, ASC_VERTICAL,
					mol_courante->nbr_atomes, NBR_ATM_VISIBLES, ATM_premier_visible,
					objet, x, y, ECHANGE);
    ATM_PrepareListe();
    return VRAI;

  case LST_ATM_LIER: /* Informations sur les liaisons de l'atome */
    modifier_liaisons(ATM_courant, mol_courante);
    return VRAI;

  case LST_ATM_EFFACER: /* Effacer l'atome slectionn */
    {
      ATM_efface(mol_courante, ATM_courant, BI_etat & K_CTRL );
      if ( fen_vue_3d != NULL ) { /* On doit refaire la vue 3D */
#ifdef AVEC_OPEN_GL_ /* Affichage de la vue 3d seulement avec Open GL */
	if (liste_molecule != 0) {
	  glNewList(liste_molecule, GL_COMPILE);
	  dessine_molecule(mol_courante, mode_vue3d);
	  glEndList(); /* On a fini le dessin */
	  glNewList(liste_liaisons, GL_COMPILE);
	  dessine_liaisons(mol_courante, mode_vue3d); /* Car a pu changer aussi */
	  glEndList();
	}
	FEN_redessin(fen_vue_3d, BI_RDS_GL);
#endif
      }
    }
    return VRAI;

  case LST_ATM_CREER: /* On veut crer un nouvel atome dans la molcule courante */
    ATM_nouveau(mol_courante, VRAI);
    return VRAI;
  }

  return FAUX;
}

/* -------- Enregistre les paramtres modifiables d'un atome -------- */

void enregistre_atome(void)
{
  int nv_Z;
  double nv_val, err_valeur;
  boolean redessin = FAUX;

  if (ATM_courant == NULL) return;

  nv_Z = OBJ_NumeroAtomique( LISTE_MOLECULES, LST_ATM_NUM_ATOM );
  if ( (nv_Z > 0) && (nv_Z != ATM_courant->num_atomique) ) {
    /* A priori, le changement de Z a pour seule consquence le changement de couleur dans la vue 3D */
    ATM_courant->num_atomique = nv_Z;
    redessin = VRAI;
  }

  nv_val = OBJ_LireNombre( LISTE_MOLECULES, LST_ATM_X, &err_valeur );
  if ( fabs(nv_val - ATM_courant->position.x) > 1e-4 ) { /* Inutile d'avoir une prcision de mieux d'1/1000 d'A ! */
    ATM_courant->position.x = nv_val;
    redessin = VRAI;
  }
  nv_val = OBJ_LireNombre( LISTE_MOLECULES, LST_ATM_Y, &err_valeur);
  if ( fabs(nv_val - ATM_courant->position.y) > 1e-4 ) {
    ATM_courant->position.y = nv_val;
    redessin = VRAI;
  }
  nv_val = OBJ_LireNombre( LISTE_MOLECULES, LST_ATM_Z, &err_valeur );
  if ( fabs(nv_val - ATM_courant->position.z) > 1e-4 ) {
    ATM_courant->position.z =  nv_val;
    redessin = VRAI;
  }
  nv_val = OBJ_LireNombre( LISTE_MOLECULES, LST_ATM_RAYON, NULL );
  if ( fabs(nv_val - ATM_courant->rayon) > 1e-4 ) {
    ATM_courant->rayon = nv_val;
    redessin = VRAI;
  }
  ATM_courant->charge = OBJ_LireNombre( LISTE_MOLECULES, LST_ATM_CHARGE, NULL );

  /* On force le redessin si ncessaire, ie si l'atome est visible */
#ifdef AVEC_OPEN_GL_
  if ( (fen_vue_3d != NULL) && (redessin == VRAI) ) {
    if (liste_molecule) {
      glNewList(liste_molecule, GL_COMPILE);
      dessine_molecule(mol_courante, mode_vue3d);
      glEndList();
    }
    FEN_redessin(fen_vue_3d, BI_RDS_GL);
  }
#endif
}

#endif

