/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2002, Emmanuel CURIS
   Cration : 20 novembre 1998

   Gnralits du programme LASE (interface graphique)
   Gestion et prparation du menu d'extraction des spectres EXAFS
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "extrait.h"
#include "fichiers.h"
#include "figures.h"		/* Inclus tracer.h */
#include "filtrage.h"
#include "graphes.h"
#include "import_xas.h" /* Pour EXTRAIT_Interface */
#include "interface.h"
#include "lissage.h" /* Besoin de SPL_xxx pour le calcul de la drive avec les splines */
#include "operations.h"
#include "statistiques.h"
#include "tri.h" /* Pour fondre() */
#include "xanes.h"

#ifdef _TOS_ /* Seule la version TOS peut imprimer */
# include "imprime.h"
#endif

#include "menu_extraire.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int CLV_men_extrait(unsigned int touche, int etat, int *renvoi);
int MEN_PreparerExtraire(void);

#ifdef AVEC_OPEN_GL_
extern void RDS_atomes_3d(Fenetre *fenetre, int x, int y, int larg, int haut);
#endif

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Prparation du menu -------- */

int MEN_PreparerExtraire(void)
{
#ifdef AVEC_TRADUCTION_
  MENU_TraduireExtraire();
#endif

  arbres[MENU].clavier = CLV_men_extrait;

  OBJ_active(MENU, SAUVER        , FORCE_NON);
  OBJ_active(MENU, MEX_EXPORTER_GRF, FORCE_NON);
  OBJ_active(MENU, TIT_FILTRAGE  , FORCE_NON);
  OBJ_active(MENU, TIT_TRACE     , FORCE_NON);
  OBJ_active(MENU, IMP_GRAPHE    , (gdos != -2)); /* Impression inactive sans GDOS */

#ifdef _TOS_ /* Les accessoires doivent tre accessibles */
  OBJ_active(MENU, MEX_V_EXTRAIT, FORCE_OUI);
#endif

#ifndef _DEVELOPPE_ /* Options disponibles uniquement dans la version de dvloppement */
  OBJ_active(MENU, MEX_V_BANQUE   , FORCE_NON);
  OBJ_active(MENU, MEX_IMPORT_SEQ  , FORCE_NON);
  OBJ_active(MENU, MEX_IMPORT_IMG  , FORCE_NON);
  OBJ_active(MENU, MEX_CREER_GRAPHE, FORCE_NON);
  OBJ_active(MENU, MEX_POINTS      , FORCE_NON);
  OBJ_active(MENU, COMP_GRAPHES    , FORCE_NON);
  OBJ_active(MENU, MEX_CREER_GROUPE, FORCE_NON);
  OBJ_active(MENU, MEX_MOD_GROUPE  , FORCE_NON);
  OBJ_active(MENU, MEX_EFF_GROUPE  , FORCE_NON);
/*   OBJ_active(MENU, MEN_CONV_ABS_FLU, FORCE_NON); */
/*   OBJ_active(MENU, MEX_ERREUR_NRJ  , FORCE_NON); */
/*   OBJ_active(MENU, MEX_ERREUR_I    , FORCE_NON); */
  OBJ_active(MENU, MEX_CORR_I      , FORCE_NON);
/*   OBJ_active(MENU, MEX_CORR_FLU_ABS, FORCE_NON); */
  OBJ_active(MENU, MEX_NETTOYER    , FORCE_NON);
  OBJ_active(MENU, MEX_ESTIME_BRUIT, FORCE_NON);
  OBJ_active(MENU, RECOMP_TF       , FORCE_NON);
/*   OBJ_active(MENU, MEX_DENSITE     , FORCE_NON); */
/*   OBJ_active(MENU, DECAL_GRAPHES   , FORCE_NON); */
#endif

  /* Initialisation russie */
  return 0;
}

/* -------- Gestion des options de menu -------- */

int MEN_extrait(unsigned int option)
{
  Graphe *retour = NULL;
  char reponse;

  if ( OBJ_active(MENU, option, QUESTION) == FAUX ) /* Option inactive */
    return BI_EVENEMENT_TRAITE;

  switch (option)
  {
  case INFOS:
    ARB_AppelerInformations();
    return BI_EVENEMENT_TRAITE;

  case MEX_V_THEORIE:
    GER_infos(CALC_THEORIQUES, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MEX_V_AJUSTE:
    GER_infos(EXF_AJUSTEMENTS, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MEX_V_STRUCTURE:
    GER_infos(STRUCTURE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MEX_V_BANQUE:
    GER_infos(INFOS_BANQUE, 0, 0);
    return BI_EVENEMENT_TRAITE;

  case MEX_V_FLUOX:
    GER_infos(INFOS_FLUOX, 0, 0);
    return BI_EVENEMENT_TRAITE;
	
  case CHARGER:        /* Charger un graphe comme graphe courant */
    retour = charger(NULL);
    break;

  case SAUVER:         /* Sauver le graphe courant */
    I_GRF_Sauver( graphe_courant );
    return BI_EVENEMENT_TRAITE;

  case MEX_IMPORT_GRF:    /* Importer un fichier (ASCII) */
    GRF_ImportASCII();    /* On importe un fichier "ASCII" plus gnral */
    break;

  case MEX_IMPORT_3D:
    MOL_ImportASCII();
    break;

  case MEX_EXPORTER_GRF:       /* Exporter un fichier "ASCII" */
    GRF_ExportASCII( graphe_courant );
    break;

  case MEN_EXPORTIMAGE: /* Exporter une image du graphe courant */
#ifdef _TOS_
    EXP_image(FAUX);
#else
    EXP_PS_GrapheUnique( graphe_courant );
#endif
    break;

  case MEX_XAS_EXTRAIRE: /* Extraire le XANES/EXAFS d'un fichier exprimental */
    (void) I_IMPORT_DonneesXAS();
    break;
	
  case QUITTER:        /* Quitter le programme */
    reponse = SYS_ERREUR( QUITTER_PROGRAMME );
    if (reponse == 1) /* On veut quitter */
      return BI_EVENEMENT_FIN_PRG;
    break;

  case MEX_CREER_GRAPHE: /* Gnrer un graphe */
    retour = I_GRF_Generer( graphe_courant );
    break;
	
  case MODE_DESSIN:    /* Paramtres de trac des graphes */
    FEN_premierplan( fenetre_onglets );
    BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_GRAPHES );
    return BI_EVENEMENT_TRAITE;

  case MEN_TT_EFFACER: /* Effacer tous les graphes en mmoire */
    GRF_tteffacer();
    return BI_EVENEMENT_TRAITE;

#ifdef _TOS_ /* Seule la version TOS peut imprimer, pour l'instant au moins */
  case IMP_GRAPHE:     /* Imprimer le graphe courant */
    imprimer(FAUX);
    break;
#endif

  case MEN_FONDRE:     /* Fusion de divers graphes */
    retour = fondre();
    break;

  case OP_DIVERSES:    /* Oprations diverses sur 1 ou plusieurs graphes */
    retour = op_diverses(graphe_courant);
    break;

  case MEN_MOYENNER:   /* Moyenne de plusieurs graphes          */
    retour = moyenner();
    break;

  case COMP_GRAPHES:		/* Comparer deux ou plusieurs graphes */
    retour = I_GRF_ComparerGraphes( graphe_courant );
    break;

  case MEN_CLC_SIGNAL: /* Calculer signal  partir de I0 et I1  */
    retour = XAN_extrait();
    break;

  case MEN_CORR_NRJ:   /* Corriger les nergies d'un spectre    */
    corr_energie(graphe_courant);
    break;
	
  case MATRICE:          /* Enlever la contribution de la matrice    */
    retour = I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_AUCUNE );
    break;

  case CHERCHE_E0:     /* Recherche de l'nergie de seuil */
    cherche_e0(graphe_courant, VRAI);
    break;

    /* ~~~~~~ Options concernant plus particulirement le XANES ~~~~~~~~~~~~~~~~~~~ */

  case XANES_TOUT:     /* Faire tout le XANES d'un coup (comme EDAXANES)   */
    retour = I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_XANES );
    break;

  case DERIVEE_UN:     /* Drive premire */
    retour = I_GRF_Derivee( graphe_courant );
    break;
	
  case DERIVEE_DEUX:   /* Drive seconde           */
    {
      double *b, *c, *d;
      int erreur;

      erreur = SPL_tt_prep(graphe_courant, &b, &c, &d);
      if (erreur != 0) { /* Calcul ne peut pas avoir lieu */
	return BI_EVENEMENT_TRAITE;
      }
      retour = DER_seconde(graphe_courant, graphe_courant->nombre_point, b, c, d);
      free(b); free(c); free(d); /* On libre la mmoire */
    }
    break;

  case XAN_NORMALISE:  /* Normalisation du XANES   */
    norm_xanes(graphe_courant);
    return BI_EVENEMENT_TRAITE;
	
  case INT_PRESEUIL:   /* Intgration du prseuil  */
    XAN_integre(graphe_courant);
    return BI_EVENEMENT_TRAITE;
	
    /* ~~~~~~ Options concernant les spectres EXAFS surtout ~~~~~~~~~~~~~~~~~~~~~~~~ */

  case EXAFS_TOUT:     /* On fait tout l'EXAFS d'un coup  */
    {
      Graphe *mu0;
      boolean type_chaine;

      if ( GR_BRUT == graphe_courant->type ) { /* Il faut aussi la matrice */
	type_chaine = I_XAS_CHAINE_XAS;
	retour = I_XAN_OterFondContinu( graphe_courant, I_XAS_CHAINE_XAS );
      } else {
	retour = graphe_courant; /* Pas besoin de la matrice, car dj fait */
	type_chaine = I_XAS_CHAINE_EXAFS;
      }
      retour = I_XAS_OterMu0_Etape1( retour, type_chaine, &mu0 );
      retour = MU2_ouvre(retour, mu0, type_chaine, &mu0);
      retour = MU3_ouvre(retour, mu0, type_chaine);
      (void) TF_ouvre(retour, type_chaine);
      TT_exafs = VRAI;
    }
    break;

  case MEN_EXF_REFAIRE: /* On reproduit l'extraction  l'identique */
    EXF_reproduit(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case OTE_MU0_1:      /* Enlever 0, premire tape   */
    /* On ne mmorise pas le graphe rsultat, car isol : on ira quand on fermera la fentre... */
    (void) I_XAS_OterMu0_Etape1( graphe_courant, I_XAS_CHAINE_AUCUNE, NULL );
    break;
	
  case OTE_MU0_2:      /* Enlever 0, deuxime tape   */
    retour = GRF_choisir(GR_MU0, FAUX, VRAI, NULL);
    retour = MU2_ouvre(graphe_courant, retour, I_XAS_CHAINE_AUCUNE, NULL);
    break;
	
  case OTE_MU0_3:      /* Enlever 0, troisime tape  */
    retour = GRF_choisir(GR_MU0, FAUX, VRAI, NULL);
    retour = MU3_ouvre(graphe_courant, retour, I_XAS_CHAINE_AUCUNE );
    break;

  case MEN_MU0_TOTAL:  /* ter 0 dj calcul         */
    MU0_ouvre(graphe_courant, GRF_choisir(GR_MU0, FAUX, VRAI, NULL));
    return BI_EVENEMENT_TRAITE;

  case MEN_EXF_NORMALIS: /* Normaliser un spectre EXAFS */
    retour = I_XAS_NormaliserEXAFS( graphe_courant );
    break;

  case MAGNIFIER:      /* Multiplier par k^n              */
    EXF_ponderer( graphe_courant, 2 );
    break;
	
  case CONVERSION:     /* Conversion E <-> k               */
    retour = convert(graphe_courant);
    break;
		
  case TF:             /* Transforme de Fourier du spectre courant */
    (void) TF_ouvre(graphe_courant, I_XAS_CHAINE_AUCUNE );
    return BI_EVENEMENT_TRAITE;

  case FILTRE_PIC:     /* TF inverse en isolant une rgion de la TF */
    TFI_ouvre(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case MEX_REPRO_TF:   /* Reproduire le calcul de la TF & de la TF inverse */
    TF_reproduit(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case CORR_AMPLITUDE: /* Correction de l'amplitude                */
    retour = corr_amplitude(graphe_courant, 0.0, 0.0, 0.0);
    break;
	
  case CORR_PHASE:     /* Correction de la phase                   */
    retour = TFI_CorrigerPhase( graphe_courant, 0, 0.0 );
    break;

    /* ~~~~~~ Options concernant l'analyse statistique de donnes ~~~~~~~~~~~~~~~~~~~~~~~~ */

  case MEN_DISP_STAND: /* Calcul de la dispersion standard */
    retour = I_GRF_Moyenner();
    break;

  case MEX_CORRELATION: /* Calcul des corrlations entre deux sries */
    retour = estime_liaison(graphe_courant, NULL);
    break;

  case MEX_HOM_VARIANCE:
    (void) condenser_erreur( graphe_courant );
    break;

  case MEX_KSMIRNOV: /* Comparaison de 2 distributions par le test de K-S */
    kolmogorov_smirnov(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case MEX_CVMISES: /* Comparaison de deux distributions par le test de C-vM */
    cramer_vonmises(graphe_courant);
    return BI_EVENEMENT_TRAITE;

  case MEX_ANVA_1F:
    I_ANVA_1facteur( graphe_courant );
    return BI_EVENEMENT_TRAITE;

    /* ~~~~~~ Options concernant la cration de figures ~~~~~~~~~~~~~~~~~~~~~~~~ */

  case MEX_SUPERPOSER: /* Superposition simple mise  jour... */
    TRC_OuvrirSuperposition();
    return BI_EVENEMENT_TRAITE;

  case MEX_FIGURE:       /* Tracer un (des) graphes         */
    FIG_OuvrirFenetre( NULL );
    return BI_EVENEMENT_TRAITE;

  default:
    break;
  }
	
  if (retour != NULL) /* Il faut redessiner */
    GRF_aller(retour);         /* On va  ce graphe comme graphe courant */
  else
    GRF_aller(graphe_courant); /* On va au graphe courant dj existant  */

  return BI_EVENEMENT_TRAITE; /* On ne quitte pas, le message a t trait */
}

/* -------- Raccourcis claviers -------- */

static int CLV_men_extrait(unsigned int touche, int etat, int *renvoi)
{
  *renvoi = 0; /* On ne quitte pas un menu de toute faon... */

  if ( etat & K_CTRL ) { /* On appuyait sur Control */
    if ( touche <= 255 )
      touche = (unsigned int) toupper( (int) touche); /* Comportement indfini pour autres codes ! */
    switch ( touche )
    {
    case 'A':   /* ^A = appeler le menu d'Ajustements            */
      GER_infos(EXF_AJUSTEMENTS, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'C':   /* ^C = Charger */
      return MEN_extrait(CHARGER);

    case 'E':   /* ^E = Extraire signal d'un fichier exprimental */
      return MEN_extrait( MEX_XAS_EXTRAIRE );

    case 'F':   /* ^F = transforme de Fourier d'un graphe        */
      return MEN_extrait(TF);

    case 'G':   /* ^G = appeler le menu de cration de structures */
      GER_infos(STRUCTURE, 0, 0);
      return BI_EVENEMENT_TRAITE;

    case 'K':   /* ^K = test de Kolmogorov-Smirnov */
      kolmogorov_smirnov(graphe_courant);
      return BI_EVENEMENT_TRAITE;

    case 'L':   /* ^L = appeler la liste des graphes */
      FEN_premierplan( fenetre_onglets );
      BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_GRAPHES );
      return BI_EVENEMENT_TRAITE;

    case 'M':   /* ^M = Meilleure droite */
      GRF_aller( I_GRF_AjusterDroite( graphe_courant ) );
      break;

    case 'Q':	  /* ^Q = Quitter */
      return MEN_extrait(QUITTER);
	
    case 'S':   /* ^S = Sauver graphe (format interne) */
      return MEN_extrait(SAUVER);

    case 'T':   /* ^T = appeler le menu des calculs Thoriques */
      GER_infos(CALC_THEORIQUES, 0, 0);
      break;

    case 'X':   /* ^X = lancer l'extraction complte du XANES */
      return MEN_extrait(XANES_TOUT);

#ifdef _DEVELOPPE_		
    default:
      printf("On appuyait sur la lettre no : %d (%c)\n", touche, (char) touche);
      break;
#endif
    }
  }

  return BI_EVENEMENT_NON_TRAITE; /* Par dfaut, message non-trait */
}
