/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2003, Emmanuel CURIS

   Gnralits du programme LASE (interface graphique)

   Gestion et prparation du menu de gestion des banques de donnes de spectres
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "systeme.h"

#include "exafs.h"

#include "esrf.h"
#include "fichiers.h"
#include "graphes.h"
#include "images.h"
#include "interface.h"

#include "menu_fluoX.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int CLV_men_fluoX(unsigned int touche, int etat, int *renvoi);
int MEN_PreparerFluoX(void);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Prparation du menu -------- */

int MEN_PreparerFluoX(void)
{
  arbres[MEN_FLUOX].clavier = CLV_men_fluoX;

# ifndef _DEVELOPPE_ /* Version de dveloppement uniquement */
  OBJ_active(MEN_FLUOX, MFX_V_BANQUE, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_ABERRANCE, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_CONTRASTE, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_HISTO, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_INTERPOLER, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_ROTATION, FORCE_NON);
  OBJ_active(MEN_FLUOX, MFX_CONVOLUER, FORCE_NON);
# endif

  /* Initialisation russie */
  return 0;
}

/* -------- Gestion des options -------- */

int MEN_fluoX(unsigned int option)
{
  int reponse;
  Graphe *retour = NULL;
  Image *image = NULL;

  switch (option)
  {
  case MFX_INFOS:
    ARB_AppelerInformations();
    return VRAI;

  case MFX_V_EXTRAIT:
    GER_infos(TRAITE_SPECTRES, 0, 0);
    return VRAI;

  case MFX_V_AJUSTE:
    GER_infos(EXF_AJUSTEMENTS, 0, 0);
    return VRAI;

  case MFX_V_THEORIE:
    GER_infos(CALC_THEORIQUES, 0, 0);
    return VRAI;

  case MFX_V_STRUCT:
    GER_infos(STRUCTURE, 0, 0);
    return VRAI;

  case MFX_V_BANQUE:
    GER_infos(INFOS_BANQUE, 0, 0);
    return VRAI;

  case MFX_CHARGER: /* Marche pour tout, normalement */
    retour = charger(NULL);
    break;

  case MFX_SPECTRE: /* Import d'un fichier venant du LURE sur D15 */
    Import_LURED15();
    return VRAI;

  case MFX_IMPORT_CARTE: /* Import d'un fichier fait sur ID22 ou ID21 */
    import_ESRF(ESRF_ID22);
    return VRAI;

  case MFX_GNUPLOT: /* Export de l'image au format GNUplot */
    IMG_export_gnuplot(image_courante);
    return VRAI;

  case MFX_QUITTER: /* Quitter le programme */
    reponse = SYS_ERREUR( QUITTER_PROGRAMME );
    if (reponse == 1) /* On veut quitter */
      return BI_EVENEMENT_FIN_PRG;
    break;

  case MFX_LISTE: /* Afficher la liste des images */
    OPTION_A_FAIRE( "Enlever cette option du menu" );
    return VRAI;

  case MFX_PROFIL: /* Estimer un profil de concentration radial (ou transversal) */
    retour = IMG_profil(image_courante);
    break;

  case MFX_PROFIL_V_IMG: /* Gnrer une image  partir d'un profil */
    image = IMG_generer(graphe_courant);
    break;

  case MFX_EFFETS_3D: /* Effets en 3D */
    image = effets_epaisseur(image_courante);
    break;

  case MFX_CONVOLUER: /* Effets de largeur du faisceau */
    image = convoluer_image(image_courante);
    break;
  }

  if (retour != NULL) GRF_aller(retour);
  if (image != NULL)  IMG_AllerImage( image );
  return VRAI;
}

/* -------- Raccourcis claviers -------- */

static int CLV_men_fluoX(unsigned int touche, int etat, int *renvoi)
{
  *renvoi = 0; /* On ne quitte pas un menu de toute faon... */

  if (etat & K_CTRL) /* On appuyait sur Control */
  {
    if ( touche <= 255 )
      touche = (unsigned int) toupper( (int) touche); /* Comportement indfini pour autres codes ! */
    switch ( touche )
    {
    case 'A':   /* ^A = appeler le menu d'Ajustements            */
      GER_infos(EXF_AJUSTEMENTS, 0, 0);
      return VRAI;

    case 'C':   /* ^C = Charger - marche aussi pour les structures 3D, maintenant... */
      return MEN_extrait(CHARGER);

    case 'E':   /* ^E = appeler le menu d'Extraction d'un graphe */
      GER_infos(TRAITE_SPECTRES, 0, 0);
      return VRAI;

    case 'L':   /* ^L = appeler la liste des images */
      FEN_premierplan( fenetre_onglets );
      BI_Onglets_ChangerActif( fenetre_onglets, BI_ONGLET_IMAGES );
      return VRAI;

    case 'Q':		/* ^Q = Quitter */
      return MEN_extrait(QUITTER);

    case 'T':   /* ^T = appeler le menu de calculs thoriques    */
      GER_infos(CALC_THEORIQUES, 0, 0);
      return VRAI;

#ifdef _DEVELOPPE_
    default:
      printf("On appuyait sur la lettre no : %d (%c)\n", touche, (char) touche);
      break;
#endif
    }
  }

  return FAUX; /* Par dfaut, message non-trait */
}
