/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 2003, Emmanuel CURIS
   Cration de ce fichier : 28 aot 2003

   Interface pour la sauvegarde des objets internes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "atomes.h"
#include "experience.h"
#include "images.h"
#include "interface.h"
#include "fichiers.h"
#include "serie.h"

#include "int_sauver.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_sauver_experience = NULL; /* Dossiers pour la sauvegarde */
static char *chemin_sauver_graphe     = NULL;
static char *chemin_sauver_image      = NULL;
static char *chemin_sauver_molecule   = NULL;

/* Chemins par dfaut : dfinies dans variables.c */
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* -------- Sauvegarde d'un graphe -------- *
   Renvoie 0    si tout va bien
           != 0 en cas de problme
 * ---------------------------------------- */

int I_GRF_Sauver(struct graphe *graphe)
{
  const char *masque;
  char *nom_fichier;
  int retour;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  if ( NULL == graphe ) return +1;

  /* Choix de l'extension */
  if ( ( graphe->type >= 0 ) && ( graphe->type < NOMBRE_TYPES_GRAPHES ) )
    masque = extensions[ graphe->type ];
  else
    masque =  "*.*";

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) titre_sauver_graphe;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_graphe;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, principale );
  /* On mmorise les chemins */
  chemin_sauver_graphe = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_graphe );

  if ( NULL == nom_fichier ) return -1;	/* Annul ou erreur */

  /* On sauvegarde dans le fichier indiqu */
  retour = GRF_SauverIsole( nom_fichier, graphe );

  /* Plus besoin du nom du fichier */
  free( nom_fichier ); nom_fichier = NULL;

  /* On renvoie le rsultat */
  return retour;
}


/* -------- Sauvegarde d'une exprience -------- *
   Renvoie 0    si tout va bien
           != 0 en cas de problme
 * ---------------------------------------- */

int I_EXP_Sauver(struct experience *experience)
{
  char *nom_fichier;
  int retour;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };
  const char *masque = "*.lase";

  if ( NULL == experience ) return +1;

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) titre_sauver_experience;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_experience;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, fenetre_onglets );
  /* On mmorise les chemins */
  chemin_sauver_experience = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_experience );

  if ( NULL == nom_fichier ) return -1;	/* Annul ou erreur */

  /* On sauvegarde dans le fichier indiqu */
  retour = EXP_SauverIsolee( nom_fichier, experience );

  /* Plus besoin du nom du fichier */
  free( nom_fichier ); nom_fichier = NULL;

  /* On renvoie le rsultat */
  return retour;
}

/* -------- Sauvegarde d'une image -------- *
   Renvoie 0    si tout va bien
           != 0 en cas de problme
 * ---------------------------------------- */

int I_IMG_Sauver(struct image *image)
{
  char *nom_fichier;
  int retour;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };
  const char *masque = "*.lase";

  if ( NULL == image ) return +1;

  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) titre_sauver_image;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_image;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, fenetre_onglets );
  /* On mmorise les chemins */
  chemin_sauver_image = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_image );

  if ( NULL == nom_fichier ) return -1;	/* Annul ou erreur */

  /* On sauvegarde dans le fichier indiqu */
  retour = IMG_SauverIsolee( nom_fichier, image );

  /* Plus besoin du nom du fichier */
  free( nom_fichier ); nom_fichier = NULL;

  /* On renvoie le rsultat */
  return retour;
}

/* -------- Sauvegarde d'une molcule -------- *
   Renvoie 0    si tout va bien
           != 0 en cas de problme
 * ---------------------------------------- */

int I_MOL_Sauver(struct molecule *molecule)
{
  char *nom_fichier;
  int retour;
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };
  const char *masque = "*.lase";

  if ( NULL == molecule ) return +1;

  /* Appel du slecteur */
  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) titre_sauver_molecule;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_molecule;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, fenetre_onglets );
  /* On mmorise les chemins */
  chemin_sauver_molecule = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_molecule );

  if ( NULL == nom_fichier ) return -1;	/* Annul ou erreur */

  /* On sauvegarde dans le fichier indiqu */
  retour = MOL_SauverIsolee( nom_fichier, molecule );

  /* Plus besoin du nom du fichier */
  free( nom_fichier ); nom_fichier = NULL;

  /* On renvoie le rsultat */
  return retour;
}
