/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1996 - 2008, Emmanuel CURIS
   Ce fichier : 19 janvier 2008 par copie de series_grf.c

   Oprations fondamentales sur les sries d'images
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "interface.h"
#include "images.h"

#include "int_series_img.txt"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

static char *chemin_sauver_serie_images  = NULL;

/* Chemins par dfaut : dfinies dans variables.c */
extern unsigned int nombre_chemins_sauvegarde; extern char **chemins_sauvegarde;

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Crer une nouvelle srie, en demandant le nom  l'utilisateur -------- */

struct serie_images *I_SERIE_IMG_Creer(struct serie_images *avant, const char *nom_propose)
{
  char *titre;
  struct serie_images *nouvelle_serie;

  /* On demande le titre de la srie  l'utilisateur */
  if ( NULL == nom_propose ) {
    if ( NULL == IMG_serie_courante ) {
      nom_propose = txt_premiere_serie_images;
    } else {
      nom_propose = IMG_serie_courante->nom;
    }
  }
  titre = BI_Demande_Texte( fen_images, &Q_NomNouvelleSerieImage, nom_propose );

  /* On cre cette srie */
  nouvelle_serie = IMG_SERIE_CreerSerie( avant, titre );

  /* On contrle que la cration a fonctionn */
  if ( NULL == nouvelle_serie ) {
#ifdef _DEVELOPPE_
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crer la srie %s\n",
		   __FILE__, __FUNCTION__, __LINE__, titre);
#endif
    (void) BI_ERR_alerte( fen_images, &ERR_CreationSerieImageImpossible, VRAI, titre );
  }

  /* On a termin : on nettoie & on renvoie la srie cre */
  free( titre ); titre = NULL;
  return nouvelle_serie;
}

/* -------- Sauver une srie, en demandant le nom du fichier  l'utilisateur -------- */

int I_SERIE_IMG_Sauver(struct serie_images *serie)
{
  int retour;
  char *nom_fichier;
  const char *masque = "*.lase";
  BI_InfosSelecteur infos_selecteur = {
    NULL,
    BI_SELECTEUR_FILTRE_UTILISATEUR | BI_SELECTEUR_AJOUTER_PREDEFINI | BI_SELECTEUR_AVEC_SAUVEGARDE | BI_SELECTEUR_MEMORISER_CHEMIN,
    NULL,
    1, NULL,
    0, NULL,
    (void (*)(unsigned int, unsigned int, const char *) ) NULL
  };

  if ( NULL == serie ) return -1;

  /* On demande le nom du fichier */
  /* Prparation du slecteur de fichiers */
  infos_selecteur.titre  = (char *) txt_sauver_serie_images;
  infos_selecteur.filtres = &masque;
  infos_selecteur.chemin = chemin_sauver_serie_images;
  infos_selecteur.nbr_predefinis = nombre_chemins_sauvegarde;
  infos_selecteur.predefinis = chemins_sauvegarde;

  nom_fichier = BI_SelecteurSauvegarde( &infos_selecteur, NULL, fen_images );
  /* On mmorise les chemins */
  chemin_sauver_serie_images = infos_selecteur.chemin;
  nombre_chemins_sauvegarde = infos_selecteur.nbr_predefinis;
  chemins_sauvegarde = infos_selecteur.predefinis;
  /* On ajoute en particulier le chemin courant  la liste des chemins */
  BI_Selecteur_AjouterChemin( &chemins_sauvegarde, &nombre_chemins_sauvegarde, chemin_sauver_serie_images );

  if ( NULL == nom_fichier ) return +1;	 /* Annulation ou erreur */

  /* On fait la sauvegarde demande */
  retour = IMG_SERIE_SauverIsolee( serie, nom_fichier, 0 );
  free( nom_fichier ); nom_fichier = NULL;
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s, %s l.%u : Erreur en sauvant la srie [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return -2;
  }

  /* On mentionne la sauvegarde dans l'historique, puisqu'elle a russi */
  HIST_Titre( 1, "Sauvegarde", "E/S" );
  HIST_Texte( 1, 0, "Fichier : %s\n", nom_fichier );
  HIST_Texte( 1, 0, "S"e_AIGU"rie sauv"e_AIGU" : %s\n", serie->nom );

  /* La sauvegarde s'est bien pass */
  return 0;
}

/* -------- Insrer un graphe dans la srie indique, ou une nouvelle -------- */

struct serie_images *I_IMG_InsererDansSerie(struct serie_images *serie_cible, Image *image_avant, Image *image_a_inserer)
{
  int retour;

  if ( NULL == image_a_inserer ) return NULL;

  /* Dtermination de la srie cible */
  if ( NULL == serie_cible ) {
    serie_cible = IMG_QuelleSerie( image_avant );
    if ( NULL == serie_cible ) {
      serie_cible = IMG_serie_courante;
    }
  }
  if ( NULL == serie_cible ) {
    /* C'est qu'aucune srie n'a t cre ! */
    /* On cre donc une nouvelle srie, la premire */
    serie_cible = I_SERIE_IMG_Creer( serie_cible, txt_nom_serie_image_resultats );
  }

  /* Peut-on ajouter des images dans la srie retenue ? */
  while ( ( NULL == serie_cible ) || ( IMG_SERIE_EstProtegee( serie_cible ) ) ) {
    int reponse;

    /* La srie ne peut recevoir d'image supplmentaire : que faire ? */
    reponse = BI_ERR_alerte( fen_images, &Q_SerieImageProtegee, VRAI, ( serie_cible != NULL ) ? ( serie_cible->nom ) : ("NULL") );
    if ( 1 == reponse ) {
      /* On cre une nouvelle srie */
      serie_cible = I_SERIE_IMG_Creer( serie_cible, txt_nom_serie_image_resultats );
    } else {
      /* On slectionne une srie parmi celles existantes */
      serie_cible = I_SERIE_IMG_Choisir( -1, ROOT, 0 );
    }
  }

  /* Est-ce que l'image indique est bien dans la srie retenue ?
     Peut tre non s'il est dans une srie protge ==> aucun message particulier ici */
  if ( image_avant != NULL ) {
    struct serie_images *serie_img_avant = NULL;

    serie_img_avant = IMG_QuelleSerie( image_avant );
    if ( serie_img_avant != serie_cible )
      image_avant = NULL;
  }

  /* On insre l'image dans la srie indique */
  retour = IMG_InsererDansSerie( image_a_inserer, serie_cible, image_avant );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'insrer l'image %s dans la srie %s [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, image_a_inserer->nom, serie_cible->nom, retour);
    return NULL;
  }

  /* On a russi : tout s'est bien pass
     On renvoie la srie [ventuellement cre] qui contient le graphe */
  return serie_cible;
}

/* -------- Choisir une srie parmi la liste des sries disponibles -------- */

struct serie_images *I_SERIE_IMG_Choisir(int numero_arbre, unsigned int numero_objet, unsigned long lgmax_texte)
{
  unsigned long i, nombre_series;
  int choix;
  char **textes = NULL;
  Serie_Images *serie;

  if ( NULL == IMG_premiere_serie ) {
    /* Aucune srie ralise : rien  choisir ! */
    if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
      OBJ_CopierTexte( numero_arbre, numero_objet, "Aucune s"e_AIGU"rie en m"e_AIGU"moire", lgmax_texte );
    }
    return NULL;
  }
  if ( NULL == IMG_premiere_serie->suivante ) {
    /* Une seule srie prsente : pas rellement de choix... */
    if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
      OBJ_CopierTexte( numero_arbre, numero_objet, IMG_premiere_serie->nom, lgmax_texte );
    }
    return IMG_premiere_serie; /* Une seule srie... */
  }

  /* On compte le nombre de sries en mmoire */
  for ( nombre_series = 0, serie = IMG_premiere_serie ; serie != NULL ; serie = serie->suivante, nombre_series++ );

  /* On prpare la table des noms des sries */
  textes = (char **) malloc( sizeof(char *) * nombre_series );
  if ( NULL == textes ) {
    ERR_Memoire( "Table des noms de sries", sizeof(char *) * nombre_series );
    return NULL;
  }
  for ( i = 0, serie = IMG_premiere_serie ; i < nombre_series ; i++, serie = serie->suivante ) {
    textes[ i ] = serie->nom;
  }

  choix = SS_menu_texte( (const char **) textes, nombre_series, "Liste des s"e_AIGU"ries d'images", numero_arbre, numero_objet );
  free( textes ); textes = NULL;
  if ( choix < 0 ) return NULL;

  for ( i = 0, serie = IMG_premiere_serie ; i < (unsigned long) choix ; i++, serie = serie->suivante );
  if ( ( numero_arbre >= 0 ) && ( numero_objet != ROOT ) ) {
    OBJ_CopierTexte( numero_arbre, numero_objet, serie->nom, lgmax_texte );
  }

  return serie;
}
