/* LASE - Logiciel d'Analyse des Spectres ExpÃ©rimentaux
   (C) 2006, Emmanuel CURIS
   CrÃ©ation : 2 juin 2006

   Interface graphique

   Soustraction d'un fond continu Ã  un graphe
   Dans le cas du XAS : Ã´ter la contribution de la matrice (Âµm)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"
#include "exafs.h"

#include "automatise.h"
#include "extrait.h"
#include "interface.h"
#include "operations.h"		/* Pour la dÃ©rivÃ©e */
#include "science.h"
#include "serie.h"
#include "tracer.h"

/* -------- Variables locales -------- */

struct infos_mu0_etape_1 {
  /* Les paramÃ¨tres nÃ©cessaires Ã  l'extraction */
  MU1_infos parametres;

  /* Infos complÃ©mentaires pour l'affichage */
  double residu;
  boolean affiche_derivees;
  Graphe *derivee_premiere, *derivee_seconde;

  /* Infos pour la post-analyse */
  unsigned int type_chaine;
};

/* -------- Les textes [isolÃ©s pour traduction] -------- */

#include "int_xas_etape1.txt"

/* -------- Fonctions locales -------- */

static struct infos_mu0_etape_1 *I_XAS_MU01_CreerInfos(Graphe *brut);

static Fenetre *I_XAS_MU01_RappelerFenetre(Graphe *brut);
static boolean  I_XAS_MU01_GrapheConvient (Graphe *brut);

static int  I_XAS_MU01_LibererFenetre(Fenetre *fenetre, boolean fermer);

static int  I_XAS_MU01_Messages(Fenetre *fen, Message *message);
static int  I_XAS_MU01_AdapterGraphes(Fenetre *fenetre, const Graphe *nv_brut, boolean nombre_de_points_a_change);
static int  I_XAS_MU01_FermerFenetre (Fenetre *fenetre, int mode);

static void I_XAS_MU01_MettreAJour(Fenetre *fenetre, const Graphe *brut, MU1_infos *infos, boolean recalculer_mu0);

static void RCT_mu01(Fenetre *fen, int x_souris, int y_souris, int vx_x, int vx_y);
static void RDS_mu01(Fenetre *fen, int x, int y, unsigned int larg, unsigned int haut);
static int  GER_mu01(unsigned int num_objet, int x, int y);
static int  CLV_mu01(Fenetre *fen, unsigned int touche, int etat);
static int  CLC_mu01(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

static int prepare_arbre_mu0_1(unsigned int numero_arbre);

/* ~-~-~-~-~-~-~-~-~ Appel par l'utilisateur ~-~-~-~-~-~-~-~-~ */

/* -------- Routine d'appel principale : ouverture de la fenÃªtre et initialisation -------- */

/* -------- Ouverture de la fenÃªtre associÃ©e Ã  cette Ã©tape ----------------------------------\
 |   Renvoie le graphe exafs extrait (qui ne devrait pas changÃ© tout au long de l'extraction |
 |                                                                                           |
 | brut = le graphe dont on veut extraire l'exafs                                            |
 | chaine = VRAI si dans une extraction chaÃ®nÃ©e -> taille fenÃªtre adaptÃ©e                    |
 | mu0 = renvoie adresse du graphe contenant le Âµ0 extrait - mÃªme remarque que pour l'exafs  |
 \------------------------------------------------------------------------------------------ */

Graphe *I_XAS_OterMu0_Etape1(Graphe *brut, unsigned int type_chaine, Graphe **mu0_estime)
{
  Fenetre *fenetre = NULL;
  struct infos_mu0_etape_1 *infos = NULL;

  /* ContrÃ´le sur le graphe */
  if ( FAUX == I_XAS_MU01_GrapheConvient( brut ) ) {
    AVERTISSEMENT( "Le graphe %s ne convient pas !", ( brut != NULL ) ? ( brut->nom ) : "NULL" );
    return NULL;
  }

  /* Est-ce que l'arbre est dÃ©jÃ  ouvert ? */
  if ( VRAI == ARB_EstOuvert( ELIM_MU0_1 ) ) {
    fenetre = I_XAS_MU01_RappelerFenetre( brut );
    if ( NULL == fenetre ) {
      /* On ne souhaite pas mettre Ã  jour la fenÃªtre avec le graphe indiquÃ© -> terminÃ© ! */
      return NULL;
    }
  }

  /* On crÃ©e les informations */
  infos = I_XAS_MU01_CreerInfos( brut );
  if ( NULL == infos ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible de crÃ©er les informations nÃ©cessaires : calcul annulÃ©\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FEN_Detruire( fenetre );	/* La fenÃªtre peut encore exister, si Ã©tait associÃ©e Ã  un ancien graphe... */
    return NULL;
  }
  infos->type_chaine = type_chaine;

  /* Ici : fenÃªtre Ã  initialiser, voire Ã  crÃ©er */
  if ( NULL == fenetre ) {
    /* Il faut bien crÃ©er la fenÃªtre, pas seulement la rÃ©initialiser */
    fenetre = FEN_CreerAvecArbre( TOUS_SF_ASC, titre_fenetre, NULL,
				  ELIM_MU0_1, BI_FEN_ARBRE_HAUT, prepare_arbre_mu0_1,
				  RDS_mu01, CLV_mu01, CLC_mu01, 
				  I_XAS_MU01_Messages, RCT_mu01, ( int (*)(Fenetre *) ) NULL);
    if ( NULL == fenetre ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de crÃ©er la fenÃªtre pour l'interface !\n",
		     __FILE__, __FUNCTION__, __LINE__);
      free( infos ); infos = NULL;
      return NULL; /* Rien Ã  faire... */
    }
    fenetre->drapeaux.redessin    = VRAI;
    fenetre->msg_masque.fermeture = VRAI; /* Messages traitÃ©s par MSG_matrice */
  }

  /* On associe le graphe Ã  la fenÃªtre */
  FEN_GRF_LierGraphe( fenetre, brut );

  /* On associe les informations Ã  la fenÃªtre & Ã  l'arbre */
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, infos );
  ARB_MettreComplement( ELIM_MU0_1, infos );

  if ( I_XAS_CHAINE_XAS == type_chaine ) { /* On ouvre dans une chaÃ®ne */
    FEN_dimensions( fenetre, -500, 0, -500, -500 ); /* Un quart d'Ã©cran, haut droit */
  } else if ( I_XAS_CHAINE_EXAFS == type_chaine ) {
    FEN_dimensions( fenetre, 0, 0, -500, -500 ); /* Un quart d'Ã©cran, haut gauche */
  }

  /* On ouvre la fenÃªtre */
  FEN_Ouvrir( fenetre );

  XAS_MU01_CalculerMu0_I( brut, &(infos->parametres) );	/* Calcul du polynÃ´me */
  XAS_MU01_ExtraireEXAFS( brut, &(infos->parametres) );	/* Extraction         */

  OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E0, infos->parametres.e_0, 0.0 );
  OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E2, infos->parametres.e_2, 0.0 );

  if ( mu0_estime != NULL ) *mu0_estime = infos->parametres.mu_0;
  return infos->parametres.exafs;
}

/* -------- Renvoie VRAI si le graphe convient, FAUX s'il n'est pas lÃ©gitime de lui Ã´ter un fond continu -------- */

static boolean I_XAS_MU01_GrapheConvient(Graphe *brut)
{
  if ( NULL == brut ) return FAUX; /* Rien Ã  faire ! */

  /* Le graphe doit Ãªtre une fonction rÃ©elle : y=f(x) */
  if ( FAUX == GRF_EstFonctionReelle( brut ) ) {
    (void) BI_ERR_alerte( principale, &ERR_FonctionReelle, VRAI, brut->nom );
    return FAUX;
  }

  if ( ( brut->type >= 0 ) && ( GR_BRUT != brut->type ) && ( GR_MOINSMAT != brut->type ) ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &ERR_TypeIncorrect, VRAI );
    if ( I_XAS_MU01_PBGRAPHE_ANNULER == reponse ) {
      return FAUX;
    }
  }

  /* On doit avoir des eV en X pour que la conversion dans l'espace des k fonctionne */
  if ( FAUX == GRF_ControlerUnite( brut, "eV", AXE_X, VRAI ) ) {
    int reponse;

    reponse = BI_ERR_alerte( principale, &ERR_UniteIncorrecte, VRAI );
    if ( I_XAS_MU01_PBGRAPHE_ANNULER == reponse ) {
      return FAUX;
    }
  }

  /* Le graphe convient, ou on accepte ses bizarreries */
  return VRAI;
}

/* -------- Que faire si la fenÃªtre est dÃ©jÃ  ouverte ? -------- */

static Fenetre *I_XAS_MU01_RappelerFenetre(Graphe *brut)
{
  Fenetre *fenetre;
  struct infos_mu0_etape_1 *infos __attribute__((__unused__));
  Graphe *graphe;
  int reponse;

  /* Quelle est la fenÃªtre associÃ©e Ã  l'arbre ? */
  fenetre = ARB_QuelleFenetre( &(arbres[ ELIM_MU0_1 ]), FAUX );

  /* Quelles sont les informations associÃ©es Ã  cette fenÃªtre ? */
  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (struct infos_mu0_etape_1 *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  /* Est-ce que le graphe est le mÃªme ? */
  if ( graphe == brut ) {
    /* On travaille sur le mÃªme graphe : on se contente de faire repasser au premier plan */
    FEN_premierplan( fenetre );
    return NULL;
  }

  /* On pose la question : faut-il rappeler,... ? */
  reponse = BI_ERR_alerte( fenetre, &Q_ReactiverMu0Etape1, VRAI, graphe->nom, brut->nom );
  switch ( reponse )
  {
  case I_XAS_MU01_RAPPELER:
    FEN_premierplan( fenetre );
    return NULL;

  case I_XAS_MU01_RELANCER:
    /* On termine les calculs pour l'ancien graphe : ferme la fenÃªtre et libÃ¨re tout */
    I_XAS_MU01_LibererFenetre( fenetre, FAUX );

    /* On rappelera donc une nouvelle fenÃªtre propre... */
    return fenetre;

  case I_XAS_MU01_ANNULER:
    /* On ne fait rien de spÃ©cial */
    return NULL;

  default:
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en posant la question, ou rÃ©ponse inconnue [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, reponse);
    return fenetre;
  }
}

/* -------- CrÃ©er et initialiser les informations associÃ©es Ã  la fenÃªtre -------- */

static struct infos_mu0_etape_1 *I_XAS_MU01_CreerInfos(Graphe *brut)
{
  struct infos_mu0_etape_1 *infos = NULL;
  unsigned int i;

  /* On rÃ©serve les informations */
  infos = (struct infos_mu0_etape_1 *) malloc( (size_t) sizeof(struct infos_mu0_etape_1) );
  if ( NULL == infos ) {
    ERR_Memoire( "ParamÃ¨tres d'extraction", sizeof( struct infos_mu0_etape_1 ) );
    return NULL;
  }

  /* On initialise les informations */
  /* a. le graphe contenant le rÃ©sultat [isolÃ©, sera mis en sÃ©rie seulement quand on aura validÃ© le calcul !] */
  infos->parametres.exafs = GRF_nv_isole( brut->nombre_point, GR_EXAFS1, GRF_FCT_REELLE );
  if ( infos->parametres.exafs == NULL ) {
    free( infos ); infos = NULL;
    return NULL;
  }
#ifdef _DEVELOPPE_
  GRF_titre( infos->parametres.exafs, "Ãtape 1 extraction EXAFS,  EXAFS en cours de calcul" );
#endif

  /* b. le graphe contenant le Âµ0 modÃ©lisÃ© [isolÃ©, mÃªme remarque] */
  infos->parametres.mu_0 = GRF_nv_isole( brut->nombre_point, GR_MU0, GRF_FCT_REELLE );
  if ( NULL == infos->parametres.mu_0 ) {
    GRF_eff_isole( infos->parametres.exafs );
    free( infos ); infos = NULL;
    return NULL;
  }
#ifdef _DEVELOPPE_
  GRF_titre( infos->parametres.exafs, "Ãtape 1 extraction EXAFS,  Âµ0 en cours de calcul" );
#endif

  if ( OBJ_choisit(ELIM_MU0_1, ELIM_MU0_RAPIDE, QUESTION) == VRAI )
    infos->parametres.methode = ELIM_MU0_RAPIDE;
  if ( OBJ_choisit(ELIM_MU0_1, ELIM_MU0_LENT  , QUESTION) == VRAI )
    infos->parametres.methode = ELIM_MU0_LENT;

  /* On cherche la valeur de E0 la plus plausible */
  infos->parametres.e_0 = cherche_e0 (brut, FAUX);                   /* E0 : maximum de la dÃ©rivÃ©e             */
  infos->parametres.e_2 = plus_proche(brut, infos->parametres.e_0 + 100.0, VRAI); /* E2 : on se place 100 eV aprÃ¨s le seuil */
  /* Dans EDAEES : utilise la valeur en 2*x_min_avant_seuil - x_max */
  infos->parametres.e_3 = brut->x_max;

  /* On calcule la fonction Ã  soustraire */
  for ( i = 0 ; i <= XAS_MU01_DEGRE_MAXIMAL ; i++ ) {
    infos->parametres.coefs[ i ] = 0.0;
  }
  infos->parametres.degre       = OBJ_LireEntier_U( ELIM_MU0_1, DEGRE_VALEUR );
  infos->parametres.deg_affiche = OBJ_LireEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE );
  infos->parametres.avec_e3     = OBJ_choisit( ELIM_MU0_1, MU1_2_PTS, QUESTION );

  /* On protÃ¨ge les graphes */
  brut->drapeaux.protege = VRAI;
  infos->parametres.exafs->drapeaux.protege = VRAI;
  infos->parametres.mu_0->drapeaux.protege = VRAI;

  /* Initialisation terminÃ©e */
  return infos;
}

/* -------- LibÃ©rer les informations associÃ©es Ã  la fenÃªtre -------- */

static int I_XAS_MU01_LibererFenetre(Fenetre *fenetre, boolean fermer)
{
  struct infos_mu0_etape_1 *infos;
  Graphe *brut;

  if ( NULL == fenetre ) return -2;  /* Ne devrait jamais arriver, mais bon... */

  /* On rÃ©cupÃ¨re les informations */
  brut   = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos  = (struct infos_mu0_etape_1 *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  /* On les dissocie de la fenÃªtre */
  FEN_GRF_LierGraphe( fenetre, NULL );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );
  ARB_MettreComplement( ELIM_MU0_1, NULL );

  /* On les libÃ¨re */
  GRF_DeprotegerGraphe( brut );

  if ( infos != NULL ) {
    Graphe *vx_exafs, *vx_mu0;

    vx_exafs = infos->parametres.exafs; infos->parametres.exafs = NULL;
    vx_mu0   = infos->parametres.mu_0 ; infos->parametres.mu_0  = NULL;

    /* On informe que le graphe ré³µltat va dispara?e */
    GRF_modifier( vx_exafs, NULL, VRAI );

    /* On efface les graphes de ré³µltats, isol?*/
    GRF_eff_isole( vx_exafs );
    GRF_eff_isole( vx_mu0 );

    /* Plus besoin des infos */
    free( infos ); infos = NULL;
  }

  /* On ferme la fen?e, si on le demande */
  if ( VRAI == fermer ) {
    FEN_DetruireFenetre( fenetre, FAUX );
  }

  /* Terminé ³ans souci */
  return 0;
}

/* ~~~~~~~~~~~~~ Gestion de la fenÃªtre d'Ã©limination de mu0, premiÃ¨re Ã©tape ~~~~~~~~~~~~~~ */

/* ------------- Redessin de la fenÃªtre d'Ã©limination de mu matrice -----------------------\
 |   Le clipping est dÃ©jÃ  installÃ© correctement lors de l'appel                            |
 |                                                                                         |
 | fenetre    : structure associÃ©e Ã  la fenÃªtre considÃ©rÃ©e                                 |
 | x, y       : coordonnÃ©es de la zone de redessin                                         |
 | larg, haut : dimensions de la zone de redessin                                          |
 \---------------------------------------------------------------------------------------- */

static void RDS_mu01(Fenetre *fenetre, int x, int y, unsigned int larg, unsigned int haut)
{
  InfosRegion2D region;
  Graphe *graphe;
  MU1_infos *infos;
  double echelle_X, echelle_Y;
  int x_barre, retour;

  /* PrÃ©paration du dessin */
  TRC_InitDessins( fenetre, x, y, larg, haut );

  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );

  if ( NULL == graphe ) {
    TRC_Message( fenetre, txt_brut_nul );
    return;
  }
  if ( ( graphe->x_max == graphe->x_min ) || ( graphe->y_max == graphe->y_min ) ) {
    TRC_Message( fenetre, txt_brut_nul );
    return;
  }

  /* On prÃ©pare la rÃ©gion 2D : aide pour les dessins aprÃ¨s en fait */
  retour = TRC_Z2D_Initialiser( &region, fenetre, graphe );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en initialisant la rÃ©gion 2D [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* On trace le graphe dans la rÃ©gion 2D et on rÃ©cupÃ¨re les Ã©chelles au passage */
  TRC_GrapheDansRegion2D( fenetre, &region, 0, graphe, NOIR, BI_CONTINU, BI_MARQUE_AUCUNE, 0, &echelle_X, &echelle_Y );

  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos ) {
    TRC_Message( fenetre, txt_infos_nul );
    return;
  }

  /* On trace la courbe Ã  soustraire - en bleu */
  TRC_Polynome( fenetre, &region, BLEU, BI_CONTINU, 0, infos->degre, infos->coefs, graphe->nombre_point, graphe->x );

  /* On met les barres */
  TRC_Couleur( ROUGE ); /* Barre rouge : E0 */
  x_barre = TRC_ConvertitX_PvE( infos->e_0, graphe->x_min, 0, echelle_X );
  TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );

  TRC_Couleur( VERT );  /* Barre verte : E2 */
  x_barre = TRC_ConvertitX_PvE( infos->e_2, graphe->x_min, 0, echelle_X );
  TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );

  if ( VRAI == infos->avec_e3 ) {
    TRC_Couleur( VERT_FONCE );  /* Barre vert foncÃ© : E3 */
    x_barre = TRC_ConvertitX_PvE( infos->e_3, graphe->x_min, 0, echelle_X );
    TRC_LigneV( fenetre, x_barre, 0, fenetre->w_haut );
  }

  /* On trace le graphe EXAFS extrait, en gris - en fin, car Ã©chelle diffÃ©rente */
  if ( (infos->exafs->x_max == infos->exafs->x_min)
    || (infos->exafs->y_max == infos->exafs->y_min) ) {
    TRC_Message( fenetre, txt_exafs_nul );
    return;
  }

  /* On calcule l'Ã©chelle nÃ©cessaire pour cela */
  retour = TRC_Z2D_Initialiser( &region, NULL, infos->exafs );
  if ( retour != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Erreur en initialisant la rÃ©gion 2D [code %d]\n",
		   __FILE__, __FUNCTION__, __LINE__, retour);
    return;
  }

  /* On trace le graphe dans la rÃ©gion 2D et on rÃ©cupÃ¨re les Ã©chelles au passage */
  TRC_GrapheDansRegion2D( fenetre, &region, 0, infos->exafs, GRIS_FONCE, BI_CONTINU, BI_MARQUE_AUCUNE, 0, &echelle_X, &echelle_Y );

  /* On trace l'axe des X associÃ©, en violet */
  TRC_Couleur( VIOLET );
  x_barre = TRC_ConvertitY_PvE( 0.0, graphe->y_min, 0, echelle_Y );
  TRC_LigneH( fenetre, 0, x_barre, fenetre->w_larg );
}

/* -------- DÃ©placement de la souris, fenÃªtre d'Ã©limination de Âµ0 (1re Ã©tape) -------- */

void RCT_mu01(Fenetre *fenetre, int x_souris, int y_souris, int vx_x, int vx_y)
{
  Graphe  *graphe;
  MU1_infos *infos = NULL;
  double x_echelle, x_reel;
  char *texte;

  /* On passe en coordonnÃ©es de la fenÃªtre */
  x_souris -= fenetre->w_x; y_souris -= fenetre->w_y;
  vx_x     -= fenetre->w_x; vx_y     -= fenetre->w_y;

  graphe = (Graphe *) FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  if (graphe == NULL) return;

  if ( graphe->x_min == graphe->x_max ) return; /* Pas de pointeur si bornes identiques... */

  TRC_Mode( BI_MODE_INVERSE );
  TRC_Couleur( NOIR );
  TRC_Style( BI_CONTINU );
  TRC_Epaisseur( 0 );
  
  /* On efface l'ancien pointeur */
  if ( ( vx_x >= 0 ) && ( vx_x <= (int) fenetre->w_larg )
    && ( vx_y >= 0 ) && ( vx_y <= (int) fenetre->w_haut ) ) { /* Il Ã©tait bien sur la fenÃªtre */
#ifdef _DEVELOPPE_
    /*    printf("Ancien %ld effacÃ© en %d %d\n", fenetre->handle, vx_x, vx_y); */
#endif
    TRC_LigneV( fenetre, vx_x, 0, fenetre->w_haut );
  }

  if ( ( y_souris < 0 ) || ( y_souris > (int) fenetre->w_haut )  /* Souris hors de la partie intÃ©ressante */
    || ( x_souris < 0 ) || ( x_souris > (int) fenetre->w_larg ) ) {
    return;
  }

  /* On met le nouveau pointeur */
  TRC_LigneV( fenetre, x_souris, 0, fenetre->w_haut );

  x_echelle = ( graphe->x_max - graphe->x_min ) / ( (double) fenetre->w_larg );
  x_reel    = TRC_ConvertitX_EvP( x_souris, graphe->x_min, 0, x_echelle );

  /* Plus de dessin Ã  faire : on repasse en mode recouvrir... */
  TRC_Mode( BI_MODE_REMPLACE );

  /* Affichage du texte : seulement si les infos sont prÃ©sentes !  */
  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos ) {
    FEN_infos( fenetre, "En cours d'ouverture" );
    return;
  }

  /* Recherche des unitÃ©s */
  if ( NULL == infos->exafs ) {
    texte = TXT_genere_chaine( infos_fenetre_sans_exafs, x_reel, GRF_QuelleUnite( graphe, AXE_X ) );
  } else {
    double k_reel, k_echelle;
    double k_du_E, E_du_k;
    const char *unite_k;

    /* Ã quelle position est-on en k ? */
    if ( infos->exafs->x_max == infos->exafs->x_min ) {
      return;
    }
    k_echelle = ( (double) fenetre->w_larg ) / ( infos->exafs->x_max - infos->exafs->x_min);
    k_reel    = infos->exafs->x_min + ( (double) ( x_souris - fenetre->w_x ) ) / k_echelle;

    /* On rÃ©cupÃ¨re l'Ã©chelle en X du graphe extrait */
    unite_k = GRF_QuelleUnite( infos->exafs, AXE_X );
    if ( NULL == unite_k ) unite_k = ANGSTROM"^-1";

    /* On fait les conversions pour faciliter la lecture */
    /* a. le X du graphe brut converti en Ã©chelle des k */
    k_du_E = CONV_E_vers_k( x_reel, infos->e_0, GRF_QuelleUnite( graphe, AXE_X ), unite_k );

    /* b. le X de l'EXAFS converti en Ã©nergie */
    E_du_k = CONV_k_vers_E( k_reel, unite_k, infos->e_0, GRF_QuelleUnite( graphe, AXE_X ) );

    /* On fait la chaÃ®ne d'informations */
    texte = TXT_genere_chaine( infos_fenetre, x_reel, GRF_QuelleUnite( graphe, AXE_X ), k_du_E, unite_k,
			                      k_reel, unite_k, E_du_k, GRF_QuelleUnite( graphe, AXE_X ) );
  }
  FEN_infos( fenetre, texte );
  free( texte ); texte = NULL;
}

/* -------------- Messages reÃ§us par la fenÃªtre d'Ã©limination de mu_0, 1re Ã©tape ---------- */

static int I_XAS_MU01_Messages(Fenetre *fenetre, Message *message)
{
  int retour = BI_EVENEMENT_NON_TRAITE;

  switch ( message->type )
  {
  case MSG_GRAPHE_MODIFIE:
    retour = I_XAS_MU01_AdapterGraphes( fenetre, (Graphe *) (message->divers.pointeur), message->divers.drapeau );
    break;

  case WM_CLOSED:
    retour = I_XAS_MU01_FermerFenetre( fenetre, message->fenetre.haut );
    break;

  default:
    AVERTISSEMENT( "Message ignorÃ© [%d]", message->type );
  }

  /* On renvoie ce que l'on a eu comme rÃ©ponse */
  return retour;
}

/* ------ Prendre en compte un changement du graphe "brut" et faire ce qu'il faut avec -------- */

static int I_XAS_MU01_AdapterGraphes(Fenetre *fenetre, const Graphe *nv_brut, boolean nombre_de_points_a_change)
{
  MU1_infos *infos;

  if ( NULL == nv_brut ) {
    /* C'est que le graphe servant de brut a Ã©tÃ© effacÃ©, par exemple. Il faut donc fermer la fenÃªtre. */
    FEN_GRF_LierGraphe( fenetre, nv_brut );  /* Mais on indique bien qu'elle n'a plus de brut ! */
    return I_XAS_MU01_FermerFenetre( fenetre, BI_FEN_FERMER_FORCER_ANNULE );
  }

  /* On rÃ©cupÃ¨re les informations */
  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );

  /* A-t-on changÃ© le nombre de points du graphe brut ? */
  if ( VRAI == nombre_de_points_a_change ) { /* OUI -> on redÃ©marre Ã  zÃ©ro, en fait */
    /* On n'efface pas les graphes car ils peuvent Ãªtre utilisÃ©s par la fenÃªtre de l'Ã©tape suivante */
    unsigned long nv_nbr_pts = nv_brut->nombre_point;

    /* On adapte le nombre de points pour le spectre EXAFS & le Âµ0 */
    if ( ( FAUX == GRF_AdapteTaille( infos->exafs, nv_nbr_pts ) ) ||
	 ( FAUX == GRF_AdapteTaille( infos->mu_0 , nv_nbr_pts ) ) ) {
      /* Si l'on ne peut pas adapter, on force la fermeture de la fenÃªtre [sans faire de calcul] */
      return I_XAS_MU01_FermerFenetre( fenetre, BI_FEN_FERMER_FORCER_ANNULE );
    }
  }

  /* Devient le nouveau graphe de rÃ©fÃ©rence */
  FEN_GRF_LierGraphe( fenetre, nv_brut );

  /* On met Ã  jour la fenÃªtre */
  I_XAS_MU01_MettreAJour( fenetre, nv_brut, infos, VRAI );

  /* On a fini, tout va bien */
  return BI_EVENEMENT_TRAITE;
}

/* ------ Fermeture de la fenÃªtre (avec ou sans calcul) -------- */

static int I_XAS_MU01_FermerFenetre(Fenetre *fenetre, int mode)
{
  char *titre = NULL;
  int retour = I_XAS_MU01_FIN_MODULE_ANNULER;
  Graphe *brut, *exafs, *mu_0;
  MU1_infos *infos;

  /* On rÃ©cupÃ¨re les informations */
  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  if ( ( NULL == brut ) || ( NULL == infos ) ) mode = BI_FEN_FERMER_FORCER_ANNULE;

  if ( BI_FEN_FERMER_FORCER_NORMAL == mode ) { /* On demande ce qu'il faut faire */
    retour = BI_ERR_alerte( fenetre, &Q_MU01_QuitterModule, VRAI );
  } else if ( BI_FEN_FERMER_FORCER_ANNULE == mode ) {
    /* On force la fermeture en annulant les calculs */
    retour = I_XAS_MU01_FIN_MODULE_FERMER;
  } else if ( BI_FEN_FERMER_FORCER_OK == mode ) {
    /* On force la fermeture en faisant les calculs [l'idÃ©al serait de demander, si opportun, si on le fait aussi sur la sÃ©rie] */
    retour = I_XAS_MU01_FIN_MODULE_SOUSTRAIRE;
  }

  if ( I_XAS_MU01_FIN_MODULE_ANNULER == retour ) { /* On annule la fermeture */
    return BI_EVENEMENT_TRAITE;
  }

  /* De toute faÃ§on, on ferme ==> on dÃ©lie les infos de la fenÃªtre */
  ARB_MettreComplement( ELIM_MU0_1, NULL );
  FEN_GRF_LierGraphe( fenetre, NULL );
  FEN_StockerComplement( fenetre, FEN_COMPLEMENT_2, NULL );
  FEN_Detruire( fenetre );

  exafs = infos->exafs;
  mu_0  = infos->mu_0;

  /* On dÃ©protÃ¨ge les graphes */
  exafs->drapeaux.protege = FAUX;
  mu_0->drapeaux.protege = FAUX;
  if ( brut != NULL ) { /* Le spectre brut existe encore - sinon, risque de seg. fault. !
			   Il a pu Ãªtre effacÃ© par exemple en fermant la fenÃªtre Âµmatrice dans l'extraction chaÃ®nÃ©e
			   ou encore en effaÃ§ant tous les graphes */
    brut->drapeaux.protege = FAUX;
  }

  if ( I_XAS_MU01_FIN_MODULE_FERMER == retour ) { /* On ferme sans soustraire */
    /* On informe que le graphe rÃ©sultat va disparaÃ®tre */
    GRF_modifier( exafs, NULL, VRAI );

    /* Et l'on fait l'effacement */
    GRF_eff_isole( exafs ); exafs = NULL;
    GRF_eff_isole( mu_0 ); mu_0 = NULL;

    free( infos ); infos = NULL;
    return BI_EVENEMENT_FEN_FERMEE;
  }

  /* On stocke les informations dans l'arbre de "rÃ©pÃ©tition" */
  OBJ_MettreNombre( EXAFS_GENERAL, EXF_VALEUR_E0  , infos->e_0, 0.0 );
  OBJ_MettreNombre( EXAFS_GENERAL, EXF_VALEUR_E2  , infos->e_2, 0.0 );
  OBJ_MettreEntier_U( EXAFS_GENERAL, EXV_DEGRE_MU0_I, infos->degre );
  OBJ_MettreEntier_U( EXAFS_GENERAL, EXF_VALEUR_K   , infos->deg_affiche );
  if (infos->methode == ELIM_MU0_RAPIDE)
    OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, FORCE_OUI);
  else if (infos->methode == ELIM_MU0_LENT)
    OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT, FORCE_OUI);
  if (infos->avec_e3 == VRAI) { /* On a tronquÃ© le spectre */
    OBJ_active(EXAFS_GENERAL, E1F_MU1_VAL_E2P, FORCE_OUI);
    OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, FORCE_OUI);
    OBJ_MettreNombre(EXAFS_GENERAL, E1F_MU1_VAL_E2P, infos->e_3, 0.0 );
  } else {
    OBJ_active(EXAFS_GENERAL, E1F_MU1_VAL_E2P, FORCE_NON);
    OBJ_choisit(EXAFS_GENERAL, E1F_MU1_2_PTS, FORCE_NON);
  }

  /* On veut extraire le spectre */
  exafs = MU1_ExtraitEXAFS( brut, infos, FAUX );

  /* On fait le traitement post-extraction */
  XAS_MU01_PostExtraction( brut, exafs, infos );

  /* On insÃ¨re le graphe rÃ©sultat dans la sÃ©rie courante, aprÃ¨s le brut */
  I_GRF_InsererDansSerie( serie_courante, brut, exafs );

  /* Idem avec le modÃ¨le de Âµ0, Ã©tape 1, juste aprÃ¨s l'exafs */
  titre = TXT_genere_chaine( nom_grf_mu0I, brut->nom ); /* nom_grf_mu0I doit contenir un %s */
  GRF_titre( mu_0, titre );
  free( titre ); titre = NULL;

  I_GRF_InsererDansSerie( serie_courante, exafs, mu_0 );
  mu_0->drapeaux.sauve = FAUX;    /* Graphe non sauvÃ©            */

  /* Traitement en sÃ©rie, au besoin */
  if ( I_XAS_MU01_FIN_MODULE_SERIE == retour ) {
    Serie_Graphes *serie;
    Graphe **graphes = NULL;
    unsigned long i, nombre_graphes = 0;
    struct automatiser *lot;

    
    /* On rÃ©cupÃ¨re la sÃ©rie du graphe dÃ©jÃ  traitÃ© */
    serie = GRF_QuelleSerie( brut ); /* Pas forcÃ©ment la courante ! */
    if ( NULL == serie ) {
      AVERTISSEMENT( "Graphe %s hors sÃ©rie !", brut->nom );
      FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    }

    /* On en extrait la liste des graphes qui conviennent : mÃªme type que le brut, mÃªme unitÃ© en X */
    graphes = GRF_ChoisirMasque( serie, brut, GRF_CHOIX_GENRE | GRF_CHOIX_TYPE, 0, &nombre_graphes );
    lot = AUTO_GRF_PreparerStructure( nombre_graphes, graphes );
    if ( NULL == lot ) {
      OPTION_A_FAIRE( "Fin propre" );
    }
    free( graphes ); graphes = NULL;

    /* PrÃ©paration de l'automatisation : fonction Ã  utiliser,... */
    lot->classe_fonction = FONCTION_EST_PAR_TYPE;
    lot->genre = GRF_FCT_REELLE;
    lot->type = GR_MOINSMAT;
    lot->numero_fonction = FCT_GRF_MOINSMAT_EXAFS;

    /* Infos nÃ©cessaires Ã  l'automatisation */
    lot->infos_necessaires = infos;
    lot->reference.graphe  = infos->exafs;

    /* On perd le rÃ©sultat dans les infos, pour Ã©viter des problÃ¨mes Ã©ventuels... */
    infos->exafs = NULL;

    /* On appelle le traitement par lots */
    retour = I_AUTOMATISE_SurListeConnue( lot, 0 );
    if ( retour != 0 ) {
      if ( retour < 0 ) {
	OPTION_A_FAIRE( "Message : erreur survenue !" );
      }

      OPTION_A_FAIRE( "Quitter proprement !" );
    }

    /* Doit-on insÃ©rer dans une nouvelle sÃ©rie ? */
    if ( lot->nom_serie != NULL ) {
      /* Oui : on la crÃ©e aprÃ¨s la sÃ©rie en cours */
      serie = SERIE_GRF_CreerSerie( serie, lot->nom_serie );
      free( lot->nom_serie ); lot->nom_serie = NULL;
    } else {
      /* On reste dans la sÃ©rie en cours */
    }

    /* On insÃ¨re tous les rÃ©sultats dans la sÃ©rie indiquÃ©e */
    serie = I_GRF_InsererDansSerie( serie, brut, lot->resultats[ 0 ].graphe );
    for ( i = 1 ; i < lot->nombre_objets ; i++ ) {
      serie = I_GRF_InsererDansSerie( serie, lot->resultats[ i - 1 ].graphe, lot->resultats[ i ].graphe );
    }
  }

  /* Plus besoin des infos d'extraction */
  free( infos ); infos = NULL;

  return BI_EVENEMENT_FEN_FERMEE;
}

/* -------------- Clic sur la fenÃªtre d'Ã©limination de mu0, 1re Ã©tape ------------------------ */

static int CLC_mu01(Fenetre *fenetre, int bouton, int n_clic __attribute__((__unused__)), int etat, int x_clic, int y_clic __attribute__((__unused__)))
{
  MU1_infos *infos;
  Graphe *graphe;
  double x_echelle;

#ifdef _TOS_
  int handle, _rien_;

  wind_get(0, WF_TOP, &handle, &n_clic, &_rien_, &_rien_);
  if (handle != fenetre->handle) {
    wind_set(fenetre->handle, WF_TOP);
    return VRAI;
  }

  if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + fenetre->w_larg)
    || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + fenetre->w_haut) ) {
    return VRAI; /* Clic en-dehors de la zone de travail */
  }
#endif

  infos  = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  graphe = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  if ( ( NULL == graphe ) || ( NULL == infos ) ) {
    AVERTISSEMENT( "Il manque des infos dans la fenÃªtre [graphe=%p, infos=%p]", (void *) graphe, (void *) infos );
    return BI_EVENEMENT_TRAITE;
  }

  x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
  if (x_echelle) {
    if (bouton == 1) { /* Clic du bouton gauche -> E0 */
      infos->e_0 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
      if ( (infos->avec_e3 == VRAI) && (infos->e_0 > infos->e_3) ) {
	double x = infos->e_3; infos->e_3 = infos->e_0; infos->e_0 = x;
	OBJ_MettreNombre( ELIM_MU0_1, MU1_VALEUR_E3, infos->e_3, 0.0 );
      }
      OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E0, infos->e_0, 0.0 );
    } else { /* Bouton droit -> E2 */
      if ( etat == 0) { /* Aucune touche spÃ©ciale */
	infos->e_2 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
	OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E2, infos->e_2, 0.0 );
      } else if ( infos->avec_e3 == VRAI ) { /* Touche spÃ©ciale, et E3 est utilisÃ©e */
	if ( (etat & K_RSHIFT) || (etat & K_LSHIFT) ) { /* Shift = place E3, Ã©chelle des E */
	  infos->e_3 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
	  if ( infos->e_3 < infos->e_0 ) {
	    double x = infos->e_3; infos->e_3 = infos->e_0; infos->e_0 = x;
	    OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E0, infos->e_0, 0.0 );
	  }
	  OBJ_MettreNombre( ELIM_MU0_1, MU1_VALEUR_E3, infos->e_3, 0.0 );
	} else if ( etat & K_CTRL ) { /* Control = place E3, Ã©chelle des k */
	  if (infos->exafs) {
	    x_echelle = (infos->exafs->x_max - infos->exafs->x_min) / ( (double) (fenetre->w_larg) );
	    if (x_echelle) {
	      double coefficient = 2.0 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rÃ©el */
	      coefficient *= EV_V_JOULE;              /* Conversion des Ã©nergies en Joules */ 

	      infos->e_3  = infos->exafs->x_min + (x_clic - fenetre->w_x) * x_echelle;
	      infos->e_3 *= M_V_ANGSTROM; /* Conversion de k en m^-1*/
	      infos->e_3 *= infos->e_3;   /* On a kÂ² */
	      infos->e_3 /= coefficient;  /* On passe en Ã©nergie (en eV) */
	      infos->e_3 += infos->e_0;
	      OBJ_MettreNombre( ELIM_MU0_1, MU1_VALEUR_E3, infos->e_3, 0.0 );
	    }
	  }
	}
      }
    }

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
    I_XAS_MU01_MettreAJour( fenetre, graphe, infos, VRAI );
  }

  return BI_EVENEMENT_TRAITE;
}

/* -------- Gestion des clics sur l'arbre --------------------------------------------------\
 |   Renvoie VRAI si l'objet a Ã©tÃ© traitÃ© [ici, toujours VRAI]                              |
 |                                                                                          |
 | num_objet = l'objet sur lequel on a cliquÃ©                                               |
 | x, y      = coordonnÃ©es de la souris _relatives Ã  l'objet cliquÃ©                         |
 \----------------------------------------------------------------------------------------- */

static int GER_mu01(unsigned int num_objet, int x, int y)
{
  MU1_infos *infos;
  Graphe *brut;
  Fenetre *fenetre;
  boolean etat;
  int reponse;
  struct infos_demande nv_valeur = { BI_CHAMP_DEMANDE_REEL, NULL, "eV",
				     BI_EDITABLE_MIN | BI_EDITABLE_MAX | BI_EDITABLE_UNITE, 0.0, 0.0, 0.0,
				     0, NULL, { 0 }, { 0 }, { 0 } };

  fenetre = ARB_QuelleFenetre(arbres + ELIM_MU0_1, FAUX); /* FenÃªtre actuelle */
  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos ) {
    AVERTISSEMENT( "Attention, pas d'infos associÃ©es Ã  la fenÃªtre [infos=%p]", (void *) infos );
    return BI_EVENEMENT_TRAITE;
  }
  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );

  switch (num_objet)
  {
  case ELIM_MU0_RAPIDE: /* MÃ©thode utilisÃ©e pour approcher l'absorption atomique */
  case ELIM_MU0_LENT:
    if (num_objet != (unsigned int) infos->methode) { /* On change de mÃ©thode */
      OBJ_choisit(ELIM_MU0_1, num_objet, FORCE_OUI);
      infos->methode = num_objet;

      /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
      I_XAS_MU01_MettreAJour( fenetre, brut, infos, VRAI );
    }
    return BI_EVENEMENT_TRAITE;

  case VALEUR_E0: /* Valeur Ã  utiliser pour E0 */
    nv_valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    nv_valeur.texte = "E0 =";
    nv_valeur.min = brut->x_min;
    if ( VRAI == infos->avec_e3 ) {
      nv_valeur.max = infos->avec_e3;
    } else {
      nv_valeur.max = brut->x_max;
    }
    nv_valeur.valeur_initiale.reel = infos->e_0;
    nv_valeur.valeur_defaut.reel   = cherche_e0( brut, FAUX );

    reponse = BI_DemandeValeur( fenetre, &Q_ValeurE0, &nv_valeur );
    if ( Q_ValeurE0.bouton_annuler == (unsigned int) reponse ) return BI_EVENEMENT_TRAITE; /* On annule */

    infos->e_0 = nv_valeur.valeur_donnee.reel;
    OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E0, infos->e_0, 0.0 );

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs ; pas besoin de recommencer Âµ0 ! */
    I_XAS_MU01_MettreAJour( fenetre, brut, infos, FAUX );
    return BI_EVENEMENT_TRAITE;

  case VALEUR_E2: /* Valeur Ã  utiliser pour E2 */
    nv_valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    nv_valeur.texte = "E2 =";
    nv_valeur.min = infos->e_0;
    nv_valeur.max = brut->x_max;
    nv_valeur.valeur_initiale.reel = infos->e_2;

    reponse = BI_DemandeValeur( fenetre, &Q_ValeurE2, &nv_valeur );
    if ( Q_ValeurE2.bouton_annuler == (unsigned int) reponse ) return BI_EVENEMENT_TRAITE; /* On annule */

    infos->e_2 = nv_valeur.valeur_donnee.reel;

    OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E2, infos->e_2, 0.0 );

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
    I_XAS_MU01_MettreAJour( fenetre, brut, infos, VRAI );
    return BI_EVENEMENT_TRAITE;
    
  case DEGRE_MOINS:
  case ELIM_MU0_FOND:
  case DEGRE_VALEUR:
  case DEGRE_PLUS:
    infos->degre = gere_ascenseur(ELIM_MU0_1, DEGRE_MOINS, ELIM_MU0_FOND, DEGRE_VALEUR, DEGRE_PLUS, ASC_HORIZONTAL,
				  1, 9, 1, (int) (infos->degre), num_objet, x, y);

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
    I_XAS_MU01_MettreAJour( fenetre, brut, infos, VRAI );
    return BI_EVENEMENT_TRAITE;

  case MU01_DEG_AFFICHE: /* DegrÃ© pour k^n * khi(k) */
    {
      int vx_degre = infos->deg_affiche;

      nv_valeur.proprietes |= BI_EDITABLE_ENTIER;
      nv_valeur.proprietes &= ~BI_EDITABLE_UNITE;
      nv_valeur.texte = "n =";
      nv_valeur.min = 0.0;
      nv_valeur.max = 5.0;
      nv_valeur.valeur_initiale.reel = (double) (infos->deg_affiche);

      reponse = BI_DemandeValeur( fenetre, &Q_Ponderation, &nv_valeur );
      if ( Q_Ponderation.bouton_annuler == (unsigned int) reponse ) return BI_EVENEMENT_TRAITE; /* On annule */

      infos->deg_affiche = (int) nv_valeur.valeur_donnee.reel;
      OBJ_MettreEntier_U(ELIM_MU0_1, MU01_DEG_AFFICHE, infos->deg_affiche);

      EXF_k_p_n(infos->exafs, infos->deg_affiche - vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , infos->deg_affiche - vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case MU1_VALEUR_E3:
    nv_valeur.proprietes |= BI_EDITABLE_MIN | BI_EDITABLE_MAX;
    nv_valeur.texte = "E3 =";
    if ( infos->e_2 < infos->e_0 ) {
      nv_valeur.min = infos->e_0;
    } else {
      nv_valeur.min = infos->e_2;
    }
    nv_valeur.max = brut->x_max;
    nv_valeur.valeur_initiale.reel = infos->e_3;

    reponse = BI_DemandeValeur( fenetre, &Q_ValeurE3, &nv_valeur );
    if ( Q_ValeurE3.bouton_annuler == (unsigned int) reponse ) return BI_EVENEMENT_TRAITE; /* On annule */

    infos->e_3 = nv_valeur.valeur_donnee.reel;

    OBJ_MettreNombre( ELIM_MU0_1, MU1_VALEUR_E3, infos->e_3, 0.0 );

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
    I_XAS_MU01_MettreAJour( fenetre, brut, infos, VRAI );
    return BI_EVENEMENT_TRAITE;

  case MU1_2_PTS:
    etat = OBJ_choisit(ELIM_MU0_1, MU1_2_PTS, ECHANGE);
    OBJ_active(ELIM_MU0_1, MU1_VALEUR_E3, etat);
    infos->avec_e3 = etat;

    /* On met Ã  jour l'affichage en fonction des nouvelles valeurs */
    I_XAS_MU01_MettreAJour( fenetre, brut, infos, VRAI );
    return BI_EVENEMENT_TRAITE;
  }

  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Gestion des raccourcis claviers -----------------\
 |   Renvoie VRAI si la touche a pu Ãªtre traitÃ©e             |
 |                                                           |
 | alt-0/4 = pondÃ©ration du signal EXAFS obtenu              |
 | shift-0 = propose le E0 par dÃ©faut (max. de la dÃ©rivÃ©e)   |
 |       0 = demande le E0 Ã  l'utilisateur                   |
 |       2 = demande le E2 Ã  l'utilisateur                   |
 | shift-2 = demande le E2' Ã  l'utilisateur, Ã©chelle des E   |
 | ctrl- 2 = demande le E2' Ã  l'utilisateur, Ã©chelle des k   |
 | espace  = calcul terminÃ©                                  |
 | -, +    = degrÃ© prÃ©cÃ©dent/suivant pour le polynÃ´me        |
 | r, l    = mÃ©thode rapide ou lente pour le calcul          |
 \---------------------------------------------------------- */

static int CLV_mu01(Fenetre *fenetre, unsigned int touche, int etat)
{
  Graphe *brut;
  MU1_infos *infos;
/*   Message message; */

  brut  = (Graphe *)    FEN_LireComplement( fenetre, FEN_INFOS_GRAPHE );
  infos = (MU1_infos *) FEN_LireComplement( fenetre, FEN_COMPLEMENT_2 );
  if ( NULL == infos ) {
    return BI_EVENEMENT_TRAITE;
  }

  switch ( touche )
  {
  case '0': /* Demander la valeur de E0 */
    if ( (etat & K_RSHIFT) || (etat & K_LSHIFT) ) {
      infos->e_0 = cherche_e0( brut, FAUX );
      OBJ_MettreNombre( ELIM_MU0_1, VALEUR_E0, infos->e_0, 0.0 );

      /* On met Ã  jour l'affichage */
      I_XAS_MU01_MettreAJour( fenetre, brut, infos, FAUX );
    } else if (etat & K_ALT) {
      int vx_degre = infos->deg_affiche;

      infos->deg_affiche = 0;
      OBJ_MettreEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE, 0 );
      EXF_k_p_n(infos->exafs, -vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , -vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    } else {
      return ARB_SimulerClic( ELIM_MU0_1, VALEUR_E0, 1 );
    }
    return BI_EVENEMENT_TRAITE;

  case '1':
    if (etat & K_ALT) {
      int vx_degre = infos->deg_affiche;
      infos->deg_affiche = 1;
      OBJ_MettreEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE, 1 );
      EXF_k_p_n(infos->exafs, 1 - vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , 1 - vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case '2':
    if ( etat == 0 )
      return ARB_SimulerClic( ELIM_MU0_1, VALEUR_E2, 1 );
    else if (etat & K_ALT) {
      int vx_degre = infos->deg_affiche;
      infos->deg_affiche = 2;
      OBJ_MettreEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE, 2 );
      EXF_k_p_n(infos->exafs, 2 - vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , 2 - vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    } else if ( infos->avec_e3 == VRAI ) {
      if ( etat & ( K_RSHIFT | K_LSHIFT ) ) {
	return ARB_SimulerClic( ELIM_MU0_1, MU1_VALEUR_E3, 1 );
      } else if (etat & K_CTRL) {
      }
    }
    return BI_EVENEMENT_TRAITE;

  case '3':
    if (etat & K_ALT) {
      int vx_degre = infos->deg_affiche;
      infos->deg_affiche = 3;
      OBJ_MettreEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE, 3 );
      EXF_k_p_n(infos->exafs, 3 - vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , 3 - vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case '4':
    if (etat & K_ALT) {
      int vx_degre = infos->deg_affiche;
      infos->deg_affiche = 4;
      OBJ_MettreEntier_U( ELIM_MU0_1, MU01_DEG_AFFICHE, 4 );
      EXF_k_p_n(infos->exafs, 4 - vx_degre, FAUX);
      EXF_k_p_n(infos->mu_0 , 4 - vx_degre, FAUX);
      GRF_modifier(infos->exafs, infos->exafs, FAUX);
      FEN_redessin(fenetre, BI_RDS_UTIL);
    }
    return BI_EVENEMENT_TRAITE;

  case '-':
    return ARB_SimulerClic( ELIM_MU0_1, DEGRE_MOINS, 1 );

  case '+':
    return ARB_SimulerClic( ELIM_MU0_1, DEGRE_PLUS , 1 );

  case 'n':
    return ARB_SimulerClic( ELIM_MU0_1, ELIM_MU0_RAPIDE, 1 );

  case 'p':
    return ARB_SimulerClic( ELIM_MU0_1, ELIM_MU0_LENT, 1 );

  case 'd':
    return ARB_SimulerClic( ELIM_MU0_1, MU1_2_PTS, 1);
  }

  /* Touche non traitÃ©e -> raccourci gÃ©nÃ©ral ? */
  return BI_EVENEMENT_NON_TRAITE;
}

/* -------- Mettre Ã  jour la fenÃªtre quand on change les paramÃ¨tres : recalculer le polynÃ´me, l'EXAFS,... -------- */

static void I_XAS_MU01_MettreAJour(Fenetre *fenetre, const Graphe *brut, MU1_infos *infos, boolean recalculer_mu0)
{
  int retour;

  /* On recalcule le modÃ¨le de Âµ0 */
  if ( VRAI == recalculer_mu0 ) {
    retour = XAS_MU01_CalculerMu0_I( brut, infos );
    if ( retour != 0 ) {
      AVERTISSEMENT( "ProblÃ¨me en calculant le modÃ¨le de Âµ0 [code %d]", retour );
      return;
    }
  }

  /* On s'en sert pour extraire l'EXAFS */
  retour = XAS_MU01_ExtraireEXAFS( brut, infos );
  if ( retour != 0 ) {
    AVERTISSEMENT( "ProblÃ¨me en utilisant le modÃ¨le de Âµ0 pour extraire l'EXAFS [code %d]", retour );
    return;
  }

  /* On redessine tout cela */
  FEN_redessin( fenetre, BI_RDS_UTIL );
}

/* -------- PrÃ©paration de l'arbre (traductions,...) -------- */

static int prepare_arbre_mu0_1(unsigned int numero_arbre __attribute__((__unused__)))
{
  if ( FAUX == arbres[ ELIM_MU0_1 ].drapeaux.init ) {
    int retour;

    retour = ARB_Initialiser( ELIM_MU0_1, "menu_exafs.html#etape_1", GER_mu01, ( int (*)() ) NULL );
    if ( retour != 0 ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans l'initialisation de l'arbre\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -1;
    }
  }

  /* On amÃ©liore les dessins de l'arbre */
  OBJ_agrandit(ELIM_MU0_1, ROOT, 0, +2);

  ARB_EnjoliverAscenseur( ELIM_MU0_1, DEGRE_MOINS, ELIM_MU0_FOND, DEGRE_VALEUR, DEGRE_PLUS, ASC_HORIZONTAL, +1);
  OBJ_deplace (ELIM_MU0_1, VALEUR_E2    ,  0, +1, DEPLACE_PIXELS);
  OBJ_deplace (ELIM_MU0_1, MU1_VALEUR_E3,  0, +2, DEPLACE_PIXELS);
  OBJ_deplace (ELIM_MU0_1, MU01_DEG_AFFICHE,  0, +1, DEPLACE_PIXELS);
  OBJ_deplace (ELIM_MU0_1, MU1_2_PTS    ,  0, +2, DEPLACE_PIXELS);

  /* On s'occupe des traductions Ã©venuelles */
#ifdef AVEC_TRADUCTION_
  met_chaine(ELIM_MU0_1, ELIM_MU0_RAPIDE, 0, txt_mu1_normal, TXT_REMPLACE);
  met_chaine(ELIM_MU0_1, ELIM_MU0_LENT, 0, txt_mu1_pondere, TXT_REMPLACE);
#endif

  arbres[ ELIM_MU0_1 ].drapeaux.prepare = VRAI;
  return 0;
}
