/* LASE - logiciel d'analyse des spectres E.X.A.F.S.
   (c) 1998, Emmanuel CURIS - Cration du fichier : 4 juin 1998

   Mthode des moindres carrs  brutale
    - pour un polynme
    - pour une fonction en A + B/x + C/x^2 + ...
   Dtermination des coefficients par rsolution du systme
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "exafs.h"
#include "graphes.h"
#include "lissage.h"

/* ***** Fonctions internes de ce module ***************************************************** */

static int degre_1(double *somme_x, double *somme_xy, double *coefs);

/* ----- Moindres carrs directe avec polynmes complets --------------------------------------\
 |  Renvoie l'erreur si le polynme a pu tre calcul correctement, -1 sinon
 |
 | poly      : le tableau des coefficients du polynme en retour ([0] + [1]*x + [2]*x^2 + ...) |
 | degre     : le degr du polynme d'interpolation (ventuellement ngatif...)
 | graphe    : le graphe sur lequel on va travailler
 | nbr_zones : le nombre d'intervalles de points valables
 | zone      : tableau des bornes des intervalles (0 : min1, 1 : max1, 2 : min2, 3 : max2,...) |
 \-------------------------------------------------------------------------------------------- */

double MC_Polynome_SurPartieGraphe(const Graphe *graphe, int degre, int nbr_zones, double *zones, double *poly)
{
  double  somme_x[22]  = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
          somme_xy[11] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
  double  x,y,x_puissance;
  double  retour = 0.0; /* Erreur pour le polynme complet */
  double  somme_y2 = 0.0; /* Somme des y^2, pour le calcul d'erreur... */
  boolean convient = VRAI;
  int     i;
  unsigned long index; /* Compteur... */
  register double *zone_x,*zone_y;
  double norme; /* Pour mettre tous les X dans une chelle plus sympa... vite des erreurs ? */

  /* On initialise les coefficients  0 */
  for (i = 0 ; i <= abs(degre) ; poly[i++] = 0);

  /* On calcule le nombre de points et Sx -> pour les moyennes en normalisant */

  if (zones == NULL) /* On veut tous les points */
  {
    register double *x;
    unsigned long i;

    somme_x[0] = graphe->nombre_point;
    for (x = graphe->x, i = 0 ; i < graphe->nombre_point ; i++, x++)
      somme_x[1] += *x;
  }
  else
  {
    register double *x;
    unsigned long i;
    int i_zone;

    for (x = graphe->x, i = 0 ; i < graphe->nombre_point ; i++, x++)
    {
      boolean convient = FAUX;
      for (i_zone = 0 ; i_zone < 2*nbr_zones ; i_zone += 2)
      {
	if ( (*x >= zones[i_zone]) && (*x <= zones[i_zone + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }

      if ( convient == VRAI ) /* On doit prendre en compte ce point */
      {
	somme_x[0]++; /* Un point en plus */
	somme_x[1] += *x;
      }
    }
  }

  if (somme_x[0] == 0.0) /* Aucun point */
  {
    LIS_ERREUR(ERR_LIS_TROP_PEU_POINTS);
    return -1.0;
  }

  if (somme_x[1] != 0)
  {
    norme = somme_x[0] / somme_x[1]; /* 1 / <x> */
    somme_x[1] = somme_x[0];
    printf("%s : normalisation, facteur %g\n", __FUNCTION__, norme );
  }
  else
    norme = 1; /* Pas envie de rflchir pour l'instant ->  voir plus tard */

  /* On calcule les autres sommes ncessaires */

  zone_x = graphe->x;
  zone_y = graphe->y;
  for (index = 0 ; index < graphe->nombre_point ; index++)
  {
    x = *(zone_x); /* Valeur du point en cours */
		
    if (zones != NULL) /* Tout point ne convient pas */
    {
      convient = FAUX;
      for (i = 0 ; i < 2*nbr_zones ; i += 2)
      {
	if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }
    }
    
    if ( convient == VRAI ) /* On est dans les bornes */
    {
      x *= norme;
      y = *(zone_y);
     
      somme_xy[0] += y;
      somme_xy[1] += x*y;
      somme_y2 += y*y;
      x_puissance = x * x;
      for (i = 2 ; i <= 2*abs(degre) ; i++)
      {
	somme_x[i]  += x_puissance;
	if (degre > 0)
	{
	  if ( i <= degre )
	    somme_xy[i] += x_puissance * y;
	}
	else
	{
	  if ( x_puissance == 0 )
	  {
	    EXF_ERREUR(DIV_ZERO, "x nul", txt_err_calc_annule);
	    return -1.0;
	  }
	  if ( i <= -degre )
	    somme_xy[i] += y / x_puissance;
	}
	x_puissance *= x;
      }
    }
    zone_x++; zone_y++;
  }

  switch (abs(degre))
  {
  case 0: /* Constante -> un seul calcul possible... */
    poly[0] = somme_xy[0] / somme_x[0];
    break;

  case 1: /* Un seul calcul possible */
    if ( degre_1(somme_x, somme_xy, poly) < 0 ) /* Erreur survenue */
      return -1.0;
    break;

  default: /* Approximation par un polynme de degr suprieur : pivot de Gauss */
    {
      double *matrice;
      int i, j;
      int deg = abs(degre);

      /* Matrice du systme */
      matrice = (double *) malloc( sizeof(double) * (deg + 2) * (deg + 1) );
      if (matrice == NULL)
      {
	SYS_ERREUR(MANQUE_MEMOIRE, "M_carr_1.c", "moindre2", "matrice");
	return -1;
      }

      /* On prpare la matrice  rsoudre */

      for (j = 0 ; j <= deg ; j++)
	for (i = 0 ; i <= deg ; i++)
	  matrice[ (deg + 2) * j + i ] = somme_x[i + j];

      for (j = 0 ; j <= deg ; j++)
	matrice[ (deg + 2) * j + (deg + 1) ] = somme_xy[j];

      /* Affichage de la matrice des sommes */
#ifdef _DEVELOPPE_
      printf("\n\n Matrice carre associe au systme\n");
      for (i = 0 ; i <= degre ; i++) {
	printf("( ");
	for (j = 0 ; j <= degre ; j++)
	  printf("%g ", somme_x[ i + j ]);

	printf(") = (%g)\n", somme_xy[ i ]);
      }
      printf("\n");
#endif

      if ( Gauss(deg, matrice, poly) < 0 ) { /* Rsolution par le pivot de Gauss */
	free(matrice);
	return -1;
      }
      free(matrice);

      /* On vrifie que c'est bien solution */
      /*      for (i = 0 ; i <= degre ; i++)
      {
	double y_th = 0;

	for (j = 0 ; j <= degre ; j++)
	  y_th += somme_x[ i + j ] * poly[j];

	printf("Y attendu : %f - calcul : %f\n", somme_xy[i], y_th);
      } */
    }
  }

  /* Prise en compte du facteur de normalisation */

  retour = 1;
  for (i = 1 ; i <= degre ; i++) {
    retour *= norme;
    poly[i] *= retour;
  }
#ifdef _DEVELOPPE_
/*   printf("%s : Coefficients trouvs = \n( %g", __FUNCTION__, poly[ 0 ] ); */
/*   for ( i = 1 ; i <= degre ; i++ ) { */
/*     printf(" , %g", poly[ i ]); */
/*   } */
/*   printf(" )\n"); */
#endif

  /* Calcul de l'erreur */
  zone_x = graphe->x; zone_y = graphe->y;
  retour = 0;
  for (index = 0 ; index < graphe->nombre_point ; index++)
  {
    x = *(zone_x);
    if (zones != NULL) /* Tout point ne convient pas */
    {
      convient = FAUX;
      for (i = 0 ; i < 2*nbr_zones ; i+=2)
      {
	if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }
    }
    if ( convient == VRAI ) /* On est dans les bornes */
    {
      y = 0;
      for (i = degre ; i > 0 ; i--)
      {
	y += poly[i] * x;
      }
      y += poly[0] - *zone_y;
      retour += y * y;
    }
    zone_x++; zone_y++;		
  }

  if (somme_y2 > 0.0)
    retour /= somme_y2;
  else
    EXF_ERREUR(DIV_ZERO, "Somme des carr"e_AIGU"s nulle", txt_err_val_inchangee);

  return retour; /* Renvoi de l'erreur */  
}

/* ---- Meilleure droite par les moindres carrs simples --------------------------------\
 | Renvoie 0 si pas de problme, <0 en cas d'erreur                                      |
 |                                                                                       |
 | somme_x  : tableau des sommes ncessaires (somme_x[n] = Somme des x^n)                |
 | somme_xy : idem (somme_xy[n] = Somme des x^n * y)                                     |
 | poly     : coefficients du polynme trouv ( P(x) = somme des poly[i] * x^i )         |
 \-------------------------------------------------------------------------------------- */

static int degre_1(double *somme_x, double *somme_xy, double *poly)
{
  double delta; /* Discriminant du systme */

#ifdef _DEVELOPPE_
  printf("degre_1 : ajustement d'une droite par les moindres carrs\n");
  printf("  Nombre de points : %g\n", somme_x[0]);
  printf("  Sx               : %g\n", somme_x[1]);
  printf("  Sx              : %g\n", somme_x[2]);
  printf("  Sy               : %g\n", somme_xy[0]);
  printf("  Sxy              : %g\n", somme_xy[1]);
#endif

  delta = somme_x[0] * somme_x[2] - somme_x[1] * somme_x[1];
  if (delta == 0) {
    (void) EXF_ERREUR(DIV_ZERO, "D"e_AIGU"terminant nul", "Calcul annul"e_AIGU);
    return -DIV_ZERO;
  }
		
  poly[0]  = somme_xy[0] * somme_x[2] - somme_x[1] * somme_xy[1];
  poly[0] /= delta;
  poly[1]  = somme_xy[1] * somme_x[0] - somme_x[1] * somme_xy[0];
  poly[1] /= delta;

#ifdef _DEVELOPPE_
  printf("%s : coefficients trouvs =\n( %g , %g )\n", __FUNCTION__, poly[ 0 ], poly[ 1 ]);
#endif

  return 0; /* Pas de problme */
}
