#include <stdio.h>
#include <stdlib.h>

#include <mfhdf.h>

int HDF_LireAttribut( int32 identifiant, unsigned int numero );

int HDF_LireAttribut( int32 identifiant, unsigned int numero )
{
  char nom_attribut[ 65 ];
  int32 type_donnees, nombre_donnees;
  intn retour;
  size_t taille;
  void *donnees;

  /* Infos sur l'attribut */
  retour = SDattrinfo( identifiant, numero, nom_attribut, &type_donnees, &nombre_donnees );
  if ( FAIL == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDfileinfo()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }
  printf("%s : %u valeurs de type %d ", nom_attribut, (unsigned int) nombre_donnees, (int) type_donnees );

  switch ( type_donnees )
  {
  case DFNT_FLOAT32:
    taille = sizeof( float );
    break;

  case DFNT_FLOAT64:
    taille = sizeof( double );
    break;

  case DFNT_UCHAR8:
  case DFNT_CHAR8:
    nombre_donnees++; /* Pour avoir un 0 en fin de chane */
  case DFNT_INT8:
  case DFNT_UINT8:
    taille = sizeof( int8 );
    break;

  case DFNT_INT16:
  case DFNT_UINT16:
  case DFNT_UCHAR16:
  case DFNT_CHAR16:
    taille = sizeof( int16 );
    break;

  case DFNT_INT32:
  case DFNT_UINT32:
    taille = sizeof( int32 );
    break;

  case DFNT_INT64:
  case DFNT_UINT64:
/*     taille = sizeof( int64 ); */
    break;

  case DFNT_INT128:
  case DFNT_UINT128:
/*     taille = sizeof( int128 ); */
    break;
  }

  donnees = (void *) calloc( nombre_donnees, taille );
  if ( NULL == donnees ) {
    printf("\n");
    (void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return +25;
  }

  /* Lecture */
  retour = SDreadattr( identifiant, numero, donnees);
  if ( FAIL == retour ) {
    printf("\n");
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDreadattr()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -50;
  }

  /* Affichage */
  if ( ( DFNT_CHAR8 == type_donnees ) || ( DFNT_UCHAR8 == type_donnees ) ) {
    printf("\"%s\"", (char *) donnees);
  } else if ( ( DFNT_INT16 == type_donnees ) || ( DFNT_UINT16 == type_donnees ) ) {
    int i;

    printf("= %d", (int) ( (int16 *) donnees)[ 0 ]);
    for ( i = 1 ; i < nombre_donnees ; i++ ) {
      printf(", %d", (int) ( (int16 *) donnees)[ i ]);
    }
  } else if ( ( DFNT_INT32 == type_donnees ) || ( DFNT_UINT32 == type_donnees ) ) {
    int i;

    printf("= %d", (int) ( (int32 *) donnees)[ 0 ]);
    for ( i = 1 ; i < nombre_donnees ; i++ ) {
      printf(", %d", (int) ( (int32 *) donnees)[ i ]);
    }
  } else if ( DFNT_FLOAT32 == type_donnees ) {
    int i;

    printf("= %g", ( (float *) donnees)[ 0 ]);
    for ( i = 1 ; i < nombre_donnees ; i++ ) {
      printf(", %g", ( (float *) donnees)[ i ]);
    }
  } else if ( DFNT_FLOAT64 == type_donnees ) {
    int i;

    printf("= %g", ( (double *) donnees)[ 0 ]);
    for ( i = 1 ; i < nombre_donnees ; i++ ) {
      printf(", %g", ( (double *) donnees)[ i ]);
    }
  }

  printf("\n");

  free(donnees);
  return 0;
}

int main(int argc, char *argv[])
{
  int32 identifiant, id_donnees;
  int32 nombre_spectres, nbr_attributs;
  intn retour;
  int i;
  char nom_donnees[ 65 ];

  if ( argc < 2 ) {
    (void) fprintf(stderr, "%s,%s l.%u : Pas de fichier indiqu\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  /* Ouverture du fichier */
  identifiant = SDstart( argv[ 1 ], DFACC_READ );
  if ( -1 == identifiant ) {
    (void) fprintf(stderr, "%s,%s l.%u : Impossible d'ouvrir %s\n",
		   __FILE__, __FUNCTION__, __LINE__, argv[ 1 ] );
    return -1;
  }

  printf("Fichier %s ouvert... ID=%d\n", argv[ 1 ], (int) identifiant );

  /* Infos sur le fichier */
  retour = SDfileinfo( identifiant, &nombre_spectres, &nbr_attributs );
  if ( FAIL == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDfileinfo()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2;
  }

  printf("Il contient %ld spectres & %ld attributs globaux\n", (long) nombre_spectres, (long) nbr_attributs);

  /* Les attributs globaux */
  for ( i = 0 ; i < nbr_attributs ; i++ ) {

    printf("\n-------- Attribut %d --------\n", i + 1 );

    HDF_LireAttribut( identifiant, i );
  }

  /* Les donnes */
  for ( i = 0 ; i < nombre_spectres ; i++ ) {
    int32 dimension, tailles[ MAX_VAR_DIMS ], type;
    int j;

    printf("\n-------- Jeu de donnes %d --------\n", i + 1);

    /* On ouvre */
    id_donnees = SDselect( identifiant, i );
    if ( FAIL == id_donnees ) {
      (void) fprintf(stderr, "%s,%s l.%u : Impossible de lire ce jeu\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -45;
    }

    /* Infos sur ce jeu de donnes */
    retour = SDgetinfo( id_donnees, nom_donnees, &dimension, tailles, &type, &nbr_attributs);
    if ( FAIL == retour ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDendaccess()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -12;
    }
    printf("%s : %u colonnes [", nom_donnees, (unsigned int) dimension);
    for ( j = 0 ; j < dimension ; j++ )
      printf("%d ", (int) tailles[ j ]);
    printf("\b], type=%d, %lu attributs\n", (int) type, (unsigned long) nbr_attributs);

    /* On lit les attributs locaux */
    for ( j = 0 ; j < nbr_attributs ; j++ ) {
      printf("\n   -~- Attribut %d -~-\n", j + 1 );

      retour = HDF_LireAttribut( id_donnees, j );
      if ( retour != 0 ) {
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans HDF_LireAttribut()\n",
		       __FILE__, __FUNCTION__, __LINE__);
      }
    }

    /* On lit les donnes */
    if ( tailles[ 0 ] > 0 ) {
      double *donnees;
      int32 positions[ MAX_VAR_DIMS ], nombre[ MAX_VAR_DIMS ];

      /* On lit tout */
      for ( j = 0 ; j < MAX_VAR_DIMS ; j++ ) {
	positions[ j ] = 0;
	nombre[ j ] = tailles[ j ];
      }

      donnees = (double *) calloc( sizeof(double), tailles[ 0 ] );
      if ( NULL == donnees ) {
	(void) fprintf(stderr, "%s,%s l.%u : Mmoire insuffisante\n",
		       __FILE__, __FUNCTION__, __LINE__);
	return +45;
      }

      /* Lecture proprement dite */
      retour = SDreaddata( id_donnees, positions, NULL, nombre, donnees );
      if ( FAIL == retour ) {
	HEprint( stderr, 0 );
	(void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDreaddata( %lu, %lu, NULL, %lu, %p)\n",
		       __FILE__, __FUNCTION__, __LINE__, (unsigned long) id_donnees, (unsigned long) positions[ 0 ], (unsigned long) nombre[ 0 ], (void *) donnees);
	free( donnees );
	return -75;
      }

      /* Affichage, pour contrle */
      printf("Donnes lues :\n%g", donnees[ 0 ]);
      for ( j = 1 ; j < tailles[ 0 ] ; j++ ) {
	printf("; %g", donnees[ j ]);
      }
      printf("\n");

      free( donnees );
    } else {
      printf("... pas de donnes\n");
    }

    /* On ferme */
    retour = SDendaccess( id_donnees );
    if ( FAIL == retour ) {
      (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDendaccess()\n",
		     __FILE__, __FUNCTION__, __LINE__);
      return -48;
    }
  }

  /* Fermeture du fichier */
  retour = SDend( identifiant );
  if ( FAIL == retour ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans SDend()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -20;
  }

  return 0;
}
