/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Calculs thθoriques de divers graphes utiles
   (c) Emmanuel CURIS 1996 - 1998

 25 mars 1998
   * Utilisation de "SYSTEME.H"

 16 mars 1998
   * Inclus messages d'erreurs EXAFS

  5 décembre 1997
   * Création de cet historique
   * Corrigé erreur dans calcul des phases théoriques : choix théorie/semi-exp pour réflecteur marchait pas
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "systeme.h"

#include "graphes.h"

#include "exafs.h"
#include "interface.h" /* Pour GRF_liste */
#include "lissage.h"
#include "science.h" /* Pour M_PI */
#include "theorie.h" /* Pour le calcul des phases - sera déplace un jour */

typedef struct {
  GRF_liste *centre, *reflecteur; /* Il faudrait remplacer par un sous-menu */
} PHA_infos;

/* ~~~~~~~~~~~~~~~~ Diverses choses pour l'extraction ~~~~~~~~~~~~~~~~~ */

/* ---------------- Absorption atomique (mu_0) ------------------------ */

Graphe *CTH_mu0(void)
{
  int reponse = 0;
  double coef_a, coef_b, e_0; /* ParamΦtres de calcul */
  double facteur;
  double x_min, x_max, pas, x;   /* Bornes de calcul et tout et tout */
  unsigned long i, nbr_pts;
  Graphe *mu_0;
  register double *zone_x, *zone_y;

  do {
    reponse = ARB_Bloquant( TH_CALC_MU0, "Calculer un "MU"0", principale, 0 );   /* On appelle l'arbre  */
    if (reponse == THMU0_ANNULE) return NULL; /* On annule le calcul */

    if (reponse == THMU0_ADAPTE) /* Suivant l'élément considéré, met les A, B et E0 idéaux */
    {
      /* Pas encore fait... */
    }
  } while (reponse != THMU0_OK);

  /* Intervalle de calcul */
  reponse = OBJ_LireIntervalle( TH_CALC_MU0, THMU0_KMIN, THMU0_KMAX, THMU0_PAS, ROOT,
				&x_min, &x_max, &pas, &nbr_pts );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans la lecture de l'intervalle\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;		
  }

  mu_0 = GRF_nouveau( NULL, nbr_pts, GR_MU0, GR_NORMAL, "Sans nom" ); /* , "Nombre d'onde", "Absorption", NULL, NULL */
  if (mu_0 == NULL)
    return NULL;

  /* Méthode de Victoreen ---> calcul en k(x) = 12,39852067 / (e0 + x^2 * 3.81)
     On pose A = C * k(0)^3 - D * k(0)^4
     et absorption en (C * k(x)^3 - D * k(x)^4) / A                       */

  coef_a = OBJ_LireNombre(TH_CALC_MU0, THMU0_COEF_C1, NULL ); /* On lit les coefficients de la Victorine */
  coef_b = OBJ_LireNombre(TH_CALC_MU0, THMU0_COEF_D1, NULL );
  e_0    = OBJ_LireNombre(TH_CALC_MU0, THMU0_SEUIL, NULL  ); /* Énergie de seuil */
  if (e_0 == 0.0)
  {
    GRF_efface(mu_0, VRAI, VRAI);
    EXF_ERREUR(DIV_ZERO, E_AIGU"nergie de seuil nulle", txt_err_calc_annule);
    return NULL;    
  }
  else
  {
    double k, k_3;
    k = 12.39852067 / e_0;
    k_3 = k * k * k;
    facteur =  coef_a * k_3 - coef_b * k * k_3; /* A * k^3 - D * k^4 */

    if (facteur == 0)
    {
      GRF_efface(mu_0, VRAI, VRAI);
      EXF_ERREUR(DIV_ZERO, "Facteur de conversion nul", txt_err_calc_annule);
      return NULL;
    }
  }

  x = x_min;
  zone_x = mu_0->x; zone_y = mu_0->y;

  for (i = 0 ; i < nbr_pts ; i++)
  {
    double k, k_3, y;

    k = 12.39852067 / (e_0 + x * x * 3.81); /* On calcule k */
    k_3 = k * k * k;
    y  = coef_a * k_3 - coef_b * k * k_3;

    *zone_x = x;
    *zone_y = y / facteur;
    
    x += pas; zone_x++; zone_y++;
  }

  GRF_bornes(mu_0, ORDONNE, DESORDONNE);

  return mu_0;
}

/* ~~~~~~~~~~~~~~~~ Composantes du signal d'E.X.A.F.S. ~~~~~~~~~~~~~~~~ */

/* ---------------- Déphasage, pour un atome central et un réflecteur - */

Graphe *CTH_phase(void)
{
  Graphe *ph_centre, *ph_reflecteur, *phase;
  PHA_infos inf_phases;
  OBJECT *adresse = arbres[CALC_PHASE].adresse;
  int reponse;
  double k_min, k_max, pas, y;
  unsigned long nbr_pts, index;
  boolean seuil_k;
  double *b,*c,*d;
  register double *x_phase, *y_phase;

  ARB_MettreComplement( CALC_PHASE, &inf_phases );

  OPTION_A_FAIRE( "ĳ Remanier profondément [inclusion FEFF et tout τa]" );
  inf_phases.centre = GRF_par_type(GR_EXAFSPHA, 0, NULL);
  inf_phases.reflecteur = GRF_par_type(GR_EXAFSPHA, 0, NULL);

  /* On rΦgle en fonction de ce que l'on vient de recevoir */

  if (inf_phases.centre == NULL) { /* Pas de liste de fichiers de phase -> calcul seulement théorique... */
    OB_STATE(adresse,PHA_CTR_EXP) |= DISABLED;     /* Calcul théorique seul possible */
    OB_STATE(adresse,PHA_CTR_EXP) &= ~SELECTED;
    OB_STATE(adresse,PHA_CTR_THEORIE) |= SELECTED;
  }
  if ( OB_STATE(adresse,PHA_CTR_THEORIE) & SELECTED ) { /* Calcul théorique */
    OB_FLAGS(adresse,PHA_CTR_1 - 1) |= HIDETREE; /* Sélecteur invisible */
    OB_FLAGS(adresse,PHA_CTR_AVANT) |= HIDETREE;
    OB_FLAGS(adresse,PHA_CTR_FOND)  |= HIDETREE;
    OB_FLAGS(adresse,PHA_CTR_APRES) |= HIDETREE;

    OB_FLAGS(adresse,PHA_CTR_NATURE) &= ~HIDETREE; /* Choix de l'élément possible */
    arbres[CALC_PHASE].premier_edit = PHA_CTR_NATURE;
  } else {
    OB_FLAGS(adresse,PHA_CTR_1 - 1) &= ~HIDETREE; /* Sélecteur visible */
    OB_FLAGS(adresse,PHA_CTR_AVANT) &= ~HIDETREE;
    OB_FLAGS(adresse,PHA_CTR_FOND)  &= ~HIDETREE;
    OB_FLAGS(adresse,PHA_CTR_APRES) &= ~HIDETREE;

    OB_FLAGS(adresse,PHA_CTR_NATURE) |= HIDETREE; /* Choix de l'élément invisible */
    arbres[CALC_PHASE].premier_edit = CALC_PHA_DEBUT;
  }

  if (inf_phases.reflecteur == NULL) /* Pas de liste de fichiers de phase -> calcul seulement théorique... */
  {
    OB_STATE(adresse,PHA_EXP) |= DISABLED;     /* Calcul théorique seul possible */
    OB_STATE(adresse,PHA_EXP) &= ~SELECTED;
    OB_STATE(adresse,PHA_THEORIE) |= SELECTED;
  }
  if ( OB_STATE(adresse,PHA_THEORIE) & SELECTED ) /* Calcul théorique pour le réflecteur */
  {
    OB_FLAGS(adresse,PHA_EXP_1 - 1) |= HIDETREE; /* Sélecteur invisible */
    OB_FLAGS(adresse,PHA_EXP_AVANT) |= HIDETREE;
    OB_FLAGS(adresse,PHA_EXP_FOND)  |= HIDETREE;
    OB_FLAGS(adresse,PHA_EXP_APRES) |= HIDETREE;
    
    OB_FLAGS(adresse,PHA_EXP_NATURE) &= ~HIDETREE; /* Choix de l'élément possible */
    if ( ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) == FAUX )
      arbres[CALC_PHASE].premier_edit = PHA_EXP_NATURE;
  } else {
    OB_FLAGS(adresse,PHA_EXP_1 - 1) &= ~HIDETREE; /* Sélecteur visible */
    OB_FLAGS(adresse,PHA_EXP_AVANT) &= ~HIDETREE;
    OB_FLAGS(adresse,PHA_EXP_FOND)  &= ~HIDETREE;
    OB_FLAGS(adresse,PHA_EXP_APRES) &= ~HIDETREE;

    OB_FLAGS(adresse,PHA_EXP_NATURE) |= HIDETREE; /* Choix de l'élément invisible */
    if ( ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) == FAUX )
      arbres[CALC_PHASE].premier_edit = CALC_PHA_DEBUT;
  }

  /* On affiche les listes éventuelles de graphes convenant */
  GRF_affiche(inf_phases.centre, CALC_PHASE, PHA_CTR_1, PHA_CTR_4, 20,
	      PHA_CTR_AVANT, PHA_CTR_APRES, PHA_CTR_FOND, PHA_CTR_BOUTON);
  GRF_affiche(inf_phases.reflecteur, CALC_PHASE, PHA_EXP_1, PHA_EXP_4, 20,
	      PHA_EXP_AVANT, PHA_EXP_APRES, PHA_EXP_FOND, PHA_EXP_BOUTON);
  
  reponse = ARB_Bloquant( CALC_PHASE, "Calculer une phase", principale, 0 );
  if ( PHA_ANNULER == reponse ) {
    free( inf_phases.centre );
    free( inf_phases.reflecteur );
    return NULL;
  }

  /* Intervalle de calcul */
  reponse = OBJ_LireIntervalle( CALC_PHASE, CALC_PHA_DEBUT, CALC_PHA_FIN, CALC_PHA_PAS, ROOT,
				&k_min, &k_max, &pas, &nbr_pts );
  if ( reponse != 0 ) {
    (void) fprintf(stderr, "%s,%s l.%u : ERREUR dans la lecture de l'intervalle\n",
		   __FILE__, __FUNCTION__, __LINE__);
    free(inf_phases.centre);
    free(inf_phases.reflecteur);
    return NULL;		
  }

  seuil_k  =  OB_STATE(adresse, PHA_K ) & SELECTED;
  seuil_k |= (OB_STATE(adresse, PHA_L1) & SELECTED); /* donc VRAI -> seuil K ou L1 -> - M_PI */
  
  phase = GRF_nouveau( NULL, nbr_pts, GR_EXAFSPHA, GR_NORMAL, "Phase th"e_AIGU"orique"/* , "Nombre d'onde k", NULL, NULL, NULL */);
  if (phase == NULL) /* Plus rien à faire */
  {
    free(inf_phases.centre);
    free(inf_phases.reflecteur);
    return NULL;				
  }

  x_phase = phase->x; y_phase = phase->y;
  for (index = 0 ; index < nbr_pts ; index++, x_phase++, y_phase++)
  {
    *x_phase = k_min; k_min += pas; /* On passe au point suivant */
    if (seuil_k)
      *y_phase = -M_PI;
    else
      *y_phase = 0;
  }

  /* On calcule la phase de l'atome central */

  if ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) /* On veut un calcul théorique, atome central */
  {
  } else {
    ph_centre = (inf_phases.centre)->graphes[(inf_phases.centre)->num_select];

    /* On calcule la SPLINE pour lisser le spectre expérimental de référence */
    y = SPL_tt_prep(ph_centre, &b, &c, &d);
    if (y == -1) /* Erreur en calculant la spline */
    {
      free(inf_phases.centre);
      free(inf_phases.reflecteur);
      free(b); free(c); free(d);
      return NULL;
    }

    for (index = 0 ; index < nbr_pts ; index++) {
      y = SPL_tt_calc(ph_centre, (phase->x)[index], b, c, d);
      (phase->y)[index] += y;
    }

    free(b); free(c); free(d); /* Plus besoin de ça */
  }
  
  /* On calcule la phase de l'atome réflecteur */

  if ( OB_STATE(adresse, PHA_THEORIE) & SELECTED ) /* On veut un calcul théorique pour le réflecteur */
  {
  } else {
    ph_reflecteur = (inf_phases.reflecteur)->graphes[(inf_phases.reflecteur)->num_select];

    /* On calcule la SPLINE pour lisser le spectre expérimental de référence */
    y = SPL_tt_prep(ph_reflecteur, &b, &c, &d);
    if (y == -1) /* Erreur en calculant la spline */
    {
      free(inf_phases.centre);
      free(inf_phases.reflecteur);
      free(b); free(c); free(d);
      return NULL;
    }

    for (index = 0 ; index < nbr_pts ; index++)
    {
      y = SPL_tt_calc(ph_reflecteur, (phase->x)[index], b, c, d);
      (phase->y)[index] += y;
    }
    
    free(b); free(c); free(d); /* Plus besoin de τa */
  }

  /* Calcul des bornes */
  
  GRF_bornes(phase, ORDONNE, DESORDONNE); /* Calcul en X inutile */

  /* On a fini */

  free(inf_phases.centre);
  free(inf_phases.reflecteur);

  return phase;
}

int GER_phase(unsigned int num_objet, int x_souris, int y_souris)
{
  PHA_infos *inf_phases;
  boolean etat;

  inf_phases = (PHA_infos *) ARB_LireComplement( CALC_PHASE );
  
  if ( (num_objet >= PHA_CTR_1) && (num_objet <= PHA_CTR_APRES) ) 
    /* Sélecteur de graphe des phases de l'atome central */
  {
    GRF_selecteur(inf_phases->centre, CALC_PHASE, PHA_CTR_1, PHA_CTR_4, 20,
		  PHA_CTR_AVANT, PHA_CTR_APRES, 
		  PHA_CTR_FOND, PHA_CTR_BOUTON, num_objet, x_souris, y_souris);
    return VRAI;
  }

  if ( (num_objet >= PHA_EXP_1) && (num_objet <= PHA_EXP_APRES) ) 
    /* Sélecteur de graphe des phases de l'atome réflecteur */
  {
    GRF_selecteur(inf_phases->reflecteur, CALC_PHASE, PHA_EXP_1, PHA_EXP_4, 20, 
		  PHA_EXP_AVANT, PHA_EXP_APRES, 
		  PHA_EXP_FOND, PHA_EXP_BOUTON, num_objet, x_souris, y_souris);
    return VRAI;
  }

  OBJ_choisit(CALC_PHASE, num_objet, ECHANGE);

  if ( (num_objet == PHA_CTR_THEORIE) || (num_objet == PHA_CTR_EXP) )
  {
    etat = OBJ_choisit(CALC_PHASE, PHA_CTR_THEORIE, QUESTION);
    OBJ_cache(CALC_PHASE, PHA_CTR_1 - 1, etat);
    OBJ_cache(CALC_PHASE, PHA_CTR_AVANT, etat);
    OBJ_cache(CALC_PHASE, PHA_CTR_FOND, etat);
    OBJ_cache(CALC_PHASE, PHA_CTR_APRES, etat);
    OBJ_cache(CALC_PHASE, PHA_CTR_NATURE, !etat);
    return VRAI;
  }

  return FAUX;
}
