/* LASE - Logiciel d'Analyse des Spectres EXAFS
   LASE - (C) 1998-2002, Emmanuel CURIS

   Calculs thoriques : ajouter du bruit  un spectre thorique
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "graphes.h"
#include "statistiques.h" /* Pour les diverses densits de probabilit */
#include "theorie.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int TH_BruitGaussien(Graphe *graphe);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -~-~-~-~-~-~-~-~ Fonctions de bruitage -~-~-~-~-~-~-~-~ */

/* -------- Bruit gaussien
   Moyenne  = les valeurs en Y du graphe
   Variance = les valeurs en Y des barres d'erreur sur les Y
*/

static int TH_BruitGaussien(Graphe *graphe)
{
  unsigned long i;

  if ( GRF_FCT_REELLE == graphe->genre_donnees ) {
    double *y, *y_err;

    for (i = 0, y = graphe->y, y_err = graphe->erreur_y->y ; i < graphe->nombre_point ; i++, y++, y_err++) {
      *y += gaussienne() * *y_err; /* Tirage d'une gaussienne centre-rduite, drduit, dcentr */
    }
  } else if ( GRF_FCT_COMPLEXE == graphe->genre_donnees ) {
    Complexe *y, *y_err;

    for (i = 0, y = (Complexe *) graphe->y, y_err = (Complexe *) graphe->erreur_y->y ; i < graphe->nombre_point ; i++, y++, y_err++) {
      y->re += gaussienne() * y_err->re; /* Tirage d'une gaussienne centre-rduite, drduit, dcentr */
      y->im += gaussienne() * y_err->im; /* Tirage d'une gaussienne centre-rduite, drduit, dcentr */
    }
  } else {
    (void) fprintf(stderr, "TH_BruitGaussien : type de donnes dans le graphe inconnu ou inutilisable\n");
    return -1;
  }

  GRF_bornes(graphe, NE_PAS_FAIRE, DESORDONNE);
  return 0; /* Tout s'est bien pass */
}

/* -~-~-~-~-~-~-~-~ Interface gnrale vue de l'extrieur -~-~-~-~-~-~-~-~ */

Graphe *TH_AjouterBruit(Graphe *graphe, int distribution, boolean en_place)
{
  int retour = -1;

  if (NULL == graphe) return NULL;

  if ( NULL == graphe->erreur_y ) {
    (void) fprintf(stderr, "TH_AjouterBruit : graphe sans barres d'erreur !\n");
    return NULL;    
  }

  if (FAUX == en_place) {
    graphe = GRF_Copier( graphe, NULL, FAUX );
    if ( NULL == graphe ) {
      (void) fprintf(stderr, "TH_AjouterBruit : impossible de copier le graphe !\n");
      return NULL;
    }
  }

  switch (distribution)
  {
  case DP_GAUSSIENNE:
    retour = TH_BruitGaussien(graphe);
    break;

  default:
    (void) fprintf(stderr, "TH_AjouterBruit : distribution inconnue ou pas encore ralise ! [%d]\n", distribution);
    break;
  }

  if (retour != 0) {
    (void) fprintf(stderr, "TH_AjouterBruit : problme en ajoutant le bruit !\n");
    if (FAUX == en_place) {
      GRF_efface(graphe, VRAI, VRAI); /* Pas besoin de cette copie ! */
      graphe = NULL;
    }
  }

  return graphe;
}

/* -~-~-~-~-~-~-~-~ Interface graphique -~-~-~-~-~-~-~-~ */

Graphe *TH_bruiter_graphe(Graphe *graphe)
{
  if (NULL == graphe) return NULL;

  return graphe;
}

