/* LASE - Logiciel d'Analyse des Spectres EXAFS
   LASE - (C) 1998-2003, Emmanuel CURIS

   Calcul thorique ou semi-empirique de la section efficace de diffusion lastique
   [= diffusion cohrente, = diffusion Rayleigh = diffusion Thomson (pour un lectron)]
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "systeme.h"

#include "chimie_utils.h" /* Analyse des formules chimiques... */
#include "absorption.h"
#include "graphes.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int TH_Diffuse_McMaster(int Z, struct graphe *graphe, double f_massique);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Appel depuis l'extrieur -------- */

int TH_DiffusionElastique(int Z, struct graphe *graphe, int methode, double f_massique)
{
  int retour;

  if ( NULL == graphe ) return -1;
  if ( ( Z < HYDROGENE ) || ( Z > CURIUM ) ) {
    (void) fprintf(stderr, "TH_DiffusionElastique : lment inconnu [%d]\n", Z);
    return -2;
  }

  if ( GRF_FCT_REELLE != graphe->genre_donnees ) {

  }

  /* Appel de la bonne mthode */
  switch ( methode )
  {
  case DIFFUSION_COEF_McMaster:
    retour = TH_Diffuse_McMaster( Z, graphe, f_massique );
    break;

  default:
    (void) fprintf(stderr, "TH_DiffusionElastique : Mthode inconnue [%d]\n", methode);
    return -4;
  }
  if ( retour != 0 ) {
    (void) fprintf(stderr, "TH_DiffusionElastique : ERREUR dans la modlisation [code %d]\n", retour);
    return -10;
  }

  /* Tout s'est bien pass */
  return 0;
}

/* -------- Estimation  l'aide des coefficients, modle de McMaster -------- */

static int TH_Diffuse_McMaster(int Z, struct graphe *graphe, double f_massique)
{
  double A, B, C, D;
  double *x, *y;
  int retour;
  unsigned long i;

  /* Lecture des coefficients */
  retour = TH_coef_McMaster( Z, DIFFUSION_COHERENTE, &A, &B, &C, &D );
  if ( FAUX == retour ) {
    (void) fprintf(stderr, "TH_Diffuse_McMaster : ERREUR dans TH_coef_McMaster\n");
    return -1;
  }

  /* Calculs */
  for ( i = 0, x = graphe->x, y = graphe->y ; i < graphe->nombre_point ; i++, x++, y++ ) {
    *y += f_massique * TH_McMaster( *x, Z, A, B, C, D );
  }

  /* Tout s'est bien pass */
  return 0;
}
