/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (C) 1998-2000, Emmanuel CURIS

   Module d'affichage graphique des rsultats

   Trac de la grille dans un rectangle des rsultats
   (Repris de TRACE.c  la racine)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "systeme.h"

#include "tracer.h"
#include "math_utils.h" /* fonction Puissance */

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static void TRC_GrilleDecimale(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe);
static void TRC_GrilleLog     (Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe);

static void TRC_Grille_Log(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe, double echelle, int p10, double val_ref);

/* ~~~~~~~~ Ralisation pratique des fonctions ~~~ */

/* -------- Trac d'une grille pour l'axe considr -------- */

void TRC_Grille(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe)
{
  if ( ( NULL == region ) || ( NULL == axe ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Paramtres absurdes\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  if ( FAUX == ( axe->proprietes & AXE_AVEC_GRILLE ) ) return; /* Pas de grille */

  /* Couleur, style de la grille */
  TRC_Couleur( axe->couleur_grille );
  TRC_Style( axe->style_grille );
  TRC_Epaisseur( 1 );

  switch ( axe->type_graduation )
  {
  case GRADUATION_AUCUNE:
    break;

  case GRADUATION_DECIMALE:
    TRC_GrilleDecimale( fenetre, region, axe );
    break;

  case GRADUATION_LOGARITHMIQUE:
    TRC_GrilleLog( fenetre, region, axe );
    break;

  case GRADUATION_PARABOLIQUE:
    OPTION_A_FAIRE( "Graduation parabolique" );
    break;

  case GRADUATION_GAUSSIENNE:
    OPTION_A_FAIRE( "Graduation gaussienne" );
    break;

  default:
    AVERTISSEMENT( "Mthode de graduation inconnue [%d]", axe->type_graduation);
    return;
  }
}

/* -~-~-~-~-~-~-~-~ Grilles dcimales classiques -~-~-~-~-~-~-~-~ */

static void TRC_GrilleDecimale(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe)
{
  int points[ 4 ] = { 0, 0, 0, 0 };
  double valeur, pas, depart, echelle;
  int chiffre;

  /* Coordonnes fixes */
  if ( axe->proprietes & AXE_VERTICAL ) {
    points[ 0 ] = region->x_BG;
    points[ 2 ] = region->x_BG + region->largeur;
    echelle = ( axe->max - axe->min ) / region->hauteur;
  } else {
    points[ 1 ] = region->y_BG;
    points[ 3 ] = region->y_BG - region->hauteur;
    echelle = ( axe->max - axe->min ) / region->largeur;
  }

  /* Quel pas utiliser ? (la puissance de 10 utilise pour le texte, a priori) */
  pas = axe->pas_graduation;
  if ( pas <= 0.0 ) {
    /* Si automatique : un cran en-dessous de la puissance optimale ! */
    pas = Puissance( 10.0, AXE_GraduationMajeure( axe->min, axe->max ) - 1 );
  }

  /* O dmarre-t-on ? */
  if ( axe->min > 0.0 ) {
    double x_p;

    x_p = axe->min / pas;
    chiffre = (int) ( x_p - floor( x_p / 10.0 ) * 10.0);
    depart  = ( floor( x_p ) - ( double ) chiffre ) * pas;
  } else {
    double x_p;

    x_p = axe->min / pas;
    chiffre = (int) ( x_p - ceil( x_p / 10.0 ) * 10.0);
    depart  = ( ceil( x_p ) - ( double ) chiffre ) * pas;
  }

  /* On saute toutes les graduations avant le dbut de l'axe */
  valeur = depart + pas * (double) chiffre;
  while ( valeur < axe->min ) {
    chiffre++;
    valeur = depart + pas * (double) chiffre;
  }

  /* Tant que l'on est dans la partie visible, on fait les traits */
  while ( valeur  <= axe->max ) {
    /* Position des points */
    if ( axe->proprietes & AXE_VERTICAL ) {
      points[ 1 ] = TRC_ConvertitY_PvE( valeur, axe->min, region->y_BG, echelle );
      if ( points[ 1 ] < (int) ( region->y_BG - region->hauteur + 1 ) ) break;
      points[ 3 ] = points[ 1 ];
    } else {
      points[ 0 ] = TRC_ConvertitX_PvE( valeur, axe->min, region->x_BG, echelle );
      if ( points[ 0 ] > (int) ( region->x_BG + region->largeur - 1 ) ) break; /* vite une grille juste en-dehors du cadre */
      points[ 2 ] = points[ 0 ];
    }

    /* Trac */
    TRC_Ligne( fenetre, points[ 0 ], points[ 1 ], points[ 2 ], points[ 3 ] );

    /* Passage au tic suivant */
    chiffre++;
    valeur = depart + pas * (double) chiffre; /* Recalcul pour viter de rester sur place si pas trop faible ! */
  }
}

/* -~-~-~-~-~-~-~-~ Grilles logarithmiques classiques -~-~-~-~-~-~-~-~ */

static void TRC_GrilleLog(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe)
{
  int p10_min, p10_max, p10, x_HD, y_HD;
  double place_puissance, dimension, echelle;
  int pos_depart, pos_fin;

  /* Prparation de la grille : partie invariante */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    dimension = (double) ( region->hauteur );
  } else { /* Axe horizontal */
    dimension = (double) ( region->largeur );
  }
  echelle = ( axe->max - axe->min ) / dimension;

  /* Coordonnes du coin oppos */
  x_HD = region->x_BG + region->largeur - 1;
  y_HD = region->y_BG - region->hauteur + 1;

  /* Quelle place occupe une puissance complte sur l'axe [= un intervalle de 1 en log] */
  place_puissance = 1.0 / echelle;
  if ( place_puissance < 10.0 ) { /* Moins de 10 pixels : seul certaines puissances seront places => sans grille */
    return;
  }

  /* Quelles sont les puissances de 10 extrmes impliques ? */
  p10_min = floor( axe->min );
  p10_max = ceil ( axe->max );

  /* Pour chacune de ces puissances, on gradue */
  for ( p10 = p10_min ; p10 < p10_max ; p10++ ) {
    pos_depart  = p10 - axe->min; /* log10( 10^n ) = n : 1er tic */
    pos_depart /= echelle;

    pos_fin = (p10 + 1) - axe->min; /* log10( 10^n+1 ) = n + 1 : dernier tic, 1er suivant */
    pos_fin /= echelle;

    /* On place le tic de fin, s'il est dans la rgion */
    if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
      pos_depart = TRC_ConvertitY_PvE( (double) (p10    ), axe->min, region->y_BG, echelle );
      pos_fin    = TRC_ConvertitY_PvE( (double) (p10 + 1), axe->min, region->y_BG, echelle );

      if ( ( pos_fin < (int) region->y_BG ) && ( pos_fin > y_HD ) ) {
	TRC_LigneH( fenetre, region->x_BG, pos_fin, region->largeur );
      }
    } else {
      pos_depart = TRC_ConvertitX_PvE( (double) (p10    ), axe->min, region->x_BG, echelle );
      pos_fin    = TRC_ConvertitX_PvE( (double) (p10 + 1), axe->min, region->x_BG, echelle );

      if ( ( pos_fin > (int) region->x_BG ) && ( pos_fin < x_HD ) ) {
	TRC_LigneV( fenetre, region->y_BG - region->hauteur, pos_fin, region->hauteur );
      }
    }

    /* On place le niveau en-dessous avec une taille infrieure */
    TRC_Grille_Log( fenetre, region, axe, echelle, p10, Puissance( 10.0, p10 ) );
  }
}

/* -------- Placer les tics logarithmiques entre deux positions --------

   p10        : la puissance de 10 que l'on reprsente
   pos_depart : la position _cran_ du tic pour le tic "1"
 */

static void TRC_Grille_Log(Fenetre *fenetre, InfosRegion2D *region, InfosAxe *axe, double echelle,
			   int p10, double val_ref)
{
  double positions[ 10 ];
  int point_avant;
  int position_ecran;
  unsigned int i;

  /* Position des tics  l'cran */
  for ( i = 0 ; i < 10 ; i++ ) {
    positions[ i ] = val_ref +  Puissance( 10.0, p10 ) * (double) i; /* Valeur dcimale */
    positions[ i ] = log10( positions[ i ] ); /* Correspondance en log */
  }

  /* Prparation */
  if ( axe->proprietes & AXE_VERTICAL ) { /* Axe vertical */
    point_avant = TRC_ConvertitY_PvE( positions[ 0 ], axe->min, region->y_BG, echelle );
  } else { /* Axe horizontal */
    point_avant = TRC_ConvertitX_PvE( positions[ 0 ], axe->min, region->x_BG, echelle );
  }

  /* On place les traits pour "2"  "9" [1 plac avant comme "x" d'un tour suprieur, "10" = "1" du suivant]  */
  if ( axe->proprietes & AXE_VERTICAL ) {
    for ( i = 1 ; i < 10 ; i++ ) {
    /* On trace les traits */
      position_ecran = TRC_ConvertitY_PvE( positions[ i ], axe->min, region->y_BG, echelle );
      if ( position_ecran >= (int) region->y_BG ) { /* On est en-dessous */
	continue;
      }

      if ( position_ecran <= (int) ( region->y_BG - region->hauteur + 1 ) ) { /* On dborde par en haut */
	return; /* La suite sera hors-rgion */
      }

      if ( ( point_avant - position_ecran ) < 4 ) return; /* Trop serrs : la suite aussi ! */
      point_avant = position_ecran;

      TRC_LigneH( fenetre, region->x_BG, position_ecran, region->largeur );
    }
  } else {
    for ( i = 1 ; i < 10 ; i++ ) {
      position_ecran = TRC_ConvertitX_PvE( positions[ i ], axe->min, region->x_BG, echelle );
      if ( position_ecran <= (int) region->x_BG ) { /* On est en-dessous */
	continue;
      }
      if ( position_ecran >= (int) ( region->x_BG + region->largeur - 1 ) ) {
	return; /* On dborde par en haut : la suite sera hors-rgion */
      }

      if ( ( position_ecran - point_avant ) < 4 ) return; /* Trop serrs : la suite aussi ! */
      point_avant = position_ecran;

      TRC_LigneV( fenetre, position_ecran, region->y_BG - region->hauteur, region->hauteur );
    }
  }
}

/* -~-~-~-~-~-~-~-~ Placer une barre  une position donne -~-~-~-~-~-~-~-~ */

/* -------- Routine d'appel gnrique --------
   Ne renvoie rien

   station : la station VDI  utiliser pour le dessin
   region  : la rgion 2D dans laquelle il faut placer la barre
   valeur  : la valeur [vraie, IE dans le repre rel et PAS de l'cran]  laquelle placer la barre
   type_axe: Si AXE_X, on place une barre verticale [valeur donne sur l'axe des X]
           : Si AXE_Y, on place une barre horizontale [valeur donne sur l'axe des Y]
   couleur : la couleur  utiliser pour dessiner la barre
   ------------------------------------------ */

void TRC_TracerBarre(Fenetre *fenetre, InfosRegion2D *region, double valeur, int type_axe, int couleur)
{
  InfosAxe *axe = NULL;
  double echelle;
  int position;

  if ( ( NULL == fenetre ) || ( NULL == region ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Paramtre absurde\n",
		   __FILE__, __FUNCTION__, __LINE__);
    FIN_ANTICIPEE( FIN_ERREUR_GRAVE );
    return;
  }

  /* On slectionne l'axe considr */
  if ( AXE_X == type_axe ) axe = &(region->axe_X);
  if ( AXE_Y == type_axe ) axe = &(region->axe_Y);
  if ( NULL == axe ) {
    AVERTISSEMENT( "Axe indiqu incorrect [%d]", type_axe );
    return;
  }

  /* On vrifie que la valeur tudie est visible */
  if ( ( valeur < axe->min ) || ( valeur > axe->max ) ) {
    /* On est en-dehors de la rgion visible */
    return;
  }

  TRC_Couleur( couleur );
  TRC_Style( SOLID );
  TRC_Epaisseur( 1 );

  /* On en dduit l'chelle */
  if ( type_axe & AXE_X ) {
    echelle  = AXE_Echelle( axe, 0, region->largeur );
    position = TRC_ConvertitX_PvE( valeur, axe->min, region->x_BG, echelle );

    TRC_Z_LigneV( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur,
		  position, region->y_BG, region->hauteur );
  }
  if ( type_axe & AXE_Y ) {
    /* Attention,  l'cran on est  l'envers (axe vers le bas) ! */
    echelle = AXE_Echelle( axe, 0, region->hauteur );
    position = TRC_ConvertitY_PvE( valeur, axe->min, region->y_BG, echelle );

    TRC_Z_LigneH( fenetre, region->x_BG, region->y_BG - region->hauteur, region->largeur, region->hauteur,
		  region->x_BG, position, region->largeur );
  }
}

