/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2002, Emmanuel CURIS

   Conversion symbole chimique -> numro atomique corresponddant
*/

#ifndef HYDROGENE

# ifndef VRAI
#  define VRAI 1
#  define FAUX 0
# endif

# ifndef AVOGADRO
#  define AVOGADRO 6.022045e23     /* mol^-1 - Rivail */
# endif

/* Les diffrents lments */

#define SYMBOLE_CHIMIQUE_INCONNU 0

#define HYDROGENE       1 /* 1s1    - 1re priode */
#define HELIUM          2 /* 1s    */
#define LITHIUM         3 /* 2s1    - 2e priode */
#define BERYLLIUM       4 /* 2s    */
#define BORE            5 /* 2s2p  */
#define CARBONE         6 /* 2s2p */
#define AZOTE           7 /* 2s2p3 */
#define OXYGENE         8 /* 2s2p4 */
#define FLUOR           9 /* 2s2p5 */
#define NEON           10 /* 2s2p6 */
#define SODIUM         11 /* 3s     - 3e priode */
#define MAGNESIUM      12 /* 3s    */
#define ALUMINIUM      13 /* 3s3p  */
#define SILICIUM       14 /* 3s3p */
#define PHOSPHORE      15 /* 3s3p3 */
#define SOUFRE         16 /* 3s3p4 */
#define CHLORE         17 /* 3s3p5 */
#define ARGON          18 /* 3s3p6 */
#define POTASSIUM      19 /* 4s     - 4e priode */
#define CALCIUM        20
#define SCANDIUM       21
#define TITANE         22
#define VANADIUM       23
#define CHROME         24
#define MANGANESE      25
#define FER            26
#define COBALT         27
#define NICKEL         28
#define CUIVRE         29
#define ZINC           30
#define GALLIUM        31
#define GERMANIUM      32
#define ARSENIC        33
#define SELENIUM       34
#define BROME          35
#define KRYPTON        36
#define RUBIDIUM       37
#define STRONTIUM      38
#define YTTRIUM        39
#define ZIRCONIUM      40
#define NIOBIUM        41
#define MOLYBDENE      42
#define TECHNETIUM     43
#define RUTHENIUM      44
#define RHODIUM        45 /* Rh */
#define PALLADIUM      46
#define ARGENT         47
#define CADMIUM        48
#define INDIUM         49
#define ETAIN          50
#define ANTIMOINE      51
#define TELLURE        52
#define IODE           53
#define XENON          54
#define CESIUM         55
#define BARYUM         56
#define LANTHANE       57
#define CERIUM         58
#define PRASEODYME     59
#define NEODYME        60
#define PROMETHIUM     61 /* Pm */
#define SAMARIUM       62
#define EUROPIUM       63
#define GADOLINIUM     64
#define TERBIUM        65
#define DYSPROSIUM     66
#define HOLMIUM        67
#define ERBIUM         68
#define THULIUM        69 /* Tm */
#define YTTERBIUM      70
#define LUTETIUM       71
#define HAFNIUM        72
#define TANTALE        73
#define TUNGSTENE      74
#define RHENIUM        75
#define OSMIUM         76
#define IRIDIUM        77
#define PLATINE        78
#define OR             79
#define MERCURE        80
#define THALLIUM       81
#define PLOMB          82
#define BISMUTH        83
#define POLONIUM       84
#define ASTATE         85
#define RADON          86
#define FRANCIUM       87
#define RADIUM         88
#define ACTINIUM       89
#define THORIUM        90 /*  vrifier - Th */
#define PROTACTINIUM   91
#define URANIUM        92
#define NEPTUNIUM      93
#define PLUTONIUM      94
#define AMERICIUM      95
#define CURIUM         96 /* Cm */
#define BERKELIUM      97 /* Bk */
#define CALIFORNIUM    98
#define EINSTEINIUM    99
#define FERMIUM       100 /* Fm */
#define MENDELEVIUM   101 /* Md */
#define NOBELIUM      102 /* No */
#define LAWRENCIUM    103 /* Lr */
#define RUTHERFORDIUM 104 /* Rf */
#define DUBNIUM       105 /* Db */
#define SEABORGIUM    106 /* Sg */
#define BOHRIUM       107 /* Bh */
#define HASSIUM       108 /* Hs */
#define MEITNERIUM    109 /* Mt */
#define DARMSTADTIUM  110 /* Ds */
#define UNUNUNIUM     111 /* Uuu */
#define UNUNBIUM      112
#define UNUNTRIUM     113
#define UNUNQUADIUM   114
#define UNUNPENTIUM   115
#define UNUNHEXIUM    116
#define UNUNSEPTIUM   117
#define UNUNOCTIUM    118

/* Conversion symbole -> numro atomique */

int ATM_num_atomique(char *symbole);

/* Donnes gnrales connues sur les atomes */

/* ************ Informations sur les atomes ************************************* */

typedef struct
{
  const char *symbole;     /* Symbole de l'lment                  */
  const char *nom;         /* Nom de l'lment - ici en franais... */
  float m_molaire;         /* Masse molaire de l'lment */
  float electronegativite; /* lectrongativit de l'lment */
  float rayon_covalent;
  float rayon_vdw;         /* Rayon de Van der Waals              */
  float m_volumique;       /* Masse volumique */
} ATM_infos;

#ifndef UTILITAIRES_CONTIENT_DONNEES_ATOMES
extern ATM_infos ATM_donnees[];
#endif

const char *ATM_Z_vers_symbole(int Z);
const char *ATM_Z_vers_nom    (int Z);

float ATM_Z_vers_rayon(int Z, int type_rayon);
float ATM_Z_vers_masse(int Z, int type_masse);

#define ATM_RAYON_COVALENT    0
#define ATM_RAYON_VANDERWAALS 1
#define ATM_RAYON_METALLIQUE  2
#define ATM_RAYON_IONIQUE     3

#define ATM_MASSE_MOLAIRE     0
#define ATM_MASSE_ATOMIQUE    1
#define ATM_MASSE_VOLUMIQUE   2

int ZATM_forme_complexes(unsigned int Z);

/* -------- Analyse de composition -------- */

typedef struct compo {
  struct compo *avant;
  float atomes[120];
} Composition;

Composition *Chimie_AnalyseFormule    (const char *chaine);
int          Chimie_ChargeBibliotheque(const char *nom_fichier);

double       Chimie_EstimeDensite(Composition *composition, int methode);
double       Chimie_MasseMolaire (Composition *composition);

/* -------- Liste de solvants ------------- */

typedef struct solvant {
  char *nom;			/* Nom du solvant [obligatoire] */
  char *formule;		/* Formule        [NULL si non-pertinent] */
  float masse_molaire;		/* en g/mol */
  float densite;		/* en g/mL */
  float T_min, T_max;		/* Tempratures extrmes d'utilisation [T_min : solidification ; T_max : bullition ; K] */
  float pKe;			/* Constante d'autoionisation */
} Solvant;

Solvant *Chimie_CreerSolvants  (unsigned long nombre_solvants);
void     Chimie_EffacerSolvants(Solvant *solvants, unsigned long nombre_solvants);
Solvant *Chimie_LireSolvants   (const char *nom_fichier, unsigned long *nombre_solvants);

/* -------- Liste de composants ----------- */

typedef struct liste_composes {
  char *nom;			/* Le nom du compos */
  char *formule;		/* Formule brute [analysable par ^^] */
  float valeur;			/* Valeur associe au compos [role dfini par l'utilisateur] */
} ListeComposes;

ListeComposes *Chimie_LireListeComposes(const char *nom_fichier, unsigned long *nombre_composes);

#endif /* HYDROGENE */
