/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   Calculs mathmatiques frquents
*/

#include <stdio.h>
#include <stdlib.h>

#include "math_utils.h"

#ifndef NMAX_COEFFICIENT_BINOMIAL
/* Essais linux Celeron/gcc : on a au moins 13 chiffres corrects pour tout p avec N <= 1029
                              on n'a aucun chiffre correct pour tout p avec N > 1029
     le nombre de chiffres corrects avec N=1029 varie suivant p (15 pour p=0,5, 14 pour p=0,4, 13 pour p<0,3)
 */
# define NMAX_COEFFICIENT_BINOMIAL 1029
#endif

/* ~-~-~-~-~-~-~-~- Dnombrement ~-~-~-~-~-~-~-~- */

/* -------- Calcul d'un coefficient binomial -------- */

double CoefficientBinomial(unsigned int N, unsigned int k)
{
  unsigned long i;
  double C = 1.0; /* Travail en double pour viter les dbordements de capacit */

  if ( N > NMAX_COEFFICIENT_BINOMIAL ) {
    (void) fprintf(stderr, "%s, %s l.%u : Valeur de N trop grande [Nmax = %u, N=%u], les valeurs seront certainement fausses\n",
		   __FILE__, __FUNCTION__, __LINE__, NMAX_COEFFICIENT_BINOMIAL, N);
    return -1.0;
  }

  if ( k > N ) return 0.0;  /* Par convention, C(n,k) = 0 si k < 0 ou k > n */
  if ( k > N/2 ) k = N - k; /* C(n,k) = C(n, n-k), on l'utilise pour tre sr d'avoir k < n-k */
  if ( 0 == k ) return 1.0; /* C(n,0) = C(n, n) = 1 */
  if ( 1 == k ) return (double) N; /* C(n,1) = C(n,n-1) = n */

  for (i = 1 ; i <= k ; i++) {
    C *= ( (double) N ) / (double) i;
    N--;
  }

  return C;
}

/* ~-~-~-~-~-~-~-~- Arithmtique, analyse ~-~-~-~-~-~-~-~- */

/* -------- Polynme ( une variable) -------- */

inline double Polynome(double x, unsigned int degre, double *coefficients)
{
  double valeur = 0.0;
  unsigned int i;

  if ( NULL == coefficients ) return 0.0;
  if ( 0.0 == x ) return coefficients[ 0 ];

				/* Schma descendant : a0 + a1*X + a2*X + ... = (a2 X + a1) * X + a0 */
  for ( i = degre ; i > 0 ; i-- ) {
    valeur += coefficients[ i ];
    valeur *= x;
  }
  valeur += coefficients[ 0 ];	/* Le terme constant ! */

  return valeur;
}

/* -------- puissance d'un nombre -------- */

inline double Puissance(double valeur, int exposant)
{
  unsigned int i;
  double p = 1.0;

  if ( exposant >  0 ) {
    for ( i = 0 ; i < (unsigned int) exposant ; i++ ) {
      p *= valeur;
    }
  } else if ( exposant < 0 ) {
    if ( 0.0 == valeur ) {
      (void) fprintf(stderr, "%s, %s l.%u : Division par zro [0^%d]\n",
		     __FILE__, __FUNCTION__, __LINE__, exposant);
      return 0.0; /* Arbitraire */
    }

    for ( i = 0 ; i < (unsigned int) -exposant ; i++ ) {
      p /= valeur;
    }
  }

  return p;
}
