/* Fonctions utilitaires gnrales
   Dveloppes pour LASE, mais plus gnriques
   (C) 1996 - 2003, Emmanuel CURIS

   Manipulations de table de valeurs (doubles)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>   /* Pour rint */
#include <string.h> /* Pour memcpy */

#include "txt_utils.h"
#include "utils_tables.h"

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int TriCroissant_D  (const void *, const void *);
static int TriDecroissant_D(const void *, const void *);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -~-~-~-~-~-~-~-~ Cration d'une table de valeurs -~-~-~-~-~-~-~-~ */

/* -------- Copier une table de valeurs -------- */

double *TABLE_CopierTable( double *table, unsigned long nbr_valeurs )
{
  double *copie;

  if ( NULL == table ) return NULL;

  copie = TABLE_CreerTable( nbr_valeurs );
  if ( NULL == copie ) {
    (void) fprintf(stderr, "TABLE_CopierTable : ERREUR dans TABLE_CreerTable()\n");
    return NULL;
  }

  memcpy( copie, table, sizeof( double ) * nbr_valeurs );

  return copie;
}

/* -------- Crer une table de valeurs -------- */

double *TABLE_CreerTable( unsigned long nbr_valeurs )
{
  double *table;

  if ( 0 == nbr_valeurs ) {
    (void) fprintf(stderr, "TABLE_CreerTable : Nombre de valeurs absurde [%lu]\n", nbr_valeurs);
    return NULL;
  }

  table = (double *) malloc( sizeof( double ) * nbr_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "TABLE_CreerTable : Mmoire insuffisante pour rserver %lu valeurs\n", nbr_valeurs);
    return NULL;
  }

  return table;
}

/* -------- Crer une table et l'initialiser avec une grille rgulire : quatre faons de dcrire la grille -------- */

double *TABLE_CreerTable_mMp( double min, double max, double pas, unsigned long *nombre )
{
  double *table, *t;
  unsigned long i, nbr_valeurs;

  /* Controles prliminaires */
  if ( max <= min ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMp : Bornes absurdes [min=%g, max=%g]\n", min, max);
    return NULL;
  }
  if ( ( pas <= 0.0 ) || ( pas > (max - min) ) ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMp : Pas absurde [%g / min=%g, max=%g]\n", pas, min, max);
    return NULL;
  }

  /* Combien de valeurs sont ncessaires ? */
  nbr_valeurs = 1 + (unsigned long) rint( (max - min) / pas );

  /* Cration de la table */
  table = TABLE_CreerTable( nbr_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMp : ERREUR dans TABLE_CreerTable()\n");
    return NULL;
  }

  /* Initialisation de la table */
  for ( i = 0, t = table ; i < nbr_valeurs ; i++, t++ ) {
    *t = min + pas * (double) i;
  }

  /* Initialisation termine et russie */
  if ( nombre != NULL ) *nombre = nbr_valeurs;
  return table;
}

double *TABLE_CreerTable_mMN( double min, double max, unsigned long nbr_valeurs )
{
  double pas;
  double *table, *t;
  unsigned long i;

  /* Controle prliminaire */
  if ( nbr_valeurs < 2 ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMN : Nombre de valeurs absurde [%lu]\n", nbr_valeurs);
    return NULL;
  }

  /* Cration de la table */
  table = TABLE_CreerTable( nbr_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMN : ERREUR dans TABLE_CreerTable()\n");
    return NULL;
  }

  /* Calcul du pas */
  pas = ( max - min ) / (double) ( nbr_valeurs - 1 );

  /* Initialisation de la table */
  for ( i = 0, t = table ; i < nbr_valeurs ; i++, t++ ) {
    *t = min + pas * (double) i;
  }

  /* Initialisation termine et russie */
  return table;
}

double *TABLE_CreerTable_mNp( double min, unsigned long nbr_valeurs, double pas )
{
  double *table, *t;
  unsigned long i;

  /* Cration de la table */
  table = TABLE_CreerTable( nbr_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "TABLE_CreerTable_mMN : ERREUR dans TABLE_CreerTable()\n");
    return NULL;
  }

  /* Initialisation de la table */
  for ( i = 0, t = table ; i < nbr_valeurs ; i++, t++ ) {
    *t = min + pas * (double) i;
  }

  /* Initialisation termine et russie */
  return table;
}

double *TABLE_CreerTable_MNp(double max, unsigned long nbr_valeurs, double pas)
{
  double *table, *t;
  unsigned long i;

  /* Cration de la table */
  table = TABLE_CreerTable( nbr_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "%s, %s l.%u : ERREUR dans TABLE_CreerTable()\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Initialisation de la table */
  for ( i = 0, t = table + nbr_valeurs - 1 ; i < nbr_valeurs ; i++, t-- ) {
    *t = max - pas * (double) ( i );
  }

  /* Initialisation termine et russie */
  return table;
}

/* -------- Crer une table de valeurs  partir d'une liste de nombres -------- */

double *TABLE_CreerTable_SV(const char *valeurs, unsigned long *nbr_valeurs, int separateur)
{
  double *table = NULL;
  unsigned long i, nombre_valeurs;
  char **txt_valeurs;

  if ( NULL == valeurs ) return NULL;
  if ( NULL == nbr_valeurs ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de mmoriser la taille de la table !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return NULL;
  }

  /* Aucune valeur tant que l'on n'a pas converti correctement la chane ! */
  *nbr_valeurs = 0;

  /* On cre une table de valeurs avec une valeur par table */
  txt_valeurs = TXT_DecomposerChaine( valeurs, separateur, &nombre_valeurs );
  if ( NULL == txt_valeurs ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de dcomposer la chane [%s]!\n",
		   __FILE__, __FUNCTION__, __LINE__, valeurs);
    return NULL;
  }

  /* On cre une table pour ces valeurs */
  table = TABLE_CreerTable( nombre_valeurs );
  if ( NULL == table ) {
    (void) fprintf(stderr, "%s, %s l.%u : Impossible de crer une table de %lu valeurs\n",
		   __FILE__, __FUNCTION__, __LINE__, nombre_valeurs);
    free( txt_valeurs[ 0 ] );
    free( txt_valeurs ); txt_valeurs = NULL;
    return NULL;
  }

  /* On convertit les valeurs de la chane */
  for ( i = 0 ; i < nombre_valeurs ; i++ ) {
    table[ i ] = TXT_CNV_vNombre_D( txt_valeurs[ i ], NULL, NULL );
  }

  /* On n'a plus besoin de la table des valeurs textuelles */
  free( txt_valeurs[ 0 ] );
  free( txt_valeurs ); txt_valeurs = NULL;

  /* On a termin ! */
  *nbr_valeurs = nombre_valeurs;
  return table;
}

/* -~-~-~-~-~-~-~-~ Tri, fusion,... de tables de valeurs -~-~-~-~-~-~-~-~ */

/* -------- Fusion de plusieurs tables en une seule [pas forcment trie] --------------\
 | Renvoie la table aprs fusion (NULL en cas d'erreur)                                 |
 |         dans total, le nombre de valeurs au total dans cette table                   |
 |                                                                                      |
 | tables      = tableau des tables  fusionner [Rq : peut contenir des NULL]           |
 | nbr_valeurs = pour chaque table, le nombre de valeurs qu'elle contient               |
 | nbr_tables  = le nombre de tables  fusionner                                        |
 | trier       = si VRAI, on suppose toutes les tables d'entre tries par X croissant  |
 |               et on veut que la table de sortie soit aussi trie par X croissant     |
 \------------------------------------------------------------------------------------- */

double *TABLE_FondreTables( double **tables, unsigned long *nbr_valeurs, unsigned int nbr_tables, int trier, unsigned long *total )
{
  double *fusion;
  unsigned int i;
  unsigned long nbr_total = 0;

  if ( ( NULL == tables ) || ( NULL == nbr_valeurs) ) return NULL;

  /* Combien de points au total ? */
  for ( i = 0 ; i < nbr_tables ; i++ ) {
    nbr_total += nbr_valeurs[ i ];
  }

  /* Cration de la table fusion */
  fusion = TABLE_CreerTable( nbr_total );
  if ( NULL == fusion ) {
    (void) fprintf(stderr, "TABLE_FondreTables : ERREUR dans TABLE_CreerTable()\n");
    return NULL;
  }
  if ( total != NULL ) *total = nbr_total;

  /* Remplissage de la table fusion */
  if ( VRAI == trier ) {
    double min;
    unsigned int j, j_min;
    unsigned long i, *idx;

    idx = (unsigned long *) calloc( nbr_tables, sizeof( unsigned long ) );
    if ( NULL == idx ) {
      (void) fprintf(stderr, "TABLE_FondreTables : Mmoire insuffisante --- Essayer sans tri !\n");
      free( fusion );
      return NULL;
    }

    /* Index de lecture dans les tables : initialiss  zro par calloc -> OK */

    for ( i = 0 ; i < nbr_total ; i++ ) {
      min = HUGE_VAL;
      j_min = nbr_tables;

      /* On cherche le minimum dans toutes les tables */
      for ( j = 0 ; j < nbr_tables ; j++ ) {
	if ( idx[ j ] < nbr_valeurs[ j ] ) { /* La table n'est pas entirement traite */
	  if ( *(tables[ j ] + idx[ j ] ) < min ) {
	    min = *(tables[ j ] + idx[ j ] );
	    j_min = j;
	  }
	}
      }

      /* Et on le place dans la table */
      fusion[ i ] = min;
      if ( j_min < nbr_tables ) {
	idx[ j_min ]++; /* Ce point est donc lu */
      } else {
	(void) fprintf(stderr, "TABLE_FondreTables : ERREUR dans la fusion de tables tries [point %lu]\n", i);
	free( fusion );
	return NULL;
      }
    }
  } else {
    unsigned int i;
    unsigned long idx_depart;

    for ( i = 0, idx_depart = 0 ; i < nbr_tables ; i++ ) {
      if ( ( tables[ i ] != NULL )  && ( nbr_valeurs[ i ] > 0 ) ) {
	memcpy( fusion + idx_depart, tables[ i ], sizeof( double ) * nbr_valeurs[ i ] );
      }

      idx_depart += nbr_valeurs[ i ];
    }
  }

  /* Fini, tout s'est bien pass */
  return fusion;
}

/* -------- Tri d'une table de valeurs -------- */

int TABLE_TrierTable( double *table, unsigned long nbr_valeurs, int sens )
{
  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) {
    (void) fprintf(stderr, "TABLE_TrierTable : table=NULL ou 0 valeurs\n");
    return -1;
  }
  if ( 1 == nbr_valeurs ) /* La table est forcment trie ! */
    return 0;

  /* Tri proprement dit */
  if ( sens >= 0 )
    qsort( table, nbr_valeurs, sizeof( double ), TriCroissant_D );
  else
    qsort( table, nbr_valeurs, sizeof( double ), TriDecroissant_D );

  /* Tout s'est bien pass */
  return 0;
}

static int TriCroissant_D(const void *v_d1, const void *v_d2)
{
  double d1, d2;

  d1 = * (const double *) v_d1;
  d2 = * (const double *) v_d2;

  if ( d1 < d2 ) return -1;
  if ( d1 > d2 ) return +1;
  return 0;
}

static int TriDecroissant_D(const void *v_d1, const void *v_d2)
{
  double d1, d2;

  d1 = * (const double *) v_d1;
  d2 = * (const double *) v_d2;

  if ( d1 < d2 ) return +1;
  if ( d1 > d2 ) return -1;
  return 0;
}

/* ~-~-~-~-~-~-~-~- Recherche des quartiles ~-~-~-~-~-~-~-~- */

/* -------- Mdiane uniquement -------- */

double TABLE_Triee_Mediane(double *table, unsigned long nbr_valeurs)
{
  double mediane;
  unsigned long milieu;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de valeurs indiques !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return 0.0;			/* Valeur comme une autre... */
  }

  /* Quel est l'index de la valeur centrale ? */
  milieu = nbr_valeurs >> 1;

  /* Le nombre de valeurs est-il pair ou impair ? */
  if ( nbr_valeurs & 1 ) {
    /* Nombre impair de valeurs : on prend la valeur centrale */
    /* Si nbr_valeurs impair [3], milieu est l'index de la valeur centrale [1 pour 0 1 2] */
    mediane = table[ milieu ];
  } else {
    /* Nombre pair de valeurs : on prend la moyenne des deux valeurs centrales */
    /* Si nbr_valeurs pairs [4], milieu est l'index de la valeur juste au-dessus [2 pour 0 1 / 2 3] */
    mediane  = table[ milieu - 1 ] + table[ milieu ];
    mediane /= 2.0;
  }

  /* On renvoie la mdiane ainsi obtenue */
  return mediane;
}

/* -------- Tous les quartiles --------
   Renvoie l'intervalle interquartile en cas de succs
           <0 en cas d'erreur
   Les trois quartiles intressants sont, par ordre croissant, dans quartiles[]
 */

double TABLE_Triee_Quartiles(double *table, unsigned long nbr_valeurs, double quartiles[ 3 ])
{
  unsigned long milieu;

  if ( ( NULL == table ) || ( 0 == nbr_valeurs ) ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas de valeurs indiques !\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -1.0;
  }
  if ( NULL == quartiles ) {
    (void) fprintf(stderr, "%s, %s l.%u : Pas d'emplacement prvu pour mmoriser les rsultats\n",
		   __FILE__, __FUNCTION__, __LINE__);
    return -2.0;
  }

  /* On calcule la mdiane de l'ensemble */
  quartiles[ 1 ] = TABLE_Triee_Mediane( table, nbr_valeurs );

  /* Index de cette mdiane */
  milieu = nbr_valeurs >> 1;
  if ( nbr_valeurs & 1 ) {
    /* Nombre de valeurs impair : les quartiles sont les mdianes des deux moitis, valeur centrale exclue */
    /* Ex : n = 5 => milieu = 2 ; 1re table : 0 1 ; 2e table : 3 4 */
    quartiles[ 0 ] = TABLE_Triee_Mediane( table, milieu );
    quartiles[ 2 ] = TABLE_Triee_Mediane( table + milieu + 1, milieu );
  } else {
    /* Nombre de valeurs pair : les quartiles sont les mdianes des deux moitis */
    /* Ex : n = 6 => milieu = 3 ; 1re table : 0 1 2 ; 2e table : 3 4 5 */
    quartiles[ 0 ] = TABLE_Triee_Mediane( table, milieu );
    quartiles[ 2 ] = TABLE_Triee_Mediane( table + milieu, milieu );
  }
  printf( "Quartiles trouvs : %g, %g\n", quartiles[ 0 ], quartiles[ 2 ] );
  /* On renvoie l'intervalle inter-quartiles */
  return quartiles[ 2 ] - quartiles[ 0 ];
}
