/* Fonctions generales de gestion des textes */

#ifndef TXT_UTILS_H_

# define TXT_UTILS_H_ /* Protection contre inclusion multiple */
# include <stdarg.h>
# include <time.h> /* Pour struct tm : description des dates & heures */

# ifndef VRAI
#  define FAUX 0
#  define VRAI 1
# endif

/* ------ Compter le nombre d'occurences d'un caractère dans une chaîne ------ */

unsigned long TXT_compte_caractere(const char *chaine, int caractere);

/* ------ Classification des caractères en sous-ensembles ------ */

int CAR_EstOperateur(int c);

/* ------ Gestion de l'UTF-8 ------
   (fonctions dans txt_utf8.h, travaillant sur un caractère donné pour le convertir en ASCII)
 */

char TXT_UTF8_vers_nombre   (const char *texte, unsigned int *n_octets);
char TXT_UTF8_vers_operateur(const char *texte, unsigned int *n_octets);

int           TXT_UTF8_LongueurCaractere(const char *caractere);
unsigned long TXT_UTF8_CompterCaracteres(const char *texte);

/* ------ Manipulation des chaînes de caractère -------- */

char *TXT_AjouteTexte (char *chaine1, const char *chaine2);
char *TXT_InsererTexte(char *texte, const char *txt_insere, unsigned long position);

char  *TXT_genere_chaine   (const char *masque, ...);
char  *TXT_vGenereChaine   (const char *masque, va_list parametres);
size_t TXT_lg_chaine_finale(const char *masque, va_list parametres);

char **TXT_DecomposerChaine(const char *texte, int separateur, unsigned long *nbr_elements);
char **TXT_GenereLignes    (char *texte, unsigned int lg_max, int separateur, unsigned long *nbr_lignes);

/* Compter le nombre de caractères d'une chaîne : en fonction du codage par défaut... */
#ifdef _latin1_
# define TXT_CompterCaracteres( t ) ( ( NULL == (t) ) ? ( 0 ) : ( strlen( t ) ) )
#endif
#ifdef _utf8_
# define TXT_CompterCaracteres( t ) TXT_UTF8_CompterCaracteres( t )
#endif

/* Compter le nombre d'octets occupés par le premier caractère de la  chaîne indiquée : en fonction du codage par défaut... */
#ifdef _latin1_
# define TXT_LongueurCaractere( t ) ( ( NULL == (t) ) ? ( 0 ) : ( 1 ) )
#endif
#ifdef _utf8_
# define TXT_LongueurCaractere( t ) TXT_UTF8_LongueurCaractere( t )
#endif

/* ------ Conversions de textes en valeurs numériques -- */

#define TXT_DOUBLE_CHAINE_VIDE (-1.0) /* Chaîne indiquée vide : on renvoie 0, mais on le signale ! */

long          TXT_CNV_vNombre_L (const char *texte, char const **fin_nombre);
unsigned long TXT_CNV_vNombre_UL(const char *texte, char const **fin_nombre);
double        TXT_CNV_vNombre_D (const char *texte, char const **fin_nombre, double *erreur);

int           TXT_DecoderExposant(const char *texte, unsigned long *decalage);

/* ------ Conversions de valeurs en textes ------------- */

#define TXT_AFFICHE_CARACTERE 0	/* %c  : un caractère      [passé comme int] */
#define TXT_AFFICHE_I         1 /* %d  : un entier         [passé comme int] */
#define TXT_AFFICHE_UI        2	/* %u  : un entier >=0     [passé comme unsigned int] */
#define TXT_AFFICHE_L         3 /* %ld : un entier         [passé comme long] */
#define TXT_AFFICHE_UL        4 /* %lu : un entier >=0     [passé comme unsigned long] */
#define TXT_AFFICHE_D         5 /* %f  : un réel           [passé comme double] */
#define TXT_AFFICHE_POURCENT  6 /* %P  : un pourcentage    [passé comme double] : x -> x*100 % */
#define TXT_AFFICHE_ERREUR    7 /* %e  : un réel avec incertitude [passé comme 2 doubles] */
#define TXT_AFFICHE_FICHIER   8 /* %F  : un nom de fichier [passé comme chaîne] */
#define TXT_AFFICHE_CHAINE    9 /* %s  : un texte          [passé comme chaîne] */
#define TXT_AFFICHE_CAR_PC   10	/* %%  : le caractère % */
#define TXT_AFFICHE_COMPLEXE 11 /* %C  : un complexe       [passé comme 2 doubles] */
#define TXT_AFFICHE_INCONNU  1000 /* Autre, ignoré */

#define TXT_CODE_AVEC_SIGNE       1 /* Bit mis : on veut un signe, même si positif */
#define TXT_CODE_ENTIER_IMPOSE    2 /* Bit mis : on impose que la partie entière occupe la place indiquée */
#define TXT_CODE_DECIMALE_IMPOSEE 4 /* Bit mis : on ne veut pas plus de chiffres qu'indiqué, même si on pourrait gagner */
#define TXT_CODE_UNITE_INDIQUEE   8 /* Bit mis : on indique une unité [passée comme chaîne, après la valeur] */
#define TXT_CODE_ALIGNE_A_DROITE 16 /* Bit mis : on impose que le nombre soit aligné à droite */

struct affiche_valeur {
  unsigned int type;		/* Type de conversion. Cf. les #define ci-dessus */
  unsigned int proprietes;	/* Masque de bits. Cf. ci-dessus */
  unsigned int n_max;		/* Nombre maximal de caractères autorisé [0 = illimité] */
  unsigned int lg_avant;	/* Nombre maximal de chiffres avant la virgule [0 = illimité] */
  unsigned int lg_apres;	/* Nombre maximal de chiffres après la virgule */
  unsigned long longueur;	/* Longueur réellement occupée */
};

const char *TXT_AnalyseCodeAffichage(const char *code, struct affiche_valeur *valeurs);

char *TXT_CNV_D_vTexte (double valeur, double erreur, unsigned int avant, unsigned int apres, unsigned int exposant,
			char *resultat, unsigned int *lg_resultat);
char *TXT_CNV_L_vTexte (long valeur, char *resultat, unsigned int *lg_resultat);
char *TXT_CNV_UL_vTexte(unsigned long valeur, char *resultat, unsigned int *lg_resultat);

unsigned long TXT_nbr_car_D (double valeur, struct affiche_valeur *controle);
unsigned long TXT_nbr_car_F (float valeur, struct affiche_valeur *controle);
unsigned long TXT_nbr_car_I (int valeur);
unsigned long TXT_nbr_car_L (long valeur);
unsigned long TXT_nbr_car_UI(unsigned int valeur);
unsigned long TXT_nbr_car_UL(unsigned long valeur);

/* ------ Manipulation de dates ------------------------ */

const char *TXT_NomDeJour(unsigned int jour);
const char *TXT_NomDeMois(unsigned int mois);

#define JANVIER   0 /* Dans struct tm, les mois commencent à 0 ! */
#define FEVRIER   1
#define MARS      2
#define AVRIL     3
#define MAI       4
#define JUIN      5
#define JUILLET   6
#define AOUT      7
#define SEPTEMBRE 8
#define OCTOBRE   9
#define NOVEMBRE 10
#define DECEMBRE 11

#define LUNDI    1
#define MARDI    2
#define MERCREDI 3
#define JEUDI    4
#define VENDREDI 5
#define SAMEDI   6
#define DIMANCHE 7

int TXT_QuelJour(const char *txt_jour);
int TXT_QuelMois(const char *txt_mois);

char *TXT_CreerDate(const char *format, struct tm *date);
int   TXT_AnalyserDate(const char *texte, const char *format, struct tm *date);

/* ------ Manipulation d'intervalles de valeurs -------- */

typedef struct {
  int debut, fin;
} Intervalle_I;

Intervalle_I *TXT_genere_intervalles (const char *chaine, unsigned long *nbr_intervalles);
int           TXT_est_dans_intervalle(int valeur, Intervalle_I *intervalles, unsigned long nbr_intervalles);

/* ------ Manipulation des unités (scientifiques) ------ */

#define UNITE_AUTRE       -2 /* Unité inconnue, complexe,... */
#define UNITE_AUCUNE      -1 /* Aucune unité particulière */

#define UNITE_COMPTAGE     0 /* Nombre de coups, d'objets, de photons,... */

#define UNITE_LONGUEUR     1 /* m */
#define UNITE_MASSE        2 /* kg, mais on utilise dans le logiciel le g par commodité */
#define UNITE_TEMPS        3 /* s */
#define UNITE_INTENSITE    4 /* A, intensité électrique */
#define UNITE_TEMPERATURE  5 /* K */
#define UNITE_QUANTITE     6 /* mol, quantité de matière */
#define UNITE_I_LUMIERE    7 /* cd, intensité lumineuse */
#define UNITE_ANGLE        8 /* rad --- pseudo-unité, car rapport de deux longueurs */
#define UNITE_ANGLE_SOLIDE 9 /* sr (stéradian) */

#define UNITE_ACTIVITE    10 /* Bq (activité radioactive) */
#define UNITE_CAPACITE    11 /* F (capacité électrique) */
#define UNITE_MAGNETIQUE  12 /* T (champ magnétique) */
#define UNITE_CHARGE      13 /* C (charge électrique) */
#define UNITE_CONDUCTANCE 14 /* S (siemens) */
#define UNITE_DOSE        15 /* Gy (dose absorbée) */
#define UNITE_ECLAIREMENT 16 /* Lx (lux, éclairement lumineux) */
#define UNITE_ENERGIE     17 /* J  = m² kg s^-2 */
#define UNITE_FLUX_MAG    18 /* Wb (flux magnétique) */
#define UNITE_FLUX_LUM    19 /* lm (lumen, flux lumineux) */
#define UNITE_FORCE       20 /* N  = m kg s^-2 */
#define UNITE_FREQUENCE   21 /* Hz = s^-1 */
#define UNITE_INDUCTANCE  22 /* H */
#define UNITE_LUMINANCE   23 /* Nt (nit) */
#define UNITE_PRESSION    24 /* Pa = m^-1 kg s^-2 */
#define UNITE_PUISSANCE   25 /* W  = m² kg s^-3 */
#define UNITE_RESISTANCE  26 /* ohm (résistance électrique) */
#define UNITE_TENSION     27 /* V (tension électrique) */
#define UNITE_VERGENCE    28 /* dioptrie = m^-1 */
#define UNITE_SURFACE     29 /* m² */
#define UNITE_VOLUME      30 /* m³ */
#define UNITE_VISCO_DYNAMIQUE 67 /* Pl (poiseuille) = Pa s */

#define UNITE_ACCELERATION        31 /* m/s² */
#define UNITE_ACC_ANGULAIRE       32 /* rad/s² */
#define UNITE_ACTION              33
#define UNITE_AIMANTATION         34
#define UNITE_CAPA_THERM          35 /* J/K */
#define UNITE_CAPA_THERM_MASSIQUE 36 /* J/K/kg */
#define UNITE_CHAMP_ELECTRIQUE    37 /* V/m */
#define UNITE_COEFF_DILATATION    38
#define UNITE_COMPRESSIBILITE     39
#define UNITE_CONDUCTIVITE        40
#define UNITE_CTE_TORSION         41
#define UNITE_DEBIT_MASSIQUE      42
#define UNITE_DEBIT_VOLUMIQUE     43
#define UNITE_DENSITE_COURANT     44
#define UNITE_DIFFUSIVITE         45
#define UNITE_EFF_LUMINEUSE       46
#define UNITE_EMITTANCE           47
#define UNITE_ENTROPIE            UNITE_CAPA_THERM
#define UNITE_EXCITATION_ELEC     49
#define UNITE_EXCITATION_MAG      50
#define UNITE_EXPO_LUMINEUSE      51
#define UNITE_IMPULSION           52
#define UNITE_MASSE_VOLUMIQUE     53 /* kg/m³ */
#define UNITE_MOMENT_FORCE        54 /* m² kg s^-2 = N m */
#define UNITE_MOMENT_CINETIQUE    55
#define UNITE_MOMENT_ELECTRIQUE   56
#define UNITE_MOMENT_INERTIE      57
#define UNITE_PERMEABILITE        58 /* H/m */
#define UNITE_PERMITTIVITE        59 /* F/m */
#define UNITE_PULSATION           UNITE_FREQUENCE /* Difficile de différencier Hz et s^-1 ! */
#define UNITE_QTE_MOUVEMENT       60
#define UNITE_RAIDEUR             61
#define UNITE_RELUCTANCE          62
#define UNITE_RESILIENCE          63
#define UNITE_RESISTIVITE         64
#define UNITE_TENSION_SUPERFICIELLE 65 /* kg s^-2 = N/m */
#define UNITE_VISCO_CINEMATIQUE     66
#define UNITE_VITESSE               68 /* m/s */
#define UNITE_VITESSE_ANGULAIRE     69 /* rad/s */
#define UNITE_VOLUME_MASSIQUE       70 /* m³/kg */

#define UNITE_MASSE_LINEIQUE        71 /* kg/m */
#define UNITE_MASSE_SURFACIQUE      72 /* kg/m² */
#define UNITE_SECTION_EFFICACE      UNITE_SURFACE
#define UNITE_CONCENTRATION         74
#define UNITE_EQUIVALENT_DOSE       75 /* Sv */
#define UNITE_NOMBRE_ONDE           UNITE_VERGENCE /* cm^-1 plutôt, mais... */

double TXT_NormaliserUnite(const char *unite, int complete, int *type_unite, char *unite_simplifiee);
char  *TXT_ProduitUnites  (const char *unite_avant, const char *unite_produit, int exposant);
int    TXT_TypeUnite      (const char *unite);

int    TXT_UniteEstSimple (const char *unite, int *exposant);
int    TXT_PuissanceUnite (char **u_simples, int *exposants, unsigned int n_termes, int exposant);

unsigned int TXT_DecomposerUnite(const char *unite, char ***monomes, int **exposants);
char        *TXT_RecomposerUnite(char **unites_simples, int *exposants, unsigned int nbr_termes);

#endif /* TXT_UTILS_H_ */
